#pragma once
#include "StdAfx.h"
#include "icVector.h"
#include "EdgeFieldManipulator.h"
#include <sstream>

using namespace std;


EdgeFieldManipulator::EdgeFieldManipulator() {
	poly = NULL;
	psf = NULL;

	weight_interpolate = .5;
	current_rotation_radian = 0;
	set_rotation_degree = 0;
	init_edgefld_option = 0;
}


void EdgeFieldManipulator::setMesh(Polyhedron &poly) {
	this->poly = &poly;
}


void EdgeFieldManipulator::releaseMesh() {
	this->poly = NULL;
}


void EdgeFieldManipulator::setStyleField(PaintingStyleField &psf) {
	this->psf = &psf;
}


void EdgeFieldManipulator::releaseStyleField() {
	this->psf = NULL;
}


void EdgeFieldManipulator::setWeight(double weight) {
	weight_interpolate = weight;
}


double EdgeFieldManipulator::getWeight() {
	return weight_interpolate;
}


void EdgeFieldManipulator::setRotationAngle(double rot_degree) {
	set_rotation_degree = rot_degree;
}


double EdgeFieldManipulator::getRotationAngle() {
	//return rotation_degree;
	return current_rotation_radian * 180. / M_PI;
}


void EdgeFieldManipulator::rotateField() {
	double old_rotation_radian = current_rotation_radian;


	if(poly == NULL) return;


	current_rotation_radian = set_rotation_degree * M_PI / 180.;

	poly->global_rotate_field(old_rotation_radian - current_rotation_radian);
}


void EdgeFieldManipulator::reflectField() {
	poly->global_reflect_field();
}


// Just smooth edge field... no field analysis is performed since this might be blended with design elements or rotated...
void EdgeFieldManipulator::smoothField(int num_iterations, double strong_vertex_ratio) {
	int i, j, k;
	int num_verts = poly->nverts;
	static const int num_bins = 200;
	unsigned int bins[num_bins];
	unsigned int which_bin, till_now = 0;
	unsigned int num_verts_threshold;
	double threshold;
	double timesize = 0.1;
	double scaled_length;

	Vertex *v;


	if(poly == NULL) return;
	
	
	// find all tensors with a 'strong' value
	if(poly->vec_ten_mode == 0) {
		double max_len = 0.0;
		
		for(i = 0; i < num_verts; i++) {
			v = poly->vlist[i];
			v->status = 0;  // cannot modify
			v->value = length(icVector2(v->vec_fld.entry[0], v->vec_fld.entry[1]));
			
			if(v->value > max_len)
				max_len = v->value;
		}

		memset(bins, 0, sizeof(unsigned int) * num_bins);

		for(i = 0; i < num_verts; i++) {
			v = poly->vlist[i];

			which_bin = (int)floor(v->value * 100 / max_len);
			
			if(which_bin < 0)
				which_bin = 0;
			
			if(which_bin > num_bins - 1)
				which_bin = num_bins - 1;
			
			bins[which_bin]++;
		}

		num_verts_threshold = (unsigned int)(num_verts * strong_vertex_ratio);
		scaled_length = max_len / 100.;

		for(i = num_bins - 1; i >= 0; i--) {
			till_now += bins[i];

			if(till_now >= num_verts_threshold) {      // only when for heart is 0.04
				threshold = i * scaled_length;
				break;
			}
		}

		for(i = 0; i < num_verts; i++) {
			v = poly->vlist[i];

			if(v->value > threshold) {
				v->status = 0;
			}
			else {
				v->status = 1;
			}
		}

		for(k = 0; k < num_iterations; k++) {   // use 300 for heart
			for(i = 0; i < num_verts; i++) {
				v = poly->vlist[i];
				v->old_vec_fld = v->vec_fld;
			}

			for(i = 0; i < num_verts; i++) {
				Vertex *temp_v = poly->vlist[i], *temp_other;
				double coeff, total_coeff = 0.0;
				temp_v->vec_fld.set(0.0);

				for(j = 0; j < temp_v->adjacent_edges->num_edges; j++) {
					Edge *temp_e = temp_v->adjacent_edges->edges[j];
					
					if(temp_e->verts[0] == temp_v) {
						coeff = temp_e->floater_lamda[0];
						temp_other = temp_e->verts[1];
					}
					else {
						coeff = temp_e->floater_lamda[1];
						temp_other = temp_e->verts[0];
					}

					temp_v->vec_fld += coeff * (temp_other->old_vec_fld - temp_v->old_vec_fld);
					total_coeff += coeff;
				}

				temp_v->vec_fld *= timesize / total_coeff;
				temp_v->vec_fld += temp_v->old_vec_fld;
			}

			for(i = 0; i < num_verts; i++) {
				v = poly->vlist[i];

				if(v->status == 0) v->vec_fld = v->old_vec_fld;
			}
		}
	}
	else {
		double max_len = 0.0;

		for(i = 0; i < num_verts; i++) {
			v = poly->vlist[i];
			v->status = 0;  // cannot modify
			v->value = length(icVector2(v->ten_fld.entry[0][0], v->ten_fld.entry[0][1]));
			if(v->value > max_len)
				max_len = v->value;
		}

		memset(bins, 0, sizeof(unsigned int) * num_bins);

		for(i = 0; i < num_verts; i++) {
			v = poly->vlist[i];
			which_bin = (int)floor(v->value * 100 / max_len);

			if(which_bin < 0)
				which_bin = 0;

			if(which_bin > num_bins - 1)
				which_bin = num_bins - 1;

			bins[which_bin]++;
		}


		num_verts_threshold = (unsigned int)(num_verts * strong_vertex_ratio);
		scaled_length = max_len / 100.;

		for(i = num_bins - 1; i >= 0; i--) {
			till_now += bins[i];

			if(till_now >= num_verts_threshold) {   // only when for heart is 0.04
				threshold = i * scaled_length;
				break;
			}
		}

		// mark vertices of 'strong' tensor
		for(i = 0; i < num_verts; i++) {
			v = poly->vlist[i];

			if(v->value > threshold) {
				v->status = 0;		// cannot modify
			}
			else {
				v->status = 1;		// can be modified...
			}
		}

		// smooth field
		for (k = 0; k < num_iterations; k++) {   // use 300 for heart
			for(i = 0; i < num_verts; i++) {
				v = poly->vlist[i];
				v->old_ten_fld = v->ten_fld;
			}

			for(i = 0; i < poly->nverts; i++) {
				Vertex *temp_v = poly->vlist[i], *temp_other;
				double coeff, total_coeff = 0.0;
				temp_v->ten_fld.set(0.0);

				for(j = 0; j < temp_v->adjacent_edges->num_edges; j++) {
					Edge *temp_e = temp_v->adjacent_edges->edges[j];
					
					if(temp_e->verts[0] == temp_v) {
						coeff = temp_e->floater_lamda[0];
						temp_other = temp_e->verts[1];
					}
					else {
						coeff = temp_e->floater_lamda[1];
						temp_other = temp_e->verts[0];
					}

					temp_v->ten_fld += coeff * (temp_other->old_ten_fld - temp_v->old_ten_fld);
					total_coeff += coeff;
				}

				temp_v->ten_fld *= timesize / total_coeff;
				temp_v->ten_fld += temp_v->old_ten_fld;
			}

			for(i = 0; i < poly->nverts; i++) {
				v = poly->vlist[i];

				if(v->status == 0) v->ten_fld = v->old_ten_fld;
			}
		}
	}
}


// normalization is performed before field analysis.
void EdgeFieldManipulator::normalizeField() {
	int num_verts = poly->nverts;
	double temp_len;
	Vertex *v;


	if(poly == NULL) return;

	if(poly->vec_ten_mode == 0) {
		for(int i = 0; i < num_verts; i++) {
			v = poly->vlist[i];

			temp_len = length(icVector2(v->vec_fld.entry[0], v->vec_fld.entry[1]));

			if(temp_len > 0.0) v->vec_fld *= 1.0 / temp_len;
		}
	}
	else {
		for(int i = 0; i < num_verts; i++) {
			v = poly->vlist[i];

			temp_len = length(icVector2(v->ten_fld.entry[0][0], v->ten_fld.entry[0][1]));
			
			if(temp_len > 0.0) v->ten_fld *= 1.0 / temp_len;
		}
	}
}


int EdgeFieldManipulator::addSink(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 0, 0, x, y, region_id, time);

	// add a global list
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addSource(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 0, 1, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addSaddle(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 0, 2, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addCCWCenter(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 0, 3, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addCWCenter(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 0, 4, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addDipole1(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;
	
	se = new SingularElement(poly->vec_ten_mode, 1, 0, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addDipole2(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 1, 1, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


int EdgeFieldManipulator::addMonkeySaddle(double x, double y, int time, int region_id) {
	SingularElement *se;


	if(poly == NULL || psf == NULL) return -1;

	se = new SingularElement(poly->vec_ten_mode, 1, 2, x, y, region_id, time);
	poly->singularelements->append(se);

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	// recompute tensor field
	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return poly->singularelements->num_singularelements - 1;
}


void EdgeFieldManipulator::editSE(double x, double y, int se_id, int edit_mode, int time) {
	float mag_ratio = 0.04f;
	double new_angle = 0;
	double proj = 0;
	double proj_min = .2;
	icVector2 temp_v, temp_dir;
	SingularElement *se = NULL;


	if(poly == NULL || psf == NULL) return;

	if(0 <= se_id && se_id < poly->singularelements->num_singularelements) {
		se = poly->singularelements->singularelements[se_id];

		switch(edit_mode) {
			case 0:
				se->Update_Position(x, y, 0.0);
				break;

			case 1:
				new_angle = atan2(y - se->center.entry[1], x - se->center.entry[0]) - PI/2;
				se->Update_Angle(new_angle);
				break;

			case 2:  // right side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = se->max_dir;
				proj = dot(temp_v, temp_dir) / mag_ratio;
				
				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(proj, se->min_mag);
				break;

			case 3:  // left side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = -se->max_dir;
				proj = dot(temp_v, temp_dir) / mag_ratio;

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(proj, se->min_mag);
				break;

			case 4:  // up side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = se->min_dir;
				proj = dot(temp_v, temp_dir) / mag_ratio;

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(se->max_mag, proj);
				break;

			case 5:  // down side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = -se->min_dir;
				proj = dot(temp_v, temp_dir) / mag_ratio;

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(se->max_mag, proj);
				break;

			case 6:  // up-right side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = se->min_dir * se->min_mag + se->max_dir * se->max_mag;
				normalize(temp_dir);
				proj = dot(temp_v, temp_dir) / mag_ratio;
				proj *= se->max_mag / sqrt(se->max_mag * se->max_mag + se->min_mag * se->min_mag);

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(proj, proj * se->min_mag / se->max_mag);
				break;

			case 7:  // up-left side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = se->min_dir * se->min_mag - se->max_dir * se->max_mag;
				normalize(temp_dir);
				proj = dot(temp_v, temp_dir)/mag_ratio;
				proj *= se->max_mag / sqrt(se->max_mag * se->max_mag + se->min_mag * se->min_mag);

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(proj, proj * se->min_mag / se->max_mag);
				break;

			case 8:  // lower-left side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = -se->min_dir * se->min_mag - se->max_dir * se->max_mag;
				normalize(temp_dir);
				proj = dot(temp_v, temp_dir)/mag_ratio;
				proj *= se->max_mag / sqrt(se->max_mag * se->max_mag + se->min_mag * se->min_mag);

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(proj, proj * se->min_mag / se->max_mag);
				break;

			case 9:  // lower-right side scale max_dir
				temp_v.set(x - se->center.entry[0], y - se->center.entry[1]);
				temp_dir = -se->min_dir * se->min_mag + se->max_dir * se->max_mag;
				normalize(temp_dir);
				proj = dot(temp_v, temp_dir) / mag_ratio;
				proj *= se->max_mag / sqrt(se->max_mag * se->max_mag + se->min_mag * se->min_mag);

				if(proj < proj_min) proj = proj_min;
				se->Update_Magnitude(proj, proj * se->min_mag / se->max_mag);
				break;
		}

		// propagate design elements
		psf->propagateDesignElements(poly->singularelements, poly->regularelements);

		// recompute tensor field
		interpolateField_noAnalyze(time, -1, init_edgefld_option);
		normalizeField();
		poly->global_rotate_field(current_rotation_radian);
	}
}


int EdgeFieldManipulator::addRegularElement(double x, double y, int time, int region_id) {
	RegularElement *re;


	if(poly == NULL || psf == NULL) return -1;

	re = new RegularElement(x, y, region_id, time);
	poly->regularelements->append(re);

	// don't interpolate nor rotate...

	return poly->regularelements->num_regularelements - 1;
}


void EdgeFieldManipulator::editRE(double x, double y, int re_id, int edit_mode, int time) {
	RegularElement *re;


	if(poly == NULL) return;


	if(0 <= re_id && re_id < poly->regularelements->num_regularelements) {
		re = poly->regularelements->regularelements[re_id];

		switch(edit_mode) {
			case 0:
				re->Update_Start_Position(x, y);
				break;

			case 1:
				re->Update_End_Position(x, y);
				break;
		}

		// propagate design elements
		psf->propagateDesignElements(poly->singularelements, poly->regularelements);

		// recompute tensor field
		interpolateField_noAnalyze(time, -1, init_edgefld_option);
		normalizeField();
		poly->global_rotate_field(current_rotation_radian);
	}
}


void EdgeFieldManipulator::selectElement(double x, double y, int &se_id, int &re_id, int time) {
	double diff_x, diff_y, current_dist, prev_dist;
	const SingularElement *se;
	const RegularElement *re;
	SingularElementList *selist = poly->singularelements;
	RegularElementList *relist = poly->regularelements;
	list<SingularElement>::const_iterator itr_se, itr_se_end;
	list<RegularElement>::const_iterator itr_re, itr_re_end;


	vector<const DEInfo*> design_element;
	

	se_id = -1;
	re_id = -1;

	if(poly == NULL || psf == NULL) return;


	psf->getDEInfo(design_element, time);
	prev_dist = 1.0e-3;
	se = NULL;

	// search singular elements
	for(size_t i = 0; i < design_element.size(); i++) {
		if(design_element[i] == NULL) continue;

		itr_se = design_element[i]->se.kf.begin();
		itr_se_end = design_element[i]->se.kf.end();

		while(itr_se != itr_se_end) {
			diff_x = x - itr_se->x;
			diff_y = y - itr_se->y;
			current_dist = diff_x * diff_x + diff_y * diff_y;

			if(current_dist < prev_dist) {
				se = &(*itr_se);
				prev_dist = current_dist;
			}

			itr_se++;
		}
	}

	if(se != NULL) {
		if((se = se->ref) != NULL) {
			for(int i = 0; i < selist->num_singularelements; i++) {
				if(se == selist->singularelements[i]) {
					se_id = i;
					return;
				}
			}
		}
	}


	prev_dist = 1.0e-3;
	re = NULL;

	// search regular elements
	for(size_t i = 0; i < design_element.size(); i++) {
		if(design_element[i] == NULL) continue;

		itr_re = design_element[i]->re.kf.begin();
		itr_re_end = design_element[i]->re.kf.end();

		while(itr_re != itr_re_end) {
			diff_x = x - itr_re->x;
			diff_y = y - itr_re->y;
			current_dist = diff_x * diff_x + diff_y * diff_y;

			if(current_dist < prev_dist) {
				re = &(*itr_re);
				prev_dist = current_dist;
			}

			itr_re++;
		}
	}

	if(re != NULL) {
		if((re = re->ref) != NULL) {
			for(int i = 0; i < relist->num_regularelements; i++) {
				if(re == relist->regularelements[i]) {
					re_id = i;
					return;
				}
			}
		}
	}
}


void EdgeFieldManipulator::selectAnalyticalElement(double x, double y, int &s_id) {
	double diff_x, diff_y, current_dist, prev_dist;
	icVector2 vec3;
	SingularityList *slist;
	Singularity *singularity;
	Triangle *tri;


	if(poly == NULL) return;

	s_id = -1;
	prev_dist = 1.0e-3;

	slist = poly->singularities;

	for(int i = 0; i < slist->num_singularities; i++) {
		singularity = slist->singularities[i];
		tri = singularity->tri;
		tri->convert_local_to_global(singularity->pos, &vec3);
		diff_x = x - vec3.entry[0];
		diff_y = y - vec3.entry[1];
		current_dist = diff_x * diff_x + diff_y * diff_y;

		if(current_dist < prev_dist) {
			s_id = i;
			prev_dist = current_dist;
		}
	}
}


int EdgeFieldManipulator::determine_within_box(icVector2 center, icVector2 min_dir, icVector2 max_dir, double min_mag, double max_mag, icVector2 pos) {
	icVector2 temp = pos - center;
	double x = fabs(dot(temp, min_dir));
	double y = fabs(dot(temp, max_dir));

	if(x < min_mag && y < max_mag) return 1;
	else return 0;
}


void EdgeFieldManipulator::determineSEEditMode(double x, double y, int &se_id, int &edit_mode) {
	icVector2 temp;
	SingularElement *se;


	if(poly != NULL && 0 <= se_id && se_id < poly->singularelements->num_singularelements) {
		temp.set(x, y);
		edit_mode = -1;

		se = poly->singularelements->singularelements[se_id];

		if(determine_within_box(se->center, se->min_dir, se->max_dir, se->min_mag*mag_ratio, se->max_mag*mag_ratio, temp) == 1) {
			edit_mode = 0;
		}
		else if(determine_within_box(se->rotate_center, se->min_dir, se->max_dir, 0.15*mag_ratio, 0.15*mag_ratio, temp) == 1) {
			edit_mode = 1;
		}
		else {
			for(int i = 0; i < 8; i++) {
				if(determine_within_box(se->stretch_center[i], se->min_dir, se->max_dir, 0.15*mag_ratio, 0.15*mag_ratio, temp) == 1) {
					edit_mode = i + 2;
					break;
				}
			}
		}

		if(edit_mode == -1) {
			se_id = -1;
		}
	}
	else {
		se_id = -1;
		edit_mode = -1;
	}
}


void EdgeFieldManipulator::determineREEditMode(double x, double y, int &re_id, int &edit_mode) {
	icVector2 temp;
	RegularElement *re;


	if(poly != NULL && 0 <= re_id && re_id < poly->regularelements->num_regularelements) {
		temp.set(x, y);
		edit_mode = -1;

		re = poly->regularelements->regularelements[re_id];

		if(determine_within_box(re->end_center, icVector2(1.0, 0.0), icVector2(0.0, 1.0), 0.15*mag_ratio, 0.15*mag_ratio, temp) == 1) {
			edit_mode = 1;
		}
		else if(determine_within_box(re->start_center, icVector2(1.0, 0.0), icVector2(0.0, 1.0), 0.15*mag_ratio, 0.15*mag_ratio, temp) == 1) {
			edit_mode = 0;
		}

		if(edit_mode == -1) {
			re_id = -1;
		}
	}
	else {
		re_id = -1;
		edit_mode = -1;
	}
}


bool EdgeFieldManipulator::removeElement(double x, double y, int time) {
	int se_id = -1, re_id = -1;
	int region_id;
	SingularElementList *selist = poly->singularelements;
	RegularElementList *relist = poly->regularelements;


	if(poly == NULL || psf == NULL) return false;

	selectElement(x, y, se_id, re_id, time);

	if(se_id >= 0) {
		region_id = selist->singularelements[se_id]->regionID;

		if(se_id < selist->num_singularelements - 1) {
			delete selist->singularelements[se_id];

			selist->singularelements[se_id] = selist->singularelements[selist->num_singularelements - 1];
			selist->singularelements[selist->num_singularelements - 1] = NULL;
		}
		else {
			delete selist->singularelements[se_id];

			selist->singularelements[se_id] = NULL;
		}

		selist->num_singularelements--;

		// propagate design elements
		psf->propagateDesignElements(poly->singularelements, poly->regularelements);

		// recompute tensor field
		interpolateField_noAnalyze(time, -1, init_edgefld_option);
		normalizeField();
		poly->global_rotate_field(current_rotation_radian);

		return true;
	}
	else if(re_id >= 0) {
		region_id = relist->regularelements[se_id]->regionID;

		if(re_id < relist->num_regularelements - 1) {
			delete relist->regularelements[se_id];

			relist->regularelements[re_id] = relist->regularelements[relist->num_regularelements - 1];
			relist->regularelements[relist->num_regularelements - 1] = NULL;
		}
		else {
			delete relist->regularelements[re_id];

			relist->regularelements[re_id] = NULL;
		}

		relist->num_regularelements--;

		// propagate design elements
		psf->propagateDesignElements(poly->singularelements, poly->regularelements);

		// recompute tensor field
		interpolateField_noAnalyze(time, -1, init_edgefld_option);
		normalizeField();
		poly->global_rotate_field(current_rotation_radian);

		return true;
	}
	else {
		return false;
	}
}


bool EdgeFieldManipulator::cancelSingularityPair(int s_id1, int s_id2) {
	if(poly == NULL) return false;


	if(0 <= s_id1 && s_id1 < poly->singularities->num_singularities && 0 <= s_id2 && s_id2 < poly->singularities->num_singularities) {
		poly->singularity_pair_cancellation_field(s_id1, s_id2);

		return true;
	}
	else {
		return false;
	}
}


bool EdgeFieldManipulator::moveSingularity(int s_id, int t_id) {
	if(poly == NULL) return false;


	if(0 <= s_id && s_id < poly->singularities->num_singularities && 0 <= t_id && t_id < poly->ntris) {
		poly->singularity_movement_field(s_id, t_id);

		return true;
	}
	else {
		return false;
	}
}


void EdgeFieldManipulator::undo() {
	if(poly != NULL) poly->undo();
}


void EdgeFieldManipulator::redo() {
	if(poly != NULL) poly->redo();
}


void EdgeFieldManipulator::setDesignElements(SingularElementList *selist, RegularElementList *relist, int time) {
	if(poly != NULL) {
		poly->setSingularElementList(selist);
		poly->setRegularElementList(relist);
		
		interpolateField_noAnalyze(time, -1, init_edgefld_option);
		normalizeField();
		poly->global_rotate_field(current_rotation_radian);
	}
}


void EdgeFieldManipulator::getDesignElements(SingularElementList *&selist, RegularElementList *&relist) {
	if(poly != NULL) {
		poly->copySingularElementList(&selist);
		poly->copyRegularElementList(&relist);
	}
}


void EdgeFieldManipulator::smoothRegionInField() {
	if(poly != NULL) {
		if(poly->region_verts->num_verts <= 2) return;

		poly->smooth_field();
	}
}


void EdgeFieldManipulator::interpolateField(int time, int region_id) {
	if(poly != NULL || psf == NULL) {
		interpolateField_noAnalyze(time, region_id, init_edgefld_option);
		normalizeField();
		poly->global_rotate_field(current_rotation_radian);
	}
}


void EdgeFieldManipulator::interpolateField_noAnalyze(int time, int region_id, int set_zero) {
	vector<const DEInfo*> de_regions;

	if(poly == NULL || psf == NULL) return;

	// retrieve all design elements defined on each region.
	psf->getDEInfo(de_regions, time);

	if(poly->vec_ten_mode == 0) {
		poly->update_vecfld(region_id, set_zero, de_regions);
	}
	else {
		poly->update_tenfld(region_id, set_zero, weight_interpolate, de_regions);
	}
}


void EdgeFieldManipulator::resetField(int time) {
	if(poly == NULL || psf == NULL) return;

	poly->singularelements->finalize();
	poly->regularelements->finalize();
	poly->singularities->finalize();
	poly->singularelements = new SingularElementList(20);
	poly->regularelements = new RegularElementList(20);
	poly->singularities = new SingularityList(20);


	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	interpolateField_noAnalyze(time, -1, init_edgefld_option);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	poly->calc_new_texture_coors();
}


// clear up all design elements within a specified region. 
void EdgeFieldManipulator::zeroField(int time, int region_id) {
	SingularElement *se;
	RegularElement *re;
	SingularElementList *se_list = poly->singularelements;
	RegularElementList *re_list = poly->regularelements;

	if(poly == NULL || psf == NULL) return;

	for(int i = 0; i < se_list->num_singularelements; ) {
		if(se_list->singularelements[i]->regionID == region_id || region_id == -1) {
			delete se_list->singularelements[i];

			// move the last element to the empty spot
			if(i != se_list->num_singularelements - 1) {
				se = se_list->singularelements[se_list->num_singularelements - 1];
				se_list->singularelements[i] = se;
			}

			se_list->num_singularelements--;
		}
		else {
			i++;
		}
	}

	for(int i = 0; i < re_list->num_regularelements; ) {
		if(re_list->regularelements[i]->regionID == region_id || region_id == -1) {
			delete re_list->regularelements[i];

			// move the last element to the empty spot
			if(i != re_list->num_regularelements - 1) {
				re = re_list->regularelements[re_list->num_regularelements - 1];
				re_list->regularelements[i] = re;
			}

			re_list->num_regularelements--;
		}
		else {
			i++;
		}
	}

	// propagate design elements
	psf->propagateDesignElements(poly->singularelements, poly->regularelements);

	interpolateField_noAnalyze(time, region_id, 1);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	poly->calc_new_texture_coors();
}


void EdgeFieldManipulator::clearSelectedRegion() {
	if(poly != NULL) {
		poly->region_verts->finalize();
		poly->region_boundary->finalize();
		poly->region_verts = new VertexList(20);
		poly->region_boundary = new CornerList(20);
		
		for(int i = 0; i < poly->ntris; i++) {
			poly->tlist[i]->clicked = 0;
		}
	}
}


void EdgeFieldManipulator::updateSelectedRegion(int selected_t_id) {
	Triangle *t;

	if(0 <= selected_t_id && selected_t_id < poly->ntris) {
		t = poly->tlist[selected_t_id];

		t->clicked = 1;
		poly->region_verts->append(t->verts[0]);
	}
}


bool EdgeFieldManipulator::replaceField(string &filename) {
	/*
	int num_verts, num_data, num_itr;
	char *content;
	long start, end;
	stringstream *buffer;
	Vertex *v = NULL;
	FILE *fp;

	if(poly == NULL || poly->vec_ten_mode == 0 || (fp = fopen(filename.c_str(), "r")) == NULL) return false;


	// Clear out all design elements
	poly->region_verts->finalize();
	poly->region_boundary->finalize();
	poly->region_verts = new VertexList(20);
	poly->region_boundary = new CornerList(20);

	fscanf(fp, "%d\n", &num_data);
	start = ftell(fp);

	fseek(fp, 0, SEEK_END);
	end = ftell(fp);

	fseek(fp, start, SEEK_SET);
	content = new char[end - start];

	fread(content, sizeof(char), end - start, fp);
	fclose(fp);

	num_verts = poly->nverts;
	num_itr = min(num_verts, num_data);


	buffer = new stringstream(content);

	// Save them in each vertex...
	for(int i = 0; i < num_itr; i++) {
		v = poly->vlist[i];
		*buffer >> v->ten_fld_painting.entry[0][0];
		*buffer >> v->ten_fld_painting.entry[0][1];
		*buffer >> v->ten_fld_painting.entry[1][0];
		*buffer >> v->ten_fld_painting.entry[1][1];
		*buffer >> v->vec_fld_painting.entry[0];
		*buffer >> v->vec_fld_painting.entry[1];
	}

	delete[] content;
	delete buffer;

	for(int i = num_itr; i < num_verts; i++) {
		v = poly->vlist[i];
		v->ten_fld.set(0.0);
		v->vec_fld.set(0.0);
	}

	poly->update_tenfld(0, 1.);
	normalizeField();
	poly->global_rotate_field(current_rotation_radian);

	return true;
	*/

	return false;
}


bool EdgeFieldManipulator::exportField(string &filename) {
	int num_verts;
	Vertex *v = NULL;
	FILE *fp;

	if(poly == NULL || (fp = fopen(filename.c_str(), "w")) == NULL) return false;


	num_verts = poly->nverts;

	fprintf(fp, "%d\n", num_verts);

	// Save them in each vertex...
	for(int i = 0; i < num_verts; i++) {
		v = poly->vlist[i];

		fprintf(fp, "%f %f %f %f %f %f\n", 
			v->ten_fld.entry[0][0], v->ten_fld.entry[0][1], v->ten_fld.entry[1][0], v->ten_fld.entry[1][0], 
			v->vec_fld.entry[0], v->vec_fld.entry[1]
			);
	}

	fclose(fp);

	return true;
}


void EdgeFieldManipulator::setInitEdgeFieldOption(int mode) {
	switch(mode) {
		case 0:
		default:
			init_edgefld_option = 0;
			break;

		case 1:
			init_edgefld_option = 1;
			break;
	}
}
