#ifndef __GLSLMANAGER__
#define __GLSLMANAGER__

#pragma once
#pragma warning( disable : 4996 )
#include <afxwin.h>
#include <math.h>
#include <vector>
#include <string>

#include "GL/glew.h"

#include "GLSLUtil.h"
#include "ShaderObject.h"
#include "ShaderProgram.h"
#include "RenderTexture.h"

using namespace std;

inline char* BUFFER_OFFSET( size_t i ) { return static_cast< char* >( NULL ) + i; }

struct GLSLVertex
{
	float x, y;
	float u, v;
};

class QuadVBO
{
public:
	QuadVBO(float width=1.f, float height=1.f, float center=0.f);
	~QuadVBO();

	void Draw(bool flip=false);
private:
	GLuint BufferId[2];
};

class GLSLManager
{
public:
	//static void Initialize(void);
	static void Initialize(string &base_path);
	static void Shutdown(void);

	static void SetRenderTarget(void);
	static void SetRenderTarget(RenderTexture* NewTarget);
	static RenderTexture* GetCurrentRenderTarget();

	static void SetShader(void);
	static void SetShader(const string &VSName, const string &GSName, const string &PSName);

	static void	ResetTextures();

	static void SetTextureToIdx(unsigned int texID, GLuint hTexture);
	static void SetTextureToIdx(unsigned int texID, GLuint hTexture, GLint min_mag_filter);
	
	static GLuint GetCurrentProgram();

private:
	static ShaderObject* CompileShader(const string &VSName);
	static vector<ShaderObject*>  ShaderObjectList;
	static vector<ShaderProgram*> ShaderProgramList;

	static void	printShaderInfoLog(GLuint obj);
	static void	printProgramInfoLog(GLuint obj);

	static ShaderProgram* CurrentProgram;
	static ShaderProgram* NullProgram;

	static FILE* fp;
	static string base_dir;
	static string shader_dir;
	static string logfile_path;
	static char tmp_current_dir[MAX_PATH];

	static RenderTexture *current_rt;
};
#endif