// MFCTEST4Dlg.cpp : t@C
//

#pragma once
#include "stdafx.h"
#include "MFCTEST4.h"
#include "MFCTEST4Dlg.h"
#include "ppm.h"
#include "OpticalFlow.h"
#include <sstream>
#include <fstream>
#include <iomanip>
#include "icVector.h"
#include "icMatrix.h"
#include <list>
#include "SimpleStroke.h"
#include "PaintingPreset.h"
#include ".\mfctest4dlg.h"
#include "ScriptProcessor.h"

#include "QDImageTools.h"

#define USE_OMP 1

#ifndef _OPENMP
#define USE_OMP 0
#endif

#if USE_OMP == 1
#include <omp.h>
#endif


using namespace std;


#ifdef _DEBUG
#define new DEBUG_NEW
#define   malloc(s)         _malloc_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__)
#endif



#define WINDOW_TITLE _T("IBFV Painting Program - ")



#define GL_WINDOW_W 512
#define GL_WINDOW_H 512
//#define GL_WINDOW_W 1024
//#define GL_WINDOW_H 1024


CMFCTEST4Dlg::CMFCTEST4Dlg(CWnd* pParent /*=NULL*/) : CDialog(CMFCTEST4Dlg::IDD, pParent), DatPath(NULL)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	the_view = NULL;
	input_video = NULL;
	input_image = NULL;
	input_width = -1;
	input_height = -1;
	total_render_time = 0;
	total_run_time = 0;

	constraint_angle = (float)degreeToRadian(30.);

	for(int i = 0; i < numTabs; i++) tabContents[i] = NULL;
}


CMFCTEST4Dlg::~CMFCTEST4Dlg() {
	list<FieldSampleImage*>::iterator it_fsi;
	list<IplImage*>::iterator it_ipl;

	if(DatPath != NULL) delete[] DatPath;

	for(int i = 0; i < numTabs; i++) delete tabContents[i];
	delete the_view;

	for(it_fsi = fsi_list.begin(); it_fsi != fsi_list.end(); it_fsi++) {
		if(*it_fsi != NULL) (*it_fsi)->finalize();
	}

	for(it_ipl = frame_list.begin(); it_ipl != frame_list.end(); it_ipl++) {
		if(*it_ipl != NULL) cvReleaseImage(&(*it_ipl));
	}

	if(old_fsi != NULL) {
		old_fsi->finalize();
	}


	delete[] stroke_counts[0];
	delete[] stroke_counts[1];

	delete[] binary_map;

	disposeCVMatKernel();
}

void CMFCTEST4Dlg::DoDataExchange(CDataExchange* pDX) {
	CDialog::DoDataExchange(pDX);
}


LPCTSTR CMFCTEST4Dlg::tabNames[] = {
	_T("Field Design / Display"), 
	_T("Painting"), 
	_T("Debugging"), 
	_T("Segmentation"),
	_T("Optical Flow"),
};


BEGIN_MESSAGE_MAP(CMFCTEST4Dlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
	ON_WM_SIZE()
	ON_NOTIFY(TCN_SELCHANGE, IDC_TAB1, OnTcnSelchangeTab1)
	ON_WM_DESTROY()
	ON_COMMAND(ID_FILE_QUIT, OnFileQuit)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_COMMAND(IDC_UNDO, OnUndo)
	ON_COMMAND(IDC_REDO, OnRedo)
	ON_COMMAND(IDC_SAVE_CONTROL, OnFileSaveControl)
	ON_COMMAND(IDC_LOAD_CONTROL, OnFileLoadControl)
	ON_COMMAND(IDC_SHOW_CTL_KNOB, OnShowCtlKnob)
	ON_COMMAND(IDC_SHOW_SEPARATRICES, OnShowSeparatrices)
	ON_COMMAND(IDC_SHOW_SINGULARITIES, OnShowSingularities)
	ON_BN_CLICKED(IDC_FORWARD, OnBnClickedForward)
	ON_BN_CLICKED(IDC_BACK, OnBnClickedBack)
	ON_BN_CLICKED(IDC_FF, OnBnClickedFf)
	ON_BN_CLICKED(IDC_REW, OnBnClickedRew)
	ON_COMMAND(ID_FILE_SAVEIMAGE, OnFileSaveImage)
	ON_COMMAND(ID_FILE_SAVEVIDEO, OnFileSaveVideo)
	ON_EN_KILLFOCUS(IDC_VIDEO_CURRENT_FRAME, OnEnKillfocusVideoCurrentFrame)
	ON_BN_CLICKED(IDC_TEMPORAL_SMOOTHING, OnBnClickedTemporalSmoothing)
	ON_BN_CLICKED(IDC_ENABLE_STROKE_TRACKING, OnBnClickedEnableStrokeTracking)
	ON_COMMAND(ID_FILE_LOADOPTICALFLOW, OnFileLoadOpticalFlow)
	ON_COMMAND(ID_FILE_UNLOADOPTICALFLOW, OnFileUnoadOpticalFlow)
	ON_COMMAND(ID_FILE_RUNSCRIPT, OnFileRunScript)
	ON_BN_CLICKED(IDC_PRINT_STAT, OnBnClickedPrintMessage)
	ON_BN_CLICKED(IDC_JUMP_NEXT_KEY, OnBnClickedJumpNextKey)
	ON_BN_CLICKED(IDC_JUMP_PREV_KEY, OnBnClickedJumpPrevKey)
	ON_BN_CLICKED(IDC_JUMP_NEXT_KEY2, OnBnClickedJumpNextKey2)
	ON_BN_CLICKED(IDC_JUMP_PREV_KEY2, OnBnClickedJumpPrevKey2)
	ON_COMMAND(ID_EDIT_CLEARMESSAGEBOX, OnEditClearMessageBox)
	ON_COMMAND(ID_FILE_LOADBINARYMAP, OnFileLoadBinaryMap)
	ON_COMMAND(ID_FILE_UNLOADBINARYMAP, OnFileUnloadBinaryMap)
	ON_COMMAND(ID_LOAD_EDGEFIELD, OnFileLoadEdgeField)
	ON_COMMAND(ID_SAVE_EDGEFIELD, OnFileSaveEdgeField)
	ON_COMMAND(ID_LOAD_VIDEOPROFILE, OnFileLoadVideoProfile)
	ON_BN_CLICKED(IDC_EDGEFIELD_REFINEMENT, &CMFCTEST4Dlg::OnBnClickedEdgefieldRefinement)
	ON_EN_KILLFOCUS(IDC_ANGLE_CONSTRAINT, &CMFCTEST4Dlg::OnEnKillfocusAngleConstraint)
	ON_NOTIFY(UDN_DELTAPOS, IDC_ANGLE_CONSTRAINT_SPIN, &CMFCTEST4Dlg::OnDeltaposAngleConstraintSpin)
	ON_COMMAND(ID_LOAD_SEGMENTATION, &CMFCTEST4Dlg::OnLoadSegmentation)
	ON_COMMAND(ID_LOAD_DATIMAGE, &CMFCTEST4Dlg::OnLoadDatimage)
	ON_COMMAND(ID_UNLOAD_DATIMAGE, &CMFCTEST4Dlg::OnUnloadDatimage)
	ON_BN_CLICKED(IDC_TEMPORAL_SMOOTHING_STROKE_COLORS, &CMFCTEST4Dlg::OnBnClickedTemporalSmoothingStrokeColors)
	ON_COMMAND(ID_ShowInput, &CMFCTEST4Dlg::OnShowinput)
	ON_COMMAND(ID_ShowOPT, &CMFCTEST4Dlg::OnShowopt)
	ON_COMMAND(ID_ShowPainting, &CMFCTEST4Dlg::OnShowpainting)
	ON_COMMAND(ID_ShowSeg, &CMFCTEST4Dlg::OnShowseg)
	ON_COMMAND(ID_ShowTensor, &CMFCTEST4Dlg::OnShowtensor)
	ON_COMMAND(ID_ShowVec, &CMFCTEST4Dlg::OnShowvec)
END_MESSAGE_MAP()


// CMFCTEST4Dlg bZ[W nh

BOOL CMFCTEST4Dlg::OnInitDialog() {
	const int len = 10;
	//FILE *file;
	stringstream ss;
	CTabCtrl* tab;

	bool start_log = false;
	fstream file_log;
	TCHAR full_path[MAX_PATH];
	string logfile_name;


	if(start_log) {
		CString msg;
		GetCurrentDirectory(MAX_PATH, full_path);
		logfile_name = full_path;
		logfile_name += _T("\\Log\\MFCTEST4Dlg_Initialization.txt");
		file_log.open(logfile_name.c_str(), ios_base::out|ios_base::trunc);

		msg.Format(_T("Cannot create a log file: %s\r\n."), logfile_name.c_str());

		if(file_log.is_open() == false) {
			MessageBox(msg);
			exit(-1);
		}
	}

	if(start_log) file_log << _T("CDialog::OnInitDialog...");
	CDialog::OnInitDialog();
	if(start_log) file_log << _T("Done") << endl;

	// ̃_CAÕACRݒ肵܂BAvP[ṼC EBhE_CAOłȂꍇA
	//  Framework ́A̐ݒIɍs܂B

	if(start_log) file_log << _T("SetIcon...");
	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);
	if(start_log) file_log << _T("Done") << endl;


	// Get size and position of the picture control
	//GetDlgItem(IDC_OPENGL)->GetWindowRect(rect);

	// Convert screen coordinates to client coordinates
	//ScreenToClient(rect);

	if(start_log) file_log << _T("SetWindowText...");
	SetWindowText(WINDOW_TITLE);
	if(start_log) file_log << _T("Done") << endl;


	// Create OpenGL Control window
	CRect rect;
	rect.top = 30;
	rect.left = 10;
	rect.right = GL_WINDOW_W + rect.left;
	rect.bottom = GL_WINDOW_H + rect.top;

	if(start_log) file_log << _T("View creation...");
	the_view = new VisView();
	the_view->Create(rect, this, _T("hexgrid_6.ply"));
	if(start_log) file_log << _T("Done") << endl;

	if(start_log) file_log << _T("View initialization...");
	the_view->stopDisplayTimer();

	the_view->setImageSynthesisMethod(0);
	the_view->setDisplayMode(VisView::TENSOR_EDGE_FIELD);
	the_view->setTensorVisMode(VisView::MAJOR);
	if(start_log) file_log << _T("Done") << endl;

	// menu setup
	if(start_log) file_log << _T("Menu Setup...");
	menu.LoadMenu(IDR_MENU1);
	SetMenu(&menu);
	
	menu.CheckMenuItem(IDC_SHOW_CTL_KNOB, MF_CHECKED);
	if(start_log) file_log << _T("Done") << endl;


	// set accelerator
	if(start_log) file_log << _T("Accelerator Setup...");
    m_hAccelTable = ::LoadAccelerators(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_ACCELERATOR1));
	if(start_log) file_log << _T("Done") << endl;


	// set up tab controls
	if(start_log) file_log << _T("Tab Creation...");
	tab = reinterpret_cast<CTabCtrl*>(GetDlgItem(IDC_TAB1));

	for(int i = 0; i < numTabs; i++) tab->InsertItem(i, tabNames[i]);


	if(start_log) file_log << _T("\n") << _T("\t1...");
	tabContents[0] = new TabContent0(this);
	if(start_log) file_log << _T("Done") << endl;
	if(start_log) file_log << _T("\t2...");
	tabContents[1] = new TabContent1(this);
	if(start_log) file_log << _T("Done") << endl;
	if(start_log) file_log << _T("\t3...");
	tabContents[2] = new TabContent2(this);
	if(start_log) file_log << _T("Done") << endl;
	if(start_log) file_log << _T("\t4...");
	tabContents[3] = new TabContent4(this);
	if(start_log) file_log << _T("Done") << endl;
	if(start_log) file_log << _T("\t5...");
	tabContents[4] = new TabContent5(this);
	if(start_log) file_log << _T("Done") << endl;

	tab->SetCurSel(1);
	OnTcnSelchangeTab1(0, 0);


	reinterpret_cast<CButton*>(GetDlgItem(IDC_PRINT_STAT))->SetCheck(0);
	dbg_timing = false;


	// set flags for media
	old_fsi = NULL;
	loadedMediaType = NOTHING_LOADED;

	if(start_log) file_log << _T("File load initialization...");
	fileLoadInitialization(_T(""));
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("Counter Setup...");
	stroke_counts[0] = new size_t[the_view->getMaxSupportedLayers()];
	stroke_counts[1] = new size_t[the_view->getMaxSupportedLayers()];
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("View initialization (2)...");
	current_frame_number = 0;
	the_view->setNumFrames(1);
	the_view->setCurrentTime(current_frame_number);
	if(start_log) file_log << _T("Done") << endl;


	//use_binary_map = false;
	if(start_log) file_log << _T("Binary map set up...");
	use_binary_map = true;
	loaded_binary_map = false;
	binary_map = new unsigned char[the_view->getWinWidth() * the_view->getWinHeight()];
	if(start_log) file_log << _T("Done") << endl;


	loaded_dats = false;


	if(start_log) file_log << _T("CVMat setup...");
	preCreateCVMatKernel();
	if(start_log) file_log << _T("Done") << endl;
	

	// Setup the OpenGL Window's timer to render
	if(start_log) file_log << _T("Starting draw loop...");
	the_view->startDisplayTimer();
	if(start_log) file_log << _T("Done") << endl;

	//Set Toolbar
	toolbar.Create(this);
	toolbar.LoadToolBar(IDR_TOOLBAR1);
	RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST,0); 


	if(start_log) file_log << _T("All initialization complete.") << endl;

	return TRUE;
}


void CMFCTEST4Dlg::OnPaint() {
	if(IsIconic()) {
		CPaintDC dc(this);

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		dc.DrawIcon(x, y, m_hIcon);
	}
	else {
		CDialog::OnPaint();
	}
}


void CMFCTEST4Dlg::OnTcnSelchangeTab1(NMHDR *pNMHDR, LRESULT *pResult)
{
	int tab_num;
	CTabCtrl* tab;
	CRect r1, r2;

	tab = reinterpret_cast<CTabCtrl*>(GetDlgItem(IDC_TAB1));
	tab_num = tab->GetCurSel();

	for(int i = 0; i < numTabs; i++) {
		tabContents[i]->ShowWindow(SW_HIDE);
	}

	tab->GetClientRect(r1);
	tab->GetWindowRect(r2);

	tab->AdjustRect(FALSE, r1);
	ScreenToClient(r2);
	r1.OffsetRect(r2.left, r2.top);
	tabContents[tab_num]->MoveWindow(r1);
	tabContents[tab_num]->ShowWindow(SW_SHOW);

	if(pResult) *pResult = 0;
}


HCURSOR CMFCTEST4Dlg::OnQueryDragIcon() {
	return static_cast<HCURSOR>(m_hIcon);
}


void CMFCTEST4Dlg::OnSize(UINT nType, int cx, int cy) {
	CDialog::OnSize(nType, cx, cy);

	if(the_view) the_view->OnSize(nType, cx, cy);
}

void CMFCTEST4Dlg::OnSetFocus(CWnd *pOldWnd) { 
	//if(the_view) {
	//	the_view->initGL();
	//}
}


void CMFCTEST4Dlg::OnFileQuit() {
	CDialog::OnOK();
}


void CMFCTEST4Dlg::OnFileClose() {
	closeFile();
}


void CMFCTEST4Dlg::OnFileOpen() {
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;
	list<FieldSampleImage*>::iterator it;


	if(the_view == NULL) return;


	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("All files {*.*}\0*.*\0PPM {.ppm}\0*.ppm\0JPG {.jpg;.jpeg}\0*.jpg;*.jpeg\0BMP {.bmp}\0*.bmp\0AVI {.avi}\0*.avi\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.lpstrFileTitle = strFileTitleName;
	ofn.nMaxFileTitle = sizeof(strFileTitleName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("ppm");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;


	if(GetOpenFileName(&ofn)) {
		openFile(strFileName, strFileTitleName);
	}

	// restart drawing
	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}


void CMFCTEST4Dlg::openFile(LPCSTR str_filename, LPCSTR str_titlename) {
	CString msg, append;
	tstring filename, fn_cp;

	if(input_video) {
		cvReleaseCapture(&input_video);
		input_video = NULL;
	}

	if(input_image) {
		delete[] input_image;
		input_image = NULL;
	}

	input_width = -1;
	input_height = -1;

	filename = tstring(str_filename);
	fn_cp = tstring(str_filename);

	for(int i = 0; i < (int)fn_cp.length(); i++) fn_cp[i] = tolower(fn_cp[i]);

	if(fn_cp.rfind(".ppm") != string::npos) {
		imgld.readPPM(filename.c_str(), &input_image, input_width, input_height);
		
		if(input_width > 0 && input_height > 0) loadedMediaType = IMAGE_LOADED;
		else loadedMediaType = NOTHING_LOADED;
	}
	else if(fn_cp.rfind(".jpeg") != string::npos || fn_cp.rfind(".jpg") != string::npos) {
		imgld.readJPG(filename.c_str(), &input_image, input_width, input_height);

		if(input_width > 0 && input_height > 0) loadedMediaType = IMAGE_LOADED;
		else loadedMediaType = NOTHING_LOADED;
	}
	else if(fn_cp.rfind(".bmp") != string::npos) {
		imgld.readBMP(filename.c_str(), &input_image, input_width, input_height);

		if(input_width > 0 && input_height > 0) loadedMediaType = IMAGE_LOADED;
		else loadedMediaType = NOTHING_LOADED;
	}
	else if(fn_cp.rfind(".avi") != string::npos || fn_cp.rfind(".wmv") != string::npos) {
		// read video media
		input_video = cvCaptureFromFile(filename.c_str());

		if(input_video) {
			input_width = (int)cvGetCaptureProperty(input_video, CV_CAP_PROP_FRAME_WIDTH);
			input_height = (int)cvGetCaptureProperty(input_video, CV_CAP_PROP_FRAME_HEIGHT);

			double fps = cvGetCaptureProperty(input_video, CV_CAP_PROP_FPS);

			// try to read the first frame
			if(cvQueryFrame(input_video)) {
				loadedMediaType = VIDEO_LOADED;
			}
			else {
				printMessage(CString(_T("Error: Failed to load video.\r\n")));
				loadedMediaType = NOTHING_LOADED;
			}
		}
		else {
			loadedMediaType = NOTHING_LOADED;
		}
	}

	if(loadedMediaType == NOTHING_LOADED) {
		fileLoadInitialization(_T(""));
		printMessage(CString(_T("Error: File not loaded correctly.\r\n")));
	}
	else {
		fileLoadInitialization(str_titlename);
	}
}

void CMFCTEST4Dlg::closeFile() {
	loadedMediaType = NOTHING_LOADED;

	fileLoadInitialization(_T(""));
}


void CMFCTEST4Dlg::fileLoadInitialization(LPCSTR filename) {
	list<FieldSampleImage*>::iterator it_fsi;
	list<IplImage*>::iterator it_ipl;
	CString str;
	CButton *cb = NULL;
	TabContent2 *tb = NULL;

	//str.Format("%d, %d", input_width, input_height);
	//MessageBox(str);

	// reset the size
	/**
	switch(loadedMediaType) {
		case NOTHING_LOADED:
			//the_view->resize(GL_WINDOW_W, GL_WINDOW_H);
			the_view->OnSize(0, GL_WINDOW_W, GL_WINDOW_H);
			break;

		case IMAGE_LOADED:
			//the_view->resize(input_width, input_height);
			the_view->OnSize(0, input_width, input_height);
			break;

		case VIDEO_LOADED:
			//the_view->resize(input_width, input_height);
			the_view->OnSize(0, input_width, input_height);
			break;
	}
	/**/

	// first reset all painting regions...
	the_view->resetPaintingRegions(true);

	use_binary_map = true;
	loaded_binary_map = false;
	bmr.clear();

	if(loaded_dats == true) {
		unloadDatImage();
	}

	loaded_dats = false;


	// initialize load-optical-flow flag
	loaded_optflow = false;
	fwd_optflow.clear();


	str = WINDOW_TITLE;
	str += filename;
	SetWindowText(str);


	for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
		the_view->clearStrokes(i);
	}


	for(it_fsi = fsi_list.begin(); it_fsi != fsi_list.end(); it_fsi++) {
		if(*it_fsi != NULL) (*it_fsi)->finalize();
	}


	fsi_list.clear();
	assert(fsi_list.size() == 0);


	if(old_fsi != NULL) {
		old_fsi->finalize();
		old_fsi = NULL;
	}



	for(it_ipl = frame_list.begin(); it_ipl != frame_list.end(); it_ipl++) {
		if(*it_ipl != NULL) cvReleaseImage(&(*it_ipl));
	}

	frame_list.clear();
	assert(frame_list.size() == 0);



	cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING));
	cb->SetCheck(FALSE);
	enable_temporal_smoothing = false;

	cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING_STROKE_COLORS));
	cb->SetCheck(FALSE);
	the_view->enableStrokeImageInput(false);

	cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_ENABLE_STROKE_TRACKING));
	cb->SetCheck(FALSE);
	enable_stroke_tracking = false;
	the_view->setStrokeRegenerationMode(0);

	cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_EDGEFIELD_REFINEMENT));
	cb->SetCheck(FALSE);
	enable_angle_constraint = false;


	switch(loadedMediaType) {
		case NOTHING_LOADED:
		default:
			if(input_video) {
				cvReleaseCapture(&input_video);
				input_video = NULL;
			}

			if(input_image) {
				delete[] input_image;
				input_image = NULL;
			}

			input_width = -1;
			input_height = -1;

			num_frames = 0;
			render_start = 0;
			render_end = 0;
			fsi_list_update_status = 0;
			frame_list_update_status = 0;

			unloadDatImage();

			the_view->setNumFrames(num_frames);
			the_view->clearField();

			// Just display a black screen
			setUpForImage(NULL, 0, 0, false);
			//the_view->setBaseImage(NULL, 0, 0);

			current_frame_number = 0;
			the_view->setCurrentTime(current_frame_number);

			GetDlgItem(IDC_VIDEO_TOTAL_FRAMES)->SetWindowText(_T("/ -"));
			GetDlgItem(IDC_VIDEO_CURRENT_FRAME)->SetWindowText(_T("-"));
			GetDlgItem(IDC_VIDEO_CURRENT_FRAME)->EnableWindow(FALSE);


			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING));
			cb->EnableWindow(FALSE);

			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_ENABLE_STROKE_TRACKING));
			cb->EnableWindow(FALSE);

			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_EDGEFIELD_REFINEMENT));
			cb->EnableWindow(FALSE);

			GetDlgItem(IDC_ANGLE_CONSTRAINT_SPIN)->EnableWindow(FALSE);
			GetDlgItem(IDC_ANGLE_CONSTRAINT)->EnableWindow(FALSE);
			GetDlgItem(IDC_ANGLE_CONSTRAINT)->SetWindowText("");


			tb = reinterpret_cast<TabContent2*>(tabContents[2]);
			::setNumberToCEdit(reinterpret_cast<CEdit*>(tb->GetDlgItem(IDC_VIDEO_RENDERING_RANGE_START)), render_start);
			::setNumberToCEdit(reinterpret_cast<CEdit*>(tb->GetDlgItem(IDC_VIDEO_RENDERING_RANGE_END)), render_end);
			cb = reinterpret_cast<CButton*>(tb->GetDlgItem(IDC_SET_VIDEO_RENDERING_RANGE));
			cb->EnableWindow(FALSE);
			cb->SetCheck(0);
			tb->OnBnClickedSetVideoRenderingRange();

			GetDlgItem(IDC_FF)->EnableWindow(FALSE);
			GetDlgItem(IDC_REW)->EnableWindow(FALSE);
			GetDlgItem(IDC_FORWARD)->EnableWindow(FALSE);
			GetDlgItem(IDC_BACK)->EnableWindow(FALSE);
			GetDlgItem(IDC_JUMP_NEXT_KEY)->EnableWindow(FALSE);
			GetDlgItem(IDC_JUMP_NEXT_KEY2)->EnableWindow(FALSE);
			GetDlgItem(IDC_JUMP_PREV_KEY)->EnableWindow(FALSE);
			GetDlgItem(IDC_JUMP_PREV_KEY2)->EnableWindow(FALSE);

			break;


		case IMAGE_LOADED:
			num_frames = max_img_seq;
			render_start = 0;
			render_end = num_frames - 1;
			fsi_list_update_status = 0;
			frame_list_update_status = 0;

			unloadDatImage();

			the_view->setNumFrames(num_frames);
			the_view->clearField();

			setUpForImage(input_image, input_width, input_height, false);

			current_frame_number = 0;
			the_view->setCurrentTime(current_frame_number);

			str.Format(_T("/ %d"), num_frames);

			GetDlgItem(IDC_VIDEO_TOTAL_FRAMES)->SetWindowText(str);
			GetDlgItem(IDC_VIDEO_CURRENT_FRAME)->SetWindowText(_T("1"));
			GetDlgItem(IDC_VIDEO_CURRENT_FRAME)->EnableWindow(TRUE);


			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING));
			cb->EnableWindow(FALSE);

			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_ENABLE_STROKE_TRACKING));
			cb->EnableWindow(FALSE);

			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_EDGEFIELD_REFINEMENT));
			cb->EnableWindow(FALSE);

			GetDlgItem(IDC_ANGLE_CONSTRAINT_SPIN)->EnableWindow(FALSE);
			GetDlgItem(IDC_ANGLE_CONSTRAINT)->EnableWindow(FALSE);
			GetDlgItem(IDC_ANGLE_CONSTRAINT)->SetWindowText("");


			tb = reinterpret_cast<TabContent2*>(tabContents[2]);
			::setNumberToCEdit(reinterpret_cast<CEdit*>(tb->GetDlgItem(IDC_VIDEO_RENDERING_RANGE_START)), render_start + 1);
			::setNumberToCEdit(reinterpret_cast<CEdit*>(tb->GetDlgItem(IDC_VIDEO_RENDERING_RANGE_END)), render_end + 1);
			cb = reinterpret_cast<CButton*>(tb->GetDlgItem(IDC_SET_VIDEO_RENDERING_RANGE));
			cb->EnableWindow(TRUE);
			cb->SetCheck(0);
			tb->OnBnClickedSetVideoRenderingRange();

			GetDlgItem(IDC_FF)->EnableWindow(TRUE);
			GetDlgItem(IDC_REW)->EnableWindow(TRUE);
			GetDlgItem(IDC_FORWARD)->EnableWindow(TRUE);
			GetDlgItem(IDC_BACK)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_NEXT_KEY)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_NEXT_KEY2)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_PREV_KEY)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_PREV_KEY2)->EnableWindow(TRUE);

			break;


		case VIDEO_LOADED:
			num_frames = (int)cvGetCaptureProperty(input_video, CV_CAP_PROP_FRAME_COUNT);
			render_start = 0;
			render_end = num_frames - 1;
			fsi_list_update_status = 3;
			frame_list_update_status = 3;

			old_fsi = new FieldSampleImage(the_view->getWinWidth(), the_view->getWinHeight());

			the_view->setNumFrames(num_frames);
			the_view->clearField();

			setUpForVideo(0, false);

			current_frame_number = 0;
			the_view->setCurrentTime(current_frame_number);

			str.Format(_T("/ %d"), num_frames);

			GetDlgItem(IDC_VIDEO_TOTAL_FRAMES)->SetWindowText(str);
			GetDlgItem(IDC_VIDEO_CURRENT_FRAME)->SetWindowText(_T("1"));
			GetDlgItem(IDC_VIDEO_CURRENT_FRAME)->EnableWindow(TRUE);


			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING));
			cb->EnableWindow(TRUE);

			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_ENABLE_STROKE_TRACKING));
			cb->EnableWindow(TRUE);

			cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_EDGEFIELD_REFINEMENT));
			cb->EnableWindow(TRUE);

			GetDlgItem(IDC_ANGLE_CONSTRAINT_SPIN)->EnableWindow(TRUE);
			GetDlgItem(IDC_ANGLE_CONSTRAINT)->EnableWindow(TRUE);
			setNumberToCEdit(reinterpret_cast<CEdit*>(GetDlgItem(IDC_ANGLE_CONSTRAINT)), radianToDegree(constraint_angle), num_prec_digits);


			// make sure if optical flow needs to be calculated
			tb = reinterpret_cast<TabContent2*>(tabContents[2]);
			::setNumberToCEdit(reinterpret_cast<CEdit*>(tb->GetDlgItem(IDC_VIDEO_RENDERING_RANGE_START)), render_start + 1);
			::setNumberToCEdit(reinterpret_cast<CEdit*>(tb->GetDlgItem(IDC_VIDEO_RENDERING_RANGE_END)), render_end + 1);
			cb = reinterpret_cast<CButton*>(tb->GetDlgItem(IDC_SET_VIDEO_RENDERING_RANGE));
			cb->EnableWindow(TRUE);
			tb->OnBnClickedSetVideoRenderingRange();

			GetDlgItem(IDC_FF)->EnableWindow(TRUE);
			GetDlgItem(IDC_REW)->EnableWindow(TRUE);
			GetDlgItem(IDC_FORWARD)->EnableWindow(TRUE);
			GetDlgItem(IDC_BACK)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_NEXT_KEY)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_NEXT_KEY2)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_PREV_KEY)->EnableWindow(TRUE);
			GetDlgItem(IDC_JUMP_PREV_KEY2)->EnableWindow(TRUE);

			break;
	}


	reinterpret_cast<TabContent1*>(tabContents[1])->OnCbnSelchangeDisplayMode();

	the_view->initializeRendering();
}


void CMFCTEST4Dlg::OnDestroy() {
	HGLRC   hrc;

	CDialog::OnDestroy();

	if(input_video) cvReleaseCapture(&input_video);
	if(input_image) delete[] input_image;

	if(the_view) the_view->stopDisplayTimer();

	hrc	= ::wglGetCurrentContext();

	::wglMakeCurrent(NULL, NULL);

	if(hrc) ::wglDeleteContext(hrc);
}


void CMFCTEST4Dlg::OnUndo() {
	the_view->undo();
}


void CMFCTEST4Dlg::OnRedo() {
	the_view->redo();
}


void CMFCTEST4Dlg::OnFileSaveControl() {
	static TCHAR strFileName[MAX_PATH];
	OPENFILENAME ofn;

	ASSERT(the_view != NULL);

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("TXT {.txt}\0*.txt\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = _T("txt");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;

	if(GetSaveFileName(&ofn)) {
		the_view->saveControlToFile(strFileName);
		MessageBox(_T("Saved control."));
	}
}

void CMFCTEST4Dlg::OnFileLoadControl() {
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	OPENFILENAME ofn;
	stringstream ss;

	ASSERT(the_view != NULL);

	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("TXT {.txt}\0*.txt\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("txt");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;

	if(GetOpenFileName(&ofn)) {
		the_view->loadControlFromFile(strFileName);
		the_view->initializeRendering();
	}

	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}


void CMFCTEST4Dlg::OnShowCtlKnob() {
	if(the_view->getShowControlKnobs() == true) {
		the_view->setShowControlKnobs(false);
		menu.CheckMenuItem(IDC_SHOW_CTL_KNOB, MF_UNCHECKED);
	}
	else {
		the_view->setShowControlKnobs(true);
		menu.CheckMenuItem(IDC_SHOW_CTL_KNOB, MF_CHECKED);
	}
}


void CMFCTEST4Dlg::OnShowSeparatrices() {
	if(the_view->getShowSeparatrices() == true) {
		the_view->setShowSeparatrices(false);
		menu.CheckMenuItem(IDC_SHOW_SEPARATRICES, MF_UNCHECKED);
	}
	else {
		the_view->setShowSeparatrices(true);
		menu.CheckMenuItem(IDC_SHOW_SEPARATRICES, MF_CHECKED);
	}
}


void CMFCTEST4Dlg::OnShowSingularities() {
	if(the_view->getShowSingularities() == true) {
		the_view->setShowSingularities(false);
		menu.CheckMenuItem(IDC_SHOW_SINGULARITIES, MF_UNCHECKED);
	}
	else {
		the_view->setShowSingularities(true);
		menu.CheckMenuItem(IDC_SHOW_SINGULARITIES, MF_CHECKED);
	}
}


void CMFCTEST4Dlg::setEdgeFieldForImage() {
	setEdgeFieldForImage(true);
}


void CMFCTEST4Dlg::setEdgeFieldForImage(bool retain_design_elements) {
	CString msg;
	bool prev_timer_status;
	clock_t t1[4], t2[4];
	int cnt = 0;

	
	if(the_view == NULL) return;

	// Stop timer loop since we alter edge field...
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();

	if(dbg_timing == true) {
		msg.Format(_T("---  Set edge field for image ---\r\n"));
		printMessage(msg);
	}


	// temporarily save design elements
	if(retain_design_elements) {
		t1[cnt] = clock();
		//the_view->saveDesignElements();
		t2[cnt] = clock();
	}

	cnt++;


	// set up regions (only .dat files are supported)
	t1[cnt] = clock();
	
	if(loaded_dats == true) {
		DatforVideo();
	}

	t2[cnt] = clock();

	cnt++;
	
	t1[cnt] = clock();
	the_view->computeFieldFromImage();
	t2[cnt] = clock();

	cnt++;

	t1[cnt] = clock();
	the_view->constructFieldInformation();
	t2[cnt] = clock();

	cnt++;


	if(dbg_timing == true) {
		if(retain_design_elements == true) {
			msg.Format(_T("Save Design Elements: %.3lf sec\r\n"), (double)(t2[0] - t1[0]) / CLOCKS_PER_SEC);
			printMessage(msg);
		}

		msg.Format(_T("Clear Field: %.3lf sec\r\n"), (double)(t2[1] - t1[1]) / CLOCKS_PER_SEC);
		printMessage(msg);

		msg.Format(_T("Compute Edge Field: %.3lf sec\r\n"), (double)(t2[2] - t1[2]) / CLOCKS_PER_SEC);
		printMessage(msg);

		msg.Format(_T("Compute Field Information: %.3lf sec\r\n"), (double)(t2[3] - t1[3]) / CLOCKS_PER_SEC);
		printMessage(msg);

		msg.Format(_T("--- End of Edge field for image ---\r\n"));
		printMessage(msg);
	}


	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}


void CMFCTEST4Dlg::setEdgeFieldForVideo() {
	setEdgeFieldForVideo(current_frame_number, true);
}


void CMFCTEST4Dlg::setEdgeFieldForVideo(int frame_number, bool retain_design_elements) {
	int cnt;
	int shift = EdgeFieldConstructor::temporal_kernel_size >> 1;
	bool prev_timer_status;
	list<FieldSampleImage*>::iterator it;
	FieldSampleImage *fsi = NULL;
	FieldSample *fs = NULL;
	clock_t t1[4], t2[4];

	if(the_view == NULL) return;


	// Stop timer loop since we alter edge field...
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();

	cnt = 0;

	// temporarily save design elements
	if(retain_design_elements) {
		t1[cnt] = clock();
		//the_view->saveDesignElements();
		t2[cnt++] = clock();
	}

	// set zero
	t1[cnt] = clock();
	//the_view->clearField();
	t2[cnt++] = clock();


	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	// currently both .dat and .bmr files are supported. the latter will be removed in future...
	if(loaded_dats == true) {
		DatforVideo();
	}
	else if(loaded_binary_map == true && use_binary_map == true) {
		t1[cnt] = clock();
		bmr.load(frame_number, binary_map);
		the_view->setRegion(binary_map, bmr.getWidth(), bmr.getHeight());
		t2[cnt++] = clock();
	}


	t1[cnt] = clock();

	// compute edge field
	if(enable_temporal_smoothing == true) {
		if(fsi_list.size() != EdgeFieldConstructor::temporal_kernel_size) {
			fsi_list_update_status = 3;
		}

		//fsi_list_update_status = 3;

		switch(fsi_list_update_status) {
			case 1:
				fsi = fsi_list.front();
				fsi_list.pop_front();

				// reuse allocated memory
				createFieldSampleImageFromFrame(frame_number + shift, fsi, true);
				fsi_list.push_back(fsi);

				break;


			case 2:
				fsi = fsi_list.back();
				fsi_list.pop_back();

				// reuse allocated memory
				createFieldSampleImageFromFrame(frame_number - shift, fsi, true);
				fsi_list.push_front(fsi);

				break;


			case 3:
				for(it = fsi_list.begin(); it != fsi_list.end(); it++) {
					(*it)->finalize();
				}

				fsi_list.clear();

				for(int i = 0; i < EdgeFieldConstructor::temporal_kernel_size; i++) {
					createFieldSampleImageFromFrame(frame_number - shift + i, &fsi, true);
					fsi_list.push_back(fsi);
				}

				break;


			default:		// Do nothing...
				break;
		}


		// reset
		fsi_list_update_status = 0;

		// apply gaussian blur over multiple frames...
		field_constructor.averageEdgeField_T(fsi_list, *(the_view->input->fld_pyramid[0]));
	}
	else {
		createFieldSampleImageFromFrame(frame_number, the_view->input->fld_pyramid[0], false);
	}

	t2[cnt++] = clock();

	if(enable_angle_constraint == true) {
		//CString txt;
		//txt.Format("frame number:%d, render_start:%d\r\n", frame_number, render_start);
		//printMessage(txt);

		if(frame_number > render_start) {
			field_constructor.refineEdgeField(*(the_view->input->fld_pyramid[0]), *old_fsi, constraint_angle);
		}

		old_fsi->copy(*(the_view->input->fld_pyramid[0]));
	}

	/*
	// I apply binary map (or segmented region map) at this point... It might be weird why at this point, but 
	// this is due to my IBFV renderer needed to know how much we want to warp texture
	if(loaded_binary_map == true && use_binary_map == true) {
		t1[cnt] = clock();
		bmr.load(frame_number, binary_map);
		the_view->setRegion(binary_map, bmr.getWidth(), bmr.getHeight());
		t2[cnt++] = clock();
	}
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	LabelforVideo();
	DatforVideo();
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	*/


	if(dbg_timing == true) {
		CString msg;

		msg.Format(_T("--- Set edge field ---\r\n"));
		printMessage(msg);

		if(retain_design_elements == true) {
			msg.Format(_T("Save Design Elements: %.3lf sec\r\n"), (double)(t2[0] - t1[0]) / CLOCKS_PER_SEC);
			printMessage(msg);
		}

		msg.Format(_T("Clear Edge Field: %.3lf sec\r\n"), (double)(t2[1] - t1[1]) / CLOCKS_PER_SEC);
		printMessage(msg);

		msg.Format(_T("Compute Edge Field: %.3lf sec\r\n"), (double)(t2[2] - t1[2]) / CLOCKS_PER_SEC);
		printMessage(msg);

		if(loaded_binary_map == true && use_binary_map == true) {
			msg.Format(_T("Set Binary Map: %.3lf sec\r\n"), (double)(t2[3] - t1[3]) / CLOCKS_PER_SEC);
			printMessage(msg);
		}

		msg.Format(_T("--- End of setting edge field ---\r\n"));
		printMessage(msg);
	}


	// analyze topology as well as compute texture coodinate
	the_view->constructFieldInformation();

	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}


void CMFCTEST4Dlg::setUpForImage(unsigned char *img, int w, int h, bool retain_design_elements) {
	bool prev_timer_status;
	CString msg;
	clock_t t1, t2;


	//if(img == NULL || w <= 0 || h <= 0 || the_view == NULL) return;
	if(the_view == NULL) return;

	if(w < 0) w = 0;
	if(h < 0) h = 0;


	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	t1 = clock();
	the_view->setBaseImage(img, w, h);
	t2 = clock();

	if(dbg_timing == true) {
		msg.Format(_T("Set Base Image: %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		printMessage(msg);
	}



	// create edge field from an image set to VisView
	t1 = clock();
	setEdgeFieldForImage(retain_design_elements);
	t2 = clock();

	if(dbg_timing == true) {
		msg.Format(_T("Set Edge Field: %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		printMessage(msg);
	}


	the_view->initializeRendering();

	if(prev_timer_status == true) the_view->startDisplayTimer();
}



void CMFCTEST4Dlg::setUpForVideo(int frame_number, bool retain_design_elements) {
	bool prev_timer_status;
	unsigned char *current_ub;
	IplImage *current;
	OpticalFlow *optflow = NULL;
	float *data = NULL;
	CString msg;
	clock_t t1, t2;

	int optflow_calc_method = 0;


	if(the_view == NULL || loadedMediaType != VIDEO_LOADED) return;


	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	// construct edge field smoothed over time
	t1 = clock();
	setEdgeFieldForVideo(frame_number, retain_design_elements);
	t2 = clock();

	if(dbg_timing == true) {
		msg.Format(_T("Set Edge Field: %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		printMessage(msg);
	}



	// set image
	t1 = clock();
	getImageFromFrame(frame_number, &current, true);
	current_ub = new unsigned char[3 * current->width * current->height];

	//Qingqing Modified 11/03/2010
	IPL2UC(current, current_ub, false);
	unsigned char* flipped=new unsigned char [current->width*current->height*3];	
	UpDownFlipImage(flipped,current_ub,current->width, current->height);	
	the_view->setBaseImage(flipped, current->width, current->height);
	delete[] flipped;
	delete[] current_ub;
	
	cvReleaseImage(&current);

	//the_view->setBaseImage((unsigned char*)(current->imageDataOrigin), current->width, current->height);
	t2 = clock();


	// supply additional image input for stroke color...
	if(the_view->isStrokeImageInputEnabled() == true) {
		setStrokeRefImage(frame_number);
	}


	if(dbg_timing == true) {
		msg.Format(_T("Set Image : %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		printMessage(msg);
	}

	if(dbg_timing == true) {
		msg.Format(_T("Frame Number: %d\r\n"), frame_number + 1);
		printMessage(msg);
	}


	if(enable_stroke_tracking == true && frame_number > render_start) {
		t1 = clock();


		// get forward optical flow (notice.. origin is at upper-left)
		if(loaded_optflow) {
			data = new float[2 * fwd_optflow.getWidth() * fwd_optflow.getHeight()];
			fwd_optflow.load(frame_number - 1, data);
		}
		else {
			optflow = new OpticalFlow(OF_DEFAULT_CRITERIA, 7);
			data = new float[2 * input_width * input_height];

			calcOptFlow(frame_number - 1, optflow, data, false, optflow_calc_method);
			delete optflow;
		}

		t2 = clock();

		if(dbg_timing == true) {
			msg.Format(_T("Load/Compute Optical Flow : %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
			printMessage(msg);
		}


		// warp strokes in each stroke list
		t1 = clock();

		for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
			the_view->shiftStrokes(i, data, input_width, input_height);
		}

		t2 = clock();

		if(dbg_timing == true) {
			msg.Format(_T("Shift All Strokes On Layers: %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
			printMessage(msg);
		}

		delete[] data;

		the_view->setStrokeRegenerationMode(1);
	}
	else {
		the_view->setStrokeRegenerationMode(0);
	}

	the_view->initializeRendering();


	if(the_view->dbg_show_optflow == true) {
		copyOptFlowForVis();	
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}



void CMFCTEST4Dlg::OnBnClickedForward() {
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	if(loadedMediaType == VIDEO_LOADED) {
		if(current_frame_number < num_frames - 1) {
			fsi_list_update_status = 1;
			frame_list_update_status = 1;
		}
		else if(current_frame_number == num_frames - 1) {
			fsi_list_update_status = 0;
			frame_list_update_status = 0;
		}
		else {
			fsi_list_update_status = 3;
			frame_list_update_status = 3;
		}
	}

	setCurrentFrame(current_frame_number + 1);

	::setNumberToCEdit(ce, current_frame_number + 1);
	updatePaintingPresetParameters();

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedBack() {
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	if(loadedMediaType == VIDEO_LOADED) {
		if(current_frame_number > 0) {
			fsi_list_update_status = 2;
			frame_list_update_status = 2;
		}
		else if(current_frame_number == 0) {
			fsi_list_update_status = 0;
			frame_list_update_status = 0;
		}
		else {
			fsi_list_update_status = 3;
			frame_list_update_status = 3;
		}
	}

	setCurrentFrame(current_frame_number - 1);

	::setNumberToCEdit(ce, current_frame_number + 1);
	updatePaintingPresetParameters();

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedFf() {
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();

	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	if(loadedMediaType == VIDEO_LOADED) {
		fsi_list_update_status = 3;
		frame_list_update_status = 3;
	}

	setCurrentFrame(num_frames - 1);

	::setNumberToCEdit(ce, current_frame_number + 1);
	updatePaintingPresetParameters();

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedRew() {
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	if(loadedMediaType == VIDEO_LOADED) {
		fsi_list_update_status = 3;
		frame_list_update_status = 3;
	}

	setCurrentFrame(0);

	::setNumberToCEdit(ce, current_frame_number + 1);
	updatePaintingPresetParameters();

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnFileSaveImage() {
	unsigned char *img;
	static TCHAR strFileName[MAX_PATH];
	OPENFILENAME ofn;

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("PPM {.ppm}\0*.ppm\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = _T("ppm");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;

	if(GetSaveFileName(&ofn)) {
		the_view->stopDisplayTimer();

		img = new unsigned char[3 * the_view->getImgWidth() * the_view->getImgHeight()];
		the_view->grabImage(&img);
		::My_SSS_SavePPM(the_view->getImgWidth(), the_view->getImgHeight(), img, strFileName);

		delete[] img;

		MessageBox(_T("Saved image."));

		the_view->startDisplayTimer();
	}
}


void CMFCTEST4Dlg::OnFileSaveVideo() {
	CString msg, append;
	static TCHAR strFileName[MAX_PATH];
	OPENFILENAME ofn;
	fstream log_file;

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("AVI {.avi}\0*.avi\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = _T("avi");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;

	SYSTEMTIME st;

	if(loadedMediaType == NOTHING_LOADED) {
		MessageBox(_T("Input not loaded."), _T("Save Video"), MB_ICONSTOP);
	}
	else if(GetSaveFileName(&ofn)) {
		GetLocalTime(&st);

		if(saveVideo(strFileName, true) == true) {
			append.Format(_T("Running time for video: %d minutes %d seconds.\r\n"), (int)total_run_time / 60, (int)total_run_time % 60);
			msg += append;

			append.Format(_T("Rendering time for all frames: %d minutes %d seconds.\r\n"), (int)total_render_time / 60, (int)total_render_time % 60);
			msg += append;

			append.Format(_T("Stroke counts\r\n"));
			msg += append;

			append.Format(_T("  At first frame, ["));
			msg += append;

			for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
				if(i == the_view->getMaxSupportedLayers() - 1) {
					append.Format(_T("%d]\r\n"), stroke_counts[0][i]);
				}
				else {
					append.Format(_T("%d, "), stroke_counts[0][i]);
				}

				msg += append;
			}


			append.Format(_T("  At last frame, ["));
			msg += append;

			for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
				if(i == the_view->getMaxSupportedLayers() - 1) {
					append.Format(_T("%d]\r\n"), stroke_counts[1][i]);
				}
				else {
					append.Format(_T("%d, "), stroke_counts[1][i]);
				}

				msg += append;
			}


			append.Format("--- Run on %4d_%02d_%02d_%02d_%02d_%02d ---\r\n", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
			msg += append;
			printMessage(msg);


			MessageBox(_T("Saved video."));
		}
		else {
			MessageBox(_T("Couldn't save video."), _T("ERROR"), MB_ICONSTOP);
		}
	}
}



BOOL CMFCTEST4Dlg::PreTranslateMessage(MSG* pMsg) {
    if(m_hAccelTable != NULL){
        if(::TranslateAccelerator(GetSafeHwnd(), m_hAccelTable, pMsg))
            return TRUE;
    }

	return CDialog::PreTranslateMessage(pMsg);
}


void CMFCTEST4Dlg::OnOK() {
	//CDialog::OnOK();
}


void CMFCTEST4Dlg::OnEnKillfocusVideoCurrentFrame() {
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	if(loadedMediaType == VIDEO_LOADED) {
		fsi_list_update_status = 3;
		frame_list_update_status = 3;
	}

	setCurrentFrame((int)::getNumberFromCEdit(ce) - 1);

	::setNumberToCEdit(ce, current_frame_number + 1);

	if(the_view->getDisplayMode() == VisView::PAINTING) {
		updatePaintingPresetParameters();
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}



/*
	Create an image at specified frame number

	Note: The origin of the created image is at top-left corner
*/
/*
void CMFCTEST4Dlg::getImageFromFrame(int frame_number, CvCapture *input, IplImage **img) {
	int num_frames;
	IplImage *tmp_img;

	assert(input && img);

	num_frames = (int)cvGetCaptureProperty(input_video, CV_CAP_PROP_FRAME_COUNT);

	frame_number = (int)::clampVal(frame_number, 0, num_frames - 1);

	cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, frame_number);
	tmp_img = cvQueryFrame(input_video);

	*img = cvCreateImage(cvGetSize(tmp_img), IPL_DEPTH_8U, 3);

	cvConvertImage(tmp_img, *img, CV_CVTIMG_SWAP_RB | CV_CVTIMG_FLIP);
}
*/

void CMFCTEST4Dlg::getImageFromFrame(int frame_number, IplImage **img, bool extrapolate) {
	int idx = -1;
	int w = -1, h = -1;
	int ch = -1, step = -1;
	int num_pixels = 0;
	IplImage *tmp_img = NULL;
	IplImage *grabbed_1 = NULL, *grabbed_2 = NULL;

	if(input_video == NULL || img == NULL) return;


	if(extrapolate == false) {
		frame_number = (int)::clampVal(frame_number, 0, num_frames - 1);
	}

	if(0 <= frame_number && frame_number < num_frames) {
		cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, frame_number);
		tmp_img = cvQueryFrame(input_video);
		*img = cvCreateImage(cvGetSize(tmp_img), IPL_DEPTH_8U, 3);
		cvConvertImage(tmp_img, *img, CV_CVTIMG_SWAP_RB | CV_CVTIMG_FLIP);
	}
	else {
		if(frame_number < 0) {
			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, 0);
			tmp_img = cvQueryFrame(input_video);
			grabbed_1 = cvCloneImage(tmp_img);

			frame_number = (int)::clampVal(-frame_number, 0, num_frames - 1);
			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, frame_number);

			tmp_img = cvQueryFrame(input_video);
			grabbed_2 = cvCloneImage(tmp_img);
		}
		else {
			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, num_frames - 1);
			tmp_img = cvQueryFrame(input_video);
			grabbed_1 = cvCloneImage(tmp_img);

			frame_number = (int)::clampVal(2 * num_frames - frame_number - 2, 0, num_frames - 1);
			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, frame_number);
			
			tmp_img = cvQueryFrame(input_video);
			grabbed_2 = cvCloneImage(tmp_img);
		}


		assert(tmp_img != NULL);

		w = grabbed_1->width;
		h = grabbed_1->height;
		ch = grabbed_1->nChannels;
		step = grabbed_1->widthStep;
		num_pixels = grabbed_1->width * grabbed_1->height;


		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = ch * j + step * i;
				grabbed_2->imageDataOrigin[idx + 0] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[idx + 0]) - (unsigned char)(grabbed_2->imageDataOrigin[idx + 0]), 
						0, 
						255
					);

				grabbed_2->imageDataOrigin[idx + 1] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[idx + 1]) - (unsigned char)(grabbed_2->imageDataOrigin[idx + 1]), 
						0, 
						255
					);

				grabbed_2->imageDataOrigin[idx + 2] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[idx + 2]) - (unsigned char)(grabbed_2->imageDataOrigin[idx + 2]), 
						0, 
						255
					);
			}
		}

		/**
		for(int i = 0; i < num_pixels; i++) {
			for(int j = 0; j < 3; j++) {
				grabbed_2->imageDataOrigin[3 * i + j] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[3 * i + j]) - (unsigned char)(grabbed_2->imageDataOrigin[3 * i + j]), 
						0, 
						255
					);
			}
		}
		/**/


		*img = cvCreateImage(cvGetSize(grabbed_2), IPL_DEPTH_8U, 3);
		cvConvertImage(grabbed_2, *img, CV_CVTIMG_SWAP_RB | CV_CVTIMG_FLIP);
		cvReleaseImage(&grabbed_1);
		cvReleaseImage(&grabbed_2);
	}
}


/*
	Create a depth image of a sequence at specified frame number

	Note: The origin of the created image is at top-left corner


	Note 2:
	  I just use the image data directly returned by cvQueryFrame().
	  I've tried to use getImageFromFrame() to make this function simple, but 
	  that makes some difference in resulting depth images... (causing different optical flow values)
*/
void CMFCTEST4Dlg::getDepthFrame(int frame_number, IplImage **current, bool extrapolate) {
	int idx = -1;
	int w = -1, h = -1;
	int ch = -1, step = -1;
	int num_pixels = 0;
	IplImage *tmp_img = NULL;
	IplImage *grabbed_1 = NULL, *grabbed_2 = NULL;


	if(input_video == NULL || current == NULL) return;


	if(extrapolate == false) {
		frame_number = (int)::clampVal(frame_number, 0, num_frames - 1);
	}

	if(0 <= frame_number && frame_number < num_frames) {
		cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, frame_number);
		tmp_img = cvQueryFrame(input_video);
		*current = cvCreateImage(cvGetSize(tmp_img), IPL_DEPTH_8U, 1);
		cvConvertImage(tmp_img, *current, CV_CVTIMG_FLIP);
	}
	else {
		if(frame_number < 0) {
			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, 0);
			tmp_img = cvQueryFrame(input_video);
			grabbed_1 = cvCloneImage(tmp_img);

			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, (int)::clampVal(-frame_number, 0, num_frames - 1));
			tmp_img = cvQueryFrame(input_video);
			grabbed_2 = cvCloneImage(tmp_img);
		}
		else {
			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, num_frames - 1);
			tmp_img = cvQueryFrame(input_video);
			grabbed_1 = cvCloneImage(tmp_img);

			cvSetCaptureProperty(input_video, CV_CAP_PROP_POS_FRAMES, (int)::clampVal(2 * num_frames - frame_number - 2, 0, num_frames - 1));	
			tmp_img = cvQueryFrame(input_video);
			grabbed_2 = cvCloneImage(tmp_img);
		}

		assert(tmp_img != NULL);


		w = grabbed_1->width;
		h = grabbed_1->height;
		ch = grabbed_1->nChannels;
		step = grabbed_1->widthStep;
		num_pixels = grabbed_1->width * grabbed_1->height;

		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = ch * j + step * i;
				grabbed_2->imageDataOrigin[idx + 0] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[idx + 0]) - (unsigned char)(grabbed_2->imageDataOrigin[idx + 0]), 
						0, 
						255
					);

				grabbed_2->imageDataOrigin[idx + 1] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[idx + 1]) - (unsigned char)(grabbed_2->imageDataOrigin[idx + 1]), 
						0, 
						255
					);

				grabbed_2->imageDataOrigin[idx + 2] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[idx + 2]) - (unsigned char)(grabbed_2->imageDataOrigin[idx + 2]), 
						0, 
						255
					);
			}
		}


		/**
		for(int i = 0; i < num_pixels; i++) {
			for(int j = 0; j < 3; j++) {
				grabbed_2->imageDataOrigin[3 * i + j] = 
					(char)::clampVal(
						2 * (unsigned char)(grabbed_1->imageDataOrigin[3 * i + j]) - (unsigned char)(grabbed_2->imageDataOrigin[3 * i + j]), 
						0, 
						255
					);
			}
		}
		/**/

		*current = cvCreateImage(cvGetSize(grabbed_2), IPL_DEPTH_8U, 1);
		cvConvertImage(grabbed_2, *current, CV_CVTIMG_FLIP);
		cvReleaseImage(&grabbed_1);
		cvReleaseImage(&grabbed_2);
	}
}



/*
	Compute optical flow

	Note: Calculated optical flow has 2D structure with origin at top-left corner

	Parameters:
		frame_number... frame number of an input video, between 0 and the number of frames minus 1

		optFlow... address of pointer to OpticalFlow.
		
		reverse_flow... set false if you want forward optical flow (frame_number -> frame_nubmer + 1)
		                setting it true results in having backward optical flow (frame_number -> frame_number - 1)
		
		method... way to compute optical flow.
*/
void CMFCTEST4Dlg::calcOptFlow(int frame_number, OpticalFlow *optFlow, CvPoint2D32f *data, bool reverse_flow /* = false */, int method /* = 0 */) {
	IplImage *current, *next;

	assert(input_video && optFlow && data);

	if(reverse_flow == false) {
		getDepthFrame(frame_number, &current);
		getDepthFrame(frame_number + 1, &next);
	}
	else {
		getDepthFrame(frame_number, &current);
		getDepthFrame(frame_number - 1, &next);
	}

	switch(method) {
		case 0:
			optFlow->LK(current, next, data);
			break;
	
		case 1:
			optFlow->PyrLK(current, next, data);
			break;

		default:
			break;
	}

	cvReleaseImage(&current);
	cvReleaseImage(&next);
}


void CMFCTEST4Dlg::calcOptFlow(int frame_number, OpticalFlow *optFlow, float *data, bool reverse_flow /* = false */, int method /* = 0 */) {
	IplImage *current, *next;

	assert(input_video && optFlow && data);

	if(reverse_flow == false) {
		getDepthFrame(frame_number, &current);
		getDepthFrame(frame_number + 1, &next);
	}
	else {
		getDepthFrame(frame_number, &current);
		getDepthFrame(frame_number - 1, &next);
	}

	switch(method) {
		case 0:
			optFlow->LK(current, next, data);
			break;
	
		case 1:
			optFlow->PyrLK(current, next, data);
			break;

		default:
			break;
	}

	cvReleaseImage(&current);
	cvReleaseImage(&next);
}


int CMFCTEST4Dlg::getFileLoadStatus() {
	return this->loadedMediaType;
}


VisView* CMFCTEST4Dlg::getVisView() {
	return this->the_view;
}


void CMFCTEST4Dlg::setVideoRenderingRange(int start, int end) {
	if(loadedMediaType != NOTHING_LOADED) {
		if(end < 0) end = num_frames - 1;
		else if(start > end) end = start;

		start = (int)::clampVal(start, 0, num_frames - 1);
		end = (int)::clampVal(end, 0, num_frames - 1);

		render_start = start;
		render_end = end;
	}
	else {
		render_start = 0;
		render_end = 0;
	}
}


int CMFCTEST4Dlg::getVideoRenderingStartFrame() {
	return render_start;
}


int CMFCTEST4Dlg::getVideoRenderingEndFrame() {
	return render_end;
}


int CMFCTEST4Dlg::getMaxFrames() {
	return num_frames;
}


bool CMFCTEST4Dlg::saveVideo(string file_name, bool show_codec_option) {
	int num_frames_to_render;
	bool prev_timer_status;
	char *imgDataOriginTempPtr, *imgDataTempPtr;
	unsigned char *grabbed_img = NULL;
	double fps;
	CvSize frame_size;
	IplImage *outIpl, *tmpIpl;
	FILE *fp = NULL;



	if(loadedMediaType == IMAGE_LOADED) {
		fps = 29.97;
	}
	else if(loadedMediaType == VIDEO_LOADED) {
		fps = (double)::cvGetCaptureProperty(input_video, CV_CAP_PROP_FPS);
	}
	else {
		return false;
	}


	total_render_time = 0;
	total_run_time = 0;

	frame_size.width = the_view->getImgWidth();
	frame_size.height = the_view->getImgHeight();
	grabbed_img = new unsigned char[3 * frame_size.width * frame_size.height];


	if(show_codec_option == false) {
		video_writer = cvCreateVideoWriter(file_name.c_str(), CV_FOURCC('D', 'I', 'B', ' '), fps, frame_size);
	}
	else {
		video_writer = cvCreateVideoWriter(file_name.c_str(), -1, fps, frame_size);
	}


	if(video_writer == NULL) return false;



	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	tmpIpl = cvCreateImage(frame_size, IPL_DEPTH_8U, 3);
	outIpl = cvCreateImage(frame_size, IPL_DEPTH_8U, 3);
	imgDataTempPtr = tmpIpl->imageData;
	imgDataOriginTempPtr = tmpIpl->imageDataOrigin;


	num_frames_to_render = render_end - render_start + 1;
	the_view->setStrokeRegenerationMode(0);


	time(&(run_time[0]));


	if(loadedMediaType == IMAGE_LOADED) {
		// Initialization... just empty all stroke lists.
		for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
			the_view->clearStrokes(i);
		}

		// enabling tracking temporarily for video generation...
		enable_stroke_tracking = true;

		for(int i = 0; i < num_frames_to_render; i++) {
			/**/
			if(enable_stroke_tracking == true) {
				the_view->setStrokeRegenerationMode((i == render_start) ? 0 : 1);
			}
			else {
				the_view->setStrokeRegenerationMode(0);
			}
			/**/

			/*
			the_view->setCurrentTime(render_start + i);
			the_view->updatePaintingStyleField();
			the_view->updateTexCoordsOnMesh();
			the_view->initializeRendering();
			*/

			setCurrentFrame(render_start + i);

			the_view->display();

			total_render_time += the_view->getRenderTime();

			the_view->display_knobs();

			the_view->grabImage(&grabbed_img);

			/*
			tmpIpl->imageData = (char*)grabbed_img;
			tmpIpl->imageDataOrigin = (char*)grabbed_img;

			cvConvertImage(tmpIpl, outIpl, CV_CVTIMG_FLIP | CV_CVTIMG_SWAP_RB);
			cvWriteFrame(video_writer, outIpl);
			*/

			UC2IPL(grabbed_img, tmpIpl, false);
			cvConvertImage(tmpIpl, outIpl, CV_CVTIMG_FLIP | CV_CVTIMG_SWAP_RB);
			cvWriteFrame(video_writer, outIpl);

			if(i == 0) {
				for(int j = 0; j < the_view->getMaxSupportedLayers(); j++) {
					if(j < the_view->getMaxSupportedLayers()) {
						stroke_counts[0][j] = the_view->getNumStrokes(j);
					}
					else {
						stroke_counts[0][j] = 0;
					}
				}
			}

			if(i == num_frames_to_render - 1) {
				for(int j = 0; j < the_view->getMaxSupportedLayers(); j++) {
					if(j < the_view->getMaxSupportedLayers()) {
						stroke_counts[1][j] = the_view->getNumStrokes(j);
					}
					else {
						stroke_counts[1][j] = 0;
					}
				}
			}
		}

		// disable...
		enable_stroke_tracking = false;
	}
	else if(loadedMediaType == VIDEO_LOADED) {
		// Initialization... just empty all stroke lists.
		for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
			the_view->clearStrokes(i);
		}

		for(int i = 0; i < num_frames_to_render; i++) {
			if(i == 0) {
				fsi_list_update_status = 3;
				frame_list_update_status = 3;
			}
			else {
				fsi_list_update_status = 1;
				frame_list_update_status = 1;
			}


			//the_view->setCurrentTime(render_start + i);
			//setUpForVideo(render_start + i, true);
			setCurrentFrame(render_start + i);

			the_view->display();

			total_render_time += the_view->getRenderTime();

			the_view->display_knobs();

			the_view->grabImage(&grabbed_img);

			/*
			tmpIpl->imageData = (char*)grabbed_img;
			tmpIpl->imageDataOrigin = (char*)grabbed_img;
			cvConvertImage(tmpIpl, outIpl, CV_CVTIMG_FLIP | CV_CVTIMG_SWAP_RB);
			cvWriteFrame(video_writer, outIpl);
			*/

			UC2IPL(grabbed_img, tmpIpl, false);
			cvConvertImage(tmpIpl, outIpl, CV_CVTIMG_FLIP | CV_CVTIMG_SWAP_RB);
			cvWriteFrame(video_writer, outIpl);


			if(i == 0) {
				for(int j = 0; j < the_view->getMaxSupportedLayers(); j++) {
					stroke_counts[0][j] = the_view->getNumStrokes(j);
				}
			}

			if(i == num_frames_to_render - 1) {
				for(int j = 0; j < the_view->getMaxSupportedLayers(); j++) {
					stroke_counts[1][j] = the_view->getNumStrokes(j);
				}
			}
		}
	}

	delete[] grabbed_img;

	time(&(run_time[1]));
	total_run_time = difftime(run_time[1], run_time[0]);

	cvReleaseVideoWriter(&video_writer);

	tmpIpl->imageData = imgDataTempPtr;
	tmpIpl->imageDataOrigin = imgDataOriginTempPtr;
	cvReleaseImage(&outIpl);
	cvReleaseImage(&tmpIpl);


	::setNumberToCEdit(reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME)), render_end + 1);
	updatePaintingPresetParameters();

	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}


	return true;
}


/*
	Create FieldSampleImage for VisView.

	If smooth flag is set, 5 by 5 spatial smoothing is applied
*/
bool CMFCTEST4Dlg::createFieldSampleImageFromFrame(int frame_number, FieldSampleImage **fsi, bool smooth) {
	if(fsi == NULL) {
		return false;
	}
	else {
		*fsi = new FieldSampleImage(the_view->getWinWidth(), the_view->getWinHeight());

		return createFieldSampleImageFromFrame(frame_number, (*fsi), smooth);
	}
}


/*
	Create FieldSampleImage for VisView.

	If smooth flag is set, 5 by 5 spatial smoothing is applied
*/
bool CMFCTEST4Dlg::createFieldSampleImageFromFrame(int frame_number, FieldSampleImage *fsi, bool smooth) {
	unsigned char *buffer;
	IplImage *image = NULL;
	FieldSample *fs = NULL;


	ASSERT(the_view != NULL);

	getImageFromFrame(frame_number, &image, true);


	if(fsi == NULL || image == NULL) {
		return false;
	}


	buffer = new unsigned char[3 * input_width * input_height];
	IPL2UC(image, buffer, false);		// no need to flip RB components.

	//Qingqing Modified 11/03/2010
	unsigned char* flipped=new unsigned char [input_width*input_height*3];	
	UpDownFlipImage(flipped,buffer,input_width, input_height);	
	
	
	the_view->setBaseImage(flipped, input_width, input_height);
	

	the_view->computeFieldFromImage(fsi);

	if(smooth == true) {
		// apply Gaussian convolution filter
		field_constructor.averageEdgeField_S(*fsi);
	}
	delete[] flipped;
	delete[] buffer;
	cvReleaseImage(&image);

	return true;
}


void CMFCTEST4Dlg::OnBnClickedTemporalSmoothing() {
	list<FieldSampleImage*>::iterator it;

	if(this->loadedMediaType == VIDEO_LOADED) {
		enable_temporal_smoothing = (reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING))->GetCheck() ? true : false);

		if(enable_temporal_smoothing == true) {
			fsi_list_update_status = 3;
		}
		else {
			for(it = fsi_list.begin(); it != fsi_list.end(); it++) {
				if(*it != NULL) (*it)->finalize();
			}

			fsi_list.clear();
			fsi_list_update_status = 0;				// just in case...
			assert(fsi_list.size() == 0);
		}

		setUpForVideo(current_frame_number, true);
	}
}


void CMFCTEST4Dlg::OnBnClickedEnableStrokeTracking() {
	if(this->loadedMediaType == VIDEO_LOADED) {
		if((enable_stroke_tracking = (reinterpret_cast<CButton*>(GetDlgItem(IDC_ENABLE_STROKE_TRACKING))->GetCheck() ? true : false)) == false) {
			setUpForVideo(current_frame_number, true);
		}
	}
}


void CMFCTEST4Dlg::OnFileLoadOpticalFlow() {
	static TCHAR strFileName[MAX_PATH];
	OPENFILENAME ofn;
	stringstream ss;

	ASSERT(the_view != NULL);

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("ofl {.ofl}\0*.ofl\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("ofl");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;
	ofn.lpstrTitle = _T("Select Optical Flow...");


	if(loadedMediaType != VIDEO_LOADED) {
		MessageBox(_T("Movie not loaded."), _T("Load Optical Flow"), MB_ICONSTOP);
	}
	else {
		if(GetOpenFileName(&ofn)) {
			loadOpticalFlow(string(ofn.lpstrFile));

			if(checkOpticalFlowLoaded() == true) {
				MessageBox(_T("Loaded optical flow"), _T("Load Optical Flow"), MB_OK);
			}
			else {
				MessageBox(_T("Failed to load optical flow"), _T("Load Optical Flow"), MB_ICONSTOP);
			}
		}
	}
}


void CMFCTEST4Dlg::loadOpticalFlow(string &filename) {
	fwd_optflow.clear();

	loaded_optflow = fwd_optflow.read(filename.c_str());
}


void CMFCTEST4Dlg::unloadOpticalFlow() {
	if(loaded_optflow == true) {
		fwd_optflow.clear();

		loaded_optflow = false;
	}
}


void CMFCTEST4Dlg::OnFileLoadBinaryMap() {
	static TCHAR strFileName[MAX_PATH];
	OPENFILENAME ofn;
	stringstream ss;

	ASSERT(the_view != NULL);

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("bmr {.bmr}\0*.bmr\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("bmr");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;
	ofn.lpstrTitle = _T("Select Binary Map...");


	if(loadedMediaType != VIDEO_LOADED) {
		MessageBox(_T("Movie not loaded."), _T("Load Binary Map"), MB_ICONSTOP);
	}
	else {
		if(GetOpenFileName(&ofn)) {
			loadBinaryMap(string(ofn.lpstrFile));

			if(checkBinaryMapLoaded() == true) {
				setUpForVideo(current_frame_number, true);

				MessageBox(_T("Loaded binary map"), _T("Load Binary Map"), MB_OK);
			}
			else {
				MessageBox(_T("Failed to load binary map"), _T("Load Binary Map"), MB_ICONSTOP);
			}
		}
	}
}


void CMFCTEST4Dlg::OnFileUnoadOpticalFlow() {
	if(loadedMediaType != VIDEO_LOADED) {
		MessageBox(_T("Movie not loaded."), _T("Unload Optical Flow"), MB_ICONSTOP);
	}
	else {
		unloadOpticalFlow();

		MessageBox(_T("Unloaded optical flow"), _T("Unoad Optical Flow"), MB_OK);
	}
}


void CMFCTEST4Dlg::OnFileUnloadBinaryMap() {
	if(loadedMediaType != VIDEO_LOADED) {
		MessageBox(_T("Movie not loaded."), _T("Unload Binary Map"), MB_ICONSTOP);
	}
	else {
		unloadBinaryMap();
		setUpForVideo(current_frame_number, true);

		MessageBox(_T("Unloaded binary map"), _T("Unload Binary Map"), MB_OK);
	}
}


bool CMFCTEST4Dlg::checkOpticalFlowLoaded() {
	return loaded_optflow;
}


void CMFCTEST4Dlg::OnFileRunScript() {
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("SCRPT {.scrpt}\0*.scrpt\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.lpstrFileTitle = strFileTitleName;
	ofn.nMaxFileTitle = sizeof(strFileTitleName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("scrpt");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;
	ofn.lpstrTitle = _T("Select Script File...");

	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();

	if(GetOpenFileName(&ofn)) {
		processScript(strFileName);
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::processScript(LPCSTR filename) {

	ScriptProcessor::process(*this, string(filename));

	TabContent1* t1 = reinterpret_cast<TabContent1*>(tabContents[1]);

	reinterpret_cast<CComboBox*>(t1->GetDlgItem(IDC_DISPLAY_MODE))->SetCurSel(1);
	reinterpret_cast<CComboBox*>(t1->GetDlgItem(IDC_TENSOR_VIS_MODE))->SetCurSel(0);

	t1->OnCbnSelchangeDisplayMode();

	MessageBox(_T("Finished"), _T("Run Script..."));
}


int CMFCTEST4Dlg::getCurrentFrame() {
	return current_frame_number;
}


void CMFCTEST4Dlg::setCurrentFrame(int val) {
	if(loadedMediaType == NOTHING_LOADED) return;

	current_frame_number = (int)::clampVal(val, 0, num_frames - 1);
	//current_frame_number = val;

	the_view->setCurrentTime(current_frame_number);


	switch(loadedMediaType) {

		// we reuse the same edge field... so just update a painting style
		case IMAGE_LOADED:
			setUpForImage(input_image, input_width, input_height, true);
			break;

		// we need to load different frame... so simply call a function to set everything up.
		case VIDEO_LOADED:
			setUpForVideo(current_frame_number, true);
			break;

		default:
			return;
	}
}



void CMFCTEST4Dlg::printMessage(CString str) {
	CEdit *box = reinterpret_cast<CEdit*>(GetDlgItem(IDC_INFO_BOX));
	CString msg;

	box->GetWindowText(msg);
	msg += str;
	box->SetWindowText(msg);
	box->LineScroll(box->GetLineCount());
}


void CMFCTEST4Dlg::enablePrintStat(bool val) {
	dbg_timing = val;
}


bool CMFCTEST4Dlg::getPrintStatEnabled() {
	return dbg_timing;
}


void CMFCTEST4Dlg::OnBnClickedPrintMessage() {
	CButton* cb = reinterpret_cast<CButton*>(GetDlgItem(IDC_PRINT_STAT));

	if(cb->GetCheck() == 1) {
		enablePrintStat(true);
	}
	else {
		enablePrintStat(false);
	}
}


void CMFCTEST4Dlg::updatePaintingPresetParameters() {
	reinterpret_cast<TabContent1*>(tabContents[1])->updatePaintingPresetParameters();
}


void CMFCTEST4Dlg::updateSelectedRegionOption() {
	reinterpret_cast<TabContent0*>(tabContents[0])->updateRegionOption();
}


void CMFCTEST4Dlg::OnBnClickedJumpNextKey() {
	int key;
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));

	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	key = the_view->findNextKey();

	if(key >= 0) {
		if(loadedMediaType == VIDEO_LOADED) {
			fsi_list_update_status = 3;
			frame_list_update_status = 3;
		}

		setCurrentFrame(key);

		::setNumberToCEdit(ce, current_frame_number + 1);
		updatePaintingPresetParameters();
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedJumpPrevKey() {
	int key;
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;


	the_view->stopDisplayTimer();

	key = the_view->findPrevKey();

	if(key >= 0) {
		if(loadedMediaType == VIDEO_LOADED) {
			fsi_list_update_status = 3;
			frame_list_update_status = 3;
		}

		setCurrentFrame(key);

		::setNumberToCEdit(ce, current_frame_number + 1);
		updatePaintingPresetParameters();
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedJumpNextKey2() {
	int key;
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));

	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	key = the_view->findNextDEKey();

	if(key >= 0) {
		if(loadedMediaType == VIDEO_LOADED) {
			fsi_list_update_status = 3;
			frame_list_update_status = 3;
		}

		setCurrentFrame(key);

		::setNumberToCEdit(ce, current_frame_number + 1);
		updatePaintingPresetParameters();
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedJumpPrevKey2() {
	int key;
	CEdit *ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_VIDEO_CURRENT_FRAME));
	bool prev_timer_status = the_view->getDisplayTimerStatus();


	if(loadedMediaType == NOTHING_LOADED) return;

	the_view->stopDisplayTimer();

	key = the_view->findPrevDEKey();

	if(key >= 0) {
		if(loadedMediaType == VIDEO_LOADED) {
			fsi_list_update_status = 3;
			frame_list_update_status = 3;
		}

		setCurrentFrame(key);

		::setNumberToCEdit(ce, current_frame_number + 1);
		updatePaintingPresetParameters();
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnEditClearMessageBox() {
	CEdit *box = reinterpret_cast<CEdit*>(GetDlgItem(IDC_INFO_BOX));

	box->SetWindowText(_T(""));
}


void CMFCTEST4Dlg::copyOptFlowForVis() {
	OpticalFlow *optflow = NULL;
	float *data = NULL;

	if(loadedMediaType == VIDEO_LOADED) {
		data = new float[2 * the_view->getWinWidth() * the_view->getWinHeight()];

		if(loaded_optflow == true) {
			fwd_optflow.load(current_frame_number, data);
		}
		else {
			optflow = new OpticalFlow(OF_DEFAULT_CRITERIA, 7);
			calcOptFlow(current_frame_number, optflow, data);
		}

		the_view->setOptFlowForVis(data, input_width, input_height);
	}

	delete data;
	delete optflow;
}


void CMFCTEST4Dlg::OnFileLoadEdgeField() {
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;

	if(the_view == NULL) return;


	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("TXT {.txt}\0*.txt\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = _T("txt");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;


	if(GetOpenFileName(&ofn)) {
		if(the_view->loadEdgeField(string(strFileName)) == true) {
			printMessage(_T("Loaded edge field\r\n"));
		}
		else {
			printMessage(_T("Failed to load edge field\r\n"));
		}

		the_view->sampleFieldToFSI();
		the_view->initializeRendering();
	}

	// restart drawing
	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}

void CMFCTEST4Dlg::OnFileSaveEdgeField() {
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;

	if(the_view == NULL) return;


	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();

	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("TXT {.txt}\0*.txt\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = _T("txt");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;


	if(GetOpenFileName(&ofn)) {
		the_view->saveEdgeField(string(strFileName));
		printMessage(_T("Saved edge field\r\n"));
	}

	// restart drawing
	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}


bool CMFCTEST4Dlg::checkBinaryMapLoaded() {
	return loaded_binary_map;
}


void CMFCTEST4Dlg::loadBinaryMap(string &filename) {

	bmr.clear();

	loaded_binary_map = bmr.read(filename.c_str());
}


void CMFCTEST4Dlg::unloadBinaryMap() {
	if(loaded_binary_map == true) {
		bmr.clear();

		loaded_binary_map = false;
	}
}

// this just simply sets a flag to enable/disable stroke tracking...
// OnBnClickedEnableStrokeTracking() does more work
void CMFCTEST4Dlg::enableStrokeTracking(bool val) {
	if(loadedMediaType == VIDEO_LOADED) {
		enable_stroke_tracking = val;
	}
}


bool CMFCTEST4Dlg::getStrokeTrackingEnabled() {
	return enable_stroke_tracking;
}


// this just simply sets a flag to enable/disable temporal smoothing...
// OnBnClickedTemporalSmoothing() does more work
void CMFCTEST4Dlg::enableTemporalSmoothing(bool val) {
	if(loadedMediaType == VIDEO_LOADED) {
		enable_temporal_smoothing = val;
	}
}


bool CMFCTEST4Dlg::getTemporalSmoothingEnabled() {
	return enable_temporal_smoothing;
}


void CMFCTEST4Dlg::getStrokeCountsAtBegin(size_t *values) {
	int num_itr = the_view->getMaxSupportedLayers();

	for(int i = 0; i < num_itr; i++) {
		values[i] = stroke_counts[0][i];
	}
}


void CMFCTEST4Dlg::getStrokeCountsAtEnd(size_t *values) {
	int num_itr = the_view->getMaxSupportedLayers();

	for(int i = 0; i < num_itr; i++) {
		values[i] = stroke_counts[1][i];
	}
}


void CMFCTEST4Dlg::OnFileLoadVideoProfile() {
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;

	if(the_view == NULL) return;


	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("SCRPT {.scrpt}\0*.scrpt\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.lpstrFileTitle = strFileTitleName;
	ofn.nMaxFileTitle = sizeof(strFileTitleName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("scrpt");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;
	ofn.lpstrTitle = _T("Select Script File...");


	if(GetOpenFileName(&ofn)) {
		if(ScriptProcessor::loadVideoProfile(*this, string(strFileName)) == false) {
			MessageBox("Error in loading a video profile.", "Load Video Profile", MB_ICONSTOP);
		}
		else {
			// set up again to make binary map effective
			setUpForVideo(current_frame_number, false);

			TabContent1* t1 = reinterpret_cast<TabContent1*>(tabContents[1]);

			reinterpret_cast<CComboBox*>(t1->GetDlgItem(IDC_DISPLAY_MODE))->SetCurSel(3);

			t1->OnCbnSelchangeDisplayMode();
		}
	}

	// restart drawing
	if(prev_timer_status == true) {
		the_view->startDisplayTimer();
	}
}


double CMFCTEST4Dlg::getTotalRunTime() {
	return total_run_time;
}


double CMFCTEST4Dlg::getTotalRenderTime() {
	return total_render_time;
}


void CMFCTEST4Dlg::enableEdgeFieldRefinement(bool val) {
	enable_angle_constraint = val;
}


bool CMFCTEST4Dlg::getEdgeFieldRefinementEnabled() {
	return enable_angle_constraint;
}


void CMFCTEST4Dlg::setRefinementConstraintAngle(float angle_radian) {
	constraint_angle = (float)::clampVal(angle_radian, 0, M_PI / 2);
}


float CMFCTEST4Dlg::getRefinementConstraintAngle() {
	return constraint_angle;
}


void CMFCTEST4Dlg::OnBnClickedEdgefieldRefinement()
{
	ASSERT(the_view != NULL);

	if(this->loadedMediaType == VIDEO_LOADED) {
		enable_angle_constraint = (reinterpret_cast<CButton*>(GetDlgItem(IDC_EDGEFIELD_REFINEMENT))->GetCheck() ? true : false);
		
		setUpForVideo(current_frame_number, true);
	}
}


void CMFCTEST4Dlg::OnEnKillfocusAngleConstraint()
{
	CEdit* ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_ANGLE_CONSTRAINT));
	float val = (float)::getNumberFromCEdit(ce);

	setRefinementConstraintAngle((float)degreeToRadian(val));

	val = (float)radianToDegree(getRefinementConstraintAngle());

	::setNumberToCEdit(ce, val, num_prec_digits);
}


void CMFCTEST4Dlg::OnDeltaposAngleConstraintSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	float delta = 0.1f;
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	CEdit* ce = reinterpret_cast<CEdit*>(GetDlgItem(IDC_ANGLE_CONSTRAINT));

	if(pNMUpDown->iDelta > 0) delta *= -1;

	::setNumberToCEdit(ce, (float)::getNumberFromCEdit(ce) + delta, num_prec_digits);

	OnEnKillfocusAngleConstraint();

	*pResult = 0;
}


void CMFCTEST4Dlg::OnLoadSegmentation()
{
	// TODO: Add your command handler code here
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;

	if(the_view == NULL) return;


	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("All files {*.*}\0*.*\0PPM {.ppm}\0*.ppm\0JPG {.jpg;.jpeg}\0*.jpg;*.jpeg\0BMP {.bmp}\0*.bmp\0AVI {.avi}\0*.avi\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.lpstrFileTitle = strFileTitleName;
	ofn.nMaxFileTitle = sizeof(strFileTitleName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("ppm");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;

	if(GetOpenFileName(&ofn)) {
		openSeg(strFileName, strFileTitleName);
	}

	// restart drawing
	if(prev_timer_status == true) the_view->startDisplayTimer();
}

void CMFCTEST4Dlg::openSeg(LPCSTR str_filename, LPCSTR str_titlename)
{
	CString msg, append;
	tstring filename, fn_cp;

	input_width = -1;
	input_height = -1;

	filename = tstring(str_filename);
	fn_cp = tstring(str_filename);

	for(int i = 0; i < (int)fn_cp.length(); i++) fn_cp[i] = tolower(fn_cp[i]);

	if(fn_cp.rfind(".ppm") != string::npos) {
		imgld.readPPM(filename.c_str(), &input_image, input_width, input_height);
		
		if(input_width > 0 && input_height > 0) loadedMediaType = IMAGE_LOADED;
		else loadedMediaType = NOTHING_LOADED;
	}

	setUpForSeg(input_image, input_width, input_height);
}

void CMFCTEST4Dlg::setUpForSeg(unsigned char* img, int w, int h)
{
	bool prev_timer_status;
	CString msg;
	clock_t t1, t2;


	if(img == NULL || w <= 0 || h <= 0 || the_view == NULL) return;


	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	t1 = clock();
	
	the_view->setSegImage(img, w,h);
	t2 = clock();

	//if(dbg_timing == true) {
		msg.Format(_T("Set Seg Image: %.3lf sec\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		printMessage(msg);
	//}
	
	


	//the_view->initializeRendering();

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnLoadDatimage() {
	// TODO: Add your command handler code here
	bool prev_timer_status;
	static TCHAR strFileName[MAX_PATH];
	static TCHAR strFileTitleName[MAX_PATH];
	OPENFILENAME ofn;

	if(the_view == NULL) return;


	// temporarily stop drawing
	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();


	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.lpstrFilter = _T("DAT {.dat}\0*.dat\0\0");
	ofn.lpstrFile = strFileName;
	ofn.nMaxFile = sizeof(strFileName);
	ofn.lpstrFileTitle = strFileTitleName;
	ofn.nMaxFileTitle = sizeof(strFileTitleName);
	ofn.Flags = OFN_FILEMUSTEXIST;
	ofn.lpstrDefExt = _T("dat");
	ofn.hwndOwner = GetSafeHwnd();
	ofn.hInstance = NULL;



/////////////////////////////////////////////////////////////////////
	// load the dat
	CString msg;

	if(GetOpenFileName(&ofn)) {
		the_view->loadDat(strFileName);
		msg.Format(_T("Set Dat Image\r\n"));
		printMessage(msg);

		/*getting '.'*/
		int count = 0;
		while(strFileName[count]!='.') count++;

		/*getting the path*/
		string filename(strFileName);
		size_t pos = filename.rfind(".");

		if(pos != string::npos) {
			assert(pos + 1 - 7 > 0); 

			if(DatPath != NULL) delete[] DatPath;
			DatPath = new char[pos + 1 - 7];

			for(size_t i = 0; i < pos - 7; i++) {
				DatPath[i] = filename.c_str()[i];
			}

			DatPath[pos - 7] = '\0';
		}

		the_view->setNumFrames(num_frames, string(DatPath));

		loaded_dats = true;

		// load the region data for the current frame
		DatforVideo();
	}


/////////////////////////////////////////////////////////////////////

	// restart drawing
	if(prev_timer_status == true) { the_view->startDisplayTimer(); }
}


void CMFCTEST4Dlg::OnUnloadDatimage() {
	unloadDatImage();

	if(loadedMediaType != VIDEO_LOADED) {
	}
	else {
		setUpForVideo(current_frame_number, true);
	}

	printMessage("Unloaded Dat Image\r\n");
}


void CMFCTEST4Dlg::unloadDatImage() {
	bool prev_timer_status;

	if(the_view == NULL) return;

	prev_timer_status = the_view->getDisplayTimerStatus();
	the_view->stopDisplayTimer();
	the_view->unloadDat();
	the_view->setNumFrames(num_frames);

	loaded_dats = false;

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::DatforVideo(void) {
	int n1 = current_frame_number, n2 = n1 + 1;
	stringstream filename;
	const string suffix = ".dat";
	bool prev_timer_status;


	if(the_view == NULL) return;

	prev_timer_status = the_view->getDisplayTimerStatus();

	if(DatPath != NULL) {
		filename.str("");
		filename << setfill('0') << DatPath << setw(3) << n1 << '_' << setw(3) << n2 << suffix;

		//MessageBox(filename.str().c_str());
		the_view->loadDat(filename.str().c_str());  
	}

	if(prev_timer_status == true) the_view->startDisplayTimer();
}


void CMFCTEST4Dlg::OnBnClickedTemporalSmoothingStrokeColors() {
	bool checked = false;
	list<IplImage*>::iterator it_ipl;


	checked = (reinterpret_cast<CButton*>(GetDlgItem(IDC_TEMPORAL_SMOOTHING_STROKE_COLORS))->GetCheck() ? true : false);


	if(checked == true) {
		the_view->enableStrokeImageInput(true);

		if(this->loadedMediaType == VIDEO_LOADED) {
			frame_list_update_status = 3;
		}

		// just call this function rather than setUpForVideo() to avoid whole computation.
		setStrokeRefImage(current_frame_number);
	}
	else {
		the_view->enableStrokeImageInput(false);

		// dispose all cached images.
		for(it_ipl = frame_list.begin(); it_ipl != frame_list.end(); it_ipl++) {
			if(*it_ipl != NULL) cvReleaseImage(&(*it_ipl));
		}

		frame_list.clear();
		frame_list_update_status = 0;				// just in case...
		assert(frame_list.size() == 0);
	}

	the_view->initializeRendering();
}


void CMFCTEST4Dlg::setStrokeRefImage(int frame_number) {
	switch(loadedMediaType) {
		case IMAGE_LOADED:
			setStrokeRefImageForImage(frame_number);
			break;

		case VIDEO_LOADED:
			setStrokeRefImageForVideo(frame_number);
			break;

		default:
			break;
	}
}


void CMFCTEST4Dlg::setStrokeRefImageForImage(int frame_number) {
	unsigned char *buffer = NULL;
	IplImage *ipl = cvCreateImage(cvSize(input_width, input_height), IPL_DEPTH_8U, 3);


	buffer = new unsigned char[3 * input_width * input_height];

	UC2IPL(input_image, ipl);

	cvFilter2D(ipl, ipl, mat_s);

	IPL2UC(ipl, buffer);

	the_view->setStrokeRefImage(buffer, input_width, input_height);

	//::My_SSS_SavePPM(input_width, input_height, input_image, "blah.ppm");

	delete[] buffer;
	cvReleaseImage(&ipl);
}


void CMFCTEST4Dlg::setStrokeRefImageForVideo(int frame_number) {
	bool extrapolation = true;
	const int num_frames = 5;		// for now, fix it.
	const int shift = num_frames >> 1;
	IplImage *ipl = NULL;
	list<IplImage*>::iterator it_ipl;

	assert(num_frames == EdgeFieldConstructor::temporal_kernel_size);

	switch(frame_list_update_status) {
		case 1:
			ipl = frame_list.front();
			frame_list.pop_front();
			cvReleaseImage(&ipl);

			getImageFromFrame(frame_number + shift, &ipl, extrapolation);
			cvFilter2D(ipl, ipl, mat_s);
			frame_list.push_back(ipl);

			break;


		case 2:
			ipl = frame_list.back();
			frame_list.pop_back();
			cvReleaseImage(&ipl);

			getImageFromFrame(frame_number - shift, &ipl, extrapolation);
			cvFilter2D(ipl, ipl, mat_s);
			frame_list.push_front(ipl);

			break;


		case 3:
			for(it_ipl = frame_list.begin(); it_ipl != frame_list.end(); it_ipl++) {
				cvReleaseImage(&(*it_ipl));
			}

			frame_list.clear();

			for(int i = 0; i < num_frames; i++) {
				getImageFromFrame(frame_number - shift + i, &ipl, extrapolation);
				cvFilter2D(ipl, ipl, mat_s);
				frame_list.push_back(ipl);
			}

			break;


		default:		// Do nothing...
			break;
	}


	// reset
	frame_list_update_status = 0;


	assert(num_frames == frame_list.size());

	if(frame_list.empty() == false) {
		int idx, idx2;
		int w, h, ch, step;
		int num_elems;
		unsigned char *img = NULL;
		double *acc = NULL;


		it_ipl = frame_list.begin();

		w = (*it_ipl)->width;
		h = (*it_ipl)->height;
		ch = (*it_ipl)->nChannels;
		step = (*it_ipl)->widthStep;
		num_elems = 3 * w * h;
		img = new unsigned char[num_elems];
		acc = new double[num_elems];

		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = 3 * (j + w * i);
				idx2 = ch * j + step * i;
				acc[idx + 0] = EdgeFieldConstructor::gaussian1D[0] * (unsigned char)(*it_ipl)->imageData[idx2 + 0];
				acc[idx + 1] = EdgeFieldConstructor::gaussian1D[0] * (unsigned char)(*it_ipl)->imageData[idx2 + 1];
				acc[idx + 2] = EdgeFieldConstructor::gaussian1D[0] * (unsigned char)(*it_ipl)->imageData[idx2 + 2];
			}
		}


		for(int k = 1; k < 5; k++) {
			it_ipl++;

			for(int i = 0; i < h; i++) {
				for(int j = 0; j < w; j++) {
					idx = 3 * (j + w * i);
					idx2 = ch * j + step * i;
					acc[idx + 0] += EdgeFieldConstructor::gaussian1D[k] * (unsigned char)(*it_ipl)->imageData[idx2 + 0];
					acc[idx + 1] += EdgeFieldConstructor::gaussian1D[k] * (unsigned char)(*it_ipl)->imageData[idx2 + 1];
					acc[idx + 2] += EdgeFieldConstructor::gaussian1D[k] * (unsigned char)(*it_ipl)->imageData[idx2 + 2];
				}
			}
		}

		for(int i = 0; i < num_elems; i++) {
			img[i] = (unsigned char)((acc[i] <= 255) ? acc[i] : 255);
		}

		delete[] acc;

		the_view->setStrokeRefImage(img, (*it_ipl)->width, (*it_ipl)->height);

		delete[] img;
	}
}


void CMFCTEST4Dlg::preCreateCVMatKernel() {
	int dim = EdgeFieldConstructor::spatial_kernel_size;

	mat_s_data = new double[dim * dim];

	for(int i = 0; i < dim; i++) {
		for(int j = 0; j < dim; j++) {
			mat_s_data[j + dim * i] = EdgeFieldConstructor::gaussian1D[i] * EdgeFieldConstructor::gaussian1D[j];
		}
	}

	mat_s = cvCreateMat(dim, dim, CV_64FC1);
	cvInitMatHeader(mat_s, dim, dim, CV_64FC1, mat_s_data);
}


void CMFCTEST4Dlg::disposeCVMatKernel() {
	if(mat_s) cvReleaseMat(&mat_s);
	if(mat_s_data) delete[] mat_s_data;
}


void CMFCTEST4Dlg::UC2IPL(unsigned char *uc, IplImage *ipl, bool swap_rb) {
	int idx, idx2;
	int w, h;
	int ch, step;

	w = ipl->width;
	h = ipl->height;
	ch = ipl->nChannels;
	step = ipl->widthStep;

	if(swap_rb == true) {
		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = 3 * (j + w * i);
				idx2 = ch * j + step * i;
				ipl->imageData[idx2 + 0] = (char)uc[idx + 2];
				ipl->imageData[idx2 + 1] = (char)uc[idx + 1];
				ipl->imageData[idx2 + 2] = (char)uc[idx + 0];
			}
		}
	}
	else {
		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = 3 * (j + w * i);
				idx2 = ch * j + step * i;
				ipl->imageData[idx2 + 0] = (char)uc[idx + 0];
				ipl->imageData[idx2 + 1] = (char)uc[idx + 1];
				ipl->imageData[idx2 + 2] = (char)uc[idx + 2];
			}
		}
	}
}


void CMFCTEST4Dlg::IPL2UC(IplImage *ipl, unsigned char *uc, bool swap_rb) {
	int idx, idx2;
	int w, h;
	int ch, step;

	w = ipl->width;
	h = ipl->height;
	ch = ipl->nChannels;
	step = ipl->widthStep;


	if(swap_rb == true) {
		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = 3 * (j + w * i);
				idx2 = ch * j + step * i;
				uc[idx + 0] = (unsigned char)ipl->imageData[idx2 + 2];
				uc[idx + 1] = (unsigned char)ipl->imageData[idx2 + 1];
				uc[idx + 2] = (unsigned char)ipl->imageData[idx2 + 0];
			}
		}
	}
	else {
		for(int i = 0; i < h; i++) {
			for(int j = 0; j < w; j++) {
				idx = 3 * (j + w * i);
				idx2 = ch * j + step * i;
				uc[idx + 0] = (unsigned char)ipl->imageData[idx2 + 0];
				uc[idx + 1] = (unsigned char)ipl->imageData[idx2 + 1];
				uc[idx + 2] = (unsigned char)ipl->imageData[idx2 + 2];
			}
		}
	}
}


void CMFCTEST4Dlg::notifyRegionSelection() {
	reinterpret_cast<TabContent1*>(tabContents[1])->updateStyleInterpolationCombo();
}



void CMFCTEST4Dlg::OnShowinput()
{
	// TODO: Add your command handler code here
	reinterpret_cast<TabContent1*>(tabContents[1])->OnShowinput();

}


void CMFCTEST4Dlg::OnShowopt()
{
	// TODO: Add your command handler code here
	reinterpret_cast<TabContent1*>(tabContents[1])->OnShowopt();
}


void CMFCTEST4Dlg::OnShowpainting()
{
	// TODO: Add your command handler code here
	reinterpret_cast<TabContent1*>(tabContents[1])->OnShowpainting();
}


void CMFCTEST4Dlg::OnShowseg()
{
	// TODO: Add your command handler code here
	reinterpret_cast<TabContent1*>(tabContents[1])->OnShowseg();
}


void CMFCTEST4Dlg::OnShowtensor()
{
	// TODO: Add your command handler code here
	reinterpret_cast<TabContent1*>(tabContents[1])->OnShowtensor();
}


void CMFCTEST4Dlg::OnShowvec()
{
	// TODO: Add your command handler code here
	reinterpret_cast<TabContent1*>(tabContents[1])->OnShowvec();
}
