// MFCTEST4Dlg.h : wb_[ t@C
//

#pragma once
#include "VisView.h"
#include "ImageLoader.h"
#include "cv.h"
#include "cxcore.h"
#include "cvaux.h"
#include "highgui.h"
#include "Resource.h"
#include "TabContent0.h"
#include "TabContent1.h"
#include "TabContent2.h"
#include "TabContent3.h"
#include "Tabcontent4.h"
#include "Tabcontent5.h"
#include "OpticalFlow.h"
#include "OpticalFlowReader.h"
#include "BinaryMapReader.h"
#include "EdgeFieldConstructor.h"
#include <commctrl.h>

#include <list>
#include <map>


using namespace std;


class VisView;

// CMFCTEST4Dlg _CAO
class CMFCTEST4Dlg : public CDialog {
private:
	VisView *the_view;
	ImageLoader imgld;
	HGLRC hRC;
	HACCEL m_hAccelTable;
	const static int numTabs = 5;
	static LPCTSTR tabNames[numTabs];
	CDialog* tabContents[numTabs];

	int loadedMediaType;

	bool enable_temporal_smoothing;
	static const int temporal_smoothing_depth = 5;		// must be an odd number;
	bool enable_stroke_tracking;


	static const int num_prec_digits = 1;
	bool enable_angle_constraint;
	float constraint_angle;
	EdgeFieldConstructor field_constructor;
	FieldSampleImage *old_fsi;


	// Variables for OpenCV 
	CvVideoWriter *video_writer;
	CvCapture *input_video;
	CvTermCriteria term;
	CvSize frame_size;

	unsigned char* input_image;
	int input_width, input_height;

	// Optical flow
	bool loaded_optflow;
	OpticalFlowReader fwd_optflow;

	// Edge field blending
	list<FieldSampleImage*> fsi_list;

	// Frames to smooth
	list<IplImage*> frame_list;


	// 0: no need to update
	// 1: pop from left, push from right
	// 2: pop from right, push from left
	// 3: update all in fsi_list
	int fsi_list_update_status;
	int frame_list_update_status;


	// video render range
	int render_start, render_end;


	// some statistics
	time_t run_time[2];
	double total_run_time;
	double total_render_time;
	size_t *stroke_counts[2];


	int current_frame_number;
	int num_frames;



	// RXgNV
public:
	CMFCTEST4Dlg(CWnd* pParent = NULL);	// WRXgN^
	virtual ~CMFCTEST4Dlg();


	// _CAO f[^
	enum { IDD = IDD_MFCTEST4_DIALOG };

	const static int NOTHING_LOADED = 0;
	const static int IMAGE_LOADED = 1;
	const static int VIDEO_LOADED = 2;


	// 
protected:
	HICON m_hIcon;
	CMenu menu;
	CToolBar toolbar;

	// ꂽAbZ[W蓖Ċ֐
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnPaint();
	afx_msg void OnSetFocus(CWnd *pOldWnd);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnDestroy();

	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV T|[g

	void fileLoadInitialization(LPCSTR filename);

	

public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);


	void openFile(LPCSTR str_filename, LPCSTR str_titlename);
	void closeFile();

	int getFileLoadStatus();
	VisView* getVisView();

	void setUpForImage(unsigned char *img, int w, int h, bool retain_design_elements);
	void setEdgeFieldForImage();
	void setEdgeFieldForImage(bool retain_design_elements);

	void setUpForVideo(int frame_number, bool retain_design_elements);
	void setEdgeFieldForVideo();
	void setEdgeFieldForVideo(int frame_number, bool retain_design_elements);

	void setStrokeRefImage(int frame_number);
	void setStrokeRefImageForImage(int frame_number);
	void setStrokeRefImageForVideo(int frame_number);

	bool createFieldSampleImageFromFrame(int frame_number, FieldSampleImage *fsi, bool smooth = false);
	bool createFieldSampleImageFromFrame(int frame_number, FieldSampleImage **fsi, bool smooth = false);

	void getImageFromFrame(int frame_number, IplImage **img, bool extrapolate = false);
	void getDepthFrame(int frame_number, IplImage **current, bool extrapolate = false);

	// optical flow estimation... 0: LK, 1: LKPyr(computes at every pixels)
	void calcOptFlow(int frame_number, OpticalFlow *optFlow, CvPoint2D32f *data, bool reverse_flow = false, int method = 0);
	void calcOptFlow(int frame_number, OpticalFlow *optFlow, float *data, bool reverse_flow = false, int method = 0);


	bool saveVideo(string file_name, bool show_codec_option = true);
	void setVideoRenderingRange(int start, int end);
	int getVideoRenderingStartFrame();
	int getVideoRenderingEndFrame();
	int getMaxFrames();

	void getStrokeCountsAtBegin(size_t *values);
	void getStrokeCountsAtEnd(size_t *values);


	bool checkOpticalFlowLoaded();
	void loadOpticalFlow(string &filename);
	void unloadOpticalFlow();

	bool checkBinaryMapLoaded();
	void loadBinaryMap(string &filename);
	void unloadBinaryMap();

	void enableStrokeTracking(bool val);
	bool getStrokeTrackingEnabled();

	void enableTemporalSmoothing(bool val);
	bool getTemporalSmoothingEnabled();

	void enableEdgeFieldRefinement(bool val);
	bool getEdgeFieldRefinementEnabled();
	void setRefinementConstraintAngle(float angle_radian);
	float getRefinementConstraintAngle();


	void updatePaintingPresetParameters();
	void updateSelectedRegionOption();


	void processScript(LPCSTR filename);

	double getTotalRunTime();
	double getTotalRenderTime();


public:
	static const int max_img_seq = 600;

	int getCurrentFrame();
	void setCurrentFrame(int val);


private:
	bool dbg_timing;


public:
	void printMessage(CString str);
	void enablePrintStat(bool val);
	bool getPrintStatEnabled();

	void copyOptFlowForVis();

	void notifyRegionSelection();

private:
	CvMat *mat_s;
	double *mat_s_data;
	void preCreateCVMatKernel();
	void disposeCVMatKernel();
	void UC2IPL(unsigned char *uc, IplImage *ipl, bool swap_rb = true);
	void IPL2UC(IplImage *ipl, unsigned char *uc, bool swap_rb = true);

private:
	bool use_binary_map;
	bool loaded_binary_map;
	unsigned char *binary_map;
	BinaryMapReader bmr;

	bool loaded_dats;


public:
	afx_msg void OnFileOpen();
	afx_msg void OnFileClose();
	afx_msg void OnFileSaveControl();
	afx_msg void OnFileLoadControl();
	afx_msg void OnFileQuit();
	afx_msg void OnTcnSelchangeTab1(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnUndo();
	afx_msg void OnRedo();
	afx_msg void OnShowCtlKnob();
	afx_msg void OnShowSeparatrices();
	afx_msg void OnShowSingularities();
	afx_msg void OnEnKillfocusVideoCurrentFrame();
	afx_msg void OnBnClickedForward();
	afx_msg void OnBnClickedBack();
	afx_msg void OnBnClickedFf();
	afx_msg void OnBnClickedRew();
	afx_msg void OnFileSaveImage();
	afx_msg void OnFileSaveVideo();
	afx_msg void OnBnClickedTemporalSmoothing();
	afx_msg void OnBnClickedEnableStrokeTracking();
	afx_msg void OnFileLoadOpticalFlow();
	afx_msg void OnFileUnoadOpticalFlow();
	afx_msg void OnFileRunScript();
	afx_msg void OnBnClickedPrintMessage();

	DECLARE_MESSAGE_MAP()

	afx_msg void OnBnClickedJumpNextKey();
	afx_msg void OnBnClickedJumpPrevKey();
	afx_msg void OnBnClickedJumpNextKey2();
	afx_msg void OnBnClickedJumpPrevKey2();

	afx_msg void OnEditClearMessageBox();
	afx_msg void OnFileLoadBinaryMap();
	afx_msg void OnFileUnloadBinaryMap();
	afx_msg void OnFileLoadEdgeField();
	afx_msg void OnFileSaveEdgeField();
	afx_msg void OnFileLoadVideoProfile();
	afx_msg void OnBnClickedEdgefieldRefinement();
	afx_msg void OnEnKillfocusAngleConstraint();
	afx_msg void OnDeltaposAngleConstraintSpin(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnLoadSegmentation();
	afx_msg void OnLoadDatimage();
	afx_msg void OnUnloadDatimage();

	void openSeg(LPCSTR str_filename, LPCSTR str_titlename);
	void setUpForSeg(unsigned char* img, int w, int h);
	void unloadDatImage();
	char* DatPath;
	void DatforVideo(void);
public:
	afx_msg void OnBnClickedTemporalSmoothingStrokeColors();
	afx_msg void OnShowinput();
	afx_msg void OnShowopt();
	afx_msg void OnShowpainting();
	afx_msg void OnShowseg();
	afx_msg void OnShowtensor();
	afx_msg void OnShowvec();
};
