#pragma once
#include "PaintingPreset.h"

class PPInterpolatable {
public:
	// make everything public...
	static const int num_elems = 16;
	float values[num_elems];
	bool mask;


	PPInterpolatable() { mask = true; zero(); }
	virtual ~PPInterpolatable() {}


	void pack(const PaintingPreset &p) {
		values[0]	= p.displacement;
		values[1]	= p.jitter_amplitude;
		values[2]	= p.stroke_pos_jitter;
		values[3]	= p.stroke_size;
		values[4]	= p.stroke_initial_opacity;
		values[5]	= (float)p.stroke_min_length;
		values[6]	= (float)p.stroke_max_length;
		values[7]	= (float)p.num_layers;
		values[8]	= p.area_error;
		values[9]	= p.post_process_val1;
		values[10]	= p.post_process_val2;
		values[11]	= p.post_process_val3;
		values[12]	= p.post_process_val4;
		values[13]	= p.stroke_color_shift[0];
		values[14]	= p.stroke_color_shift[1];
		values[15]	= p.stroke_color_shift[2];
	}


	void unpack(PaintingPreset &p) {
		p.displacement				= values[0];
		p.jitter_amplitude			= values[1];
		p.stroke_pos_jitter			= values[2];
		p.stroke_size				= values[3];
		p.stroke_initial_opacity	= values[4];
		p.stroke_min_length			= (int)(values[5] + .5f);
		p.stroke_max_length			= (int)(values[6] + .5f);
		p.num_layers				= (int)(values[7] + .5f);
		p.area_error				= values[8];
		p.post_process_val1			= values[9];
		p.post_process_val2			= values[10];
		p.post_process_val3			= values[11];
		p.post_process_val4			= values[12];
		p.stroke_color_shift[0]		= values[13];
		p.stroke_color_shift[1]		= values[14];
		p.stroke_color_shift[2]		= values[15];
	}


	void copy(const PPInterpolatable &p) {
		*this = p;
		this->mask = p.mask;
	}


	void zero() {
		values[0]	= 0;
		values[1]	= 0;
		values[2]	= 0;
		values[3]	= 0;
		values[4]	= 0;
		values[5]	= 0;
		values[6]	= 0;
		values[7]	= 0;
		values[8]	= 0;
		values[9]	= 0;
		values[10]	= 0;
		values[11]	= 0;
		values[12]	= 0;
		values[13]	= 0;
		values[14]	= 0;
		values[15]	= 0;
	}


	// just copy only values. not mask.
	inline PPInterpolatable &operator=(const PPInterpolatable &p) {
		this->values[0]		= p.values[0];
		this->values[1]		= p.values[1];
		this->values[2]		= p.values[2];
		this->values[3]		= p.values[3];
		this->values[4]		= p.values[4];
		this->values[5]		= p.values[5];
		this->values[6]		= p.values[6];
		this->values[7]		= p.values[7];
		this->values[8]		= p.values[8];
		this->values[9]		= p.values[9];
		this->values[10]	= p.values[10];
		this->values[11]	= p.values[11];
		this->values[12]	= p.values[12];
		this->values[13]	= p.values[13];
		this->values[14]	= p.values[14];
		this->values[15]	= p.values[15];

		return *this;	
	}


	inline PPInterpolatable &operator+=(const PPInterpolatable &p) {
		this->values[0]		+= p.values[0];
		this->values[1]		+= p.values[1];
		this->values[2]		+= p.values[2];
		this->values[3]		+= p.values[3];
		this->values[4]		+= p.values[4];
		this->values[5]		+= p.values[5];
		this->values[6]		+= p.values[6];
		this->values[7]		+= p.values[7];
		this->values[8]		+= p.values[8];
		this->values[9]		+= p.values[9];
		this->values[10]	+= p.values[10];
		this->values[11]	+= p.values[11];
		this->values[12]	+= p.values[12];
		this->values[13]	+= p.values[13];
		this->values[14]	+= p.values[14];
		this->values[15]	+= p.values[15];

		return *this;	
	}

	inline PPInterpolatable &operator-=(const PPInterpolatable &p) {
		this->values[0]		-= p.values[0];
		this->values[1]		-= p.values[1];
		this->values[2]		-= p.values[2];
		this->values[3]		-= p.values[3];
		this->values[4]		-= p.values[4];
		this->values[5]		-= p.values[5];
		this->values[6]		-= p.values[6];
		this->values[7]		-= p.values[7];
		this->values[8]		-= p.values[8];
		this->values[9]		-= p.values[9];
		this->values[10]	-= p.values[10];
		this->values[11]	-= p.values[11];
		this->values[12]	-= p.values[12];
		this->values[13]	-= p.values[13];
		this->values[14]	-= p.values[14];
		this->values[15]	-= p.values[15];

		return *this;	
	}

	inline PPInterpolatable &operator*=(const float val) {
		this->values[0]		= this->values[0] * val;
		this->values[1]		= this->values[1] * val;
		this->values[2]		= this->values[2] * val;
		this->values[3]		= this->values[3] * val;
		this->values[4]		= this->values[4] * val;
		this->values[5]		= this->values[5] * val;
		this->values[6]		= this->values[6] * val;
		this->values[7]		= this->values[7] * val;
		this->values[8]		= this->values[8] * val;
		this->values[9]		= this->values[9] * val;
		this->values[10]	= this->values[10] * val;
		this->values[11]	= this->values[11] * val;
		this->values[12]	= this->values[12] * val;
		this->values[13]	= this->values[13] * val;
		this->values[14]	= this->values[14] * val;
		this->values[15]	= this->values[15] * val;

		return *this;
	}

	inline bool operator<(const PPInterpolatable &p) {
		return 
			(this->values[0]	>= p.values[0]) ? false : 
			(this->values[1]	>= p.values[1]) ? false : 
			(this->values[2]	>= p.values[2]) ? false : 
			(this->values[3]	>= p.values[3]) ? false : 
			(this->values[4]	>= p.values[4]) ? false : 
			(this->values[5]	>= p.values[5]) ? false : 
			(this->values[6]	>= p.values[6]) ? false : 
			(this->values[7]	>= p.values[7]) ? false : 
			(this->values[8]	>= p.values[8]) ? false :  
			(this->values[9]	>= p.values[9]) ? false : 
			(this->values[10]	>= p.values[10]) ? false : 
			(this->values[11]	>= p.values[11]) ? false : 
			(this->values[12]	>= p.values[12]) ? false : 
			(this->values[13]	>= p.values[13]) ? false : 
			(this->values[14]	>= p.values[14]) ? false : 
			(this->values[15]	>= p.values[15]) ? false : 
			true;
	}

	inline bool operator<(const float &val) {
		return 
			(this->values[0]	>= val) ? false : 
			(this->values[1]	>= val) ? false : 
			(this->values[2]	>= val) ? false : 
			(this->values[3]	>= val) ? false : 
			(this->values[4]	>= val) ? false : 
			(this->values[5]	>= val) ? false : 
			(this->values[6]	>= val) ? false : 
			(this->values[7]	>= val) ? false : 
			(this->values[8]	>= val) ? false : 
			(this->values[9]	>= val) ? false : 
			(this->values[10]	>= val) ? false : 
			(this->values[11]	>= val) ? false : 
			(this->values[12]	>= val) ? false : 
			(this->values[13]	>= val) ? false : 
			(this->values[14]	>= val) ? false : 
			(this->values[15]	>= val) ? false : 
			true;
	}


	inline bool operator<=(const PPInterpolatable &p) {
		return 
			(this->values[0]	> p.values[0]) ? false : 
			(this->values[1]	> p.values[1]) ? false : 
			(this->values[2]	> p.values[2]) ? false : 
			(this->values[3]	> p.values[3]) ? false : 
			(this->values[4]	> p.values[4]) ? false : 
			(this->values[5]	> p.values[5]) ? false : 
			(this->values[6]	> p.values[6]) ? false : 
			(this->values[7]	> p.values[7]) ? false : 
			(this->values[8]	> p.values[8]) ? false :  
			(this->values[9]	> p.values[9]) ? false : 
			(this->values[10]	> p.values[10]) ? false : 
			(this->values[11]	> p.values[11]) ? false : 
			(this->values[12]	> p.values[12]) ? false : 
			(this->values[13]	> p.values[13]) ? false : 
			(this->values[14]	> p.values[14]) ? false : 
			(this->values[15]	> p.values[15]) ? false : 
			true;
	}


	inline bool operator<=(const float &val) {
		return 
			(this->values[0]	> val) ? false : 
			(this->values[1]	> val) ? false : 
			(this->values[2]	> val) ? false : 
			(this->values[3]	> val) ? false : 
			(this->values[4]	> val) ? false : 
			(this->values[5]	> val) ? false : 
			(this->values[6]	> val) ? false : 
			(this->values[7]	> val) ? false : 
			(this->values[8]	> val) ? false : 
			(this->values[9]	> val) ? false : 
			(this->values[10]	> val) ? false : 
			(this->values[11]	> val) ? false : 
			(this->values[12]	> val) ? false : 
			(this->values[13]	> val) ? false : 
			(this->values[14]	> val) ? false : 
			(this->values[15]	> val) ? false : 
			true;
	}


	inline bool operator>(const PPInterpolatable &p) {
		return 
			(this->values[0]	<= p.values[0]) ? false : 
			(this->values[1]	<= p.values[1]) ? false : 
			(this->values[2]	<= p.values[2]) ? false : 
			(this->values[3]	<= p.values[3]) ? false : 
			(this->values[4]	<= p.values[4]) ? false : 
			(this->values[5]	<= p.values[5]) ? false : 
			(this->values[6]	<= p.values[6]) ? false : 
			(this->values[7]	<= p.values[7]) ? false : 
			(this->values[8]	<= p.values[8]) ? false :  
			(this->values[9]	<= p.values[9]) ? false : 
			(this->values[10]	<= p.values[10]) ? false : 
			(this->values[11]	<= p.values[11]) ? false : 
			(this->values[12]	<= p.values[12]) ? false : 
			(this->values[13]	<= p.values[13]) ? false : 
			(this->values[14]	<= p.values[14]) ? false : 
			(this->values[15]	<= p.values[15]) ? false : 
			true;
	}


	inline bool operator>(const float &val) {
		return 
			(this->values[0]	<= val) ? false : 
			(this->values[1]	<= val) ? false : 
			(this->values[2]	<= val) ? false : 
			(this->values[3]	<= val) ? false : 
			(this->values[4]	<= val) ? false : 
			(this->values[5]	<= val) ? false : 
			(this->values[6]	<= val) ? false : 
			(this->values[7]	<= val) ? false : 
			(this->values[8]	<= val) ? false : 
			(this->values[9]	<= val) ? false : 
			(this->values[10]	<= val) ? false : 
			(this->values[11]	<= val) ? false : 
			(this->values[12]	<= val) ? false : 
			(this->values[13]	<= val) ? false : 
			(this->values[14]	<= val) ? false : 
			(this->values[15]	<= val) ? false : 
			true;
	}


	inline bool operator>=(const PPInterpolatable &p) {
		return 
			(this->values[0]	< p.values[0]) ? false : 
			(this->values[1]	< p.values[1]) ? false : 
			(this->values[2]	< p.values[2]) ? false : 
			(this->values[3]	< p.values[3]) ? false : 
			(this->values[4]	< p.values[4]) ? false : 
			(this->values[5]	< p.values[5]) ? false : 
			(this->values[6]	< p.values[6]) ? false : 
			(this->values[7]	< p.values[7]) ? false : 
			(this->values[8]	< p.values[8]) ? false :  
			(this->values[9]	< p.values[9]) ? false : 
			(this->values[10]	< p.values[10]) ? false : 
			(this->values[11]	< p.values[11]) ? false : 
			(this->values[12]	< p.values[12]) ? false : 
			(this->values[13]	< p.values[13]) ? false : 
			(this->values[14]	< p.values[14]) ? false : 
			(this->values[15]	< p.values[15]) ? false : 
			true;
	}


	inline bool operator>=(const float &val) {
		return 
			(this->values[0]	< val) ? false : 
			(this->values[1]	< val) ? false : 
			(this->values[2]	< val) ? false : 
			(this->values[3]	< val) ? false : 
			(this->values[4]	< val) ? false : 
			(this->values[5]	< val) ? false : 
			(this->values[6]	< val) ? false : 
			(this->values[7]	< val) ? false : 
			(this->values[8]	< val) ? false : 
			(this->values[9]	< val) ? false : 
			(this->values[10]	< val) ? false : 
			(this->values[11]	< val) ? false : 
			(this->values[12]	< val) ? false : 
			(this->values[13]	< val) ? false : 
			(this->values[14]	< val) ? false : 
			(this->values[15]	< val) ? false : 
			true;
	}


	inline void _max(const PPInterpolatable &p) {
#define _ppmax(a, b)            (((a) > (b)) ? (a) : (b))
		this->values[0]		= _ppmax(this->values[0],	p.values[0]);
		this->values[1]		= _ppmax(this->values[1],	p.values[1]);
		this->values[2]		= _ppmax(this->values[2],	p.values[2]);
		this->values[3]		= _ppmax(this->values[3],	p.values[3]);
		this->values[4]		= _ppmax(this->values[4],	p.values[4]);
		this->values[5]		= _ppmax(this->values[5],	p.values[5]);
		this->values[6]		= _ppmax(this->values[6],	p.values[6]);
		this->values[7]		= _ppmax(this->values[7],	p.values[7]);
		this->values[8]		= _ppmax(this->values[8],	p.values[8]);
		this->values[9]		= _ppmax(this->values[9],	p.values[9]);
		this->values[10]	= _ppmax(this->values[10],	p.values[10]);
		this->values[11]	= _ppmax(this->values[11],	p.values[11]);
		this->values[12]	= _ppmax(this->values[12],	p.values[12]);
		this->values[13]	= _ppmax(this->values[13],	p.values[13]);
		this->values[14]	= _ppmax(this->values[14],	p.values[14]);
		this->values[15]	= _ppmax(this->values[15],	p.values[15]);
#undef _ppmax
	}


	inline void _min(const PPInterpolatable &p) {
#define _ppmin(a, b)            (((a) < (b)) ? (a) : (b))
		this->values[0]		= _ppmin(this->values[0],	p.values[0]);
		this->values[1]		= _ppmin(this->values[1],	p.values[1]);
		this->values[2]		= _ppmin(this->values[2],	p.values[2]);
		this->values[3]		= _ppmin(this->values[3],	p.values[3]);
		this->values[4]		= _ppmin(this->values[4],	p.values[4]);
		this->values[5]		= _ppmin(this->values[5],	p.values[5]);
		this->values[6]		= _ppmin(this->values[6],	p.values[6]);
		this->values[7]		= _ppmin(this->values[7],	p.values[7]);
		this->values[8]		= _ppmin(this->values[8],	p.values[8]);
		this->values[9]		= _ppmin(this->values[9],	p.values[9]);
		this->values[10]	= _ppmin(this->values[10],	p.values[10]);
		this->values[11]	= _ppmin(this->values[11],	p.values[11]);
		this->values[12]	= _ppmin(this->values[12],	p.values[12]);
		this->values[13]	= _ppmin(this->values[13],	p.values[13]);
		this->values[14]	= _ppmin(this->values[14],	p.values[14]);
		this->values[15]	= _ppmin(this->values[15],	p.values[15]);
#undef _ppmin
	}


	inline PPInterpolatable &operator*=(const double val) {
		return (*this *= (const float)val);
	}

	inline PPInterpolatable &operator*=(const int val) {
		return (*this *= (const float)val);
	}

	inline const PPInterpolatable operator+(const PPInterpolatable &p) const {
		return (PPInterpolatable(*this) += p);
	}

	inline const PPInterpolatable operator-(const PPInterpolatable &p) const {
		return (PPInterpolatable(*this) -= p);
	}

	inline const PPInterpolatable operator*(const float val) const {
		return (PPInterpolatable(*this) *= val);
	}

	inline const PPInterpolatable operator*(const double val) const {
		return (PPInterpolatable(*this) *= (const float)val);
	}

	inline const PPInterpolatable operator*(const int val) const {
		return (PPInterpolatable(*this) *= (const float)val);
	}

	inline bool operator<(const double &val) { return (*this < (float)val); }
	inline bool operator<=(const double &val) { return (*this <= (float)val); }
	inline bool operator>(const double &val) { return (*this > (float)val); }
	inline bool operator>=(const double &val) { return (*this >= (float)val); }
	
	inline const PPInterpolatable interpolate(const PPInterpolatable &operand, float blend_factor) {
		return PPInterpolatable(*this) * blend_factor + operand * (1.f - blend_factor);
	}
};


inline const PPInterpolatable operator*(const double val, const PPInterpolatable &p) { return p * val; }
inline const PPInterpolatable operator*(const float val, const PPInterpolatable &p) { return p * val; }
inline const PPInterpolatable operator*(const int val, const PPInterpolatable &p) { return p * (float)val; }

