#pragma once
#include <math.h>

class PaintingPreset {
public:
	// setting for color compositions
	int color_composition_method;		// not interpolatable
	int alpha;							// not interpolatable


	// setting for stroke length
	float displacement;


	// setting for color jittering
	float jitter_amplitude;


	// setting for strokes
	float stroke_pos_jitter;
	float stroke_gridsize;			// not interpolatable due to temporal coherency restriction
	float stroke_size;
	float stroke_initial_opacity;
	int stroke_min_length;
	int stroke_max_length;

	float stroke_color_shift[3];		// HSV

	// setting for multi-layer
	int num_layers;
	float area_error;


	// setting for post-processing
	int post_process_operation;		// not interpolatable
	float post_process_val1;
	float post_process_val2;
	float post_process_val3;
	float post_process_val4;


	inline void zero();				// we define this function after defining zero structure...
};



const static PaintingPreset PRESET_ZERO = {
	0, 
	0, 

	0, 

	0, 

	0, 
	0, 
	0, 
	0, 
	0, 
	0, 

	0, 
	0, 
	0, 

	0, 
	0, 

	0, 
	0, 
	0, 
	0, 
	0, 
};


inline void PaintingPreset::zero() {
	*this = PRESET_ZERO;
}



const static PaintingPreset PRESET_NONE = {
	0, 
	0, 

	0, 
	
	0, 	

	0, 
	0, 
	0, 
	255, 
	0, 
	0, 

	0, 
	0, 
	0,   

	1,		// single layer 
	0, 


	0, 
	0, 
	0, 
	0, 
	0, 
};


const static PaintingPreset PRESET_WATERCOLOR = {
	1, 
	0,		// 


	0.003f,	// displacement


	15,		// jitter amplitude


	70,		// some position jitter
	6,		//
	12,		// 
	180, 
	3, 
	3, 

	0, 
	0, 
	0, 

	1, 
	255, 


	0, 
	100, 
	1, 
	1, 
	6, 
};


const static PaintingPreset PRESET_POINTILLISM = {
	1,		// 
	0,		// 


	0.,		// displacement


	10,		// jitter amplitude


	70, 
	6,		//
	12,		// 
	255, 
	0, 
	0, 

	0, 
	0, 
	0, 

	1, //3, 
	255, 


	1, 
	0, 
	0, 
	0, 
	0, 
};


const static PaintingPreset PRESET_FRACTURE2 = {
	1,		// 
	0,		// 


	0.000,	// displacement


	5,		// jitter amplitude


	70, 
	4,		//
	8,		// 
	255, 
	0, 
	0, 

	0, 
	0, 
	0, 

	1, 
	255, 


	1, 
	45, 
	10, 
	0.2f, 
	1, 
};


const static PaintingPreset PRESET_VANGOGH_SP = {
	1,		// overwrite
	0,		// no blending... can specify any number


	0.0025f,	// displacement


	45,		// jitter amplitude


	30,		// positon jitter
	6,		//
	16,		// 
	255, 
	4, 
	4, 

	0, 
	0, 
	0, 

	1, 
	255, 


	1, 
	45, 
	5, 
	20, 
	1, 
};


const static PaintingPreset PRESET_IMPRESSIONISM_SP = {
	1,		// overwrite
	20,		// 


	0.005f,	// displacement


	20,		// jitter amplitude


	30,		// some position jitter
	6,		//
	16,		// 
	255, 
	8, 
	8, 

	0, 
	0, 
	0, 

	3, 
	30, 


	1, 
	45, 
	10, 
	15, 
	.7f, 
};


const static PaintingPreset PRESET_COLORWASH_SP = {
	1,		// overwrite
	20,		// 


	0.005f,	// displacement


	40,		// jitter amplitude


	30,		// some position jitter
	6,		//
	16,		// 
	128, 
	8, 
	8, 

	0, 
	0, 
	0, 

	3, 
	40, 


	1, 
	45, 
	3, 
	0, 
	0, 
};



void interpolatePaintingPreset(const PaintingPreset &p1, const PaintingPreset &p2, float param, PaintingPreset &result, int func_type = 0);

