#pragma once
#include "PaintingStyleNode.h"
#include "TransitionalPaintingPreset.h"
#include "PaintingPreset.h"
#include "tendes_plane.h"
#include "RegionTransformation.h"
#include <list>
#include <vector>
#include <utility>
#include <fstream>
#include <sstream>
#include <iomanip>


using namespace std;


#define DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT 0
#define DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT 0


class PaintingRegion {
private:
	int id;
	TransitionalPaintingPreset t_preset;
	PaintingPreset current_preset;
	RegionTransformation rt;


	int num_frames;
	bool diffusion;										// set true if any element exists
	bool use_zero_field;								// use the zero field as the initial field or not
	bool apply_transformation;


	typedef struct transformation {
		double x0, y0, theta, dx, dy, dtheta;			// position, translation and orientation

		transformation() {
			x0 = 0;
			y0 = 0;
			theta = 0;
			dx = 0;
			dy = 0;
			dtheta = 0;
		}

	} transformation;


	vector<transformation> transport;
	vector<DEInfo> design_elements;						// propagated design elements

	bool buildRegionMotions(const string &file_path = string());


public:
	static const float dehighlight_color[4];
	static const float highlight_color[4];

	list< pair<int, int> > points;		// a list of points within this region


	PaintingRegion(int id);
	int getID();
	void setID(int id) { this->id = id; }


	// operations for style parameter keyframes
	bool setKey(PaintingPreset p, int t = 0);
	PaintingPreset getKey(int t = 0);
	void setCurrentKey(PaintingPreset p);
	PaintingPreset getCurrentKey();

	void copyKeys(const PaintingRegion &src);
	size_t getNumKeys() { return t_preset.getSize(); }		// can be used to decide to use diffusion for style paramters

	void retrieveAllKeys(vector< pair<int, PaintingPreset> > &presets) { t_preset.getAllKeys(presets); }
	void setAllKeys(const vector< pair<int, PaintingPreset> > &presets) { t_preset.setAllKeys(presets); }

	void retrieveAllInterpolationFuncs(vector< pair<int, TransitionalPaintingPreset::INTERPOLATION_TYPE> > &funcs) { t_preset.getAllInterpolationFuncs(funcs); }
	void setAllInterpolationFuncs(const vector< pair<int, TransitionalPaintingPreset::INTERPOLATION_TYPE> > &funcs) { t_preset.setAllInterpolationFuncs(funcs); }

	int getNextKey(int t = 0);
	int getPrevKey(int t = 0);
	bool removeKey(int t);

	TransitionalPaintingPreset::INTERPOLATION_TYPE getStyleInterpolationType(int t = 0) { return t_preset.getInterpolationFunc(t); }
	void setStyleInterpolationType(TransitionalPaintingPreset::INTERPOLATION_TYPE type, int t = 0) { t_preset.setInterpolationFunc(type, t); }


	// operations for design elements


	bool setDesignElemHolderSize(int size, const string &file_path = string());
	size_t getDesignElemHolderSize() { return num_frames; }

	bool propagateDesignElems(SingularElementList *se, RegularElementList *re);
	bool propagateDesignElems(SingularElement **se, size_t num_se, RegularElement **re, size_t num_re);

	const DEInfo *getDesignElems(int frame) { return (frame < 0) ? NULL : ((frame >= num_frames) ? NULL : &(design_elements[frame])); }
	bool isKeyFrameOfDesignElems(int frame) { const DEInfo *de; return ((de = getDesignElems(frame)) == NULL ? false : (de->numKF() > 0)); }

	bool getDiffusionFlag() { return diffusion; }

	void setUseZeroField(bool val) { use_zero_field = val; }
	bool getUseZeroField() { return use_zero_field; }

	void setApplyTransformation(bool val) { apply_transformation = val; }
	bool getApplyTransformation() { return apply_transformation; }

	int getNextDEKey(int t = 0);
	int getPrevDEKey(int t = 0);

private:

	void copyDesignElements(const list<SingularElement> &se_from, list<SingularElement> &se_to, const list<RegularElement> &re_from, list<RegularElement> &re_to, int n1, bool forward);



#if DEBUG_SAVE_ELLIPSETRANSF_AS_TEXT == 1
	fstream d_fstr_ellip;
#endif

#if DEBUG_SAVE_ELEMENTS_INFO_AS_TEXT == 1
	fstream d_fstr_de;
#endif

};

