#pragma once

class RGBComparator {
public:

	static enum FUNC_TYPE {
		MIN, MAX, 
	};

	RGBComparator() { setEvalFunc(MIN); }
	virtual ~RGBComparator() {}


	inline RGBComparator::FUNC_TYPE getEvalFunc() { return func; }

	inline void setEvalFunc(RGBComparator::FUNC_TYPE func);

	inline bool eval(const unsigned char rgb0[3], const unsigned char rgb1[3]);

private:
	FUNC_TYPE func;

	bool (RGBComparator::*comp)(const unsigned char c0[3], const unsigned char c1[3]);

	inline bool isMin(const unsigned char c0[3], const unsigned char c1[3]);
	inline bool isMax(const unsigned char c0[3], const unsigned char c1[3]);

	inline int calcInten(const unsigned char c[3]) { return c[0] * c[0] + c[1] * c[1] + c[2] * c[2]; }
};




inline bool RGBComparator::eval(const unsigned char rgb0[3], const unsigned char rgb1[3]) {
	return (this->*comp)(rgb0, rgb1);
}


inline void RGBComparator::setEvalFunc(RGBComparator::FUNC_TYPE func) {
	switch(func) {
		case MIN:
			comp = &RGBComparator::isMin;
			break;

		case MAX:
			comp = &RGBComparator::isMax;
			break;

		default:
			return;
	}

	this->func = func;
}


inline bool RGBComparator::isMin(const unsigned char c0[3], const unsigned char c1[3]) {
	int inten0 = calcInten(c0);
	int inten1 = calcInten(c1);

	if(inten0 < inten1) return true;
	else if(inten1 < inten0) return false;
	else if(c0[1] < c1[1]) return true;
	else if(c0[1] == c1[1] && c0[0] < c1[0]) return true;
	else if(c0[1] == c1[1] && c0[0] == c1[0] && c0[2] < c1[2]) return true;
	else return false;
}


inline bool RGBComparator::isMax(const unsigned char c0[3], const unsigned char c1[3]) {
	int inten0 = calcInten(c0);
	int inten1 = calcInten(c1);

	if(inten0 > inten1) return true;
	else if(inten1 > inten0) return false;
	else if(c0[1] > c1[1]) return true;
	else if(c0[1] == c1[1] && c0[0] > c1[0]) return true;
	else if(c0[1] == c1[1] && c0[0] == c1[0] && c0[2] > c1[2]) return true;
	else return false;
}

