#include "StdAfx.h"
#include "RenderTexture.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

void CHECK_FRAMEBUFFER_STATUS() 
{
	GLenum status; 
	status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT); 
	//fprintf(stderr, "%x\n", status);
	switch(status) { 
		case GL_FRAMEBUFFER_COMPLETE_EXT: 
			fprintf(stderr,"framebuffer complete!\n");
			break; 
		case GL_FRAMEBUFFER_UNSUPPORTED_EXT: 
			fprintf(stderr,"framebuffer GL_FRAMEBUFFER_UNSUPPORTED_EXT\n");
			/* you gotta choose different formats */ 
			assert(0); 
			break; 
		case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT: 
			fprintf(stderr,"framebuffer INCOMPLETE_ATTACHMENT\n");
			break; 
		case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT: 
			fprintf(stderr,"framebuffer FRAMEBUFFER_MISSING_ATTACHMENT\n");
			break; 
		case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT: 
			fprintf(stderr,"framebuffer FRAMEBUFFER_DIMENSIONS\n");
			break; 
		case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT: 
			fprintf(stderr,"framebuffer INCOMPLETE_FORMATS\n");
			break; 
		case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT: 
			fprintf(stderr,"framebuffer INCOMPLETE_DRAW_BUFFER\n");
			break; 
		case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT: 
			fprintf(stderr,"framebuffer INCOMPLETE_READ_BUFFER\n");
			break; 
		case GL_FRAMEBUFFER_BINDING_EXT:
			fprintf(stderr,"framebuffer BINDING_EXT\n");
			break;
		default:
			/* programming error; will fail on all hardware */
			assert(0);
	}
}


RenderTexture::RenderTexture(unsigned int width, unsigned int height, GLint mag_min_filter, GLint internal_format) {
	glGenTextures(1, &hTexture);
	glBindTexture(GL_TEXTURE_2D, hTexture);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_min_filter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, mag_min_filter);
	glTexImage2D(GL_TEXTURE_2D, 0, internal_format, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	glBindTexture(GL_TEXTURE_2D, 0);


	glGenRenderbuffersEXT(1, &hDepth);
	glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, hDepth);
	glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_STENCIL_EXT, width, height);


	glGenFramebuffersEXT(1, &hFBO);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, hFBO);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, hTexture, 0);


	glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, hDepth);
	CHECK_FRAMEBUFFER_STATUS();


	glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, hDepth);
	CHECK_FRAMEBUFFER_STATUS();


	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
}


RenderTexture::RenderTexture(GLuint TextureHandle)
{
	hTexture = TextureHandle;
	hDepth = 0;

	glGenFramebuffersEXT(1, &hFBO);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, hFBO);
	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, hTexture, 0);
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
}

RenderTexture::~RenderTexture()
{
	glDeleteTextures(1, &hTexture);
	glDeleteRenderbuffersEXT(1, &hDepth);
	glDeleteFramebuffersEXT(1, &hFBO);
}