#pragma once
#include "stdafx.h"
#include "ScriptProcessor.h"
#include "VisView.h"
#include "Util.h"
#include "ppm.h"
#include "PaintingPreset.h"



bool ScriptProcessor::process(CMFCTEST4Dlg &program_dlg, string &scriptfilename, bool make_log) {
	int r_start = -1, r_end = -1;
	bool input_loaded = false;
	bool output_dir_specified = false;
	fstream  batch_file_fstr;
	fstream log_file_fstr;
	string log_file_path;
	string out_dir_path;
	string line, command, arg;
	size_t arg_start, arg_end;
	SYSTEMTIME st;
	CString msg, append;
	VisView *the_view;


	if((the_view = program_dlg.getVisView()) == NULL) return false;


	batch_file_fstr.open(scriptfilename.c_str(), ios_base::in);

	if(batch_file_fstr.is_open() == false) {
		return false;
	}

	if(make_log == true) {
		log_file_path = scriptfilename;
		log_file_path = log_file_path.substr(0, log_file_path.rfind('\\') + 1);

		GetLocalTime(&st);
		log_file_path += _T("LOG_");
		append.Format("%4d_%02d_%02d_%02d_%02d_%02d", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
		log_file_path += append.GetString();
		log_file_path += _T(".txt");

		log_file_fstr.open(log_file_path.c_str(), ios::out);

		append.Format(_T("Loaded Batch File: %s.\r\n"), scriptfilename.c_str());
		msg += append;
		append.Format(_T("Created Log File: %s.\r\n"), log_file_path.c_str());
		msg += append;
		program_dlg.printMessage(msg);
	}

	program_dlg.closeFile();

	while(!batch_file_fstr.eof()) {
		::getline(batch_file_fstr, line);

		command = line.substr(0, line.find(' '));

		// ignore if a line is a comment or contains no string.
		if(command.length() == 0 || command.at(0) == '#') continue;

		// load input
		if(command == _T("load_image") || command == _T("load_video")) {
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				arg = line.substr(arg_start, arg_end - arg_start + 1);

				program_dlg.openFile(arg.c_str(), _T(""));

				input_loaded = ((program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::NOTHING_LOADED) ? false : true);
			}
		}

		// load optical flow
		else if(command == _T("load_opflw")){
			if(input_loaded && program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == string("NULL")) {
						program_dlg.unloadOpticalFlow();
					}
					else {
						program_dlg.loadOpticalFlow(arg);

						if(make_log == true &&  program_dlg.checkOpticalFlowLoaded() == false) {
							append.Format(_T("Failed to load optical flow \"%s\"\r\n"), arg.c_str());
							log_file_fstr << append;
							log_file_fstr.flush();
						}
					}
				}
			}
		}

		// load controls (design elements)
		else if(command == _T("load_ctrls")) {
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				arg = line.substr(arg_start, arg_end - arg_start + 1);

				if(arg == "NULL") {
					the_view->resetField();
					the_view->sampleFieldToFSI();

					if(the_view->getEdgeFieldType() == VisView::TENSOR) {
						the_view->makeBlendMap();
					}
				}
				else {
					the_view->loadControlFromFile(arg.c_str());
				}
			}
		}

		// load a binary map (or, a segmentation/region map)
		else if(command == _T("load_bin_map")) {
			if(input_loaded && program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == string("NULL")) {
						program_dlg.unloadBinaryMap();
					}
					else {
						program_dlg.loadBinaryMap(arg);

						if(make_log == true && program_dlg.checkBinaryMapLoaded() == false) {
							append.Format(_T("Failed to load a binary map \"%s\"\r\n"), arg.c_str());
							log_file_fstr << append;
							log_file_fstr.flush();
						}
					}
				}
			}
		}

		// specify output directory
		else if(command == _T("output_dir")){
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				string test;
				FILE *fp = NULL;

				arg = line.substr(arg_start, arg_end - arg_start + 1);
				test = arg;
				test += "\\blah.txt";

				if((fp = fopen(test.c_str(), _T("w"))) != NULL) {
					output_dir_specified = true;
					out_dir_path = arg;
					fclose(fp);
					remove(test.c_str());

					if(make_log == true) {
						append.Format(_T("Output directory \"%s\"\r\n"), arg.c_str());
						log_file_fstr << append;
						log_file_fstr.flush();
					}
				}
				else {
					output_dir_specified = false;

					if(make_log == true) {
						append.Format(_T("Unable to set as the output directory \"%s\"\r\n"), arg.c_str());
						log_file_fstr << append;
						log_file_fstr.flush();
					}
				}
			}
		}

		// render as IBFV visualization
		else if(command == _T("mode_vis")){
			the_view->setDisplayMode(VisView::TENSOR_EDGE_FIELD);
			the_view->setTensorVisMode(VisView::MAJOR);

			the_view->setVisIterations(20);
		}

		// render as a painting
		else if(command == _T("mode_paint")){
			the_view->setDisplayMode(VisView::PAINTING);

			the_view->setPaintIterations(10);
		}

		// display singularities when saving output
		else if(command == _T("show_singularities")){
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				arg = line.substr(arg_start, arg_end - arg_start + 1);

				if(arg == _T("ON")) {
					the_view->setShowSingularities(true);
				}
				else if(arg == _T("OFF")) {
					the_view->setShowSingularities(false);
				}
			}
		}

		// close input
		else if(command == _T("unload")){
			program_dlg.closeFile();
			input_loaded = false;
		}


		else if(command == _T("goto")) {
			int val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			program_dlg.setCurrentFrame(val - 1);

			if(make_log == true) {
				if(val != program_dlg.getCurrentFrame()) {
					append.Format(_T("Out of frame: %d. Clamped to %d\r\n"), val, program_dlg.getCurrentFrame());
					log_file_fstr << append;
					log_file_fstr.flush();
				}
			}
		}


		// save an image painting
		else if(command == _T("render_image")) {
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start && input_loaded && output_dir_specified) {
				string save_file_name(out_dir_path);
				arg = line.substr(arg_start, arg_end - arg_start + 1);
				FILE *fp;


				save_file_name += _T("\\");
				save_file_name += arg;

				if((fp = fopen(save_file_name.c_str(), "r")) != NULL) {
					fclose(fp);

					if(make_log == true) {
						append.Format(_T("Error: File already exists \"%s\". Skip rendering.\r\n"), arg.c_str());
						log_file_fstr << append;
						log_file_fstr.flush();
					}
				}
				else {
					clock_t t1, t2;
					unsigned char *img;

					the_view->initializeRendering();

					t1 = clock();
					the_view->display();
					t2 = clock();

					the_view->grabImage(&img, 3);
					::My_SSS_SavePPM(the_view->getWinWidth(), the_view->getWinHeight(), img, save_file_name.c_str());

					delete[] img;
					
					if(make_log == true) {
						append.Format(_T("Saved image: %s\r\n"), save_file_name.c_str());
						log_file_fstr << append;

						append.Format(_T("  Render time: %.3lf seconds.\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
						log_file_fstr << append;
						log_file_fstr.flush();
					}
				}

			}
		}

		// save a video painting
		else if(command == _T("render_video")) {
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start && input_loaded && output_dir_specified) {
				string save_file_name(out_dir_path);
				arg = line.substr(arg_start, arg_end - arg_start + 1);
				FILE *fp;


				save_file_name += _T("\\");
				save_file_name += arg;

				if((fp = fopen(save_file_name.c_str(), "r")) != NULL) {
					fclose(fp);

					if(make_log == true) {
						append.Format(_T("Error: File already exists \"%s\". Skip rendering.\r\n"), arg.c_str());
						log_file_fstr << append;
						log_file_fstr.flush();
					}
				}
				else {
					double run_time, render_time;

					program_dlg.saveVideo(save_file_name, false);

					if(make_log == true) {
						size_t *counts_begin, *counts_end;

						counts_begin = new size_t[the_view->getMaxSupportedLayers()];
						counts_end = new size_t[the_view->getMaxSupportedLayers()];

						program_dlg.getStrokeCountsAtBegin(counts_begin);
						program_dlg.getStrokeCountsAtEnd(counts_end);

						run_time = program_dlg.getTotalRunTime();
						render_time = program_dlg.getTotalRenderTime();


						append.Format(_T("Saved video: %s\r\n"), save_file_name.c_str());
						log_file_fstr << append;

						append.Format(_T("  Running time for video: %d minutes %d seconds.\r\n"), (int)run_time / 60, (int)run_time % 60);
						log_file_fstr << append;

						append.Format(_T("  Rendering time for all frames: %d minutes %d seconds.\r\n"), (int)render_time / 60, (int)render_time % 60);
						log_file_fstr << append;


						append.Format(_T("  Stroke counts\r\n"));
						log_file_fstr << append;

						append.Format(_T("    At first frame, ["));
						log_file_fstr << append;


						for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
							if(i == the_view->getMaxSupportedLayers() - 1) {
								append.Format(_T("%d]\r\n"), counts_begin[i]);
							}
							else {
								append.Format(_T("%d, "), counts_begin[i]);
							}

							log_file_fstr << append;
						}

						append.Format(_T("    At last frame, ["));
						log_file_fstr << append;

						for(int i = 0; i < the_view->getMaxSupportedLayers(); i++) {
							if(i == the_view->getMaxSupportedLayers() - 1) {
								append.Format(_T("%d]\r\n"), counts_end[i]);
							}
							else {
								append.Format(_T("%d, "), counts_end[i]);
							}

							log_file_fstr << append;
						}

						log_file_fstr.flush();

						delete[] counts_begin;
						delete[] counts_end;
					}
				}
			}
			else {

				if(make_log == true) {
					append.Format(_T("\r\nError: Input not loaded or outpur directory not specified\r\n"));
					log_file_fstr << append;
					log_file_fstr.flush();
				}
			}
		}

		// specify rendering range
		else if(command == _T("render_range")) {
			if(input_loaded == true) {
				int render_start, render_end;
				stringstream ss;

				ss.str(line.substr(line.find(' ') + 1));
				ss >> r_start;
				ss >> r_end;

				program_dlg.setVideoRenderingRange(r_start - 1, r_end - 1);
				render_start = program_dlg.getVideoRenderingStartFrame();
				render_end = program_dlg.getVideoRenderingEndFrame();

				if(make_log == true) {
					if(render_start != r_start - 1 || (r_end > 0 && render_end != r_end - 1)) {
						append.Format(_T("Warning: Out of range: [%d, %d]. Retargeted to [%d, %d]\r\n"), r_start, r_end, render_start + 1, render_end + 1);
						log_file_fstr << append;
					}

					log_file_fstr.flush();
				}
			}
		}

		// turn on edge field smoothing over time
		else if(command == _T("temporal_smoothing")) {
			if(program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("ON")) {
						program_dlg.enableTemporalSmoothing(true);
					}
					else if(arg == _T("OFF")) {
						program_dlg.enableTemporalSmoothing(false);
					}
				}
			}
		}

		// turn on stroke tracking (so strokes will follow object motion)
		else if(command == _T("stroke_tracking")) {
			if(program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("ON")) {
						program_dlg.enableStrokeTracking(true);
					}
					else if(arg == _T("OFF")) {
						program_dlg.enableStrokeTracking(false);
					}
				}
			}
		}

		// use temporal constraint on orientation
		else if(command == _T("edgefield_constraint")) {
			if(program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("ON")) {
						program_dlg.enableEdgeFieldRefinement(true);
					}
					else if(arg == _T("OFF")) {
						program_dlg.enableEdgeFieldRefinement(false);
					}
				}
			}
		}

		// specify threshold for orientation constraint
		else if(command == _T("edgefield_constraint_angle")) {
			float val;
			stringstream ss;

			if(program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				program_dlg.setRefinementConstraintAngle((float)degreeToRadian(val));
			}
		}

		// apply painting style
		else if(command == _T("set")) {
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				arg = line.substr(arg_start, arg_end - arg_start + 1);
				//program_dlg.setCurrentFrame(0);

				if(arg == _T("VANGOGH")) {
					the_view->setKey(PRESET_VANGOGH_SP);
				}
				else if(arg == _T("IMPRESSIONISM")) {
					the_view->setKey(PRESET_IMPRESSIONISM_SP);
				}
				else if(arg == _T("COLORISTWASH")) {
					the_view->setKey(PRESET_COLORWASH_SP);
				}
				else if(arg == _T("POINTILLISM")) {
					the_view->setKey(PRESET_POINTILLISM);
				}
				else if(arg == _T("FRACTURE")) {
					the_view->setKey(PRESET_FRACTURE2);
				}
				else {
					if(make_log == true) {
						append.Format(_T("Warning: Unrecognized style \"%s\"\r\n"), arg.c_str());
						log_file_fstr << append;
						log_file_fstr.flush();
					}
				}
			}
		}

		// specify amount of smoothing in computing gradient from input
		else if(command == _T("edgefield_src_img_smoothing")) {
			int val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			if(val < 0) {
				if(make_log == true) {
					append.Format(_T("Warning: Invalid edgefield source image smoothing (%d). Set to 0\r\n"), val);
					log_file_fstr << append;
					log_file_fstr.flush();
				}

				val = 0;
			}

			the_view->setEdgeFieldSmoothingIterations(val);
		}

		// specify amount of smoothing on edge field
		else if(command == _T("edgefield_on_mesh_smoothing")) {
			int val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			if(val < 0) {
				if(make_log == true) {
					append.Format(_T("Warning: Invalid edgefield smoothing (%d). Set to 0\r\n"), val);
					log_file_fstr << append;
					log_file_fstr.flush();
				}

				val = 0;
			}

			the_view->setImgSmoothingIterationsForEdgeField(val);
		}

		// specify fraction of vertices to retain strong edge field values during edge field smoothing
		else if(command == _T("edgefield_strong_vert_ratio")) {
			float val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			if(val < 0 || val > 1) {
				if(make_log == true) {
					append.Format(_T("Warning: Invalid edgefield smoothing (%f). Clamped\r\n"), val);
					log_file_fstr << append;
					log_file_fstr.flush();
				}

				val = (float)::clampVal(val, 0, 1);
			}

			the_view->strong_vertex_ratio = val;
		}

		// specify stepsize for warping (for vis. only)
		else if(command == _T("vis_warp_displacement")) {
			double val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			if(val < 0) {
				if(make_log == true) {
					append.Format(_T("Warning: Invalid warp displacement (%.3lf). Set to 0\r\n"), val);
					log_file_fstr << append;
					log_file_fstr.flush();
				}

				val = 0;
			}

			the_view->setVisDisplacement(val);
			the_view->updateKey();
		}

		// specify number of iterations of warping (for vis. only)
		else if(command == _T("vis_warp_iterations")) {
			int val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			if(val < 0) {
				if(make_log == true) {
					append.Format(_T("Warning: Invalid warp iterations (%d). Set to 0\r\n"), val);
					log_file_fstr << append;
					log_file_fstr.flush();
				}

				val = 0;
			}

			the_view->setVisIterations(val);
		}

		// specify blending factor (for vis. only)
		else if(command == _T("vis_warp_alpha")) {
			int val;
			stringstream ss;

			if(the_view->getVisColorCompositionMethod() == VisView::ALPHA_BLEND) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setVisBlendAlpha(val);
			}
		}


		// specify number of iterations of warping for painting
		else if(command == _T("paint_warp_iterations")) {
			int val;
			stringstream ss;

			ss.str(line.substr(line.find(' ') + 1));
			ss >> val;

			if(val < 0) {
				if(make_log == true) {
					append.Format(_T("Warning: Invalid warp iterations (%d). Set to 0\r\n"), val);
					log_file_fstr << append;
					log_file_fstr.flush();
				}

				val = 0;
			}

			the_view->setPaintIterations(val);
		}

		// specify amount for stroke color jitter
		else if(command == _T("stroke_color_jitter")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_color_jitter;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify amount for stroke warp displacement
		else if(command == _T("stroke_displacement")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_displacemnt;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify stroke size
		else if(command == _T("stroke_size")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_size;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify stroke min. length
		else if(command == _T("stroke_min_length")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_min_length;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify stroke max. length
		else if(command == _T("stroke_max_length")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_max_length;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify stroke spacing
		else if(command == _T("stroke_gridsize")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_gridsize;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify stroke opacity
		else if(command == _T("stroke_opacity")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_opacity;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify amount of position jitter for strokes
		else if(command == _T("stroke_pos_jitter")) {
			stringstream ss;
			StrokeParams sp;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				sp = the_view->getStrokeParams();

				ss.str(line.substr(line.find(' ') + 1));
				ss >> sp.stroke_position_jitter;

				the_view->setStrokeParams(sp);
				the_view->updateKey();
			}
		}

		// specify the number of layers for painting
		else if(command == _T("layers")) {
			int val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setNumEnabledLayers(val);
				the_view->updateKey();
			}
		}

		// specify area error threshold
		else if(command == _T("layers_area_error")) {
			float val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setAreaErrorThreshold(val);
				the_view->updateKey();
			}
		}

		// apply post processing
		else if(command == _T("post_process")) {
			if(the_view->getDisplayMode() == VisView::PAINTING) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("ON")) {
						the_view->enablePostProcessing(true);
					}
					else if(arg == _T("OFF")) {
						the_view->enablePostProcessing(false);
					}
				}
			}
		}

		// specify type of post processing
		else if(command == _T("post_process_type")) {
			if(the_view->getDisplayMode() == VisView::PAINTING) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("WATERCOLOR")) {
						the_view->setPostProcessingOperation(0);
						the_view->updateKey();
					}
					else if(arg == _T("EMBOSSING")) {
						the_view->setPostProcessingOperation(1);
						the_view->updateKey();
					}
				}
			}
		}

		// set a value for post processing
		else if(command == _T("post_process_param1")) {
			float val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setPostProcessingParam(0, val);
				the_view->updateKey();
			}
		}

		// set a value for post processing
		else if(command == _T("post_process_param2")) {
			float val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setPostProcessingParam(1, val);
				the_view->updateKey();
			}
		}

		// set a value for post processing
		else if(command == _T("post_process_param3")) {
			float val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setPostProcessingParam(2, val);
				the_view->updateKey();
			}
		}

		// set a value for post processing
		else if(command == _T("post_process_param4")) {
			float val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setPostProcessingParam(3, val);
				the_view->updateKey();
			}
		}

		else if(command == _T("embossing_type")) {
			int val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setEmbossingType(val);
			}
		}

		else if(command == _T("stroke_order")) {
			int val;
			stringstream ss;

			if(the_view->getDisplayMode() == VisView::PAINTING) {
				ss.str(line.substr(line.find(' ') + 1));
				ss >> val;

				the_view->setStrokeOrder(val);
			}
		}

		else if(command == _T("ordering_range")) {
			if(the_view->getDisplayMode() == VisView::PAINTING) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("WHOLE")) {
						the_view->setOrderingRange(false);
					}
					else if(arg == _T("PARTIAL")) {
						the_view->setOrderingRange(true);
					}
					else {
						if(make_log == true) {
							append.Format(_T("Warning: Unknown option \"%s\".\r\n"), arg.c_str());
							log_file_fstr << append;
							log_file_fstr.flush();
						}
					}
				}
			}
		}

		// renderer
		else if(command == _T("renderer")) {
			if(the_view->getDisplayMode() == VisView::PAINTING) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;

				if(arg_end > arg_start) {
					arg = line.substr(arg_start, arg_end - arg_start + 1);

					if(arg == _T("IBFV")) {
						the_view->setRenderingMethod(VisView::IBFV);
					}
					else if(arg == _T("SPLINE")) {
						the_view->setRenderingMethod(VisView::SPLINE);
					}
					else {
						if(make_log == true) {
							append.Format(_T("Warning: Unknown renderer \"%s\".\r\n"), arg.c_str());
							log_file_fstr << append;
							log_file_fstr.flush();
						}
					}
				}
			}
		}

		// for unknown commands...
		else {
			if(make_log == true) {
				arg_start = line.find('"', 0) + 1;
				arg_end = line.find('"', arg_start) - 1;
				arg = line.substr(arg_start, arg_end - arg_start + 1);
				append.Format(_T("Warning: Unknown command: \"%s\"\r\n"), command.c_str());
				log_file_fstr << append;
				log_file_fstr.flush();
			}
		}
	}

	batch_file_fstr.close();


	if(make_log == true) {
		log_file_fstr.close();
	}

	program_dlg.closeFile();

	return true;
}


bool ScriptProcessor::loadVideoProfile(CMFCTEST4Dlg &program_dlg, string &scriptfilename) {
	bool err_flag = false;
	VisView *the_view;
	string line, command, arg;
	size_t arg_start, arg_end;
	fstream  batch_file_fstr;



	if((the_view = program_dlg.getVisView()) == NULL) return false;

	batch_file_fstr.open(scriptfilename.c_str(), ios_base::in);

	if(batch_file_fstr.is_open() == false) {
		return false;
	}


	program_dlg.closeFile();

	// load a video file
	::getline(batch_file_fstr, line);
	command = line.substr(0, line.find(' '));

	if(command == _T("load_video")) {
		arg_start = line.find('"', 0) + 1;
		arg_end = line.find('"', arg_start) - 1;

		if(arg_end > arg_start) {
			arg = line.substr(arg_start, arg_end - arg_start + 1);

			program_dlg.openFile(arg.c_str(), scriptfilename.c_str());

			err_flag = ((program_dlg.getFileLoadStatus() == CMFCTEST4Dlg::VIDEO_LOADED) ? false : true);
		}
	}
	else {
		err_flag = true;
	}


	if(err_flag == false) {
		::getline(batch_file_fstr, line);
		command = line.substr(0, line.find(' '));

		if(command == _T("load_opflw")){
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				arg = line.substr(arg_start, arg_end - arg_start + 1);

				if(arg != string("NULL")) {
					program_dlg.loadOpticalFlow(arg);

					err_flag = !program_dlg.checkOpticalFlowLoaded();
				}
			}
		}
		else {
			err_flag = true;
		}
	}

	if(err_flag == false) {
		::getline(batch_file_fstr, line);
		command = line.substr(0, line.find(' '));

		if(command == _T("load_bin_map")){
			arg_start = line.find('"', 0) + 1;
			arg_end = line.find('"', arg_start) - 1;

			if(arg_end > arg_start) {
				arg = line.substr(arg_start, arg_end - arg_start + 1);

				if(arg != string("NULL")) {
					program_dlg.loadBinaryMap(arg);

					err_flag = !program_dlg.checkBinaryMapLoaded();
				}
			}
		}
		else {
			err_flag = true;
		}
	}


	return !err_flag;
}

