#pragma once

#include "StdAfx.h"
#include <sstream>
#include <limits>
#include <algorithm>
#include "VisView.h"
#include "trackball.h"
#include "tmatrix.h"
#include "ppm.h"
#include "PerlinNoise.h"
#include "ImageLoader.h"
#include "PaintingStyleNode.h"
#include "DesignElementVis.h"
#include "KeyFrameLoader.h"
#include "renderer_debug.h"
#include <map>


#define USE_OMP 0

#ifndef _OPENMP
#define USE_OMP 0
#endif

#if USE_OMP == 1
#include <omp.h>
#endif


// some timing macro
#if D_SAVESTAT_VISVIEW == 1
#include "debug_timing.h"
#else 
#include "debug_no_timing.h"
#endif


int d_render_num_steps = 0, d_render_num_postprocessing = 0, d_render_num_heightmapcomps = 0;
__int64 **d_render_step = NULL, **d_render_postprocessing = NULL, ** d_render_heightmapcomps;





// remove those macros... to avoid compile errors.
#undef min
#undef max

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#define   malloc(s)         _malloc_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__)
#endif


#define BUFSIZE 512
#define	NPN 128
#define NPIX  512
#define SCALE 4.0


#define GL_CLAMP_TO_EDGE					0x812F


int    alpha  = (int)(0.01*255);
int    beta = (int)(0.01*255);

int    iframe = 0; 
int    Npat   = 1;
float tmax   = NPIX/(SCALE*NPN);
float dmax   = SCALE/NPIX;
float mag_ratio = 0.04f;
float stepsize;

int warp_counter;



const double VisView::init_max_displacemnt = 255./NPIX;



void setSizeToPOT(int &w, int &h) {
	int w_pot = 1;
	int h_pot = 1;

	while(w > w_pot) w_pot *= 2;
	while(h > h_pot) h_pot *= 2;

	w = w_pot;
	h = h_pot;
}


// static member
TCHAR VisView::full_path[MAX_PATH];

VisView::VisView() : segmentation_image(NULL)
, segmImageDefined(false)
, flag_merge(0)
, current_id(0)
, flag_click_merge(0)

, VideoSegmentation(0)
{
}


void VisView::Create(CRect rect, CWnd *parent, LPSTR filename) {
	int rw, rh;
	FILE *file = NULL;

	bool start_log = false;
	fstream file_log;
	string logfile_name;


	ASSERT(filename != NULL);
	ASSERT(parent != NULL);

	// Get path...
	GetCurrentDirectory(MAX_PATH, full_path);


	if(start_log) {
		CString msg;
		logfile_name = full_path;
		logfile_name += _T("\\Log\\Visview_Initialization.txt");
		file_log.open(logfile_name.c_str(), ios_base::out|ios_base::trunc);

		msg.Format("Cannot create a log file: %s\r\n.", logfile_name.c_str());

		if(file_log.is_open() == false) {
			MessageBox(msg);
			exit(-1);
		}
	}


	// check POT
	if(start_log) file_log << _T("Check POT...");
	rw = rect.Width();
	rh = rect.Height();
	setSizeToPOT(rw, rh);
	rect.right = rect.left + rw;
	rect.bottom = rect.top + rh;
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("Registering window class, create a window...");
	CString className = AfxRegisterWndClass
		(CS_HREDRAW | CS_VREDRAW | CS_OWNDC, NULL, (HBRUSH)GetStockObject(BLACK_BRUSH), NULL);

	CreateEx(0, className, _T("OpenGL"), WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, rect, parent, 0);
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("Misc. window setting...");
	m_originalRect = rect;

	this->OnSize(0, rect.Width(), rect.Height());

	hWnd = parent;
	this->parent = reinterpret_cast<CMFCTEST4Dlg*>(hWnd);

	// set dimension
	this->win_width = rect.Width();
	this->win_height = rect.Height();
	num_win_pixels = win_width * win_height;

	img_width = 0;
	img_height = 0;
	num_img_pixels = 0;
	if(start_log) file_log << _T("Done") << endl;


	// load mesh data
	if(start_log) file_log << _T("Load mesh data...");
	file = _tfopen(filename, "r");

	ASSERT(file != NULL);

	if(file != NULL) {
		poly = new Polyhedron(file);
		poly->get_model_from_file();
		field_manipulator.setMesh(*poly);		// set mesh first... we set a style field later.
	}
	else {
		MessageBox(_T("File not loaded"), _T(""), MB_OK);
	}

	fclose(file);
	if(start_log) file_log << _T("Done") << endl;


	// OpenGL initialization
	if(start_log) file_log << _T("OpenGL Initializaion...");
	oglInitialize();
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("GLEW Initializaion...");
	GLenum err = glewInit();
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("View Initializaion...");
	initGLView();
	if(start_log) file_log << _T("Done") << endl;


	// setup GLSL manager after OpenGL initialization...
	if(start_log) file_log << _T("GLSLManager Initializaion...");
	GLSLManager::Initialize(ToString(full_path));
	if(start_log) file_log << _T("Done") << endl;


	ibfv_renderer = NULL;
	spline_renderer = NULL;


	if(start_log) file_log << _T("Buffer Initializaion...");
	initializeBuffer();
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("Renderer Initializaion...");
	initializeRenderer();
	if(start_log) file_log << _T("Done") << endl;


	if(start_log) file_log << _T("Data Initializaion...");
	initializeData();
	if(start_log) file_log << _T("Done") << endl;

	// initialize timer (to NULL)
	m_unpTimer = NULL;

	if(start_log) file_log << _T("All initialization complete.") << endl;
}


void VisView::initializeBuffer() {
	// allocate memory for images/textures...
	original_image = new unsigned char[3 * num_win_pixels];
	memset(original_image, 0, sizeof(unsigned char) * 3 * num_win_pixels);

	stroke_ref_img = new unsigned char[3 * num_win_pixels];
	memset(stroke_ref_img, 0, sizeof(unsigned char) * 3 * num_win_pixels);

	blend_map = new float[num_win_pixels];
	painting_style_field = new PaintingStyleField(win_width, win_height);
	input = new FieldSampleImagePyramid(win_width, 64);		// assume that win_width == win_height

	painted_image = new unsigned char[3 * num_win_pixels];
	memset(painted_image, 0, sizeof(unsigned char) * 3 * num_win_pixels);

	painted_image_copy = new unsigned char[3 * num_win_pixels];
	memset(painted_image_copy, 0, sizeof(unsigned char) * 3 * num_win_pixels);

	for(int i = 0; i < 8; i++) pixels_all[i] = new unsigned char[4 * num_win_pixels];

	vis_image = new unsigned char[4 * num_win_pixels];
	height_map = new unsigned char[3 * num_win_pixels];


	// create textures for watercolor effects...
	post_processing_map = new double[num_win_pixels];
	pigment_map = new double[num_win_pixels];
	turbulance_map = new double[num_win_pixels];
	paper_texture_map = new double[num_win_pixels];


	noise_pattern_image = new unsigned char[4 * num_win_pixels];


	offscreen = new RenderTexture(win_width, win_height);
	offscreen_blendmap = new RenderTexture(win_width, win_height);
	quad = new QuadVBO();


	// hand-drawn curve data
	traced_points.clear();
	curves.clear();


	// We are not going to modify vertices on mesh nor change the number of vertices, so we find all visible triangles and store 
	//their positions to avoid excessive calls on glVertex()
	visible_tri_idx = new int[poly->ntris];

	getVisibilitForTriangles(num_visible_triangles, visible_tri_idx);

	visible_tri_vertex_coords = new float[2 * 3 * num_visible_triangles];

	for(int i = 0; i < 8; i++) {
		visible_tri_vertex_tex_coords[i] = new float[2 * 3 * num_visible_triangles];
	}

	updateMeshVisibility();


	optflow = (CvPoint2D32f *)cvAlloc(sizeof(CvPoint2D32f) * num_win_pixels);


	d_render_num_steps = 4;
	d_render_num_postprocessing = 2;
	d_render_num_heightmapcomps = 2;
	D_INIT_TIMING(d_render_step, d_render_num_steps);
	D_INIT_TIMING(d_render_postprocessing, d_render_num_postprocessing);
	D_INIT_TIMING(d_render_heightmapcomps, d_render_num_heightmapcomps);
}


void VisView::initializeRenderer() {
	if(ibfv_renderer == NULL) {
		ibfv_renderer = new IBFVRenderer(win_width, win_height, max_supported_layers);
	}
	else {
		ibfv_renderer->resize(win_width, win_height);
	}

	if(spline_renderer == NULL) {
		spline_renderer = new SplineRenderer(win_width, win_height, max_supported_layers);
	}
	else {
		spline_renderer->resize(win_width, win_height);
	}
}


void VisView::initializeData() {
	int idx;
	float alpha = .3f;


	dt = .25;
	dt_itr = 0;


	num_frames = 1;
	current_time = 0;
	render_time = 0;
	QueryPerformanceFrequency((LARGE_INTEGER*)&timer_freq);

	stroke_regen_mode = 0;
	warp_iterations_counter = 0;
	num_warping_for_paint = 10;
	num_warping_for_vis = 20;


	use_stroke_ref_img = false;


	field_manipulator.setStyleField(*painting_style_field);		// set style field.
	selected_region = painting_style_field->getRegion(0, 0);


	act_state = ACT_STATE::NO_ACTION;  // no action
	old_act_state = ACT_STATE::EDIT_ELEMENT;  // no action
	edit_mode = -1;
	mouse_mode = MOUSE_MODE::NO_MOUSE_ACTION;
	show_separatrices = false;
	show_control_knobs = true;
	show_singularities = false;


	num_sample = 1;
	testparam_f = .55f;


	// set default render target
	GLSLManager::SetRenderTarget(offscreen);


	img_smoothing_for_edgefield_iterations = init_img_smoothing_for_edgefield_iterations;
	edge_field_smoothing_iterations = init_edge_field_smoothing_iterations;
	strong_vertex_ratio = 0.06;

	current_painting_step = PAINTING_STEP::PAINTING_DONE;


	desired_width = 3;

	current_se_id = -1;
	selist = NULL;
	relist = NULL;

	enable_post_processing = true;
	update_post_processing_map = false;
	createStaticTexturesForWaterColorEffect();


	setDisplayMode(TENSOR_EDGE_FIELD);
	setTensorVisMode(MAJOR);
	poly->vec_ten_mode = 1;
	setImageSynthesisMethod(0);


	// create IBFV noise texture
	NPN_size = init_NPN_size;
	::dmax = (float)init_max_displacemnt;

	setVisDisplacement(0.005);

	setVisBlendAlpha(50);
	setPaintBlendAlpha(50);

	color_composition_method_for_vis = COLOR_COMPOSITION::ALPHA_BLEND;
	color_composition_method_for_paint = COLOR_COMPOSITION::ALPHA_BLEND;


	curve_color[0] = curve_color[1] = curve_color[3] = 1.f;
	curve_color[2] = 0;

	show_multistyle_regions = true;
	show_curves = true;
	draw_traces = false;

	run_once = true;


	idx = 0;

	for(int i = 0; i < win_height; i++) {
		for(int j = 0; j < win_width; j++) {
			optflow[idx].x = (float)j;
			optflow[idx].y = (float)i;
			idx++;
		}
	}

	stroke_order = 1;
	partial_ordering = true;
	embossing_type = 1;


	ibfv_renderer->setParams(&ibfv_param);
	spline_renderer->setParams(&spline_param);
	setRenderingMethod(SPLINE);


	makeNoisePatterns();
	setBackgroundPattern();


	dbg_show_layer = false;
	dbg_show_layer_idx = 0;
	dbg_save_stroke_placement_mask = false;
	dbg_show_only_initial_stroke_image = false;
	dbg_output_stroke_map = false;
	dbg_save_advection_image = false;
	dbg_show_optflow = false;

	d_tri_spec = NULL;
}


void VisView::updateMeshVisibility() {
	int idx_6 = 0;
	Triangle *t;

	for(int i = 0; i < num_visible_triangles; i++, idx_6 += 6) {
		t = poly->tlist[visible_tri_idx[i]];

		visible_tri_vertex_coords[idx_6 + 0] = (float)t->verts[0]->x;
		visible_tri_vertex_coords[idx_6 + 1] = (float)t->verts[0]->y;
		visible_tri_vertex_coords[idx_6 + 2] = (float)t->verts[1]->x;
		visible_tri_vertex_coords[idx_6 + 3] = (float)t->verts[1]->y;
		visible_tri_vertex_coords[idx_6 + 4] = (float)t->verts[2]->x;
		visible_tri_vertex_coords[idx_6 + 5] = (float)t->verts[2]->y;
	}

	updateTexCoordsOnMesh();
}


void VisView::finalizeBuffer() {
	vector<PATH*>::iterator itr;


	input->finalize();

	GLSLManager::SetRenderTarget();

	delete offscreen;
	delete offscreen_blendmap;
	delete quad;


	for(int i = 0; i < 8; i++) delete[] pixels_all[i];


	delete[] height_map;
	delete[] vis_image;
	delete[] original_image;
	delete[] stroke_ref_img;
	delete[] painted_image;
	delete[] painted_image_copy;
	delete[] blend_map;
	delete[] post_processing_map;
	delete[] pigment_map;
	delete[] turbulance_map;
	delete[] paper_texture_map;

	delete painting_style_field;

	if(selist) selist->finalize();
	if(relist) relist->finalize();


	delete[] noise_pattern_image;

	for(itr = curves.begin(); itr != curves.end(); itr++) {
		delete (*itr);
	}

	delete[] visible_tri_vertex_coords;
	delete[] visible_tri_idx;

	for(int i = 0; i < 8; i++) {
		delete[] visible_tri_vertex_tex_coords[i];
	}

	cvFree((void**)&optflow);


	D_FIN_TIMING(d_render_step)
	D_FIN_TIMING(d_render_postprocessing)
	D_FIN_TIMING(d_render_heightmapcomps)
}


void VisView::finalizeRenderer() {
	delete ibfv_renderer;
	delete spline_renderer;
}


VisView::~VisView() {
	stopDisplayTimer();
	finalizeBuffer();
	finalizeRenderer();

	if(this->poly) {
		this->poly->finalize();
		delete poly;
	}

	GLSLManager::Shutdown();
	wglMakeCurrent(NULL, NULL);
}


bool VisView::resize(int width, int height) {
	bool prev_timer_status = getDisplayTimerStatus();

	if(0 <= width || 0 <= height) return false;


	stopDisplayTimer();

	/**
	CRect rect;
	GetClientRect(&rect);
	ClientToScreen(rect);
	MoveWindow(rect.left, rect.top, width, height, TRUE);
	/**/

	finalizeBuffer();

	win_width = width;
	win_height = height;
	num_win_pixels = win_width * win_height;

	img_width = 0;
	img_height = 0;
	num_img_pixels = 0;

	initGLView();

	initializeBuffer();
	createStaticTexturesForWaterColorEffect();

	if(prev_timer_status == true) startDisplayTimer();

	return true;
}



BEGIN_MESSAGE_MAP(VisView, CWnd)
ON_WM_PAINT()
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_TIMER()
ON_WM_MOUSEMOVE()
ON_WM_MOUSEWHEEL()
ON_WM_MOUSEACTIVATE()
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_MBUTTONDOWN()
ON_WM_MBUTTONUP()
ON_WM_RBUTTONDOWN()
END_MESSAGE_MAP()


void VisView::OnPaint() {
	//CPaintDC dc(this); // device context for painting
	
	ValidateRect(NULL);

	// Do not call CWnd::OnPaint() for painting messages
}


int VisView::OnCreate(LPCREATESTRUCT lpCreateStruct) {
	if(CWnd::OnCreate(lpCreateStruct) == -1) return -1;

	return 0;
}

void VisView::oglInitialize() {
	// Initial Setup:
	ASSERT(poly != NULL);

	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof (PIXELFORMATDESCRIPTOR), // strcut size 
		1,                              // Version number
		PFD_DRAW_TO_WINDOW |			// Flags, draw to a window,
			PFD_SUPPORT_OPENGL |		// use OpenGL
			PFD_DOUBLEBUFFER,
			PFD_TYPE_RGBA,				// RGBA pixel values
		32,								// 32-bit color
		0, 0, 0,						// RGB bits & shift sizes.
		0, 0, 0,						// Don't care about them
		8, 0,							// No alpha buffer info
		0, 0, 0, 0, 0,					// No accumulation buffer
		32,								// 32-bit depth buffer	
		8,								// No stencil buffer
		0,								// No auxiliary buffers
		PFD_MAIN_PLANE,					// Layer type
		0,								// Reserved (must be 0)
		0,								// No layer mask
		0,								// No visible mask
		0								// No damage mask
	};

	// Get device context only once.
	hdc = GetDC()->m_hDC;

	// Pixel format.
	m_nPixelFormat = ChoosePixelFormat(hdc, &pfd);
	SetPixelFormat(hdc, m_nPixelFormat, &pfd);

	// Create the OpenGL Rendering Context.
	hrc = wglCreateContext(hdc);
	wglMakeCurrent(hdc, hrc);



	// Basic Setup:

	// Set color to use when clearing the background.
	glClearColor(0.f, 0.f, 0.f, 1.f);
	glClearDepth(1.f);


	// Depth function setting
	glDisable(GL_DEPTH_TEST);

	/* select clearing color */ 
	glShadeModel(GL_FLAT);
	glPolygonMode(GL_FRONT, GL_FILL);

	// may need it
	glPixelStorei(GL_PACK_ALIGNMENT,1);

	glEnable(GL_NORMALIZE);
	glEnable(GL_POLYGON_OFFSET_FILL);
	glDisable(GL_DITHER);
	glPolygonOffset(1., 1.);

	
	// culling
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);

	if(poly->orientation == 0) 
		glFrontFace(GL_CW);
	else 
		glFrontFace(GL_CCW);
}


void VisView::OnTimer(UINT nIDEvent) {
	switch (nIDEvent)
	{
		case 1:
		{
			// Draw a scene
			display(true);

			// Draw knobs and singularities
			display_knobs();

			if(show_multistyle_regions == true) {
				displayMultiStyleRegions();
			}

			if(show_curves == true) {
				displayMultiStyleCurves();
			}

			if(draw_traces == true) {
				displayTracedPoints();
			}

			if(dbg_show_optflow == true) {
				displayOptFlowArrows();
			}

			// draw to screen
			GLSLManager::SetRenderTarget();
			glEnable(GL_TEXTURE_2D);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glBindTexture(GL_TEXTURE_2D, offscreen->GetTextureHandle());
			quad->Draw();
			glBindTexture(GL_TEXTURE_2D, 0);
			glDisable(GL_TEXTURE_2D);
			GLSLManager::SetRenderTarget(offscreen);

			// Swap front/back buffers
			SwapBuffers(hdc);

			break;
		}

		default:
			break;
	}

	CWnd::OnTimer(nIDEvent);
}


void VisView::OnSize(UINT nType, int cx, int cy) {
	CWnd::OnSize(nType, cx, cy);
	//initGL();
	//resize(cx, cy);
}


void VisView::OnMouseMove(UINT nFlags, CPoint point) {
	CString msg;
	int idx_3 = 0;
	int x, y;

	if(nFlags == MK_LBUTTON) {
		///////////////////////////////////////////////////////////////////////////
	
	CString str;
	if(current_display_mode==SEGMENTATION && flag_merge==1)
	{

		
		
		
			current_id=seg_img.getRegionID((int)point.x, (int)point.y);
			
			if(current_id==regionID1);
			else
			{
				regionID2=current_id;
				seg_img.merge(segmentation_image,regionID1,regionID2);
				current_id=seg_img.getRegionID((int)point.x, (int)point.y);
		        regionID1=current_id;
				
			}

			
			//str.Format(_T("Selected region %d.\r\n"), regionID2);		
		   // parent->printMessage(str);
			//str.Format(_T("Merging.\r\n"));		
		    //parent->printMessage(str);
			/*str.Format(_T("Done Merge.\r\n"));		
		    parent->printMessage(str);*/
		}
	///////////////////////////////////////////////////////////////////////////

		/**
		if(this->act_state == EDIT_ELEMENT) {
			switch(this->mouse_mode){
				case MOUSE_MODE::DOWN:			
					create_user_spec(point.x, point.y);
					break;
			}
		}
		/**/
	}
	else if(nFlags == MK_MBUTTON) {
		if(VideoSegmentation == 0) {
			draw_traces = false;
			path_maker.update((int)point.x, win_height - 1 - (int)point.y);
			draw_traces = true;
		}
	}
	else if(nFlags == MK_CONTROL) {
		if(parent->getPrintStatEnabled() == true) {
			x = (int)point.x;
			y = win_height - 1 - (int)point.y;
			idx_3 = 3 * (x + win_width * y);
			msg.Format(_T("@(%3d, %3d), (r: %3d, g: %3d, b %3d), "), x, y, painted_image[idx_3 + 0], painted_image[idx_3 + 1], painted_image[idx_3 + 2]);
			parent->printMessage(msg);
			msg.Format(_T("intensity: %.3f\r\n"), 
				sqrt((double)(painted_image[idx_3 + 0] * painted_image[idx_3 + 0] + 
				painted_image[idx_3 + 1] * painted_image[idx_3 + 1] + 
				painted_image[idx_3 + 2] * painted_image[idx_3 + 2]))
			);
			parent->printMessage(msg);
		}
	}

	CWnd::OnMouseMove(nFlags, point);
}


void VisView::OnLButtonDown(UINT nFlags, CPoint point) {
	///////////////////////////////////////////////////////////////////////////
	
	CString str;
	if(current_display_mode==SEGMENTATION)
	{
		merge_time1 = clock();
		flag_merge=1;
		regionID1=0;
		regionID2=0;
		current_id=seg_img.getRegionID((int)point.x, (int)point.y);
		regionID1=current_id;

		//setSegmentationRegion();
		/*switch(flag_click_merge)
		{
		case 0:
			regionID1=0;
			regionID2=0;
			regionID1=seg_img.getRegionID((int)point.x, (int)point.y);

			str.Format(_T("Selected region %d.\r\n"), regionID1);		
			parent->printMessage(str);


			flag_click_merge=1;break;

		case 1:
			regionID2=seg_img.getRegionID((int)point.x, (int)point.y);


			str.Format(_T("Selected region %d.\r\n"), regionID2);		
		    parent->printMessage(str);
			str.Format(_T("Merging.\r\n"));		
		    parent->printMessage(str);

			
			seg_img.merge(segmentation_image,regionID1,regionID2);


			str.Format(_T("Done Merge.\r\n"));		
		    parent->printMessage(str);


			flag_click_merge=0;break;
		}*/

		
	
	}


	///////////////////////////////////////////////////////////////////////////
	if(this->mouse_mode == MOUSE_MODE::NO_MOUSE_ACTION) {
		this->old_act_state = this->act_state;
		this->create_user_spec(point.x, point.y);

		if(this->act_state == ADD_REGULAR_ELEMENT) {
			this->act_state = EDIT_ELEMENT;
			this->edit_mode = 1;
		}

		this->mouse_mode = MOUSE_MODE::DOWN;  // down
	}

	CWnd::OnLButtonDown(nFlags, point);
}


void VisView::OnLButtonUp(UINT nFlags, CPoint point) {
	///////////////////////////////////////////////////////////////////////////////
	if(current_display_mode==SEGMENTATION)
	{
		flag_merge=0;
		merge_time2 = clock();
		double mergeTime=(double)(merge_time2 - merge_time1) / CLOCKS_PER_SEC;
		CString str;
		str.Format(_T("Merge time: %.3lf sec.\r\n"), mergeTime);
	    parent->printMessage(str);
		
	    //field_sampler.sampleTenFieldRegionToMesh(*poly,*(input->fld_pyramid[0]),seg_img.RegionID);

	

	}
	///////////////////////////////////////////////////////////////////////////////

	if(this->act_state == SINGULARITY_PAIR_CANCELLATION) {
		this->create_user_spec(point.x, point.y);
	}
	else if(this->act_state == MOVE_SINGULARITY) {
		this->create_user_spec(point.x, point.y);
	}
	else if (this->act_state == EDIT_ELEMENT) {
		this->create_user_spec(point.x, point.y);
	}

	this->mouse_mode = MOUSE_MODE::NO_MOUSE_ACTION;
	this->act_state = this->old_act_state;

	CWnd::OnLButtonUp(nFlags, point);
}


void VisView::OnMButtonDown(UINT nFlags, CPoint point) {
	if(VideoSegmentation == 0) {
		draw_traces = true;
		path_maker.setPathContainer(traced_points);
		path_maker.start((int)point.x, win_height - 1 - (int)point.y);
	}

	CWnd::OnMButtonDown(nFlags, point);
}


void VisView::OnMButtonUp(UINT nFlags, CPoint point) {
	size_t point_count_thrshold = 3;
	bool tmp_show_multistyle_regions;
	bool prev_timer_status = getDisplayTimerStatus();
	PaintingRegion *pr;
	PATH *new_path;
	CString str;

	stopDisplayTimer();

	tmp_show_multistyle_regions = show_multistyle_regions;
	show_multistyle_regions = false;

	/////////////////////////////////////////////////////////////////////////////////////////////
      int current_region_id;
		

	if(VideoSegmentation == 1 && dat.DatImageDefined == true) {
		current_region_id = dat.getID((int)point.x, (int)point.y);
		int x = (int)point.x;
		int y = (int)point.y;

		str.Format(_T("Selected region %d at (%d,%d).\r\n"), current_region_id, x, y);
		parent->printMessage(str);
	}
	else {
		if(current_display_mode == SEGMENTATION) {
			setSegmentationRegion();
			current_region_id=seg_img.getRegionID((int)point.x, (int)point.y);
			str.Format(_T("Selected region %d.\r\n"), current_region_id);

			parent->printMessage(str);
		}
	}

    
/////////////////////////////////////////////////////////////////////////////////////////////////


	// Construct a new region as long as we've got enough number of points to make one.
	if(VideoSegmentation == 0 && traced_points.size() > point_count_thrshold) {
		selected_region = NULL;
		pr = painting_style_field->createNewRegion(traced_points, current_time);
		
		updateRegionInfoOnMesh();
		resetPropagation();


		if(pr != NULL) {
			new_path = new PATH(traced_points);
			curves.push_back(new_path);
		}

		if(parent->getPrintStatEnabled() == true) {
			if(pr != NULL) {
				str.Format(_T("Created a new region.\r\n"));
			}
			else {
				str.Format(_T("Failed to create a region.\r\n"));
			}

			parent->printMessage(str);
		}

		initializeRendering();
	}
	else {
		pr = painting_style_field->selectRegion((int)point.x, win_height - 1 - (int)point.y);

		if(pr == selected_region && painting_style_field->getNumRegions() > 1) {
			selected_region = NULL;

			if(parent->getPrintStatEnabled() == true) {
				str.Format(_T("Deselected region.\r\n"));
				parent->printMessage(str);
			}
		}
		else {
			selected_region = pr;
			parent->updatePaintingPresetParameters();

			if(parent->getPrintStatEnabled() == true) {
				str.Format(_T("Selected region %d.\r\n"), selected_region->getID());
				parent->printMessage(str);
			}
		}

		parent->notifyRegionSelection();
	}

	parent->updateSelectedRegionOption();

	path_maker.releasePathContainer();
	show_multistyle_regions = tmp_show_multistyle_regions;

	draw_traces = false;

	if(prev_timer_status == true) startDisplayTimer();

	CWnd::OnMButtonUp(nFlags, point);
}


void VisView::OnRButtonDown(UINT nFlags, CPoint point) {
	int t_id = this->pick_object(point.x, point.y);
	Vertex **v;
	CString msg;


	if((nFlags & MK_SHIFT) != 0) {
		field_manipulator.clearSelectedRegion();

		if(0 <= t_id && t_id < poly->ntris) {
			d_tri_spec = poly->tlist[t_id];
			v = &(d_tri_spec->verts[0]);
			msg.Format(
				"T[%d]:\r\n\t{%d,%d,(%.3f, %.3f),(%.3f, %.3f)}\r\n\t{%d,%d,(%.3f, %.3f),(%.3f, %.3f)}\r\n\t{%d,%d,(%.3f, %.3f),(%.3f, %.3f)}\r\n", 
				t_id, 
				v[0]->index, v[0]->regionID, v[0]->x, v[0]->y, v[0]->ten_fld.entry[0][0], v[0]->ten_fld.entry[0][1], 
				v[1]->index, v[1]->regionID, v[1]->x, v[1]->y, v[1]->ten_fld.entry[0][0], v[1]->ten_fld.entry[0][1], 
				v[2]->index, v[2]->regionID, v[1]->x, v[2]->y, v[2]->ten_fld.entry[0][0], v[2]->ten_fld.entry[0][1]
			);

			parent->printMessage(msg);
		}
		else {
			d_tri_spec = NULL;
		}
	}
	else {
		d_tri_spec = NULL;
	}

	CWnd::OnRButtonDown(nFlags, point);
}


int VisView::OnMouseActivate(CWnd* pDesktopWnd, UINT nHitTest, UINT message) {
	return CWnd::OnMouseActivate(pDesktopWnd, nHitTest, message);
}


int VisView::processHits(GLint hits, GLuint buffer[])
{
	GLint i;
	GLuint j;
	GLuint names, *ptr;
	double smallest_depth=1.0e+20, current_depth;
	int seed_id=-1; 
	unsigned char need_to_update;
	CString str;


	ptr = (GLuint *) buffer;
	for (i = 0; i < hits; i++) {  /* for each hit  */
		need_to_update = 0;
		names = *ptr;
		ptr++;
		
		current_depth = (double) *ptr/0x7fffffff;
		if (current_depth < smallest_depth) {
			smallest_depth = current_depth;
			need_to_update = 1;
		}

		ptr++;
		current_depth = (double) *ptr/0x7fffffff;
		if (current_depth < smallest_depth) {
			smallest_depth = current_depth;
			need_to_update = 1;
		}

		ptr++;

		for (j = 0; j < names; j++) {  /* for each name */
			if (need_to_update == 1)
				seed_id = *ptr - 1;
		}
	}


	if(parent->getPrintStatEnabled() == true) {
		str.Format(_T("Selected triangle %d.\r\n"), seed_id);
		parent->printMessage(str);
	}

	field_manipulator.updateSelectedRegion(seed_id);

	return seed_id;
}


void VisView::initGLView() {
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glViewport(0, 0, win_width, win_height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity(); 
	glOrtho(-0.5, 0.5, -0.5, 0.5, 0.0, 40.0);
	glMatrixMode(GL_MODELVIEW);

	glGetIntegerv(GL_VIEWPORT, viewport);
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	glGetDoublev(GL_PROJECTION_MATRIX, projection);

	glDisable(GL_BLEND);

	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glDisable(GL_TEXTURE_2D);
	glShadeModel(GL_FLAT);
}


int VisView::pick_object(int x, int y) {
	GLuint selectBuf[BUFSIZE];
	GLint hits;
	GLint viewport[4];

	glGetIntegerv(GL_VIEWPORT, viewport);

	glSelectBuffer(BUFSIZE, selectBuf);
	(void) glRenderMode(GL_SELECT);

	glInitNames();
	glPushName(0);

	gluPickMatrix((GLdouble) x, (GLdouble) (viewport[3] - y), 1.0, 1.0, viewport);
	display_triangles();
	glFlush();

	hits = glRenderMode(GL_RENDER);

	initGLView();

	return processHits(hits, selectBuf);
}



/*
	replace contents in drawing buffers with a background pattern
*/
void VisView::setBackgroundPattern() {
	int num_elems = 4 * num_win_pixels;
	unsigned char *copy_from, *copy_to;


	switch(current_display_mode) {
		case VECTOR_EDGE_FIELD:
		case TENSOR_EDGE_FIELD:
			copy_from = noise_pattern_image;
			copy_to = vis_image;

			for(int i = 0; i < num_elems; i++) {
				copy_to[i] = copy_from[i];
			}

			for(int i = 0; i < 8; i++) {
				copy_to = pixels_all[i];

				for(int j = 0; j < num_elems; j++) {
					copy_to[j] = copy_from[j];
				}
			}

			break;

		default:
			return;
	}
}


/*
	Change resolution size for noise patterns
*/
void VisView::setNPNSize(int npn_size) {
	if(current_display_mode == VisView::VECTOR_EDGE_FIELD || current_display_mode == VisView::TENSOR_EDGE_FIELD) {
		switch(npn_size) {
			case 128:
			case 256:
				NPN_size = npn_size;
				break;

			default:
				NPN_size = init_NPN_size;
				break;
		};

		makeNoisePatterns();
	}
}


int VisView::getNPNsize() {
	return NPN_size;
}


void VisView::makeNoisePatterns() {
	int t;
	int idx;
	int num_elems;
	int lut[256];
	int *phase;
	GLubyte *pat;
	int color_used[2][3] = {{255, 255, 255}, {0, 0, 0}};


	num_elems = NPN_size * NPN_size;
	phase = new int[num_elems];
	pat = new GLubyte[4 * num_elems];


	srand(1);

	for(int i = 0; i < 256; i++) {
		lut[i] = i < 127 ? 0 : 255;	
	}

	for(int i = 0; i < num_elems; i++) {
		phase[i] = rand() % 256;
	}


	t = 256;

	for(int i = 0; i < num_elems; i++) {
		idx = 4 * i;
		pat[idx + 0] = lut[(t + phase[i]) % 255] * color_used[0][0] / 255 + (255 - lut[(t + phase[i]) % 255]) * color_used[1][0] /255;
		pat[idx + 1] = lut[(t + phase[i]) % 255] * color_used[0][1] / 255 + (255 - lut[(t + phase[i]) % 255]) * color_used[1][1] /255;
		pat[idx + 2] = lut[(t + phase[i]) % 255] * color_used[0][2] / 255 + (255 - lut[(t + phase[i]) % 255]) * color_used[1][2] /255;
		pat[idx + 3] = blend_alpha_for_vis;
	}



	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, NPN_size, NPN_size, 0, GL_RGBA, GL_UNSIGNED_BYTE, pat);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	quad->Draw();

	glReadPixels(0, 0, win_width, win_height, GL_RGBA, GL_UNSIGNED_BYTE, noise_pattern_image);
	glDisable(GL_TEXTURE_2D);

	delete[] phase;
	delete[] pat;
}


void VisView::create_user_spec(int x, int y) {
	double s, t;


	s = x / (double)win_width - .5;
	t = .5 - y / (double)win_height;


	switch (act_state) {
		//case 0:
		case ADD_SINK:
			current_se_id = field_manipulator.addSink(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 1:
		case ADD_SOURCE:
			current_se_id = field_manipulator.addSource(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 2:
		case ADD_SADDLE:
			current_se_id = field_manipulator.addSaddle(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 3:
		case ADD_CCW_CENTER:
			current_se_id = field_manipulator.addCCWCenter(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 4:
		case ADD_CW_CENTER:
			current_se_id = field_manipulator.addCWCenter(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 11:
		case ADD_DIPOLE1:
			current_se_id = field_manipulator.addDipole1(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 12:
		case ADD_DIPOLE2:
			current_se_id = field_manipulator.addDipole2(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 13:
		case ADD_MONKEY_SADDLE:
			current_se_id = field_manipulator.addMonkeySaddle(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());
			current_re_id = -1;

			break;

		//case 8:
		case SELECT_ELEMENT:
			field_manipulator.selectElement(s, t, current_se_id, current_re_id, current_time);
			break;

		//case 9:
		case SINGULARITY_PAIR_CANCELLATION:
			field_manipulator.selectAnalyticalElement(s, t, current_analytical_cp_id);

			if(mouse_mode == MOUSE_MODE::NO_MOUSE_ACTION) {
				last_analytical_cp_id = current_analytical_cp_id;
				break;
			}

			field_manipulator.cancelSingularityPair(current_analytical_cp_id, last_analytical_cp_id);
			break;

		//case 10:
		case MOVE_SINGULARITY:
			if (mouse_mode == MOUSE_MODE::NO_MOUSE_ACTION) {
				field_manipulator.selectAnalyticalElement(s, t, current_analytical_cp_id);
				break;
			}

			field_manipulator.moveSingularity(current_analytical_cp_id, pick_object(x, y));
			break;

		//case 6:   // wind force
		case ADD_REGULAR_ELEMENT:
			current_se_id = -1;
			current_re_id = field_manipulator.addRegularElement(s, t, current_time, (selected_region == NULL) ? -1 : selected_region->getID());

			break;

		//case 7:
		case REMOVE_ELEMENT:
			field_manipulator.removeElement(s, t, current_time);
			current_se_id = -1;
			current_re_id = -1;

			break;

		//case 5:
		case EDIT_ELEMENT:
			if (mouse_mode == MOUSE_MODE::NO_MOUSE_ACTION) {
				if (current_se_id == -1 && current_re_id == -1) {
					field_manipulator.selectElement(s, t, current_se_id, current_re_id, current_time);
					return;
				}

				if(current_se_id >= 0) {
					field_manipulator.determineSEEditMode(s, t, current_se_id, edit_mode);

					if(edit_mode < 0) {
						field_manipulator.selectElement(s, t, current_se_id, current_re_id, current_time);
					}

					return;
				}

				if(current_re_id >= 0) {
					field_manipulator.determineREEditMode(s, t, current_re_id, edit_mode);

					if(edit_mode < 0) {
						field_manipulator.selectElement(s, t, current_se_id, current_re_id, current_time);
					}

					return;
				}

				return;
			}

			if(mouse_mode == MOUSE_MODE::DOWN) { // down
				if(current_se_id >= 0) {
					field_manipulator.editSE(s, t, current_se_id, edit_mode, current_time);
				}

				if(current_re_id >= 0) {
					field_manipulator.editRE(s, t, current_re_id, edit_mode, current_time);
				}
			}

			/**
			CString msg;
			msg.Format("Mode: %d, Edit: %d", mouse_mode, edit_mode);
			MessageBox(msg);
			/**/
			break;
	}


	switch(act_state) {
		case EDIT_ELEMENT:
			if(mouse_mode == MOUSE_MODE::NO_MOUSE_ACTION) break;
			else if((current_se_id == -1 && current_re_id == -1) || edit_mode < 0) break;

		case ADD_SINK: 
		case ADD_SOURCE:  
		case ADD_SADDLE:  
		case ADD_CCW_CENTER:  
		case ADD_CW_CENTER:  
		case REMOVE_ELEMENT:  
		case ADD_DIPOLE1:  
		case ADD_DIPOLE2:  
		case ADD_MONKEY_SADDLE: 

			// diffuse edge field based on the current region setting, and put the result back to mesh
			sampleFieldToFSI();
			input->fld_pyramid[0]->blur(*painting_style_field);
			field_sampler.sampleToMesh(*poly, *(input->fld_pyramid[0]));

			updateTexCoordsOnMesh();
			sampleFieldToFSI();
			makeBlendMap();

			break;


		default:
			break;
	}

	initializeRendering();

	/**
	CString msg;
	msg.Format("%d\r\n", poly->regularelements->num_regularelements);
	parent->printMessage(msg);
	/**/
}


void VisView::display_one_flow_image(int which_coordinates, int num_itr) {
	int num_verts = 3 * num_visible_triangles;

	unsigned char *target_tex_image;
	unsigned char *tmp = NULL;
	unsigned char *tmp_arr[2];
	int blend_alpha;
	COLOR_COMPOSITION composition_method;


	if(current_display_mode != VisView::VECTOR_EDGE_FIELD && 
		current_display_mode != VisView::TENSOR_EDGE_FIELD
		) return;


	glEnable(GL_TEXTURE_2D);
	glClear(GL_COLOR_BUFFER_BIT);


	tmp = new unsigned char[4 * num_win_pixels];

	target_tex_image = noise_pattern_image;
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	blend_alpha = blend_alpha_for_vis;
	composition_method = color_composition_method_for_vis;


	tmp_arr[0] = target_tex_image;
	tmp_arr[1] = tmp;

	for(int h = 0; h < num_itr; h++) {
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, win_width, win_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixels_all[which_coordinates]);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glVertexPointer(2, GL_FLOAT, 0, visible_tri_vertex_coords);
		glTexCoordPointer(2, GL_FLOAT, 0, visible_tri_vertex_tex_coords[which_coordinates]);
		glDrawArrays(GL_TRIANGLES, 0, num_verts);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisableClientState(GL_VERTEX_ARRAY);
		glReadPixels(0, 0, win_width, win_height, GL_RGBA, GL_UNSIGNED_BYTE, tmp);

		alphablendColor(tmp_arr, blend_alpha, win_width, win_height, pixels_all[which_coordinates]);
	}

	delete[] tmp;

	glDisable(GL_TEXTURE_2D);
}


void VisView::renderVectorField() {
	int num_elems = 4 * num_win_pixels;

	assert(current_display_mode == VECTOR_EDGE_FIELD || current_display_mode == TENSOR_EDGE_FIELD);


	for(int ten_dir = 0; ten_dir < 2; ten_dir++) {
		display_one_flow_image(ten_dir);
	}

	switch(image_composition_method) {
		case 0:
			for(int i = 0; i < num_elems; i++) {
				vis_image[i] = pixels_all[1][i];
			}

			break;

		case 1:
			for(int i = 0; i < num_elems; i++) {
				vis_image[i] = (pixels_all[0][i] + pixels_all[1][i]) / 2;
			}

			break;

		default:
			break;
	}
}


void VisView::renderTensorField() {
	int pp;
	int op_type = 0;
	int num_elems = 4 * num_win_pixels;
	unsigned char *tmp_arr[4];
	COLOR_COMPOSITION composition_method;


	assert(current_display_mode == VECTOR_EDGE_FIELD || current_display_mode == TENSOR_EDGE_FIELD);

	if(num_enabled_warped_images < 1 || num_enabled_warped_images == 3 || num_enabled_warped_images > 4) {
		return;
	}


	tmp_arr[0] = pixels_all[coords[0]];
	tmp_arr[1] = pixels_all[coords[1]];
	tmp_arr[2] = pixels_all[coords[2]];
	tmp_arr[3] = pixels_all[coords[3]];

	composition_method = color_composition_method_for_vis;


	if(current_tensor_vis_mode == MAJOR_MINOR) {
		for(int i = 0; i < 8; i++) {
			display_one_flow_image(i);
		}

		for(int i = 0; i < num_win_pixels; i++) {
			pp = 4 * i;

			vis_image[pp + 1] = (unsigned char)(
				((pixels_all[2][pp] + pixels_all[6][pp]) * blend_map[i] + (pixels_all[3][pp] + pixels_all[7][pp]) * (1.f - blend_map[i])) 
				/ 2);

			vis_image[pp + 2] = vis_image[pp + 0] = (unsigned char)(
				((pixels_all[0][pp] + pixels_all[4][pp]) * blend_map[i] + (pixels_all[1][pp] + pixels_all[5][pp]) * (1.f - blend_map[i])) 
				/ 2);

		}
	}
	else {
		for(int i = 0; i < num_enabled_warped_images; i++) {
			display_one_flow_image(coords[i]);
		}

		if(num_enabled_warped_images == 1) {
			for(int i = 0; i < num_elems; i++) {
				vis_image[i] = pixels_all[coords[0]][i];
			}
		}
		else {
			if(num_enabled_warped_images == 4) op_type = 0;
			else if(num_enabled_warped_images == 2 && coords[0] + 4 == coords[1]) op_type = 1;
			else op_type = 2;

			weightblendColor(tmp_arr, blend_map, win_width, win_height, vis_image, op_type);
		}

		if(copy_back == true) {
			for(int i = 0; i < num_enabled_warped_images; i++) {
				for(int j = 0; j < num_elems; j++) {
					pixels_all[coords[i]][j] = vis_image[j];
				}
			}
		}
	}
}


/*
	All paintings are generated using this function.
*/
void VisView::displayPainting() {
	int num_elems = 3 * num_win_pixels;
	bool flag_copy = true;
	unsigned char *layer_img_ptr;
	CString msg;
	__int64 t0[2], t1[2];

	assert(current_display_mode == VisView::PAINTING);


	D_SET_COUNTER_VAR(d_cnt_idx, 0)

	D_TICK(d_render_step, d_cnt_idx)

	// update parameter values.
	setRendererParameter();
	current_renderer->setParams(current_painting_param);

	D_TOCK(d_render_step, d_cnt_idx++)

	if(current_painting_step != PAINTING_STEP::PAINTING_DONE) {
		if(current_painting_step == PAINTING_STEP::IMAGE_PAINTING) {
			QueryPerformanceCounter((LARGE_INTEGER*)&(t0[0]));

			current_renderer->setGenMode(stroke_regen_mode);
			current_renderer->setRenderMode(0);
			current_renderer->render(painted_image);

			for(int i = 0; i < num_elems; i++) {
				painted_image_copy[i] = painted_image[i];
			}

			current_painting_step = PAINTING_STEP::POST_PROCESSING;

			flag_copy = false;		// avoid extra copy-back to pixel_total[]

			if(enable_post_processing == true) {
				update_post_processing_map = true;
			}

			QueryPerformanceCounter((LARGE_INTEGER*)&(t1[0]));
		}

		if(current_painting_step == PAINTING_STEP::POST_PROCESSING) {
			QueryPerformanceCounter((LARGE_INTEGER*)&(t0[1]));

			if(flag_copy == true) {
				for(int i = 0; i < num_elems; i++) {
					painted_image[i] = painted_image_copy[i];
				}
			}

			if(enable_post_processing == true) {
				applyPostProcessing();
			}

			QueryPerformanceCounter((LARGE_INTEGER*)&(t1[1]));
		}

		current_painting_step = PAINTING_STEP::PAINTING_DONE;

		if(parent->getPrintStatEnabled() == true) {
			CString msg;
			msg.Format(_T("Rendering: \t%.3lf sec.\r\n"), (double)(t1[0] - t0[0]) / (double)timer_freq);
			parent->printMessage(msg);

			if(enable_post_processing == true) {
				msg.Format(_T("Post Processing: \t%.3lf sec.\r\n"), (double)(t1[1] - t0[1]) / (double)timer_freq);
				parent->printMessage(msg);
			}
		}
	}


	D_COPY_VAL(t0[0], d_render_step[0][1])
	D_COPY_VAL(t1[0], d_render_step[1][1])
	D_COPY_VAL(t0[1], d_render_step[0][2])
	D_COPY_VAL(t1[1], d_render_step[1][2])
	D_UPDATE_COUNTER_VAR(d_cnt_idx, 3)
	D_TICK(d_render_step, d_cnt_idx)


	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glEnable(GL_TEXTURE_2D);

	switch(renderer) {
		case IBFV:
			if(dbg_show_layer == true) {
				if(dbg_show_only_initial_stroke_image == true) {
					layer_img_ptr = (unsigned char*)((IBFVRenderer*)current_renderer)->getInitialImage(dbg_show_layer_idx);
				}
				else {
					layer_img_ptr = (unsigned char*)((IBFVRenderer*)current_renderer)->getLayerImage(dbg_show_layer_idx);
				}

				if(layer_img_ptr != NULL) {
					glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, layer_img_ptr);
				}
			}
			else {
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGB, GL_UNSIGNED_BYTE, painted_image);
			}

			break;

		case SPLINE:
			if(dbg_show_layer == true) {
				if(dbg_show_only_initial_stroke_image == true) {
					layer_img_ptr = (unsigned char*)((SplineRenderer*)current_renderer)->getInitialImage(dbg_show_layer_idx);

					if(layer_img_ptr != NULL) {
						glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, layer_img_ptr);
					}
				}
				else {
					// ...!!!
					glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGB, GL_UNSIGNED_BYTE, painted_image);
				}
			}
			else {
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGB, GL_UNSIGNED_BYTE, painted_image);
			}

			break;
	}

	quad->Draw();

	glDisable(GL_TEXTURE_2D);


	D_TOCK(d_render_step, d_cnt_idx++)
}


/*
	Use this function to visualize edge field.
*/
void VisView::displayEdgeField() {
	assert(current_display_mode == VisView::VECTOR_EDGE_FIELD || current_display_mode == VisView::TENSOR_EDGE_FIELD);

	switch(current_painting_step) {
		case PAINTING_STEP::IMAGE_PAINTING:
		case PAINTING_STEP::POST_PROCESSING:

			current_painting_step = IMAGE_PAINTING;
			update_post_processing_map = false;

			// update painting and check if it satisfies criteria
			if(warp_iterations_counter < num_warping_for_vis) {
				if(poly->vec_ten_mode == 0) {
					renderVectorField();
				}
				else {
					renderTensorField();
				}

				warp_iterations_counter++;
			}
			else {
				current_painting_step = PAINTING_DONE;
			}

			glEnable(GL_TEXTURE_2D);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, vis_image);
			quad->Draw();
			glDisable(GL_TEXTURE_2D);

			break;


		case PAINTING_STEP::PAINTING_DONE:

			glEnable(GL_TEXTURE_2D);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			quad->Draw();
			glDisable(GL_TEXTURE_2D);

			break;
	}
}


double VisView::getRenderTime() {
	return render_time;
}

/*
	Display a current image
*/
void VisView::display(bool print_stat) {
	CString msg, append;
	__int64 time1, time2;
	bool flag = false;

	// for timing...
	string path;
	FILE *d_fp = NULL;
	int d_step_idx = 0;
	int d_postprocess_idx = 0;
	int d_heightmap_idx = 0;


	render_time = 0;

	switch(current_display_mode) {
		case VECTOR_EDGE_FIELD:
		case TENSOR_EDGE_FIELD:
			displayEdgeField();
			break;

		case PAINTING:
			if(print_stat == true && current_painting_step != PAINTING_STEP::PAINTING_DONE) flag = true;

			QueryPerformanceCounter((LARGE_INTEGER*)&time1);

			displayPainting();				// create painting

			QueryPerformanceCounter((LARGE_INTEGER*)&time2);
			render_time = (double)(time2 - time1) / (double)timer_freq;

			if(flag == true) {
				append.Format(_T("Render Image: \t%.3lf sec.\r\n"), render_time);
				msg += append;

				append.Format(_T("Strokes used: \t["));
				msg += append;

				for(int i = 0; i < max_supported_layers; i++) {
					if(i == max_supported_layers - 1) {
						append.Format(_T("%d]\r\n"), getNumStrokes(i));
					}
					else {
						append.Format(_T("%d, "), getNumStrokes(i));
					}

					msg += append;
				}

				parent->printMessage(msg);


#if D_SAVESTAT_VISVIEW == 1
				path = getFullPath();
				path.append("\\Render_stat.txt");
				d_fp = fopen(path.c_str(), "a");

				d_step_idx = 0;
				d_postprocess_idx = 0;

				switch(renderer) {
					case IBFV:
						fprintf(d_fp, "Renderer: IBFV\n");
						break;

					case SPLINE:
						fprintf(d_fp, "Renderer: Spline\n");
						break;

					default:
						fprintf(d_fp, "Renderer: Unknown\n");
						break;
				}

				fprintf(d_fp, "\n");
				fprintf(d_fp, "Update input for a renderer:         \t%.3f\n", D_TIME(d_render_step, d_step_idx, timer_freq)); d_step_idx++;
				fprintf(d_fp, "Render a color image, copy:          \t%.3f\n", D_TIME(d_render_step, d_step_idx, timer_freq)); d_step_idx++;
				fprintf(d_fp, "Post processing:                     \t\n", D_TIME(d_render_step, d_step_idx, timer_freq)); d_step_idx++;

				switch(post_processing_operation) {
					case 0:
						fprintf(d_fp, "  Update an edge darking map:        \t%.3f\n", D_TIME(d_render_postprocessing, d_postprocess_idx, timer_freq)); d_postprocess_idx++;
						fprintf(d_fp, "  Apply:                             \t%.3f\n", D_TIME(d_render_postprocessing, d_postprocess_idx, timer_freq)); d_postprocess_idx++;
						break;
					case 1:
						fprintf(d_fp, "  Compute a height map:              \t\n", D_TIME(d_render_postprocessing, d_postprocess_idx, timer_freq)); d_postprocess_idx++;
						fprintf(d_fp, "    Render a height image:           \t%.3f\n", D_TIME(d_render_heightmapcomps, d_heightmap_idx, timer_freq)); d_heightmap_idx++;
						fprintf(d_fp, "    Compute:                         \t%.3f\n", D_TIME(d_render_heightmapcomps, d_heightmap_idx, timer_freq)); d_heightmap_idx++;
						fprintf(d_fp, "  Apply:                             \t%.3f\n", D_TIME(d_render_postprocessing, d_postprocess_idx, timer_freq)); d_postprocess_idx++;
						break;
					case 2:
						fprintf(d_fp, "  (Unknown operation...):            \t%.3f\n"); d_postprocess_idx++; d_postprocess_idx++;
						break;
				}

				fprintf(d_fp, "Copy image from a renderer:          \t%.3f\n", D_TIME(d_render_step, d_step_idx, timer_freq)); d_step_idx++;
				fprintf(d_fp, "-------------------------------------------------\n");
				fprintf(d_fp, "Rendering time:                      \t%.3f\n", render_time);
				fprintf(d_fp, "\n\n");
				fclose(d_fp);

#endif
			}

			break;

		case BLACK:
			glDisable(GL_TEXTURE_2D);
			glColor3f(0, 0, 0);
			quad->Draw();

			break;

		case INPUT_IMAGE:
			glEnable(GL_TEXTURE_2D);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGB, GL_UNSIGNED_BYTE, original_image);
			quad->Draw(true);

			break;

		case BLENDMAP:
			glEnable(GL_TEXTURE_2D);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_LUMINANCE, GL_FLOAT, blend_map);
			quad->Draw();

			break;

		case SEGMENTATION:
			glEnable(GL_TEXTURE_2D);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGB, GL_UNSIGNED_BYTE, segmentation_image);
            quad->Draw(true); 
			break;

		case LABELIMAGE:
		case DATIMAGE:
			if(VideoSegmentation == 1 && dat.DatImageDefined == true) {
				glEnable(GL_TEXTURE_2D);
				glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, dat.imgdat->width,dat.imgdat->height, 0, GL_RGB, GL_UNSIGNED_BYTE, dat.DatImage);
				quad->Draw(true);
			}
			else {
				glDisable(GL_TEXTURE_2D);
				glColor3f(0, 0, 0);
				quad->Draw();
			}

			break;

		case STROKE_REF_IMAGE:
			if(use_stroke_ref_img == true) {
				glEnable(GL_TEXTURE_2D);
				glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
				glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, win_width, win_height, 0, GL_RGB, GL_UNSIGNED_BYTE, stroke_ref_img);
			}
			else {
				glDisable(GL_TEXTURE_2D);
				glColor3f(0, 0, 0);
			}

			quad->Draw();

			break;

		default:
			break;
	}
}


void VisView::display_triangles() 
{
	Triangle *temp_t;
	icVector3 color, arrow_dir, x_dir, y_dir;

	glClear(GL_COLOR_BUFFER_BIT);
	glDisable(GL_TEXTURE_2D);

	glColor3f(1.0, 1.0, 0.0);

	for(int i = 0; i < poly->ntris; i++) {
		glLoadName(i + 1);
		glBegin(GL_TRIANGLES);
		
		temp_t = poly->tlist[i];
		
		for(int j = 0; j < 3; j++) {
			glVertex2f((float)(temp_t->verts[j]->x*2.), (float)(temp_t->verts[j]->y*2.));
		}

		glEnd();
	}

	glFinish();
}


void VisView::global_rotate_field() {
	field_manipulator.rotateField();

	copyTexCoordsList();
	sampleFieldToFSI();
	makeBlendMap();
}


void VisView::global_reflect_field() {
	field_manipulator.reflectField();

	copyTexCoordsList();
	sampleFieldToFSI();
	makeBlendMap();
}


/*
	Smooth field enclosed by a user-specified region
*/
void VisView::smoothClosedField() {
	field_manipulator.smoothRegionInField();

	copyTexCoordsList();
	sampleFieldToFSI();
	makeBlendMap();
}


void VisView::sampleFieldToFSI(FieldSampleImage *fsi) {
	FieldSampleImage *target;

	if(fsi == NULL) target = input->fld_pyramid[0];
	else target = fsi;

	field_sampler.sampleToFSI(*poly, *target);
}


void VisView::display_knobs() {
	glDisable(GL_TEXTURE_2D);

	if(poly->vec_ten_mode == 0) {
		display_vector_knobs();
	}
	else if(poly->vec_ten_mode == 1) {
		display_tensor_knobs();
	}
}


// ... haven't worked on it yet.
void VisView::display_vector_knobs() {
	int num_elems;
	icVector3 color;
	SingularElement *se;
	RegularElement *re;
	Singularity *s;
	Vertex *v1, *v2;


	if(show_separatrices == true) {
		num_elems = poly->singularities->num_singularities;

		for(int i = 0; i < num_elems; i++) {
			s = poly->singularities->singularities[i];
			DesignElementVis::drawSeparatrix(*s, icVector3(.75, .25, 0), icVector3(.25, .75, 0), 3);
		}
	}

	if(show_control_knobs == true) {
		num_elems = poly->singularelements->num_singularelements;

		for(int i = 0; i < num_elems; i++) {
			se = poly->singularelements->singularelements[i];

			switch(se->type) {
				case 0:
					color.set(1.0, 0.0, 0.0);
					break;

				case 1:
					color.set(0.0, 1.0, 0.0);
					break;

				case 2:
					color.set(1.0, 1.0, 0.0);
					break;

				case 3:
					color.set(0.0, 1.0, 1.0);
					break;

				case 4:
					color.set(1.0, 0.0, 1.0);
					break;
			}

			DesignElementVis::drawSquareKnob(*se, color, (current_se_id == i) ? true : false);
		}


		num_elems = poly->regularelements->num_regularelements;
		color.set(0.0, 1.0, 1.0);

		for(int i = 0; i < num_elems; i++) {
			re = poly->regularelements->regularelements[i];

			DesignElementVis::drawArrow(*re, color, (current_re_id == i) ? true : false, false);
		}
	}


	if(show_singularities == true) {
		num_elems = poly->singularities->num_singularities;

		for(int i = 0; i < num_elems; i++) {
			s = poly->singularities->singularities[i];

			switch(s->singularity_type) {
				case 0:
					color.set(0.0, 1.0, 0.0);
					break;

				case 1:
					color.set(1.0, 0.0, 0.0);
					break;

				case 2:
					color.set(0.0, 0.0, 1.0);
					break;

				case 3:
					color.set(0.0, 1.0, 1.0);
					break;

				case 4:
					color.set(1.0, 0.0, 1.0);
					break;

				case 5:
					color.set(1.0, 1.0, 0.0);
					break;
			}

			DesignElementVis::drawSingularity(*s, color);
		}
	}

	glColor3f(1.0, 1.0, 1.0);
	glLineWidth((GLfloat)desired_width);
	glBegin(GL_LINES);

	for(int i = 0; i < poly->region_verts->num_verts; i++) {
		v1 = poly->region_verts->verts[i];
		v2 = poly->region_verts->verts[(i+1) % poly->region_verts->num_verts];

		glVertex2d(v1->x, v1->y);
		glVertex2d(v2->x, v2->y);
	}

	glEnd();
}


void VisView::display_tensor_knobs() {
	int num_elems;
	float highlight_factor = .7f;
	icVector3 color;
	Singularity *s;
	Vertex *v1, *v2;
	vector<const DEInfo*> de_regions;

	DEInfo *de_info = NULL;
	PaintingRegion *pr;
	const SEInfo *se_info = NULL;
	const REInfo *re_info = NULL;
	list<SingularElement>::const_iterator itr_se, itr_se_end;
	list<RegularElement>::const_iterator itr_re, itr_re_end;

	SingularElementList *selist = poly->singularelements;
	RegularElementList *relist = poly->regularelements;


	if(show_separatrices == true) {
		num_elems = poly->singularities->num_singularities;

		if(current_tensor_vis_mode != MINOR) {
			for(int i = 0; i < num_elems; i++) {
				s = poly->singularities->singularities[i];
				DesignElementVis::drawSeparatrix(*s, icVector3(.75, .25, 0), icVector3(.25, .75, 0), 1);
			}
		}

		if(current_tensor_vis_mode != MAJOR) {
			for(int i = 0; i < num_elems; i++) {
				s = poly->singularities->singularities[i];
				DesignElementVis::drawSeparatrix(*s, icVector3(.75, .25, 0), icVector3(.25, .75, 0), 2);
			}
		}
	}

	if(show_control_knobs == true) {
		painting_style_field->getDEInfo(de_regions, current_time);

		for(size_t i = 0; i < de_regions.size(); i++) {
			if(de_regions[i] == NULL) {		// no region with this id
				continue;
			}
			else if((pr = painting_style_field->getRegion(de_regions[i]->region_id)) == NULL) {		// 
				continue;
			}
			else if(pr->points.size() == 0) {	// no pixel visible
				continue;
			}

			se_info = &(de_regions[i]->se);
			re_info = &(de_regions[i]->re);

			for(int j = 0; j < 3; j++) {
				switch(j) {
					case 0:
						itr_se = se_info->fwd_elems.begin();
						itr_se_end = se_info->fwd_elems.end();
						itr_re = re_info->fwd_elems.begin();
						itr_re_end = re_info->fwd_elems.end();
						break;

					case 1:
						itr_se = se_info->bwd_elems.begin();
						itr_se_end = se_info->bwd_elems.end();
						itr_re = re_info->bwd_elems.begin();
						itr_re_end = re_info->bwd_elems.end();
						break;

					case 2:
						itr_se = se_info->kf.begin();
						itr_se_end = se_info->kf.end();
						itr_re = re_info->kf.begin();
						itr_re_end = re_info->kf.end();
						break;
				}


				// draw singular elements
				while(itr_se != itr_se_end) {
					switch(itr_se->type) {
						case 0:
							color.set(1.0, 0.0, 0.0);
							break;

						case 1:
							color.set(0.0, 1.0, 0.0);
							break;

						case 2:
							color.set(1.0, 1.0, 0.0);
							break;

						case 3:
							color.set(0.0, 1.0, 1.0);
							break;

						case 4:
							color.set(1.0, 0.0, 1.0);
							break;

						default:
							color.set(1.0, 1.0, 1.0);
					}

					if(j != 2) {
						color.entry[0] *= highlight_factor;
						color.entry[1] *= highlight_factor;
						color.entry[2] *= highlight_factor;
					}

					DesignElementVis::drawSquareKnob(*itr_se, color, (current_se_id >= 0 && (*itr_se).ref == selist->singularelements[current_se_id]) ? true : false);
					itr_se++;
				}


				color.set(0.0, 1.0, 1.0);

				if(j != 2) {
					color.entry[0] *= highlight_factor;
					color.entry[1] *= highlight_factor;
					color.entry[2] *= highlight_factor;		
				}

				// draw regular elements
				while(itr_re != itr_re_end) {
					DesignElementVis::drawArrow(*itr_re, color, (current_re_id >= 0 && (*itr_re).ref == relist->regularelements[current_re_id]) ? true : false, true);
					itr_re++;
				}
			}
		}
	}


	if(show_singularities == true) {
		num_elems = poly->singularities->num_singularities;

		for(int i = 0; i < num_elems; i++) {
			s = poly->singularities->singularities[i];

			if(s->vecten_index == 1) color.set(1.f, 1.f, 0);
			else color.set(0, 0, 1.f);

			DesignElementVis::drawSingularity(*s, color);
		}
	}


	glColor3f(1.0, 1.0, 1.0);
	glLineWidth((GLfloat)desired_width);
	glBegin(GL_LINES);

	for(int i = 0; i < poly->region_verts->num_verts; i++) {
		v1 = poly->region_verts->verts[i];
		v2 = poly->region_verts->verts[(i+1) % poly->region_verts->num_verts];

		glVertex2d(v1->x, v1->y);
		glVertex2d(v2->x, v2->y);
	}

	glEnd();
}


/*
	Compute edge field from an image
*/
bool VisView::computeFieldFromImage(FieldSampleImage *fsi) {
	FieldSampleImage *target;
	clock_t t1, t2;


	if(fsi == NULL) target = input->fld_pyramid[0];
	else target = fsi;

	if(target->w != win_width || target->h != win_height) {
		return false;
	}

	t1 = clock();
	field_constructor.computeEdgeField(original_image, *target, img_smoothing_for_edgefield_iterations);
	t2 = clock();

	if(parent->getPrintStatEnabled() == true) {
		CString msg;
		msg.Format(_T("Edge Field Extraction: %.3lf sec.\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		parent->printMessage(msg);
	}

	return true;
}


void VisView::constructFieldInformation() {
	clock_t t1[9], t2[9];
	int cnt = 0;
	int num_verts = poly->nverts;
	Vertex *v;


	// sample from pixels to mesh
	t1[cnt] = clock();
	field_sampler.sampleToMesh(*poly, *(input->fld_pyramid[0]));


////////////////////////////////////////////////////////////////////////////////////////////
	if(VideoSegmentation == 1) {
		//if(dat.DatImageDefined) {
		//	field_sampler.sampleTenFieldRegionToMesh(*poly, *(input->fld_pyramid[0]), dat.imgdat->labels);
		//}
	}
	/*else
	{
		if(current_display_mode==SEGMENTATION)
			field_sampler.sampleTenFieldRegionToMesh(*poly,*(input->fld_pyramid[0]),seg_img.RegionID);

	}*/
	//////////////////////////////////////////////////////////////////////////////////////////////	

	t2[cnt] = clock();

	cnt++;

	// make the edge field dominated by strong values
	t1[cnt] = clock();
	field_manipulator.smoothField(edge_field_smoothing_iterations, strong_vertex_ratio);
	t2[cnt] = clock();

	cnt++;

	// copy for backup (used to Polyhedron::calc_new_texture_coors()...)
	t1[cnt] = clock();
	if(poly->vec_ten_mode == 0) {
		for(int i = 0; i < num_verts; i++) {
			v = poly->vlist[i];
			v->vec_fld_painting = v->vec_fld;
		}
	}
	else {
		for(int i = 0; i < num_verts; i++) {
			v = poly->vlist[i];
			v->ten_fld_painting = v->ten_fld;
		}
	}

	t2[cnt] = clock();

	cnt++;

	// blend with design elements, if any
	t1[cnt] = clock();
	field_manipulator.interpolateField(current_time);
	t2[cnt] = clock();

	cnt++;


	// diffuse edge field based on the current region setting, and put the result back to mesh
	t1[cnt] = clock();
	sampleFieldToFSI();
	input->fld_pyramid[0]->blur(*painting_style_field);
	field_sampler.sampleToMesh(*poly, *(input->fld_pyramid[0]));
	t2[cnt] = clock();

	cnt++;


	// update current painting style on domain
	t1[cnt] = clock();
	updatePaintingStyleField();
	t2[cnt] = clock();

	cnt++;

	// recompute coordinates
	t1[cnt] = clock();
	updateTexCoordsOnMesh();
	t2[cnt] = clock();

	cnt++;

	// sample back to pixels (they are used to determine stroke orientations)
	t1[cnt] = clock();
	sampleFieldToFSI();
	t2[cnt] = clock();

	cnt++;


	// weight map
	t1[cnt] = clock();
	makeBlendMap();
	t2[cnt] = clock();


	/**/
	if(parent->getPrintStatEnabled() == true) {
		CString msg;

		msg.Format(_T("--- Construct edge field information ---\r\n"));
		parent->printMessage(msg);

		msg.Format(_T("Sampling edge field to mesh: \t%.3lf sec\r\n"), (double)(t2[0] - t1[0]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Smoothing: \t%.3lf sec\r\n"), (double)(t2[1] - t1[1]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Copy edge field: \t%.3lf sec\r\n"), (double)(t2[2] - t1[2]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Set design elements: \t%.3lf sec\r\n"), (double)(t2[3] - t1[3]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Heat diffusion: \t%.3lf sec\r\n"), (double)(t2[4] - t1[4]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Update painting style field: \t%.3lf sec\r\n"), (double)(t2[5] - t1[5]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Compute new texture coordinate: \t%.3lf sec\r\n"), (double)(t2[6] - t1[6]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);

		msg.Format(_T("Sample to FSI: \t%.3lf sec\r\n"), (double)(t2[7] - t1[7]) / CLOCKS_PER_SEC);
		parent->printMessage(msg);


		if(getEdgeFieldType() == VisView::TENSOR) {
			msg.Format(_T("Compute blend map: \t%.3lf sec\r\n"), (double)(t2[8] - t1[8]) / CLOCKS_PER_SEC);
			parent->printMessage(msg);
		}

		msg.Format(_T("--- End of construct edge field information ---\r\n"));
		parent->printMessage(msg);
	}
	/**/
}


void VisView::saveDesignElements() {
	if(selist) selist->finalize();
	if(relist) relist->finalize();

	field_manipulator.getDesignElements(selist, relist);
}


void VisView::setDesignElements() {
	field_manipulator.setDesignElements(selist, relist, current_time);
	if(selist) selist->finalize();
	if(relist) relist->finalize();

	current_se_id = -1;
	current_re_id = -1;
	current_analytical_cp_id = -1;

	selist = NULL;
	relist = NULL;
}


void VisView::makeBlendMap() {
	int i, j;
	bool prev_timer_status = getDisplayTimerStatus();

	if(poly->vec_ten_mode != 1) return;

	stopDisplayTimer();

	GLSLManager::SetRenderTarget(offscreen_blendmap);
	glShadeModel(GL_SMOOTH);
	glDisable(GL_TEXTURE_2D);

	glClearColor (0.5, 0.5, 0.5, 1.0);  // background for rendering color coding and lighting
	glClear(GL_COLOR_BUFFER_BIT);

	glBegin(GL_TRIANGLES);
	for(i = 0; i < poly->ntris; i++) {
		Triangle *temp_t = poly->tlist[i];
	
		for (j=0; j<3; j++) {
			icVector2 temp_vec = temp_t->verts[j]->vec_fld;
			normalize(temp_vec);
			double tt;
			if (length(temp_vec)>0.0) {
				tt = temp_vec.entry[0]*temp_vec.entry[0];
			} else {
				tt = 0.5;
			}
			glColor3d(tt, tt, tt);
			glVertex2d(temp_t->verts[j]->x, temp_t->verts[j]->y);
		}
	}
	glEnd();
	glReadPixels(0, 0, win_width, win_height, GL_RED, GL_FLOAT, blend_map);

	GLSLManager::SetRenderTarget(offscreen);

	if(prev_timer_status == true) {
		startDisplayTimer();
	}
}


void VisView::getVisibilitForTriangles(int &num_visibles, int * (&tri_idx)) {
	int num_tris = poly->ntris;
	Triangle *t;
	Vertex *v[3];

	num_visibles = 0;

	for(int i = 0; i < num_tris; i++) {
		t = poly->tlist[i];
		v[0] = t->verts[0];
		v[1] = t->verts[1];
		v[2] = t->verts[2];

		if((fabs(v[0]->x) <= .5 && fabs(v[0]->y) <= .5) || (fabs(v[1]->x) <= .5 && fabs(v[1]->y) <= .5) || (fabs(v[2]->x) <= .5 && fabs(v[2]->y) <= .5)) {
			t->visible = true;
			tri_idx[num_visibles++] = t->index;
		}
		else {
			t->visible = false;
		}
	}

	for(int i = num_visibles; i < poly->ntris; i++) {
		tri_idx[i] = -1;
	}
}


void VisView::stopDisplayTimer() {
	if(m_unpTimer != NULL) {
		this->KillTimer(m_unpTimer);
		m_unpTimer = NULL;
	}
}

void VisView::startDisplayTimer() {
	if(m_unpTimer == NULL) {
		m_unpTimer = this->SetTimer(1, 1, 0); 
	}
}

bool VisView::getDisplayTimerStatus() {
	return (m_unpTimer != NULL) ? true : false;
}


/*
	return the displaying image. origin is at the lower left corner
*/
void VisView::grabImage(unsigned char **img, int origin) {
	int idx1, idx2;
	int elems;
	bool prev_timer_status;
	unsigned char *tmp = NULL;


	if(img == NULL || *img == NULL) return;

	prev_timer_status = getDisplayTimerStatus();
	stopDisplayTimer();

	// flip upside down if needed
	if(origin == 0) {
		elems = img_width * img_height;
		tmp = new unsigned char[elems];
		glReadPixels(0, win_height - img_height, img_width, img_height, GL_RGB, GL_UNSIGNED_BYTE, tmp);

		for(int i = 0; i < img_height; i++) {
			for(int j = 0; j < img_width; j++) {
				idx1 = 3 * (j + img_width * i);
				idx2 = 3 * (j + img_width * (img_height - 1 - i));

				*img[idx1] = tmp[idx2];
				*img[idx1 + 1] = tmp[idx2 + 1];
				*img[idx1 + 2] = tmp[idx2 + 2];
			}
		}

		delete[] tmp;
	}
	else {
		glReadPixels(0, win_height - img_height, img_width, img_height, GL_RGB, GL_UNSIGNED_BYTE, *img);
	}

	if(prev_timer_status == true) {
		startDisplayTimer();
	}
}


void VisView::setEdgeFieldSmoothingIterations(int num_itr) {
	edge_field_smoothing_iterations = (num_itr >= 0) ? num_itr : 0;
}


int VisView::getEdgeFieldSmoothingIterations() {
	return edge_field_smoothing_iterations;
}


void VisView::setImgSmoothingIterationsForEdgeField(int num_itr) {
	img_smoothing_for_edgefield_iterations = (num_itr >= 0) ? num_itr : 0;
}


int VisView::getImgSmoothingIterationsForEdgeField() {
	return img_smoothing_for_edgefield_iterations;
}


void VisView::initializeRendering() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(current_display_mode == VisView::VECTOR_EDGE_FIELD || current_display_mode == VisView::TENSOR_EDGE_FIELD) {
		warp_iterations_counter = 0;
	}
	
	current_painting_step = PAINTING_STEP::IMAGE_PAINTING;

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::resetField() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	current_re_id = -1;
	current_se_id = -1;
	current_analytical_cp_id = -1;

	field_manipulator.resetField(current_time);

	// all design elements are removed so we don't do any heat diffusion.

	updateTexCoordsOnMesh();
	sampleFieldToFSI();
	makeBlendMap();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::clearField() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	current_re_id = -1;
	current_se_id = -1;
	current_analytical_cp_id = -1;

	field_manipulator.zeroField(current_time, (selected_region == NULL) ? -1 : selected_region->getID());
	
	// all design elements are removed so we don't do any heat diffusion.

	updateTexCoordsOnMesh();
	sampleFieldToFSI();
	makeBlendMap();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setActionState(VisView::ACT_STATE action) {
	this->act_state = action;
}


void VisView::setImageSynthesisMethod(int type) {
	bool directions[4];		// x-, y-, x+, y+

	this->image_composition_method = type;

	if(poly->vec_ten_mode == 1) {
		switch(type) {
			case 0:
				directions[0] = true;
				directions[1] = true;
				directions[2] = true;
				directions[3] = true;
				copy_back = false;
				break;

			case 1:
				directions[0] = true;
				directions[1] = true;
				directions[2] = true;
				directions[3] = true;
				copy_back = true;
				break;

			case 2:
				directions[0] = false;
				directions[1] = false;
				directions[2] = true;
				directions[3] = true;
				copy_back = false;
				break;

			case 3:
				directions[0] = false;
				directions[1] = false;
				directions[2] = true;
				directions[3] = true;
				copy_back = true;
				break;

			case 4:
				directions[0] = true;
				directions[1] = false;
				directions[2] = true;
				directions[3] = false;
				copy_back = false;
				break;

			case 5:
				directions[0] = true;
				directions[1] = false;
				directions[2] = true;
				directions[3] = false;
				copy_back = true;
				break;

			case 6:
				directions[0] = false;
				directions[1] = false;
				directions[2] = true;
				directions[3] = false;
				copy_back = false;
				break;

			case 7:
				directions[0] = true;
				directions[1] = false;
				directions[2] = false;
				directions[3] = false;
				copy_back = false;
				break;

			case 8:
				directions[0] = false;
				directions[1] = false;
				directions[2] = false;
				directions[3] = true;
				copy_back = false;
				break;

			case 9:
				directions[0] = false;
				directions[1] = true;
				directions[2] = false;
				directions[3] = false;
				copy_back = false;
				break;

			case 10:
				directions[0] = false;
				directions[1] = true;
				directions[2] = false;
				directions[3] = true;
				copy_back = false;
				break;

			case 11:
				directions[0] = true;
				directions[1] = true;
				directions[2] = false;
				directions[3] = false;
				copy_back = false;
				break;

			default:
				directions[0] = false;
				directions[1] = false;
				directions[2] = false;
				directions[3] = false;
				copy_back = false;
				break;
		}


		num_enabled_warped_images = 0;

		for(int i = 0; i < 4; i++) {
			if(directions[i] == true) {
				coords[num_enabled_warped_images++] = 2 * (i >> 1) + i + ((current_tensor_vis_mode == VisView::MINOR) ? 2 : 0);
			}
		}
	}
}

void VisView::setGlobalRotationAngle(double degree) {
	field_manipulator.setRotationAngle(degree);
}


void VisView::setShowControlKnobs(bool val) {
	show_control_knobs = val;
}

void VisView::setShowSingularities(bool val) {
	show_singularities = val;
}

void VisView::setShowSeparatrices(bool val) {
	show_separatrices = val;
}

void VisView::setShowMultiStyleRegions(bool val) {
	show_multistyle_regions = val;
}

void VisView::setShowCurves(bool val) {
	show_curves = val;
}

bool VisView::getShowSeparatrices() {
	return show_separatrices;
}

bool VisView::getShowSingularities() {
	return show_singularities;
}

bool VisView::getShowControlKnobs() {
	return show_control_knobs;
}

bool VisView::getShowMultiStyleRegions() {
	return show_multistyle_regions;
}

bool VisView::getShowCurves() {
	return show_curves;
}


void VisView::createStaticTexturesForWaterColorEffect() {
	int num_octaves;
	double freq;
	double persistence;
	double amp;

	int idx, idx2;
	int lut[256];
	int *phase;
	int color_used[2] = {255, 0};

	ImageLoader il;
	int w, h;
	unsigned char *paper_texture;
	CString tex_path;


	//-------------------------------------------------------//
	//  Turbulance map, using the Perlin noise
	//-------------------------------------------------------//
	num_octaves = 5;
	freq = 0.02;
	persistence = 2.;
	amp = 1.;

	for(int i = 0; i < num_win_pixels; i++) {
		turbulance_map[i] = 0;
	}

	if(persistence <= 0) persistence = 1.;

	for(int m = 0; m < num_octaves; m++) {
		for(int i = 0; i < win_height; i++) {
			for(int j = 0; j < win_width; j++) {
				turbulance_map[j + win_width * i] += ImprovedNoise::noise(j * freq, i * freq, 0) * amp;
			}
		}

		amp /= persistence;
		freq *= persistence;
	}



	//-------------------------------------------------------//
	//  Grain noise, simulating pigment dispersion
	//-------------------------------------------------------//
	num_octaves = 3;
	freq = 1.5;
	persistence = 2.5;
	amp = 2.;

	phase = new int[num_win_pixels];

	srand(1);

	for(int i = 0; i < 256; i++) lut[i] = (i < 64) ? 0 : (i < 128) ? 85 : (i < 192) ? 170 : 255;

	for(int i = 0; i < num_win_pixels; i++) {
		phase[i] = rand() % 256;
	}

	for(int i = 0; i < num_win_pixels; i++) {
		pigment_map[i] = 0;
	}

	for(int m = 0; m < num_octaves; m++) {
		for(int i = 0; i < win_height; i++) {
			for(int j = 0; j < win_width; j++) {
				idx = j + win_width * i;
				idx2 = ((int)(j * freq) % win_width) + win_width * ((int)(i * freq) % win_height);

				pigment_map[idx] += 
					amp * (
					lut[phase[idx2] % 255] * color_used[0] / (255. * 255.) + 
					(255 - lut[phase[idx2] % 255]) * color_used[1] / (255. * 255.) 
					- .5);
			}
		}

		amp /= persistence;
		freq *= persistence;
	}

	delete[] phase;


	//-------------------------------------------------------//
	//  Create paper texture
	//-------------------------------------------------------//

	// for now, let's assume that image dimension is 512 x 512...
	tex_path = CString(full_path) + _T("/watercolorpaper.jpg");
	il.readJPG((LPCTSTR)tex_path, &paper_texture, w, h);

	if(w <= 0) {
		MessageBox(_T("Paper texture was not found"), _T(""), MB_ICONWARNING);
		return;
	}

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, w, h, 0, GL_RGB, GL_UNSIGNED_BYTE, paper_texture);
	quad->Draw();
	glDisable(GL_TEXTURE_2D);

	delete[] paper_texture;
	paper_texture = new unsigned char[3 * win_width * win_height];

	glReadPixels(0, 0, win_width, win_height, GL_RGB, GL_UNSIGNED_BYTE, paper_texture);

	for(int	i = 0; i < num_win_pixels; i++) {
		paper_texture_map[i] = calcIntensity(&(paper_texture[3 * i])) / 255. - .5;
	}

	delete[] paper_texture;
}


void VisView::createEdgeDarkingMap() {
	int idx;
	double dhdx, dhdy, grad;
	double scale = 1. / (255 * 2. * 3.);

	for(int i = 0; i < win_height; i++) {
		for(int j = 0; j < win_width; j++) {
			idx = j + win_width * i;
			grad = 0;

			for(int k = 0; k < 3; k++) {
				// simple gradient estimation
				if(j <= 0) {
					dhdx = painted_image[3 * (idx + 1) + k] - painted_image[3 * (idx + 0) + k];
				}
				else if(j >= win_width - 1) {
					dhdx = painted_image[3 * (idx + 0) + k] - painted_image[3 * (idx - 1) + k];
				}
				else {
					dhdx = painted_image[3 * (idx - 1) + k] - painted_image[3 * (idx + 1) + k];
				}

				if(i <= 0) {
					dhdy = painted_image[3 * (idx + win_width) + k] - painted_image[3 * (idx + 0) + k];
				}
				else if(i >= win_height - 1) {
					dhdy = painted_image[3 * (idx + 0) + k] - painted_image[3 * (idx - win_width) + k];
				}
				else {
					dhdy = painted_image[3 * (idx - win_width) + k] - painted_image[3 * (idx + win_width) + k];
				}

				grad += abs(dhdx) + abs(dhdy);
			}

			post_processing_map[j + win_width * i] = grad * scale;

		}
	}
}


void VisView::createHeightMap() {
	int idx = 0;
	float scale = 1.f / 255.f;
	unsigned char *tmp = NULL;


	D_SET_COUNTER_VAR(d_cnt_idx, 0)
	D_TICK(d_render_heightmapcomps, d_cnt_idx)

	current_renderer->setRenderMode(1);
	current_renderer->render(height_map);

	D_TOCK(d_render_heightmapcomps, d_cnt_idx++)
	D_TICK(d_render_heightmapcomps, d_cnt_idx)

	switch(embossing_type) {
		case 0:
			for(int i = 0; i < num_win_pixels; i++, idx += 3) {
				post_processing_map[i] = height_map[idx] * scale;
			}

			/**
			My_SSS_SavePPM(win_width, win_height, height_map, "height_map.ppm");
			/**/

			break;

		case 1:
			for(int i = 0; i < num_win_pixels; i++, idx += 3) {
				post_processing_map[i] = calcIntensity(&(height_map[idx])) * scale;
			}


			/**
			My_SSS_SavePPM(win_width, win_height, height_map, "secondary_image.ppm");
			tmp = new unsigned char[3 * num_win_pixels];
			idx = 0;
			for(int i = 0; i < num_win_pixels; i++, idx += 3) {
				tmp[idx] = calcIntensity(&(height_map[idx]));
				tmp[idx+1] = tmp[idx];
				tmp[idx+2] = tmp[idx];
			}
			My_SSS_SavePPM(win_width, win_height, tmp, "pseudo_height_map.ppm");
			delete[] tmp;
			/**/

			break;

		default:
			break;
	}

	D_TOCK(d_render_heightmapcomps, d_cnt_idx++)
}




void VisView::setPostProcessingOperation(int type) {
	PaintingRegion *pr;
	PaintingPreset p;
	bool prev_timer_status = getDisplayTimerStatus();


	stopDisplayTimer();

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
		type = PaintingStyleField::mp_postprocessing_type;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();
		p.post_process_operation = type;
		pr->setCurrentKey(p);
		painting_style_field->applyPaintingForRegion(pr, p);
	}

	post_processing_operation = type;
	update_post_processing_map = true;

	if(current_painting_step == PAINTING_STEP::PAINTING_DONE) {
		current_painting_step = PAINTING_STEP::POST_PROCESSING;
	}

	if(prev_timer_status == true) startDisplayTimer();
}


int VisView::getPostProcessingOperation() {
	return post_processing_operation;
}


void VisView::setPostProcessingParam(int param, float val) {
	PaintingRegion *pr;
	PaintingPreset p;
	bool prev_timer_status = getDisplayTimerStatus();


	if(param < 0 || param >= 4) return;

	stopDisplayTimer();

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();

		switch(param) {
			case 0:
				p.post_process_val1 = val;
				break;

			case 1:
				p.post_process_val2 = val;
				break;

			case 2:
				p.post_process_val3 = val;
				break;

			case 3:
				p.post_process_val4 = val;
				break;

			default:
				break;
		}

		pr->setCurrentKey(p);
		painting_style_field->applyPaintingForRegion(pr, p);
	}


	if(post_processing_operation == 1 && (param == 2 || param == 3)) {
		update_post_processing_map |= true;
	}
	else {
		update_post_processing_map |= false;
	}

	if(current_painting_step == PAINTING_STEP::PAINTING_DONE) {
		current_painting_step = PAINTING_STEP::POST_PROCESSING;
	}


	if(prev_timer_status == true) startDisplayTimer();
}


float VisView::getPostProcessingParam(int param) {
	PaintingRegion *pr;


	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		switch(param) {
			case 0:
				return pr->getCurrentKey().post_process_val1;

			case 1:
				return pr->getCurrentKey().post_process_val2;

			case 2:
				return pr->getCurrentKey().post_process_val3;

			case 3:
				return pr->getCurrentKey().post_process_val4;

			default:
				return 0;
		}
	}
	else {
		return -1.f;
	}
}


void VisView::applyPostProcessing() {
	int num_elems = 3 * num_win_pixels;

	if(current_display_mode != VisView::PAINTING) return; 


	D_SET_COUNTER_VAR(d_cnt_idx, 0)

	switch(post_processing_operation) {
		case 0:
			D_TICK(d_render_postprocessing, d_cnt_idx)

			if(update_post_processing_map == true) {
				createEdgeDarkingMap();
				update_post_processing_map = false;
			}

			D_TOCK(d_render_postprocessing, d_cnt_idx++)
			D_TICK(d_render_postprocessing, d_cnt_idx)

			applyPigmentDensityVariation(EDGE_DARKING);
			applyPigmentDensityVariation(HIGH_FREQ);
			applyPigmentDensityVariation(LOW_FREQ);
			applyPigmentDensityVariation(PAPER_NOISE);

			D_TOCK(d_render_postprocessing, d_cnt_idx++)

			break;

		case 1:
			D_TICK(d_render_postprocessing, d_cnt_idx)

			// create height field
			if(update_post_processing_map == true) {
				createHeightMap();
				update_post_processing_map = false;
			}

			D_TOCK(d_render_postprocessing, d_cnt_idx++)
			D_TICK(d_render_postprocessing, d_cnt_idx)

			// lighting
			applyLighting();

			D_TOCK(d_render_postprocessing, d_cnt_idx++)

			break;
	}
}


void VisView::applyPigmentDensityVariation(VisView::PDV_TYPE type) {
	int preset_idx = 0;
	float val;
	double c, r;
	double *map;
	double scale = .1;
	PaintingStyleNode *psn = painting_style_field->getStyleNode();

	switch(type) {
		case EDGE_DARKING:
			map = post_processing_map;
			break;

		case HIGH_FREQ:
			map = pigment_map;
			break;

		case LOW_FREQ:
			map = turbulance_map;
			break;

		case PAPER_NOISE:
			map = paper_texture_map;
			break;

		default:
			return;
	}

	for(int i = 0; i < num_win_pixels; i++) {
		switch(type) {
			case EDGE_DARKING:
				val = psn[i].preset.post_process_val1;
				break;

			case HIGH_FREQ:
				val = psn[i].preset.post_process_val2;
				break;

			case LOW_FREQ:
				val = psn[i].preset.post_process_val3;
				break;

			case PAPER_NOISE:
				val = psn[i].preset.post_process_val4;
				break;

			default:
				return;
		}

		r = scale * val * map[i];

		for(int j = 0; j < 3; j++) {
			// clamp into the range (0, 1)
			c = ::clampVal((double)painted_image[3 * i + j], 1, 255 - 1) / 255;

			c = c * (1 - (1 - c) * r) * 255;
			painted_image[3 * i + j] = (unsigned char)::clampVal(c, 0, 255);
		}
	}
}


void VisView::applyLighting() {
	int idx, idx_3;
	double rad = M_PI / 180.;
	double ka, kd, ks, nl, sh, sh2, hl;
	double kssh, kdnl;
	double dhdx, dhdy;
	double light_angle;
	icVector3 normal, light, view, reflection;
	PaintingStyleNode *psn = painting_style_field->getStyleNode();

	ka = .2928932;
	kd = 1.;
	ks = 50.;
	hl = 5.;

	view.set(0, 0, 1.);

	for(int i = 0; i < win_height; i++) {
		for(int j = 0; j < win_width; j++) {
			idx = j + win_width * i;

			// simple gradient estimation
			if(j <= 0) {
				dhdx = (post_processing_map[idx + 1] - post_processing_map[idx]) * .5;
			}
			else if(j >= win_width - 1) {
				dhdx = (post_processing_map[idx] - post_processing_map[idx - 1]) * .5;
			}
			else {
				dhdx = (post_processing_map[idx - 1] - post_processing_map[idx + 1]) * .5;
			}


			if(i <= 0) {
				dhdy = (post_processing_map[idx + win_width] - post_processing_map[idx]) * .5;
			}
			else if(i >= win_height - 1) {
				dhdy = (post_processing_map[idx] - post_processing_map[idx - win_width]) * .5;
			}
			else {
				dhdy = (post_processing_map[idx - win_width] - post_processing_map[idx + win_width]) * .5;
			}


			light_angle = psn[idx].preset.post_process_val1 * rad;
			light.set(-1. * cos(light_angle), -1. * sin(light_angle), 1);
			::normalize(light);

			dhdx *= psn[idx].preset.post_process_val2;
			dhdy *= psn[idx].preset.post_process_val2;

			normal.set(-1. * dhdx, -1. * dhdy, 1);
			::normalize(normal);
			nl = ::dot(light, normal);

			reflection = 2 * nl * normal - light;
			::normalize(reflection);

			// hl == 5...
			sh = max(0., ::dot(reflection, view));
			sh2 = sh * sh;
			sh = sh2 * sh2 * sh;
			//sh = pow(sh, hl);

			kssh = ks * sh;
			kdnl = kd * nl;

			idx_3 = 3 * idx;
			painted_image[idx_3] = (unsigned char)::clampVal(ka * painted_image[idx_3] + kdnl * painted_image[idx_3] + kssh, 0, 255);
			painted_image[idx_3 + 1] = (unsigned char)::clampVal(ka * painted_image[idx_3 + 1] + kdnl * painted_image[idx_3 + 1] + kssh, 0, 255);
			painted_image[idx_3 + 2] = (unsigned char)::clampVal(ka * painted_image[idx_3 + 2] + kdnl * painted_image[idx_3 + 2] + kssh, 0, 255);
		}
	}
}


bool VisView::setNumEnabledLayers(int new_num_layers) {
	bool return_status;
	bool prev_timrer_status = getDisplayTimerStatus();
	PaintingRegion *pr;
	PaintingPreset p;

	stopDisplayTimer();


	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();
		p.num_layers = (int)::clampVal(new_num_layers, 1, max_supported_layers);
		pr->setCurrentKey(p);
		painting_style_field->applyPaintingForRegion(pr, p);

		return_status = true;
	}
	else {
		return_status = false;
	}


	if(prev_timrer_status == true) startDisplayTimer();

	return return_status;
}


int VisView::getNumEnabledLayers() {
	PaintingRegion *pr;

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		return pr->getCurrentKey().num_layers;
	}
	else {
		return -1;
	}
}


int VisView::getMaxSupportedLayers() {
	return max_supported_layers;
}


size_t VisView::getNumStrokes(int layer_idx) {
	return current_renderer->getNumStrokes(layer_idx);
}


void VisView::clearStrokes(int layer_idx) {
	if(0 <= layer_idx && layer_idx < max_supported_layers) {
		current_renderer->clearSeeds(layer_idx);
	}
}


void VisView::setStrokeRegenerationMode(int mode) {
	switch(mode) {
		case 0:
		case 1:
			stroke_regen_mode = mode;
			current_renderer->setGenMode(mode);
			break;

		default:
			break;
	}
}


void VisView::setBaseImage(unsigned char *img, int w, int h) {
	bool prev_timer_status = getDisplayTimerStatus();
	unsigned char *dst = NULL;

	stopDisplayTimer();


	if(img == NULL || w <= 0 || h <= 0) {
		memset(original_image, 0, sizeof(unsigned char) * 3 * num_win_pixels);
		img_width = win_width;
		img_height = win_height;
		num_img_pixels = num_win_pixels;
	}
	else {
		push_pull(img, &dst, w, h, win_width, win_height);

		if(original_image) delete original_image;

		// set the new image
		original_image = dst;

		img_width = (w > win_width) ? win_width : w;
		img_height = (h > win_height) ? win_height : h;
		num_img_pixels = img_width * img_height;
	}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setStrokeRefImage(const unsigned char *img, int w, int h) {
	int idx1, idx2;
	bool prev_timer_status = getDisplayTimerStatus();
	unsigned char *dst = NULL;

	stopDisplayTimer();

	if(img == NULL || w <= 0 || h <= 0) {
	}
	else {
		push_pull((unsigned char*)img, &dst, w, h, win_width, win_height);

		// upside down the image at this point... (directly passed to a renderer)
		for(int i = 0; i < win_height; i++) {
			idx1 = 3 * win_width * i;
			idx2 = 3 * win_width * (win_height - 1 - i);

			for(int j = 0; j < win_width; j++) {
				stroke_ref_img[idx1++] = dst[idx2++];
				stroke_ref_img[idx1++] = dst[idx2++];
				stroke_ref_img[idx1++] = dst[idx2++];
			}
		}

		delete[] dst;
	}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setStrokeParams(StrokeParams stroke_params) {
	PaintingRegion *pr;
	PaintingPreset p;
	float prev_dp;

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();

		prev_dp = p.displacement;

		p.stroke_size = ((stroke_params.stroke_size >= 0) ? stroke_params.stroke_size : 0);

		if(painting_style_field->getNumRegions() == 1) {
			p.stroke_gridsize = ((stroke_params.stroke_gridsize >= 1) ? stroke_params.stroke_gridsize : 1);
		}
		else {
			painting_style_field->setStrokeSpacingForMultiStyle(stroke_params.stroke_gridsize);
		}

		p.stroke_pos_jitter = (float)::clampVal(stroke_params.stroke_position_jitter, -100, 100);
		p.jitter_amplitude = (float)::clampVal(stroke_params.stroke_color_jitter, 0, 255);
		p.stroke_initial_opacity = (float)::clampVal(stroke_params.stroke_opacity, 1, 255);
		p.stroke_min_length = (stroke_params.stroke_min_length >= 0) ? stroke_params.stroke_min_length : 0;
		p.stroke_max_length = (stroke_params.stroke_max_length >= 0) ? stroke_params.stroke_max_length : 0;

		p.stroke_color_shift[0] = (float)::clampVal(stroke_params.stroke_color_shift[0], -360, 360);
		p.stroke_color_shift[1] = (float)::clampVal(stroke_params.stroke_color_shift[1], -1.f, 1.f);
		p.stroke_color_shift[2] = (float)::clampVal(stroke_params.stroke_color_shift[2], -1.f, 1.f);

		p.displacement = (float)::clampVal(stroke_params.stroke_displacemnt, 0, ::dmax);

		pr->setCurrentKey(p);
		painting_style_field->applyPaintingForRegion(pr, p);

		if(prev_dp != p.displacement) {
			updateTexCoordsOnMesh();

			/**
			FILE *fp = fopen("blah.txt", "a");
			fprintf(fp, "%.10f, %.10f", prev_dp, p.displacement);
			fclose(fp);
			/**/
		}
	}
}


StrokeParams VisView::getStrokeParams() {
	PaintingRegion *pr;
	PaintingPreset p;
	StrokeParams sp;

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();

		sp.stroke_size = p.stroke_size;

		if(painting_style_field->getNumRegions() == 1) {
			sp.stroke_gridsize = p.stroke_gridsize;
		}
		else {
			//sp.stroke_gridsize = mp_stroke_spacing;
			sp.stroke_gridsize = painting_style_field->getStrokeSpacingForMultiStyle();
		}

		sp.stroke_position_jitter = p.stroke_pos_jitter;
		sp.stroke_opacity = p.stroke_initial_opacity;
		sp.stroke_color_jitter = p.jitter_amplitude;
		sp.stroke_min_length = p.stroke_min_length;
		sp.stroke_max_length = p.stroke_max_length;

		sp.stroke_color_shift[0] = p.stroke_color_shift[0];
		sp.stroke_color_shift[1] = p.stroke_color_shift[1];
		sp.stroke_color_shift[2] = p.stroke_color_shift[2];

		sp.stroke_displacemnt = p.displacement;

		return sp;
	}
	else {

		sp.stroke_size = 0;
		sp.stroke_gridsize = 0;
		sp.stroke_position_jitter = 0;
		sp.stroke_opacity = 0;
		sp.stroke_color_jitter = 0;
		sp.stroke_min_length = 0;
		sp.stroke_max_length = 0;

		sp.stroke_color_shift[0] = 0;
		sp.stroke_color_shift[1] = 0;
		sp.stroke_color_shift[2] = 0;

		sp.stroke_displacemnt = 0;

		return sp;
	}
}


void VisView::setInterpolateWeight(double weight) {
	field_manipulator.setWeight(weight);
}


double VisView::getInterpolateWeight() {
	return field_manipulator.getWeight();
}


void VisView::interpolateField() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	field_manipulator.interpolateField(current_time);

	copyTexCoordsList();
	sampleFieldToFSI();
	makeBlendMap();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::clearSelectedRegion() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	field_manipulator.clearSelectedRegion();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::loadControlFromFile(const char *filename) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(poly->vec_ten_mode == 1) {
		KeyFrameLoader::load(string(filename), *poly, *painting_style_field);

		// propagate design elements
		resetPropagation();

		updatePaintingStyleField();

		// set the post processing operation... 
		setPostProcessingOperation(painting_style_field->getRegion(0, 0)->getCurrentKey().post_process_operation);

		updateRegionInfoOnMesh();
	}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::saveControlToFile(const char *filename) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(poly->vec_ten_mode == 1) {
		KeyFrameLoader::save(string(filename), *poly, *painting_style_field);
	}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::undo() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	field_manipulator.undo();

	updateTexCoordsOnMesh();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::redo() {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	field_manipulator.redo();

	updateTexCoordsOnMesh();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setDisplayMode(VisView::DISPLAY_MODE mode) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	switch(mode) {
		case VECTOR_EDGE_FIELD:
			if(poly->vec_ten_mode == 1) {
				poly->vec_ten_mode = 0;
				poly->convert_tenfld_to_vecfld();
				field_sampler.sampleToFSI(*poly, *(input->fld_pyramid[0]));
			}

			break;

		case TENSOR_EDGE_FIELD:
			if(poly->vec_ten_mode == 0) {
				poly->vec_ten_mode = 1;
				poly->convert_vecfld_to_tenfld();
				field_sampler.sampleToFSI(*poly, *(input->fld_pyramid[0]));
			}

			break;

		case PAINTING:
			if(poly->vec_ten_mode == 0) {
				poly->vec_ten_mode = 1;
				poly->convert_vecfld_to_tenfld();
				field_sampler.sampleToFSI(*poly, *(input->fld_pyramid[0]));
			}

			current_tensor_vis_mode = VisView::MAJOR;

			break;

		default:
			break;
	}

	current_display_mode = mode;
	
	updateTexCoordsOnMesh();

	if(prev_timer_status == true) startDisplayTimer();
}


VisView::DISPLAY_MODE VisView::getDisplayMode() {
	return current_display_mode;
}


void VisView::setTensorVisMode(VisView::TENSOR_VIS_MODE mode) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(current_display_mode != VisView::PAINTING) {
		current_tensor_vis_mode = mode;
		setImageSynthesisMethod(image_composition_method);
	}

	if(prev_timer_status == true) startDisplayTimer();
}


VisView::TENSOR_VIS_MODE VisView::getTensorVisMode() {
	return current_tensor_vis_mode;
}


VisView::EDGE_FIELD_TYPE VisView::getEdgeFieldType() {
	if(poly->vec_ten_mode == 0) return VECTOR;
	else return TENSOR;
}


void VisView::setAreaErrorThreshold(float val) {
	PaintingRegion *pr;
	PaintingPreset p;
	bool prev_timer_status = getDisplayTimerStatus();


	stopDisplayTimer();

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();
		p.area_error = (float)::clampVal(val, 0, 255);
		pr->setCurrentKey(p);
		painting_style_field->applyPaintingForRegion(pr, p);
	}

	if(prev_timer_status == true) startDisplayTimer();
}


float VisView::getAreaErrorThreshold() {
	PaintingRegion *pr;

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		return pr->getCurrentKey().area_error;
	}
	else {
		return -1.f;
	}
}


void VisView::shiftStrokes(int layer_idx, float *shift, int width, int height) {
	current_renderer->pushSeeds(layer_idx, shift, width, height);
}


PaintingPreset VisView::getKey() {
	PaintingRegion *pr;

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		return pr->getKey(current_time);
	}
	else {
		return PRESET_NONE;
	}
}


void VisView::updateKey() {
	PaintingRegion *pr;
	PaintingPreset preset;
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		preset = pr->getCurrentKey();
		pr->setKey(preset, current_time);

		painting_style_field->applyPaintingForRegion(pr, preset);
		updateTexCoordsOnMesh();
	}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setKey(PaintingPreset preset) {
	CString msg;
	clock_t t1, t2;
	bool prev_timer_status = getDisplayTimerStatus();
	PaintingRegion *pr;
	PaintingPreset p;


	stopDisplayTimer();


	t1 = clock();

	setDisplayMode(PAINTING);

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;

		preset.color_composition_method = PaintingStyleField::mp_color_composition;
		preset.alpha = PaintingStyleField::mp_alpha;
		preset.post_process_operation = PaintingStyleField::mp_postprocessing_type;
	}

	if(pr != NULL) {
		p = preset;

		color_composition_method_for_paint = (COLOR_COMPOSITION)p.color_composition_method;
		post_processing_operation = p.post_process_operation;
		blend_alpha_for_paint = (int)::clampVal(p.alpha, 0, 255);
		update_post_processing_map = true;


		p.displacement = (float)::clampVal(p.displacement, 0, ::dmax);
		p.jitter_amplitude = (float)::clampVal(p.jitter_amplitude, 0, max(win_width, win_height));
		p.num_layers = (int)::clampVal(p.num_layers, 1, max_supported_layers);

		p.stroke_size = ((p.stroke_size >= 0) ? p.stroke_size : 0);
		p.stroke_gridsize = ((p.stroke_gridsize >= 1) ? p.stroke_gridsize : 1);
		p.stroke_pos_jitter = (float)::clampVal(p.stroke_pos_jitter, -100, 100);
		p.stroke_initial_opacity = (float)::clampVal(p.stroke_initial_opacity, 1, 255);

		pr->setKey(p, current_time);

		painting_style_field->applyPaintingForRegion(pr, p);

		updateTexCoordsOnMesh();
	}

	t2 = clock();

	//if(parent->getPrintStatEnabled() == true) {
		//msg.Format(_T("Update style parameters to a region: \t%.3lf sec.\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		//parent->printMessage(msg);

		//msg.Format(_T("Inserted at frame %d\r\n"), current_time + 1);
		//parent->printMessage(msg);
	//}


	if(prev_timer_status == true) startDisplayTimer();
}


/*
	This function is called whenever stepsize of vertex (, which is a member of Vertex class) is modified.
	Otherwise, you can simply call poly->calc_new_texture_coors(), then copyTexCoordsList()...
*/
void VisView::updateTexCoordsOnMesh() {
	int px, py;
	int w_1 = win_width - 1;
	int h_1 = win_height - 1;
	int num_verts = poly->nverts;
	Vertex *v;

	clock_t t1, t2;
	CString msg;
	PaintingStyleNode *psn = painting_style_field->getStyleNode();
	bool prev_timer_status = getDisplayTimerStatus();


	stopDisplayTimer();

	t1 = clock();

	switch(current_display_mode) {
		case PAINTING:

			for(int i = 0; i < num_verts; i++) {
				v = poly->vlist[i];
				px = (int)::clampVal((v->x + .5) * win_width, 0, w_1);
				py = (int)::clampVal((v->y + .5) * win_height, 0, h_1);
				v->stepsize = (float)(psn[px + win_width * py].preset.displacement);
			}

			poly->calc_new_texture_coors();	
			copyTexCoordsList();

			break;

		case VECTOR_EDGE_FIELD:
		case TENSOR_EDGE_FIELD:
			for(int i = 0; i < num_verts; i++) {
				poly->vlist[i]->stepsize = ::stepsize;
			}

			poly->calc_new_texture_coors();
			copyTexCoordsList();

			break;

		default:
			break;
	}

	t2 = clock();


	//if(parent->getPrintStatEnabled() == true) {
		//msg.Format(_T("Updated texture coodrinates: \t%.3lf sec.\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		//parent->printMessage(msg);
	//}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::copyTexCoordsList() {
	int idx;
	float *tex_coords_array;
	double *v_st;
	Triangle *t;

	for(int i = 0; i < 8; i++) {
		idx = 0;
		tex_coords_array = visible_tri_vertex_tex_coords[i];

		for(int j = 0; j < num_visible_triangles; j++) {
			t = poly->tlist[visible_tri_idx[j]];

			v_st = t->verts[0]->new_ts[i];
			tex_coords_array[idx++] = (float)v_st[0] + 0.5f;
			tex_coords_array[idx++] = (float)v_st[1] + 0.5f;

			v_st = t->verts[1]->new_ts[i];
			tex_coords_array[idx++] = (float)v_st[0] + 0.5f;
			tex_coords_array[idx++] = (float)v_st[1] + 0.5f;

			v_st = t->verts[2]->new_ts[i];
			tex_coords_array[idx++] = (float)v_st[0] + 0.5f;
			tex_coords_array[idx++] = (float)v_st[1] + 0.5f;
		}
	}
}


void VisView::updatePaintingStyleField() {
	clock_t t1, t2;
	CString msg;
	bool prev_timer_status = getDisplayTimerStatus();


	stopDisplayTimer();

	t1 = clock();
	painting_style_field->updatePaintingOnField(current_time);
	t2 = clock();

	//if(parent->getPrintStatEnabled() == true) {
		//msg.Format(_T("Updated styles to all regions: \t%.3lf sec.\r\n"), (double)(t2 - t1) / CLOCKS_PER_SEC);
		//parent->printMessage(msg);
	//}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::displayTracedPoints() {
	bool tmp_draw_traces = draw_traces;
	size_t num_pts = traced_points.size();
	pair<int, int> pt;


	draw_traces = false;

	glDisable(GL_TEXTURE_2D);
	glColor3f(0.f, 1.f, 1.f);
	glLineWidth(3.f);
	glBegin(GL_LINE_STRIP);

	for(size_t i = 0; i < num_pts; i++) {
		pt = traced_points.at(i);
		glVertex2f(pt.first / (float)win_width - .5f, pt.second / (float)win_height - .5f);
	}

	glEnd();

	draw_traces = tmp_draw_traces;
}


void VisView::setTraceMode(PathMaker::MODE new_mode) {
	path_maker.setMode(new_mode);
}


PathMaker::MODE VisView::getTraceMode() {
	return path_maker.getMode();
}


void VisView::displayMultiStyleRegions() {
	bool tmp_show_multistyle_regions = show_multistyle_regions;
	PaintingRegion *pr;

	show_multistyle_regions = false;

	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glPointSize(1.f);
	glBegin(GL_POINTS);


	for(int i = 0; i < win_height; i++) {
		for(int j = 0; j < win_width; j++) {
			pr = painting_style_field->getRegion(j, i);

			if(selected_region == NULL) {
				glColor4fv(pr->highlight_color);
			}
			else if(pr != NULL) {
				if(pr == selected_region) {
					glColor4fv(pr->highlight_color);
				}
				else {
					glColor4fv(pr->dehighlight_color);
				}
			}
			else {
				glColor4fv(pr->dehighlight_color);
			}

			glVertex2f((float)j / win_width - .5f, (float)i / win_height - .5f);
		}
	}

	glEnd();
	glDisable(GL_BLEND);

	show_multistyle_regions = tmp_show_multistyle_regions;
}


void VisView::displayMultiStyleCurves() {
	bool tmp_show_curves = show_curves;
	float oow = 1.f / win_width;
	float ooh = 1.f / win_height;
	vector<PATH*>::iterator itr;
	PATH::iterator itrp;
	PATH *path;


	show_curves = false;

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glColor4fv(curve_color);
	glLineWidth(2.f);

	for(itr = curves.begin(); itr != curves.end(); itr++) {
		path = *itr;
		glBegin(GL_LINE_LOOP);

		for(itrp = path->begin(); itrp != path->end(); itrp++) {
			// Maybe gluUnproject()/gluProject() will help better to convert points between window coordinate system and world coordinate system
			// if we work on non-square video clip? Just a thought....
			glVertex2f((float)(itrp->first) * oow - .5f, (float)(itrp->second) * ooh - .5f);
		}

		glEnd();
	}

	show_curves = tmp_show_curves;
}


size_t VisView::getNumMultiStyles() {
	return painting_style_field->getNumRegions();
}

int VisView::getNumFrames() {
	return num_frames;
}


void VisView::setNumFrames(int num_frames, const string &file_path) {
	if(num_frames <= 0) num_frames = 1;

	this->num_frames = num_frames;
	
	current_time = 0;

	assert(painting_style_field != NULL);

	painting_style_field->setNumFrames(num_frames, file_path);
}


void VisView::setCurrentTime(int time) {
	current_time = ((time >= 0) ? time : 0);
}


int VisView::getCurrentTime() {
	return current_time;
}


int VisView::findNextKey() {
	if(selected_region == NULL) {
		return -1;
	}
	else {
		return selected_region->getNextKey(current_time);
	}
}


int VisView::findPrevKey() {
	if(selected_region == NULL) {
		return -1;
	}
	else {
		return selected_region->getPrevKey(current_time);
	}
}


bool VisView::removeKey() {
	if(selected_region == NULL) {
		return false;
	}
	else if(selected_region->removeKey(current_time) == true) {
		if(painting_style_field->updatePaintingOnField(current_time) == true) {
			updateTexCoordsOnMesh();
			return true;
		}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}


int VisView::findNextDEKey() {
	if(selected_region == NULL) {
		return -1;
	}
	else {
		return selected_region->getNextDEKey(current_time);
	}
}


int VisView::findPrevDEKey() {
	if(selected_region == NULL) {
		return -1;
	}
	else {
		return selected_region->getPrevDEKey(current_time);
	}
}



void VisView::alphablendColor(unsigned char **images, unsigned char blend_val, int width, int height, unsigned char *output) {
	int idx;
	int num_win_pixels = width * height;
	unsigned char blend_val_complement = 255 - blend_val;
	unsigned char *img0 = images[0];
	unsigned char *img1 = images[1];


	idx = 0;

	for(int i = 0; i < num_win_pixels; i++, idx += 4) {
		output[idx + 0] = (blend_val * img0[idx + 0] + blend_val_complement * img1[idx + 0]) / 255;
		output[idx + 1] = (blend_val * img0[idx + 1] + blend_val_complement * img1[idx + 1]) / 255;
		output[idx + 2] = (blend_val * img0[idx + 2] + blend_val_complement * img1[idx + 2]) / 255;
		output[idx + 3] = (blend_val * img0[idx + 3] + blend_val_complement * img1[idx + 3]) / 255;
	}
}


void VisView::weightblendColor(unsigned char **images, float *weight_map, int width, int height, unsigned char *output, int op) {
	int idx;
	int num_win_pixels = width * height;
	int num_elems = 4 * num_win_pixels;
	unsigned char *img0 = images[0];
	unsigned char *img1 = images[1];
	unsigned char *img2 = NULL;
	unsigned char *img3 = NULL;


	switch(op) {
		case 0:						// composite 4 images...
			idx = 0;
			img2 = images[2];
			img3 = images[3];

			for(int i = 0; i < num_elems; i++) {
				idx = i / 4;

				output[i] = (unsigned char)(((img0[i] + img2[i]) * weight_map[idx] + (img1[i] + img3[i]) * (1 - weight_map[idx])) * .5);
			}

			break;


		case 1:						// blending x+ and x-, or y+ and y-
			for(int i = 0; i < num_elems; i++) {
				output[i] = (unsigned char)((img0[i] + img1[i]) / 2);
			}

			break;

		case 2:						// otherwise... (blending x and y images)
			for(int i = 0; i < num_elems; i++) {
				idx = i / 4;

				output[i] = (unsigned char)((img0[i] * weight_map[idx] + img1[i] * (1 - weight_map[idx])));
			}

			break;

		default:
			return;
	}
}


void VisView::resetPaintingRegions(bool retain_style) {
	bool tmp_show_curves = show_curves;
	bool tmp_show_multistyle_regions = show_multistyle_regions;
	vector<PATH*>::iterator itr;


	show_curves = false;
	show_multistyle_regions = false;
	selected_region = NULL;

	// remove curves
	for(itr = curves.begin(); itr != curves.end(); itr++) {
		delete (*itr);
	}

	curves.clear();


	// ... and reset regions
	painting_style_field->flatten(retain_style);
	updateRegionInfoOnMesh();

	selected_region = painting_style_field->getRegion(0, 0);

	if(parent->getPrintStatEnabled() == true) {
		parent->printMessage(_T("Reset Painting\r\n"));
	}

	show_curves = tmp_show_curves;
	show_multistyle_regions = tmp_show_multistyle_regions;
}


void VisView::setRegion(unsigned char *binary_map, int width, int height) {
	bool tmp_show_curves = show_curves;
	bool tmp_show_multistyle_regions = show_multistyle_regions;
	bool prev_timer_status = getDisplayTimerStatus();
	vector<PATH*>::iterator itr;


	if(binary_map == NULL || width <= 0 || height <= 0) return;

	stopDisplayTimer();

	show_curves = false;
	show_multistyle_regions = false;
	selected_region = NULL;

	// remove curves
	for(itr = curves.begin(); itr != curves.end(); itr++) {
		delete (*itr);
	}

	curves.clear();


	// ... and reset regions
	painting_style_field->setRegion(binary_map, width, height);

	if(parent->getPrintStatEnabled() == true) {
		parent->printMessage(_T("Set region\r\n"));
	}

	show_curves = tmp_show_curves;
	show_multistyle_regions = tmp_show_multistyle_regions;	

	if(prev_timer_status == true) startDisplayTimer();
}



void VisView::displayOptFlowArrows() {
	const float a = .01f;
	const int spacing = 16;
	const int hs = spacing / 2;
	int num_grids_x = win_width / spacing;
	int num_grids_y = win_height / spacing;
	int idx;

	float x1, y1;
	float x2, y2;
	float dx, dy;

	float angle;
	float threshold = 0;


	glDisable(GL_TEXTURE_2D);

	glLineWidth(2.f);
	glColor3f(1.f, 0, 0);
	glBegin(GL_LINES);

	threshold *= threshold;

	for(int i = 0; i < num_grids_y; i++) {
		idx = (i * spacing + hs) * win_width + hs;

		for(int j = 0; j < num_grids_x; j++) {
			x1 = (float)j * spacing + hs;
			y1 = (float)win_height - i * spacing - hs;
			x2 = optflow[idx].x;
			y2 = (float)win_height - optflow[idx].y;

			dx = x2 - x1;
			dy = y2 - y1;


			if(dx * dx + dy * dy > threshold) {
			//if(true) {
				x1 = x1 / win_width - .5f;
				y1 = y1 / win_height - .5f;
				x2 = x2 / win_width - .5f;
				y2 = y2 / win_height - .5f;

				angle = atan2(-dy, -dx);

				glVertex2f(x1, y1);
				glVertex2f(x2, y2);

				angle += (float)M_PI / 4.f;

				glVertex2f(x2, y2);
				glVertex2f(x2 + a * cos(angle), y2 + a * sin(angle));

				angle -= (float)M_PI / 2.f;

				glVertex2f(x2, y2);
				glVertex2f(x2 + a * cos(angle), y2 + a * sin(angle));
			}

			idx += spacing;
		}
	}

	glEnd();
}


void VisView::setOptFlowForVis(float *data, int width, int height) {
	int wl, hl;
	int idx0, idx1;
	bool tmp_show_optflow = dbg_show_optflow;



	if(width <= 0 || height <= 0) return;

	idx0 = 0;
	wl = (win_width < width) ? win_width : width;
	hl = (win_height < height) ? win_height : height;

	for(int i = 0; i < hl; i++) {
		idx1 = 2 * i * width;

		for(int j = 0; j < wl; j++) {
			optflow[idx0].x = data[idx1++];
			optflow[idx0].y = data[idx1++];
			idx0++;
		}

		for(int j = wl; j < win_width; j++) {
			optflow[idx0].x = (float)j;
			optflow[idx0].y = (float)i;
			idx0++;
		}
	}

	for(int i = hl; i < win_height; i++) {
		for(int j = 0; j < win_width; j++) {
			optflow[idx0].x = (float)j;
			optflow[idx0].y = (float)i;
			idx0++;
		}
	}

	dbg_show_optflow = tmp_show_optflow;
}


bool VisView::loadEdgeField(string &filename) {
	bool return_status;
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	return_status = field_manipulator.replaceField(filename);

	copyTexCoordsList();
	sampleFieldToFSI();
	makeBlendMap();

	if(prev_timer_status == true) startDisplayTimer();

	return return_status;
}


bool VisView::saveEdgeField(string &filename) {
	bool return_status;
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	return_status = field_manipulator.exportField(filename);


	if(prev_timer_status == true) startDisplayTimer();

	return return_status;
}


void VisView::enablePostProcessing(bool val) {
	bool prev_timrer_status = getDisplayTimerStatus();


	stopDisplayTimer();

	enable_post_processing = val;

	if(current_painting_step == PAINTING_STEP::PAINTING_DONE) {
		current_painting_step = PAINTING_STEP::POST_PROCESSING;
	}

	if(prev_timrer_status == true) startDisplayTimer();
}


bool VisView::getPostProcessingEnabled() {
	return enable_post_processing;
}


void VisView::setStrokeOrder(int type) {
	bool prev_timrer_status = getDisplayTimerStatus();

	stroke_order = (int)::clampVal(type, 0, 2);

	initializeRendering();

	if(prev_timrer_status == true) startDisplayTimer();
}


void VisView::setEmbossingType(int type) {
	bool prev_timrer_status = getDisplayTimerStatus();

	embossing_type = (int)::clampVal(type, 0, 1);
	update_post_processing_map = true;

	if(current_painting_step == PAINTING_STEP::PAINTING_DONE) {
		current_painting_step = PAINTING_STEP::POST_PROCESSING;
	}

	if(prev_timrer_status == true) startDisplayTimer();
}


void VisView::setOrderingRange(bool partial_or_whole) {
	bool prev_timrer_status = getDisplayTimerStatus();

	partial_ordering = partial_or_whole;

	initializeRendering();

	if(prev_timrer_status == true) startDisplayTimer();
}


void VisView::setRenderingMethod(VisView::RENDERER method) {
	bool prev_timrer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	renderer = method;

	switch(method) {
		case IBFV:
			current_renderer = ibfv_renderer;
			current_painting_param = &ibfv_param;
			break;

		case SPLINE:
			current_renderer = spline_renderer;
			current_painting_param = &spline_param;
			break;

		default:
			break;
	}

	initializeRendering();

	if(prev_timrer_status == true) startDisplayTimer();

}


void VisView::setRenderingOption_b(RENDERING_OPTION option, bool val) {
	bool prev_timrer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(ibfv_renderer != NULL) {
		switch(option) {
			case IBFV_USE_GL_NEAERST_IN_WARP: 
				ibfv_renderer->setWarpingOption(val);
				break;
		}
	}

	initializeRendering();

	if(prev_timrer_status == true) startDisplayTimer();
}


bool VisView::getRenderingOption_b(RENDERING_OPTION option) {
	if(ibfv_renderer != NULL) {
		switch(option) {
			case IBFV_USE_GL_NEAERST_IN_WARP: 
				return ibfv_renderer->getWarpingOption();
				break;

			default:
				return false;
				break;
		}
	}
	else {
		return false;
	}
}


void VisView::setRenderingOption_uc(RENDERING_OPTION option, unsigned char val) {
	bool prev_timrer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	if(ibfv_renderer != NULL) {
		switch(option) {
			case IBFV_BGFG: 
				ibfv_renderer->setBgFgThreshold(val);
				break;
		}
	}

	initializeRendering();

	if(prev_timrer_status == true) startDisplayTimer();
}


unsigned char VisView::getRenderingOption_uc(RENDERING_OPTION option) {
	if(ibfv_renderer != NULL) {
		switch(option) {
			case IBFV_BGFG: 
				return ibfv_renderer->getBgFgThreshold();
				break;

			default:
				return 0;
				break;
		}
	}
	else {
		return 0;
	}
}


void VisView::setVisIterations(int num_itr) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	num_warping_for_vis = ((num_itr >= 0) ? num_itr : 0);

	initializeRendering();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setPaintIterations(int num_itr) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	num_warping_for_paint = ((num_itr >= 0) ? num_itr : 0);

	initializeRendering();

	if(prev_timer_status == true) startDisplayTimer();
}


int VisView::getVisIterations() { return num_warping_for_vis; }
int VisView::getPaintIterations() { return num_warping_for_paint; }

void VisView::setVisDisplacement(double dp) {
	bool prev_timer_status = getDisplayTimerStatus();

	if(poly == NULL) return;

	stopDisplayTimer();

	::stepsize = (float)::clampVal(dp, 0, ::dmax);

	if(current_display_mode == VisView::VECTOR_EDGE_FIELD || current_display_mode == VisView::TENSOR_EDGE_FIELD) {
		updateTexCoordsOnMesh();
	}

	if(prev_timer_status == true) startDisplayTimer();
}


double VisView::getVisDisplacement() { return ::stepsize; }

void VisView::setVisBlendAlpha(int alpha) {
	bool prev_timer_status = getDisplayTimerStatus();

	stopDisplayTimer();

	blend_alpha_for_vis = (int)::clampVal(alpha, 0, 255);

	makeNoisePatterns();

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setPaintBlendAlpha(int alpha) {
	PaintingRegion *pr;
	PaintingPreset p;
	bool prev_timer_status = getDisplayTimerStatus();


	stopDisplayTimer();

	blend_alpha_for_paint = (int)::clampVal(alpha, 0, 255);

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		p = pr->getCurrentKey();
		p.alpha = blend_alpha_for_paint;
		pr->setCurrentKey(p);
		painting_style_field->applyPaintingForRegion(pr, p);
	}

	if(prev_timer_status == true) startDisplayTimer();
}


int VisView::getVisBlendAlpha() { return blend_alpha_for_vis; }
int VisView::getPaintBlendAlpha() { return blend_alpha_for_paint; }

void VisView::setVisColorCompositionMethod(VisView::COLOR_COMPOSITION composition) {
	// do nothing... (only alpha-blending mode is used)
	return;
}


void VisView::setPaintColorCompositionMethod(VisView::COLOR_COMPOSITION composition) {
	// do nothing...(only overwriting mode is used)
	return;
}


VisView::COLOR_COMPOSITION VisView::getVisColorCompositionMethod() { return color_composition_method_for_vis; }
VisView::COLOR_COMPOSITION VisView::getPaintColorCompositionMethod() {return color_composition_method_for_paint; }


void VisView::setRendererParameter() {
	switch(renderer) {
		case IBFV:
			ibfv_param.edge_pixel = input->fld_pyramid[0];
			ibfv_param.num_iterations = num_warping_for_paint;
			ibfv_param.seed_order = stroke_order;
			ibfv_param.partial_ordering = partial_ordering;
			ibfv_param.source_img = original_image;
			ibfv_param.time = current_time;
			ibfv_param.style = painting_style_field;
			ibfv_param.mesh_info.mesh = poly;
			ibfv_param.mesh_info.num_visible_triangles = num_visible_triangles;
			ibfv_param.mesh_info.visible_tri_idx = visible_tri_idx;
			ibfv_param.mesh_info.visible_tri_vertex_coords = visible_tri_vertex_coords;

			for(int i = 0; i < 8; i++) {
				ibfv_param.mesh_info.visible_tri_vertex_tex_coords[i] = visible_tri_vertex_tex_coords[i];
			}

			ibfv_param.image_composition.blend_map = blend_map;

			for(int i = 0; i < 4; i++) {
				ibfv_param.image_composition.coords[i] = coords[i];
			}

			ibfv_param.image_composition.copy_back = copy_back;
			ibfv_param.image_composition.image_composition_method = image_composition_method;
			ibfv_param.image_composition.num_enabled_warped_images = num_enabled_warped_images;

			ibfv_param.stroke_ref_img = use_stroke_ref_img ? stroke_ref_img : NULL;
			ibfv_param.embossing_type = embossing_type;

			break;

		case SPLINE:
			spline_param.source_img = original_image;
			spline_param.style = painting_style_field;
			spline_param.time = current_time;
			spline_param.edge_pixel = input->fld_pyramid[0];
			spline_param.seed_order = stroke_order;
			spline_param.partial_ordering = partial_ordering;
			spline_param.embossing_type = embossing_type;
			spline_param.viewport = viewport;
			spline_param.modelview = modelview;
			spline_param.projection = projection;

			spline_param.stroke_ref_img = use_stroke_ref_img ? stroke_ref_img : NULL;

			break;

		default:
			break;
	}
}

void VisView::getSegmentation(float Spatial, int Range, int MinSize)
{
	clock_t time1, time2;
	CString str;
	if(segmImageDefined) delete [] segmentation_image;
	segmentation_image = new byte [win_width*win_height*3];
    time1 = clock();
	seg_img.getSegImage(original_image, segmentation_image,win_width,win_height,Spatial, Range, MinSize);
	time2 = clock();
	seg_time = (double)(time2 - time1) / CLOCKS_PER_SEC;

    str.Format(_T("Segmentation Image: %.3lf sec.\r\n"), seg_time);
	parent->printMessage(str);


	segmImageDefined = true;


	


}

void VisView::setSegmentationRegion(void)
{
	bool tmp_show_curves = show_curves;
	bool tmp_show_multistyle_regions = show_multistyle_regions;
	bool prev_timer_status = getDisplayTimerStatus();
	vector<PATH*>::iterator itr;


	stopDisplayTimer();

	show_curves = false;
	show_multistyle_regions = false;
	selected_region = NULL;

	// remove curves
	for(itr = curves.begin(); itr != curves.end(); itr++) {
		delete (*itr);
	}

	curves.clear();


	
	
	// ... and reset regions
	if(current_display_mode==SEGMENTATION)
	{
		painting_style_field->SetSegmentationRegion( seg_img.RegionID);
	}
	
	
	

	if(parent->getPrintStatEnabled() == true) {
		parent->printMessage(_T("Set region\r\n"));
	}

	show_curves = tmp_show_curves;
	show_multistyle_regions = tmp_show_multistyle_regions;	

	if(prev_timer_status == true) startDisplayTimer();

}

void VisView::setSegImage(unsigned char* img, int w, int h)
{
	bool prev_timer_status = getDisplayTimerStatus();
	unsigned char *dst = NULL;

	stopDisplayTimer();

	

	if(img == NULL || w <= 0 || h <= 0) {
		memset(segmentation_image, 0, sizeof(unsigned char) * 3 * num_win_pixels);
	}
	else {
		push_pull(img, &dst, w, h, win_width, win_height);

		if(segmentation_image) delete segmentation_image;

		// set the new image
		segmentation_image = dst;

		seg_img.setRegionID(segmentation_image,w,h);
	}

	if(prev_timer_status == true) startDisplayTimer();


}


void VisView::loadDat(const char *filename) {
	bool prev_timer_status = getDisplayTimerStatus();
	stopDisplayTimer();
	
	dat.getDat(filename);
	setTransformation();
	VideoSegmentation = 1;

	if(prev_timer_status == true) startDisplayTimer();	
}


void VisView::unloadDat() {
	bool prev_timer_status = getDisplayTimerStatus();
	stopDisplayTimer();

	VideoSegmentation = 0;
	painting_style_field->flatten(true);
	selected_region = painting_style_field->getRegion(0, 0);


	syncRegionInfoOnMesh();

	// just simply make all vertices have region ID 0
	// maybe I will need more elegant function to synchronize data between mesh and pixels...
	//for(int i = 0; i < poly->nverts; i++) {
	//	poly->vlist[i]->regionID = 0;
	//}

	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::setTransformation() {
	int prev_selected_region_id = -1;
	size_t prev_num_regions;
	bool tmp_show_curves = show_curves;
	bool tmp_show_multistyle_regions = show_multistyle_regions;
	bool prev_timer_status = getDisplayTimerStatus();
	vector<PATH*>::iterator itr;


	stopDisplayTimer();

	if(dat.DatImageDefined == true) {
		show_curves = false;
		show_multistyle_regions = false;

		if(selected_region != NULL) {
			prev_selected_region_id = selected_region->getID();
		}

		prev_num_regions = painting_style_field->getNumRegions();
		selected_region = NULL;

		// remove curves
		for(itr = curves.begin(); itr != curves.end(); itr++) {
			delete (*itr);
		}

		curves.clear();

		
		// ... and reset regions
		painting_style_field->setLabelsToRegions(dat);		// to pixels
		syncRegionInfoOnMesh();								// ... then pixels
		//field_sampler.sampleTenFieldRegionToMesh(*poly, *(input->fld_pyramid[0]), dat.imgdat->labels);
		

		if(parent->getPrintStatEnabled() == true) {
			parent->printMessage(_T("Set region\r\n"));
		}

		show_curves = tmp_show_curves;
		show_multistyle_regions = tmp_show_multistyle_regions;

		// re-select the previous region
		if(prev_num_regions == painting_style_field->getNumRegions()) {
			selected_region = painting_style_field->selectRegion(prev_selected_region_id);
		}

	}
	else {
		parent->printMessage(_T("Failed to set region. Corresponding Dat file not found\r\n"));
	}


	if(prev_timer_status == true) startDisplayTimer();
}


void VisView::syncRegionInfoOnMesh() {
	int px, py;
	int w_1 = win_width - 1;
	int h_1 = win_height - 1;
	Vertex *v;
	PaintingRegion *pr;
	PaintingStyleNode *fsn;

	fsn = painting_style_field->getStyleNode();

	for(int i = 0; i < poly->nverts; i++) {
		v = poly->vlist[i];

		/**
		if(v->x < -.5 || v->x > .5) continue;
		if(v->y < -.5 || v->y > .5) continue;

		// compute a coordinate of pixel (no sign flip for y is necessary)
		px = (int)((v->x + .5) * win_width);
		py = (int)((v->y + .5) * win_height);
		/**/

		/**/
		px = (int)::clampVal((v->x + .5) * w_1, 0, w_1);
		py = (int)::clampVal((v->y + .5) * h_1, 0, h_1);
		/**/


		pr = painting_style_field->getRegion(px, py);

		assert(pr != NULL);

		v->regionID = pr->getID();						// copy ID
		v->use_zero_value = pr->getUseZeroField();		// copy flag
	}
}


void VisView::updateRegionInfoOnMesh() {
	syncRegionInfoOnMesh();

	field_manipulator.interpolateField(current_time);

	sampleFieldToFSI();
	input->fld_pyramid[0]->blur(*painting_style_field);
	field_sampler.sampleToMesh(*poly, *(input->fld_pyramid[0]));

	updateTexCoordsOnMesh();
	sampleFieldToFSI();
	makeBlendMap();
}


void VisView::setKeyInterpolationFunc(TransitionalPaintingPreset::INTERPOLATION_TYPE type) {
	bool prev_timer_status = getDisplayTimerStatus();
	PaintingRegion *pr;

	stopDisplayTimer();


	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		pr->setStyleInterpolationType(type, current_time);
		painting_style_field->updatePaintingOnField(current_time);

		updateTexCoordsOnMesh();
	}

	if(prev_timer_status == true) startDisplayTimer();
}


TransitionalPaintingPreset::INTERPOLATION_TYPE VisView::getKeyInterpolationFunc() {
	PaintingRegion *pr;

	if(painting_style_field->getNumRegions() == 1) {
		pr = painting_style_field->getRegion(0, 0);
	}
	else {
		pr = selected_region;
	}

	if(pr != NULL) {
		return pr->getStyleInterpolationType(current_time);
	}
	else {
		return TransitionalPaintingPreset::UNDEFINED;
	}
}



#undef D_SET_COUNTER_VAR
#undef D_UPDATE_COUNTER_VAR
#undef D_COPY_VAL
#undef D_INIT_TIMING
#undef D_TICK
#undef D_TOCK
#undef D_TIME
#undef D_FIN_TIMING
