#ifndef icComplex_is_defined
#define icComplex_is_defined

class icComplex;

#include <math.h>
#include <stdlib.h>
#include "icVector.H"

class icComplex {
public:
	inline icComplex();
	inline icComplex(double rr, double ii);
  inline icComplex &set      (double rr, double ii);
  inline icComplex &set      (const icComplex &that);  
  inline icComplex &operator=(const icComplex &that); 
  inline int operator!=(const icComplex &that)const; 
  inline int operator==(const icComplex &that)const; 
  inline icComplex &operator+=(const icComplex &that);
  inline icComplex &operator-=(const icComplex &that);
  inline icComplex &operator*=(const icComplex &that);
  inline icComplex &operator/=(const icComplex &that);
public:
  double r, i;
};

inline double norm(const icComplex &a);

inline icComplex::icComplex() {
  r = i = 0.0;
}

inline icComplex::icComplex(double rr, double ii) {
  r = rr;
	i = ii;
}

inline icComplex &icComplex::set(double rr, double ii) {
	this->r = rr;
	this->i = ii;
  return (*this);
}

inline icComplex &icComplex::set(const icComplex &that) {
  return (*this)=that;
}

inline icComplex &icComplex::operator=(const icComplex &that) {
  r = that.r;
  i = that.i;
  return (*this);
};

inline int icComplex::operator==(const icComplex &that)const {
  return ( r == that.r && i == that.i);
}

inline int icComplex::operator!=(const icComplex &that)const {
  return ( r != that.r || i != that.i);;
}

inline icComplex &icComplex::operator+=(const icComplex &that) {
  r += that.r; 
  i += that.i; 
  return (*this);
}

inline icComplex &icComplex::operator-=(const icComplex &that) {
  r -= that.r; 
  i -= that.i; 
  return (*this);
}

inline icComplex &icComplex::operator*=(const icComplex &that) {
	double old_r = r, old_i = i;
  r = old_r*that.r - old_i*that.i; 
  i = old_i*that.r - old_r*that.i; 
  return (*this);
}

inline icComplex &icComplex::operator/=(const icComplex &that) {
	if (norm(that) == 0.0) {
		r = i = 0.0;
		return (*this);
	}
	double old_r = r, old_i = i;
	double len = 1.0/(that.r*that.r+that.i*that.i);
	r = (old_r*that.r+old_i*that.i)*len;
	i = (old_i*that.r-old_r*that.i)*len;
  return (*this);
}

inline icComplex operator+(const icComplex &a,const icComplex &b) {
  return (icComplex(a)+=b);
}

inline icComplex operator-(const icComplex &a,const icComplex &b) {
  return (icComplex(a)-=b);
}

inline icComplex operator*(const icComplex &a,const icComplex &b) {
  return (icComplex(a)*=b);
}

inline icComplex operator/(const icComplex &a,const icComplex &b) {
  return (icComplex(a)/=b);
}

inline icComplex conjugate(const icComplex &a) {
  return icComplex(a.r, -a.i);
}

inline double norm(const icComplex &a) {
  return sqrt(a.r*a.r+a.i*a.i);
}

#endif
