#ifndef __IMAGE_PYRAMID_H__
#define __IMAGE_PYRAMID_H__

#include "icVector.H"
#include "icMatrix.H"
#include "Diffusion.h"
#include "PaintingStyleField.h"


class FieldSample {
public:
	icVector2 fld_st, vec_fld, image_gradient;
	icMatrix2x2 ten_fld;
};

class FieldSampleImage {
public:
	int w, h;
	FieldSample *fld;

	FieldSampleImage(int ww, int hh) {
		w = ww;
		h = hh;
		fld = new FieldSample[w * h];

		initializeBlurSetting();
	}

	FieldSampleImage(FieldSampleImage &fsi) {
		w = fsi.w;
		h = fsi.h;
		fld = new FieldSample[w * h];

		copy(fsi);
		initializeBlurSetting();
	}

	void copy(FieldSampleImage &fsi) {
		int num_elems = w * h;

		if(w != fsi.w || h != fsi.h) return;

		for(int i = 0; i < num_elems; i++) {
			fld[i] = fsi.fld[i];
		}
	}

	bool blur(PaintingStyleField &psf);

	void finalize() {
		delete[] fld;
		delete(this);
	}

private:
	bool diffuse;								// internal flag
	int diffuse_num_itr;
	float diffuse_stepsize;
	Diffusion<TensorInterpolatable> ef_blur;	// blur edge field


	void initializeBlurSetting() {
		ef_blur.blurWithPyramid(true);											// enable pyramid method
		ef_blur.enableConvergenceCheck(true);									// check convergence at each pyramid level
		ef_blur.setBlurMethod(Diffusion<TensorInterpolatable>::GAUSS_SEIDEL);	// use Gauss Seidel method
		diffuse_num_itr = 1;													// Pyramid level keep iterates at each level until values converge... so set 1 for this.
		diffuse_stepsize = 10.f;												// use a large timestep for Gauss Seidel

		diffuse = true;
	}
};

class FieldSampleImagePyramid {
public:
	int numlevels;
	FieldSampleImage **fld_pyramid;

	FieldSampleImagePyramid(int imagesize, int minsize) {
		numlevels = 1;
		int initial_size = imagesize;
		int current_size = imagesize;

		while(current_size > minsize) {
			numlevels++;
			current_size /= 2;
		}

		fld_pyramid = new FieldSampleImage*[numlevels];
		current_size = imagesize;
		
		for(int i = 0; i < numlevels; i++) {
			fld_pyramid[i] = new FieldSampleImage(current_size, current_size);
			current_size /= 2;
		}
	}

	void resetField() {
		for(int i = 0; numlevels; i++) {
			fld_pyramid[i]->fld->fld_st.set(0, 0);
			fld_pyramid[i]->fld->image_gradient.set(0, 0);
			fld_pyramid[i]->fld->ten_fld.set(0, 0, 0, 0);
			fld_pyramid[i]->fld->vec_fld.set(0, 0);
		}
	}

	void finalize() {
		for(int i = 0; i < numlevels; i++) {
			fld_pyramid[i]->finalize();
		}

		delete(fld_pyramid);
		delete(this);
	}
};

#endif /* __IMAGE_PYRAMID_H__ */
