#include <math.h>
#define NRANSI
#include "nrutil.h"
#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4

//double ERRCON = pow(5.0/SAFETY, 1.0/PGROW);

// tensor field
void rkqs_tensor(double vec_a, double vec_b, double vec_c, double vec_d, double vec_e, double vec_f, double y[], double dydx[], int n, double *x, double htry, double eps,
	double yscal[], double *hdid, double *hnext,
	void (*derivs)(double, double, double, double, double, double, double, double [], double[], double []))
{
	void rkck_tensor(double, double, double, double, double, double, double y[], double dydx[], int n, double x, double h,
		double yout[], double yerr[], void (*derivs)(double, double, double, double, double, double, double, double [], double[], double []));
	int i;
	double errmax,h,xnew,*yerr,*ytemp;

	yerr=::vector(1,n);
	ytemp=::vector(1,n);
	h=htry;
	for (;;) {
		rkck_tensor(vec_a, vec_b, vec_c, vec_d, vec_e, vec_f, y,dydx,n,*x,h,ytemp,yerr,derivs);
		errmax=0.0;
		for (i=1;i<=n;i++) errmax=FMAX(errmax,fabs(yerr[i]/yscal[i]));
		errmax /= eps;
		if (errmax > 1.0) {
			h=SAFETY*h*pow(errmax,PSHRNK);
			if (h < 0.1*h) h *= 0.1;
			xnew=(*x)+h;
			if (xnew == *x) 
				nrerror("stepsize underflow in rkqs");
			continue;
		} else {
			if (errmax > ERRCON) *hnext=SAFETY*h*pow(errmax,PGROW);
			else *hnext=5.0*h;
			*x += (*hdid=h);
			for (i=1;i<=n;i++) y[i]=ytemp[i];
			(*derivs)(*x, vec_a, vec_b, vec_c, vec_d, vec_e, vec_f, y, dydx, dydx);
			break;
		}
	}
	free_vector(ytemp,1,n);
	free_vector(yerr,1,n);
}

// vector field
void rkqs(double vec_a, double vec_b, double vec_c, double vec_d, double vec_e, double vec_f, double y[], double dydx[], int n, double *x, double htry, double eps,
	double yscal[], double *hdid, double *hnext,
	void (*derivs)(double, double, double, double, double, double, double, double [], double []))
{
	void rkck(double, double, double, double, double, double, double y[], double dydx[], int n, double x, double h,
		double yout[], double yerr[], void (*derivs)(double, double, double, double, double, double, double, double [], double []));
	int i;
	double errmax,h,xnew,*yerr,*ytemp;

	yerr=::vector(1,n);
	ytemp=::vector(1,n);
	h=htry;
	for (;;) {
		rkck(vec_a, vec_b, vec_c, vec_d, vec_e, vec_f, y,dydx,n,*x,h,ytemp,yerr,derivs);
		errmax=0.0;
		for (i=1;i<=n;i++) errmax=FMAX(errmax,fabs(yerr[i]/yscal[i]));
		errmax /= eps;
		if (errmax > 1.0) {
			h=SAFETY*h*pow(errmax,PSHRNK);
			if (h < 0.1*h) h *= 0.1;
			xnew=(*x)+h;
			if (xnew == *x) 
				nrerror("stepsize underflow in rkqs");
			continue;
		} else {
			if (errmax > ERRCON) *hnext=SAFETY*h*pow(errmax,PGROW);
			else *hnext=5.0*h;
			*x += (*hdid=h);
			for (i=1;i<=n;i++) y[i]=ytemp[i];
			break;
		}
	}
	free_vector(ytemp,1,n);
	free_vector(yerr,1,n);
}

void rkqs(double y[], double dydx[], int n, double *x, double htry, double eps,
	double yscal[], double *hdid, double *hnext,
	void (*derivs)(double, double [], double []))
{
	void rkck(double y[], double dydx[], int n, double x, double h,
		double yout[], double yerr[], void (*derivs)(double, double [], double []));
	int i;
	double errmax,h,xnew,*yerr,*ytemp;

	yerr=::vector(1,n);
	ytemp=::vector(1,n);
	h=htry;
	for (;;) {
		rkck(y,dydx,n,*x,h,ytemp,yerr,derivs);
		errmax=0.0;
		for (i=1;i<=n;i++) errmax=FMAX(errmax,fabs(yerr[i]/yscal[i]));
		errmax /= eps;
		if (errmax > 1.0) {
			h=SAFETY*h*pow(errmax,PSHRNK);
			if (h < 0.1*h) h *= 0.1;
			xnew=(*x)+h;
			if (xnew == *x) 
				nrerror("stepsize underflow in rkqs");
			continue;
		} else {
			if (errmax > ERRCON) *hnext=SAFETY*h*pow(errmax,PGROW);
			else *hnext=5.0*h;
			*x += (*hdid=h);
			for (i=1;i<=n;i++) y[i]=ytemp[i];
			break;
		}
	}
	free_vector(ytemp,1,n);
	free_vector(yerr,1,n);
}

#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON
#undef NRANSI
