/*

Generate global coordinates for a 3D model

Eugene Zhang, Nov. 2001

*/

#pragma once
#pragma warning (once: 4101 4018 4244 4311 4312)

#include "StdAfx.h"
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

/**/
#ifdef _DEBUG
//#define new DEBUG_NEW
#define   malloc(s)         _malloc_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__)
#endif
/**/

#include <math.h>
#include <string.h>
#include <time.h>
#include "ply.h"
#include "tendes_plane.h"
#include "tendes_plane_io.h"
#include "myqueue.h"
#include "line_minimization.h"
#include "svdcmp.h"
#include "jacobi.h"
#include "rkck.h"
#include "rkqs.h"
#include "odeint.h"
#include "cubic.h"


extern float  dmax;
extern float  stepsize;


#define MAX_STORE_SIZE 100

int lamda_method = 1;
static int kmax=MAX_STORE_SIZE, kount;
static double *xp, **yp, XP[MAX_STORE_SIZE+1], *Yp[3], Yp1[MAX_STORE_SIZE+1], Yp2[MAX_STORE_SIZE+1], dxsav=10.0/MAX_STORE_SIZE;
static double **dydxp, *DYDXp[3], DYDXp1[MAX_STORE_SIZE+1], DYDXp2[MAX_STORE_SIZE+1];

static PlyFile *in_ply;

#define MAX_TEXTURE_SIZE 1024

FILE *this_file;

double error_threshold = 1.0e-13;
double error_threshold_ode = 1.0e-12;
double ode_step = 1.0e-4;
double vector_clip_size = 1.0e-2;
double saddle_out = 1.0e-5;

char model_name[128];
int max_error = 3;
int curvature_angle = 0; 
int max_f = 1;
int max_alpha = 7;
int max_beta = 1;
int num_steps = 1;
double time_step = 1.0;
double f_max = 1.0;
double alpha_max = 5.0;
double beta_max = 1.0e-3; // 1.0e-6;
int num_iterations = 0;

int num_sample = 4;  // use 1, 2, 4, 8
double checker_size;

// used for doing minimization
float *ZZ[4], z1[4], z2[4], z3[4];
float *vv[4], v1[4], v2[4], v3[4];
float ww[4];

int step_talk = 0;
void initialize_system();

void read_file();
void write_file();

int solve_linear_CG(double *sa, int *ija, double *x, double *b, unsigned int n, double tol);
int solve_linear_BiCG(double *sa, int *ija, double *x, double *b, unsigned int n, double tol);

/******************************************************************************
Main program.
******************************************************************************/

SkipList* initialize_skiplist_queue(int (*compare_proc)(char *, char *), int nelements)
{
  int max_levels = (int) (log ((double) nelements) / log(2.0));
  if (max_levels < 10)
    max_levels = 10;
	return new SkipList(compare_proc, max_levels);
}

void finalize_skiplist_queue(SkipList **the_queue)
{	
	while ((*the_queue)->count()>0) {
		char *this_c = (*the_queue)->first();
		(*the_queue)->remove(this_c);
	}
	delete(*the_queue);
	(*the_queue) = NULL;
}

/******************************************************************************
Read in a polyhedron from a file.
******************************************************************************/

Polyhedron::Polyhedron(FILE *file) {
	int i,j;
	int elem_count;
	char *elem_name;

	vlist = NULL;
	tlist = NULL;
	elist = NULL;
	clist = NULL;
	blist = NULL;
	singularities = NULL;
	ls = NULL;
	singularelements = NULL;
	regularelements = NULL;
	region_verts = NULL;
	region_boundary = NULL;
	vertex_queue = NULL;
	vert_other = NULL;
	face_other = NULL;
	for(i = 0; i < 3; i++) move_trace[i] = NULL;


	/*** Read in the original PLY object ***/
	in_ply = read_ply(file);

	for(i = 0; i < in_ply->num_elem_types; i++) {

	/* prepare to read the i'th list of elements */
	elem_name = setup_element_read_ply(in_ply, i, &elem_count);

	if(equal_strings ("vertex", elem_name)) {
		/* create a vertex list to hold all the vertices */
		nverts = max_verts = elem_count;
		vlist = new Vertex *[nverts];

		for(j = 0; j < nverts; j++) vlist[i] = NULL;

		/* set up for getting vertex elements */
		setup_property_ply (in_ply, &vert_props[0]);
		setup_property_ply (in_ply, &vert_props[1]);
		setup_property_ply (in_ply, &vert_props[2]);
		vert_other = get_other_properties_ply (in_ply, offsetof(Vertex_io,other_props));

		/* grab all the vertex elements */
		for(j = 0; j < nverts; j++) {
			Vertex_io vert;
			get_element_ply(in_ply, (void *) &vert);

			/* copy info from the "vert" structure */
			vlist[j] = new Vertex(vert.x, vert.y, vert.z);
			vlist[j]->other_props = vert.other_props;

			vlist[j]->ten_fld.set(0, 0, 0, 0);
			vlist[j]->ten_fld_painting.set(0, 0, 0, 0);
			vlist[j]->old_ten_fld.set(0, 0, 0, 0);
			vlist[j]->undo_ten_fld.set(0, 0, 0, 0);
			vlist[j]->redo_ten_fld.set(0, 0, 0, 0);
			
			vlist[j]->vec_fld.set(0, 0);
			vlist[j]->vec_fld_painting.set(0, 0);
			vlist[j]->old_vec_fld.set(0, 0);
			vlist[j]->undo_vec_fld.set(0, 0);
			vlist[j]->redo_vec_fld.set(0, 0);
		}
	}
	else if (equal_strings ("face", elem_name)) {
		/* create a list to hold all the face elements */
		ntris = max_tris = elem_count;
		tlist = new Triangle *[ntris];

		for(j = 0; j < ntris; j++) tlist[j] = NULL;

		/* set up for getting face elements */
		setup_property_ply(in_ply, &face_props[0]);
		face_other = get_other_properties_ply (in_ply, offsetof(Face_io,other_props));

		/* grab all the face elements */
		for(j = 0; j < elem_count; j++) {
			Face_io face;

			get_element_ply(in_ply, (void *) &face);

			if(face.nverts != 3) {
				fprintf(stderr, "Face has %d vertices (should be three).\n", face.nverts);
				exit(-1);
			}

			/* copy info from the "face" structure */
			tlist[j] = new Triangle;
			tlist[j]->nverts = 3;
			tlist[j]->verts[0] = (Vertex *) face.verts[0];
			tlist[j]->verts[1] = (Vertex *) face.verts[1];
			tlist[j]->verts[2] = (Vertex *) face.verts[2];
			tlist[j]->other_props = face.other_props;

			free(face.verts);
		}
	}
	else
		get_other_element_ply(in_ply);
	}

	/* close the file */
	close_ply(in_ply);
	free_ply(in_ply);

	/* fix up vertex pointers in triangles */
	for(i = 0; i < ntris; i++) {
		tlist[i]->verts[0] = vlist[(int) tlist[i]->verts[0]];
		tlist[i]->verts[1] = vlist[(int) tlist[i]->verts[1]];
		tlist[i]->verts[2] = vlist[(int) tlist[i]->verts[2]];
	}

	/* get rid of triangles that use the same vertex more than once */
	for(i = ntris - 1; i >= 0; i--) {
		Triangle *tri = tlist[i];
		Vertex *v0 = tri->verts[0];
		Vertex *v1 = tri->verts[1];
		Vertex *v2 = tri->verts[2];

		if(v0 == v1 || v1 == v2 || v2 == v0) {
			free (tlist[i]);
			ntris--;
			tlist[i] = tlist[ntris];
		}
	}
}


/******************************************************************************
Write out a polyhedron to a file.
******************************************************************************/

void Polyhedron::write_file(FILE *file)
{
  int i;
  PlyFile *ply;
  char **elist;
  int num_elem_types;

  /*** Write out the transformed PLY object ***/

  elist = get_element_list_ply (in_ply, &num_elem_types);
  ply = write_ply (file, num_elem_types, elist, in_ply->file_type);

  /* describe what properties go into the vertex elements */

  describe_element_ply (ply, "vertex", nverts);
  describe_property_ply (ply, &vert_props[0]);
  describe_property_ply (ply, &vert_props[1]);
  describe_property_ply (ply, &vert_props[2]);
  describe_property_ply (ply, &vert_props[6]);
  describe_property_ply (ply, &vert_props[7]);
  describe_property_ply (ply, &vert_props[8]);
//  describe_other_properties_ply (ply, vert_other, offsetof(Vertex_io,other_props));

  describe_element_ply (ply, "face", ntris);
  describe_property_ply (ply, &face_props[0]);


//  describe_other_properties_ply (ply, face_other,
//                                offsetof(Face_io,other_props));

//  describe_other_elements_ply (ply, in_ply->other_elems);

  copy_comments_ply (ply, in_ply);
	char mm[1024];
	sprintf(mm, "modified by patchvizply");
	  append_comment_ply (ply, mm);
  copy_obj_info_ply (ply, in_ply);

  header_complete_ply (ply);

  /* set up and write the vertex elements */
  put_element_setup_ply (ply, "vertex");
  for (i = 0; i < nverts; i++) {
    Vertex_io vert;

    /* copy info to the "vert" structure */
    vert.x = vlist[i]->x;
    vert.y = vlist[i]->y;
    vert.z = vlist[i]->z;
		vert.vx = vlist[i]->vec_fld.entry[0];
		vert.vy = vlist[i]->vec_fld.entry[1];
		vert.vz = vlist[i]->vec_fld.entry[2];
    vert.other_props = vlist[i]->other_props;

    put_element_ply (ply, (void *) &vert);
  }

  /* index all the vertices */
  for (i = 0; i < nverts; i++)
    vlist[i]->index = i;

  /* set up and write the face elements */
  put_element_setup_ply (ply, "face");

  Face_io face;
  face.verts = new int[3];
  
  for (i = 0; i < ntris; i++) {

    /* copy info to the "face" structure */
    face.nverts = 3;
    face.verts[0] = tlist[i]->verts[0]->index;
    face.verts[1] = tlist[i]->verts[1]->index;
    face.verts[2] = tlist[i]->verts[2]->index;
    face.other_props = tlist[i]->other_props;

    put_element_ply (ply, (void *) &face);
  }

  put_other_elements_ply (ply);

  close_ply (ply);
  free_ply (ply);
}

void initialize_system()
{
	ZZ[1] = z1;
	ZZ[2] = z2;
	ZZ[3] = z3;
	vv[1] = v1;
	vv[2] = v2;
	vv[3] = v3;
}


void Polyhedron::initialize(){
	int i, j;
	icVector3 v1, v2;

	create_pointers();

	calc_edge_length();

	for (i=0; i<nedges; i++)
		elist[i]->clicked = 0;

	ncorners = max_corners = ntris*3;
	clist = (Corner **)malloc(sizeof(Corner *)*ncorners);
	for (i=0; i<ncorners; i++) {
		clist[i] = (Corner *)malloc(sizeof(Corner));
		clist[i]->index = i;
		clist[i]->v = tlist[i/3]->verts[i % 3];
		clist[i]->t = tlist[i/3];
		clist[i]->e = tlist[i/3]->edges[(i + 1) % 3]; 
	}
	for (i=0; i<ncorners; i++) {
		switch (i % 3) {
		case 0:
			clist[i]->p = clist[i+2];
			clist[i]->n = clist[i+1];
			break;

		case 1:
			clist[i]->p = clist[i-1];
			clist[i]->n = clist[i+1];
			break;

		case 2:
			clist[i]->p = clist[i-1];
			clist[i]->n = clist[i-2];
			break;
		}
	}
	for (i=0; i<ntris; i++) {
		for (j=0; j<3; j++) {
			tlist[i]->corners[j] = clist[3*i+j];
		}
	}
		 
	singularelements = new SingularElementList(20);
	regularelements = new RegularElementList(20);
	singularities = new SingularityList(20);
	ls = new LineSegmentList(20);
	region_boundary = new CornerList(20);
	region_verts = new VertexList(20);
	for (i=0; i<3; i++)
		move_trace[i] = new LineSegmentList(20);
}

void Polyhedron::finalize(){
	for(int i = 0; i < 3; i++) {
		if(move_trace[i]) move_trace[i]->finalize();
	}

	if(region_verts) region_verts->finalize();
	if(region_boundary) region_boundary->finalize();
	if(singularelements) singularelements->finalize();
	if(regularelements) regularelements->finalize();
	if(singularities) singularities->finalize();

	for(int i = 0; i < ncorners; i++) {
		if(clist[i]) delete(clist[i]);
	}

	for(int i = 0; i < ntris; i++) {
		if(tlist[i]) delete(tlist[i]);
	}

	for(int i = 0; i < nedges; i++) {
		if(elist[i]) {
			if(elist[i]->tris) delete(elist[i]->tris);
			delete(elist[i]);
		}
	}

	for(int i = 0; i < nverts; i++) {
		if(vlist[i]) {
			if(vlist[i]->tris) delete(vlist[i]->tris);
			if(vlist[i]->adjacent_edges) vlist[i]->adjacent_edges->finalize();
			//if(vlist[i]->other_props) delete(vlist[i]->other_props);
			delete(vlist[i]);
		}
	}

	if(clist) delete[] clist;
	if(tlist) delete[] tlist;
	if(elist) delete[] elist;
	if(vlist) delete[] vlist;

	/*
	if(!vert_other)
		delete(vert_other);
	if (!face_other)
		delete(face_other);
		*/

	if(vert_other) {
		freePlyOtherProp(vert_other);
	}

	if(face_other) {
		freePlyOtherProp(face_other);
	}

	if(ls) {
		ls->finalize();
	}
}

/******************************************************************************
Find out if there is another face that shares an edge with a given face.

Entry:
  f1    - face that we're looking to share with
  v1,v2 - two vertices of f1 that define edge

Exit:
  return the matching face, or NULL if there is no such face
******************************************************************************/

Triangle *Polyhedron::find_common_edge(Triangle *f1, Vertex *v1, Vertex *v2)
{
  int i,j;
  Triangle *f2;
  Triangle *adjacent = NULL;

  /* look through all faces of the first vertex */

  for (i = 0; i < v1->ntris; i++) {
    f2 = v1->tris[i];
    if (f2 == f1)
      continue;
    /* examine the vertices of the face for a match with the second vertex */
    for (j = 0; j < f2->nverts; j++) {

      /* look for a match */
      if (f2->verts[j] == v2) {

#if 0
	/* watch out for triple edges */

        if (adjacent != NULL) {

	  fprintf (stderr, "model has triple edges\n");

	  fprintf (stderr, "face 1: ");
	  for (k = 0; k < f1->nverts; k++)
	    fprintf (stderr, "%d ", f1->iverts[k]);
	  fprintf (stderr, "\nface 2: ");
	  for (k = 0; k < f2->nverts; k++)
	    fprintf (stderr, "%d ", f2->iverts[k]);
	  fprintf (stderr, "\nface 3: ");
	  for (k = 0; k < adjacent->nverts; k++)
	    fprintf (stderr, "%d ", adjacent->iverts[k]);
	  fprintf (stderr, "\n");

	}

	/* if we've got a match, remember this face */
        adjacent = f2;
#endif

#if 1
	/* if we've got a match, return this face */
        return (f2);
#endif

      }
    }
  }

  return (adjacent);
}


/******************************************************************************
Create an edge.

Entry:
  v1,v2 - two vertices of f1 that define edge
******************************************************************************/

void Polyhedron::create_edge(Vertex *v1, Vertex *v2)
{
  int i,j;
  Triangle *f;

  /* make sure there is enough room for a new edge */

  if (nedges >= max_edges) {

    max_edges += 100;
    Edge **list = new Edge *[max_edges];

    /* copy the old list to the new one */
    for (i = 0; i < nedges; i++)
      list[i] = elist[i];

    /* replace list */
    delete (elist);
    elist = list;
  }

  /* create the edge */

  elist[nedges] = new Edge;
  Edge *e = elist[nedges];
  e->index = nedges;
  e->verts[0] = v1;
  e->verts[1] = v2;
  nedges++;

  /* count all triangles that will share the edge, and do this */
  /* by looking through all faces of the first vertex */

  e->ntris = 0;

  for (i = 0; i < v1->ntris; i++) {
    f = v1->tris[i];
    /* examine the vertices of the face for a match with the second vertex */
    for (j = 0; j < 3; j++) {
      /* look for a match */
      if (f->verts[j] == v2) {
        e->ntris++;
        break;
      }
    }
  }

  /* make room for the face pointers (at least two) */
  if (e->ntris < 2)
    e->tris = new Triangle *[2];
  else
    e->tris = new Triangle *[e->ntris];

  /* create pointers from edges to faces and vice-versa */

  e->ntris = 0; /* start this out at zero again for creating ptrs to tris */

  for (i = 0; i < v1->ntris; i++) {

    f = v1->tris[i];

    /* examine the vertices of the face for a match with the second vertex */
    for (j = 0; j < 3; j++)
      if (f->verts[j] == v2) {

        e->tris[e->ntris] = f;
        e->ntris++;

        if (f->verts[(j+1)%3] == v1)
          f->edges[j] = e;
        else if (f->verts[(j+2)%3] == v1)
          f->edges[(j+2)%3] = e;
        else {
          fprintf (stderr, "Non-recoverable inconsistancy in create_edge()\n");
          exit (-1);
        }

        break;  /* we'll only find one instance of v2 */
      }

  }
}


/******************************************************************************
Create edges.
******************************************************************************/

void Polyhedron::create_edges()
{
  int i,j;
  Triangle *f;
  Vertex *v1,*v2;
  double count = 0;

  /* count up how many edges we may require */

  for (i = 0; i < ntris; i++) {
    f = tlist[i];
    for (j = 0; j < f->nverts; j++) {
      v1 = f->verts[j];
      v2 = f->verts[(j+1) % f->nverts];
      Triangle *result = find_common_edge (f, v1, v2);
      if (result)
        count += 0.5;
      else
        count += 1;
    }
  }

  /*
  printf ("counted %f edges\n", count);
  */

  /* create space for edge list */

  max_edges = (int) (count + 10);  /* leave some room for expansion */
  elist = new Edge *[max_edges];
  nedges = 0;

  /* zero out all the pointers from faces to edges */

  for (i = 0; i < ntris; i++)
    for (j = 0; j < 3; j++)
      tlist[i]->edges[j] = NULL;

  /* create all the edges by examining all the triangles */

  for (i = 0; i < ntris; i++) {
    f = tlist[i];
    for (j = 0; j < 3; j++) {
      /* skip over edges that we've already created */
      if (f->edges[j])
        continue;
      v1 = f->verts[j];
      v2 = f->verts[(j+1) % f->nverts];
      create_edge (v1, v2);
    }
  }
}


/******************************************************************************
Create pointers from vertices to faces.
******************************************************************************/

void Polyhedron::vertex_to_tri_ptrs()
{
  int i,j;
  Triangle *f;
  Vertex *v;

  /* zero the count of number of pointers to faces */

  for (i = 0; i < nverts; i++)
    vlist[i]->max_tris = 0;

  /* first just count all the face pointers needed for each vertex */

  for (i = 0; i < ntris; i++) {
    f = tlist[i];
    for (j = 0; j < f->nverts; j++)
      f->verts[j]->max_tris++;
  }

  /* allocate memory for face pointers of vertices */

  for (i = 0; i < nverts; i++) {
    vlist[i]->tris = (Triangle **)
		      malloc (sizeof (Triangle *) * vlist[i]->max_tris);
    vlist[i]->ntris = 0;
  }

  /* now actually create the face pointers */

  for (i = 0; i < ntris; i++) {
    f = tlist[i];
    for (j = 0; j < f->nverts; j++) {
      v = f->verts[j];
      v->tris[v->ntris] = f;
      v->ntris++;
    }
  }
}


/******************************************************************************
Find the other triangle that is incident on an edge, or NULL if there is
no other.
******************************************************************************/

Triangle *Polyhedron::other_triangle(Edge *edge, Triangle *tri)
{
  /* search for any other triangle */

  for (int i = 0; i < edge->ntris; i++)
    if (edge->tris[i] != tri)
      return (edge->tris[i]);

  /* there is no such other triangle if we get here */
  return (NULL);
}


/******************************************************************************
Order the pointers to faces that are around a given vertex.

Entry:
  v - vertex whose face list is to be ordered
******************************************************************************/

void Polyhedron::order_vertex_to_tri_ptrs(Vertex *v)
{
  int i,j;
  Triangle *f;
  Triangle *fnext;
  int nf;
  int vindex;
  int boundary;
  int count;

  nf = v->ntris;
  f = v->tris[0];

  /* go backwards (clockwise) around faces that surround a vertex */
  /* to find out if we reach a boundary */

  boundary = 0;

  for (i = 1; i <= nf; i++) {

    /* find reference to v in f */
    vindex = -1;
    for (j = 0; j < f->nverts; j++)
      if (f->verts[j] == v) {
	vindex = j;
	break;
      }

    /* error check */
    if (vindex == -1) {
      fprintf (stderr, "can't find vertex #1\n");
      exit (-1);
    }

    /* corresponding face is the previous one around v */
    fnext = other_triangle (f->edges[vindex], f);

    /* see if we've reached a boundary, and if so then place the */
    /* current face in the first position of the vertice's face list */

    if (fnext == NULL) {
      /* find reference to f in v */
      for (j = 0; j < v->ntris; j++)
        if (v->tris[j] == f) {
	  v->tris[j] = v->tris[0];
	  v->tris[0] = f;
	  break;
	}
      boundary = 1;
      break;
    }

    f = fnext;
  }

  /* now walk around the faces in the forward direction and place */
  /* them in order */

  f = v->tris[0];
  count = 0;

  for (i = 1; i < nf; i++) {

    /* find reference to vertex in f */
    vindex = -1;
    for (j = 0; j < f->nverts; j++)
      if (f->verts[(j+1) % f->nverts] == v) {
	vindex = j;
	break;
      }

    /* error check */
    if (vindex == -1) {
      fprintf (stderr, "can't find vertex #2\n");
      exit (-1);
    }

    /* corresponding face is next one around v */
    fnext = other_triangle (f->edges[vindex], f);

    /* break out of loop if we've reached a boundary */
    count = i;
    if (fnext == NULL) {
      break;
    }

    /* swap the next face into its proper place in the face list */
    for (j = 0; j < v->ntris; j++)
      if (v->tris[j] == fnext) {
	v->tris[j] = v->tris[i];
	v->tris[i] = fnext;
	break;
      }

    f = fnext;
  }
}


/******************************************************************************
Find the index to a given vertex in the list of vertices of a given face.

Entry:
  f - face whose vertex list is to be searched
  v - vertex to return reference to

Exit:
  returns index in face's list, or -1 if vertex not found
******************************************************************************/

int Polyhedron::face_to_vertex_ref(Triangle *f, Vertex *v)
{
  int j;
  int vindex = -1;

  for (j = 0; j < f->nverts; j++)
    if (f->verts[j] == v) {
      vindex = j;
      break;
    }

  return (vindex);
}

/******************************************************************************
Create various face and vertex pointers.
******************************************************************************/

void Polyhedron::create_pointers() {
	/* index the vertices and triangles */

	for(int i = 0; i < nverts; i++) {
		vlist[i]->index = i;
	}

	for(int i = 0; i < ntris; i++) {
		tlist[i]->index = i;
	}


	/* create pointers from vertices to triangles */
	vertex_to_tri_ptrs();


	/* make edges */
	create_edges();


	/* order the pointers from vertices to faces */
	for(int i = 0; i < nverts; i++){
		//		if (i %1000 == 0)
		//			fprintf(stderr, "ordering %d of %d vertices\n", i, nverts);
		order_vertex_to_tri_ptrs(vlist[i]);
	}

	/* index the edges */
	for(int i = 0; i < nedges; i++){
		//		if (i %1000 == 0)
		//			fprintf(stderr, "indexing %d of %d edges\n", i, nedges);
		elist[i]->index = i;
	}

	for(int i = 0; i < nverts; i++) {
		find_all_edges_around_vertex(i);
	}
}

void Polyhedron::calc_bounding_sphere()
{
  unsigned int i;
  icVector3 min, max;

  for (i=0; i<nverts; i++) {
    if (i==0)  {
			min.set(vlist[i]->x, vlist[i]->y, vlist[i]->z);
			max.set(vlist[i]->x, vlist[i]->y, vlist[i]->z);
    }
    else {
			if (vlist[i]->x < min.entry[0])
			  min.entry[0] = vlist[i]->x;
			if (vlist[i]->x > max.entry[0])
			  max.entry[0] = vlist[i]->x;
			if (vlist[i]->y < min.entry[1])
			  min.entry[1] = vlist[i]->y;
			if (vlist[i]->y > max.entry[1])
			  max.entry[1] = vlist[i]->y;
			if (vlist[i]->z < min.entry[2])
			  min.entry[2] = vlist[i]->z;
			if (vlist[i]->z > max.entry[2])
			  max.entry[2] = vlist[i]->z;
		}
  }
  center = (min + max) * 0.5;
  radius = length(center - min);
}

void Polyhedron::calc_edge_length()
{
	int i;
	icVector3 v1, v2;

	for (i=0; i<nedges; i++) {
		v1.set(elist[i]->verts[0]->x, elist[i]->verts[0]->y, elist[i]->verts[0]->z);
		v2.set(elist[i]->verts[1]->x, elist[i]->verts[1]->y, elist[i]->verts[1]->z);
		elist[i]->length = length(v1-v2);
	}
}

void Polyhedron::calc_face_normals_and_area()
{
	unsigned int i, j, k, l;
	icVector3 v0, v1, v2;
  Triangle *temp_t;
	double length[3];
	Corner *temp_c;

	area = 0.0;
	for (i=0; i<ntris; i++){
		temp_c = clist[i*3+0];
		length[0] = temp_c->e->length;
		temp_c = clist[i*3+1];
		length[1] = temp_c->e->length;
		temp_c = clist[i*3+2];
		length[2] = temp_c->e->length;

		double temp_s = (length[0] + length[1] + length[2])/2.0;
		tlist[i]->area = sqrt(temp_s*(temp_s-length[0])*(temp_s-length[1])*(temp_s-length[2]));

		area += tlist[i]->area;
		temp_t = tlist[i];
		v1.set(vlist[tlist[i]->verts[0]->index]->x, vlist[tlist[i]->verts[0]->index]->y, vlist[tlist[i]->verts[0]->index]->z);
		v2.set(vlist[tlist[i]->verts[1]->index]->x, vlist[tlist[i]->verts[1]->index]->y, vlist[tlist[i]->verts[1]->index]->z);
		v0.set(vlist[tlist[i]->verts[2]->index]->x, vlist[tlist[i]->verts[2]->index]->y, vlist[tlist[i]->verts[2]->index]->z);
		tlist[i]->normal = cross(v0-v1, v2-v1);
		normalize(tlist[i]->normal);
		tlist[i]->center = (v0+v1+v2)*(1/3.0);
	}

//	outFile.open("data.txt");
	double haha;
	for (j=0; j<=0; j++) {
		for (k=-0; k<=0; k++) { 
			for (l=0; l<=0; l++) {
				haha = 0.0;
				icVector3 test((double)j, (double)k, (double)l);
				test *= radius*100;
				test += center;
				for (i=0; i<ntris; i++){
					icVector3 cent(vlist[tlist[i]->verts[0]->index]->x, vlist[tlist[i]->verts[0]->index]->y, vlist[tlist[i]->verts[0]->index]->z);
					haha += dot(test-cent, tlist[i]->normal)*tlist[i]->area;
				}
				haha /= area;
				if ((j==0) && (k==0) && (l==0)){
					if (haha<0) {
						orientation = 0;
					}
					else {
						orientation = 1;
					}
				}
/*
				if (haha<0.0)
					outFile << "x = " << j << ", y = " << k << ", z = " << l << " and negative value = " << haha << "\n";
				else
					outFile << "x = " << j << ", y = " << k << ", z = " << l << " and positive value = " << haha << "\n";
*/
			}
		}
	}
//	outFile.close();
	if (orientation == 1){
		for (i=0; i<ntris; i++)
			tlist[i]->normal *= -1.0;
	}
}

void sort(unsigned int *A, unsigned int *B, unsigned int *C, unsigned int sid, unsigned int eid){
  unsigned int i;
	unsigned int *tempA, *tempB, *tempC;
	unsigned int current1, current2, current0;

  if (sid>=eid)
		return;
	sort(A, B, C, sid, (sid+eid)/2);
	sort(A, B, C, (sid+eid)/2+1, eid);
	tempA = (unsigned int *)malloc(sizeof(unsigned int)*(eid-sid+1));
	tempB = (unsigned int *)malloc(sizeof(unsigned int)*(eid-sid+1));
	tempC = (unsigned int *)malloc(sizeof(unsigned int)*(eid-sid+1));
	for (i=0; i<eid-sid+1; i++){
		tempA[i] = A[i+sid];
		tempB[i] = B[i+sid];
		tempC[i] = C[i+sid];
	}
	current1 = sid;
	current2 = (sid+eid)/2+1;
	current0 = sid;
	while ((current1<=(sid+eid)/2) && (current2<=eid)){
		if (tempA[current1-sid] < tempA[current2-sid]) {
			A[current0] = tempA[current1-sid];
			B[current0] = tempB[current1-sid];
			C[current0] = tempC[current1-sid];
			current1++;		
		}
		else if (tempA[current1-sid] > tempA[current2-sid]){
			A[current0] = tempA[current2-sid];
			B[current0] = tempB[current2-sid];
			C[current0] = tempC[current2-sid];
			current2++;		
		}
		else {
			if (tempB[current1-sid] < tempB[current2-sid]) {
				A[current0] = tempA[current1-sid];
				B[current0] = tempB[current1-sid];
				C[current0] = tempC[current1-sid];
				current1++;		
			} else {
				A[current0] = tempA[current2-sid];
				B[current0] = tempB[current2-sid];
				C[current0] = tempC[current2-sid];
				current2++;		
			}
		}
		current0++;
	}
	if (current1<=(sid+eid)/2){
		for (i=current1; i<=(sid+eid)/2; i++){
			A[current0] = tempA[i-sid];
			B[current0] = tempB[i-sid];
			C[current0] = tempC[i-sid];
			current0++;
		}
	}
	if (current2<=eid){
		for (i=current2; i<=eid; i++){
			A[current0] = tempA[i-sid];
			B[current0] = tempB[i-sid];
			C[current0] = tempC[i-sid];
			current0++;
		}
	}

	free(tempA);
	free(tempB);
	free(tempC);
}

void Polyhedron::find_opposite_corners()
{
	unsigned int *corners, *vlist1, *vlist2, prev_v1, prev_v2;
	int *oppo;
	int i;

	oppo = (int *)malloc(sizeof(int)*ntris*3);
	corners = (unsigned int *)malloc(sizeof(unsigned int)*ntris*3);
	vlist1 = (unsigned int *)malloc(sizeof(unsigned int)*ntris*3);
	vlist2 = (unsigned int *)malloc(sizeof(unsigned int)*ntris*3);

	for (i=0; i<ntris; i++) {
		Triangle *temp_t = tlist[i];
		corners[3*i] = 3*i;
		corners[3*i+1] = 3*i+1;
		corners[3*i+2] = 3*i+2;
		if (temp_t->verts[1]->index > temp_t->verts[2]->index){
			vlist1[3*i] = temp_t->verts[1]->index;
			vlist2[3*i] = temp_t->verts[2]->index;
		} else {
			vlist1[3*i] = temp_t->verts[2]->index;
			vlist2[3*i] = temp_t->verts[1]->index;
		}
		if (temp_t->verts[2]->index > temp_t->verts[0]->index){
			vlist1[3*i+1] = temp_t->verts[2]->index;
			vlist2[3*i+1] = temp_t->verts[0]->index;
		} else {
			vlist1[3*i+1] = temp_t->verts[0]->index;
			vlist2[3*i+1] = temp_t->verts[2]->index;
		}
		if (temp_t->verts[0]->index > temp_t->verts[1]->index){
			vlist1[3*i+2] = temp_t->verts[0]->index;
			vlist2[3*i+2] = temp_t->verts[1]->index;
		} else {
			vlist1[3*i+2] = temp_t->verts[1]->index;
			vlist2[3*i+2] = temp_t->verts[0]->index;
		}
	}
	sort(vlist1, vlist2, corners, 0, ntris*3-1);
	prev_v1 = 1000;  // something not 0 would be fine since the first pair in the sequence	 is (0, 0)
	prev_v2 = 1000;
	for (i=0; i<ntris*3; i++)
		oppo[i] = -1;
	for (i=0; i<ntris*3; i++){
		if ((prev_v1 == vlist1[i]) && (prev_v2 == vlist2[i])){
			oppo[corners[i]] = corners[i-1];
			oppo[corners[i-1]] = corners[i];
		} 
		prev_v1 = vlist1[i];
		prev_v2 = vlist2[i];
	}
	free(corners);
	free(vlist1);
	free(vlist2);
	for (i=0; i<ncorners; i++) {
		if (oppo[i] >= 0)
			clist[i]->o = clist[oppo[i]];
		else
			clist[i]->o = NULL;
	}
	free(oppo);
}

// assume already triangulated
void Polyhedron::calc_corner_angles(){
	unsigned int i;
  Triangle *temp_t;
	Corner *temp_c;
	double length[3];

	for (i=0; i<ntris; i++){		
		temp_t = tlist[i];
		temp_c = clist[i*3+0];
		length[0] = temp_c->e->length;
		temp_c = clist[i*3+1];
		length[1] = temp_c->e->length;
		temp_c = clist[i*3+2];
		length[2] = temp_c->e->length;

		double temp_s = (length[0]+length[1]+length[2])/2;
		temp_t->angle[0] = 2*acos(sqrt(temp_s*(temp_s-length[0])/(length[1]*length[2])));
		temp_t->angle[1] = 2*acos(sqrt(temp_s*(temp_s-length[1])/(length[2]*length[0])));
		temp_t->angle[2] = 2*acos(sqrt(temp_s*(temp_s-length[2])/(length[0]*length[1])));
	}
}

void n_add(double *a, double *b, double *c, unsigned int n)
{
	unsigned long i;
	double temp;

	temp = 0.0;
	for (i=0; i<n; i++)
		c[i] = a[i]+b[i];
}

void n_minus(double *a, double *b, double *c, unsigned int n)
{
	unsigned long i;
	double temp;

	temp = 0.0;
	for (i=0; i<n; i++)
		c[i] = a[i]-b[i];
}

double n_dot(double *a, double *b, unsigned int n)
{
	unsigned long i;
	double temp;

	temp = 0.0;
	for (i=0; i<n; i++)
		temp += a[i]*b[i];
	return temp;
}

int multi_A_b(double *sa, int *ija, double *b, double *x, unsigned int n)
{
	unsigned long i, k;

	if (ija[0] != n+1)
		return 0;  // dimension mismatch
	for (i=0; i<n; i++) {
		b[i] = sa[i]*x[i];
		for (k=ija[i]; k<ija[i+1]; k++)
			b[i] += sa[k]*x[ija[k]];
	}
	return 1;
}

int multi_AT_b(double *sa, int *ija, double *b, double *x, unsigned int n)
{
	unsigned long i, j, k;

	if (ija[0] != n+1)
		return 0;
	for (i=0; i<n; i++)
		b[i] = sa[i]*x[i];
	for (i=0; i<n; i++) {
		for (k=ija[i]; k<ija[i+1]; k++) {
			j = ija[k];
			b[j] += sa[k]*x[i];
		}
	}
	return 1;
}

double get_error(double *sa, int *ija, double *x, double *b, unsigned int n)
{
	double error;
	double *t1, *t2;

	t1 = new double[n];
	t2 = new double[n];

	multi_A_b(sa, ija, t1, x, n);
	n_minus(t1, b, t2, n);
	error = sqrt(n_dot(t2, t2, n)/n_dot(b, b, n));
	delete(t1);
	delete(t2);
	return error;
}

int solve_linear_CG(double *sa, int *ija, double *x, double *b, unsigned int n, double tol)
{
	double *r1, *p1, alpha, beta, error, old_r1, current_r1;
	double *t1;
	long i;

	if (ija[0] != n+1)
		return 0;  // dimension mismatch
	r1 = new double[n];
	p1 = new double[n];
	t1 = new double[n];
	for (i=0; i<n; i++) {
		r1[i] = p1[i] = b[i];
		x[i] = 0.0;
	}
	error = get_error(sa, ija, x, b, n); 
	current_r1 = n_dot(r1, r1, n);
	old_r1 = current_r1;
	while (error > tol) {
		multi_A_b(sa, ija, t1, p1, n);
		alpha = old_r1/n_dot(p1, t1, n);
		for (i=0; i<n; i++) {
			x[i] += alpha*p1[i];
			r1[i] -= alpha*t1[i];
		}
		current_r1 = n_dot(r1, r1, n);
		beta = current_r1/old_r1;
		for (i=0; i<n; i++) {
			p1[i] = r1[i] + beta*p1[i];
		}
		old_r1 = current_r1;
		error = get_error(sa, ija, x, b, n); 
	}
	delete(r1);
	delete(p1);
	delete(t1);
	return 1;
}

int solve_linear_BiCG(double *sa, int *ija, double *x, double *b, unsigned int n, double tol)
{
	double *r1, *r2, *p1, *p2, alpha, beta, error, old_r1_r2, current_r1_r2;
	double *t1, *t2;
	long i;

	if (ija[0] != n+1)
		return 0;  // dimension mismatch
	r1 = new double[n];
	r2 = new double[n];
	p1 = new double[n];
	p2 = new double[n];
	t1 = new double[n];
	t2 = new double[n];
	for (i=0; i<n; i++) {
		r1[i] = r2[i] = p1[i] = p2[i] = b[i];
		x[i] = 0.0;
	}
	error = get_error(sa, ija, x, b, n); 
	current_r1_r2 = n_dot(r1, r2, n);
	old_r1_r2 = current_r1_r2;
	while (error > tol) {
		multi_A_b(sa, ija, t1, p1, n);
		multi_AT_b(sa, ija, t2, p2, n);
		alpha = old_r1_r2/n_dot(p2, t1, n);
		for (i=0; i<n; i++) {
			x[i] += alpha*p1[i];
			r1[i] -= alpha*t1[i];
			r2[i] -= alpha*t2[i];
		}
		current_r1_r2 = n_dot(r1, r2, n);
		beta = current_r1_r2/old_r1_r2;
		for (i=0; i<n; i++) {
			p1[i] = r1[i] + beta*p1[i];
			p2[i] = r2[i] + beta*p2[i];
		}
		old_r1_r2 = current_r1_r2;
		error = get_error(sa, ija, x, b, n); 
	}
	delete(r1);
	delete(r2);
	delete(p1);
	delete(p2);
	delete(t1);
	delete(t2);
	return 1;
}

Polyhedron::Polyhedron()
: regionID(0)
{
	nverts = nedges = ntris = 0;
	max_verts = max_tris = 50;

	vlist = new Vertex *[max_verts];
	tlist = new Triangle *[max_tris];
	elist = NULL;
	clist = NULL;
	blist = NULL;
	singularities = NULL;
	ls = NULL;
	singularelements = NULL;
	regularelements = NULL;
	region_verts = NULL;
	region_boundary = NULL;
	vertex_queue = NULL;
	vert_other = NULL;
	face_other = NULL;
	for(int i = 0; i < 3; i++) move_trace[i] = NULL;

	//ignore_at_singularity = false;
}

double Polyhedron::calc_triangle_area(icVector2 q0, icVector2 q1, icVector2 q2)
{
	return fabs((q1.entry[0]-q0.entry[0])*(q2.entry[1]-q0.entry[1]) - (q2.entry[0]-q0.entry[0])*(q1.entry[1]-q0.entry[1]))/2.0;
}


void Polyhedron::find_all_edges_around_vertex(int vertid)
{
	Vertex *temp_v = vlist[vertid];
	int i, j;

	temp_v->adjacent_edges = new EdgeList(10);

	for (i=0; i<temp_v->ntris; i++) {
		Triangle *temp_t = temp_v->tris[i];
		for (j=0; j<3; j++) {
			Edge *temp_e = temp_t->edges[j];
			if (temp_e->verts[0] == temp_v || temp_e->verts[1] == temp_v)
				temp_e->touched = 0;
		}
	}

	for (i=0; i<temp_v->ntris; i++) {
		Triangle *temp_t = temp_v->tris[i];
		for (j=0; j<3; j++) {
			Edge *temp_e = temp_t->edges[j];
			if (temp_e->verts[0] == temp_v || temp_e->verts[1] == temp_v) {
				temp_e->touched++;
				temp_v->adjacent_edges->insert_unique(temp_e);
			}
		}
	}

	// look to see whether there is an edge that is adjacent to v only once, i.e., v is on the boundary
	for (i=0; i<temp_v->adjacent_edges->num_edges; i++) {
		if (temp_v->adjacent_edges->edges[i]->touched == 1)
			break;
	}

	if (i == temp_v->adjacent_edges->num_edges) { // temp_v is interior vertex

	} else {  // temp_v is a boundary vertex

	}
}

void print_this (char *ptr1)
{
  Vertex *v1 = (Vertex *) ptr1;
	fprintf(stderr, "index=%d\n", v1->index);
}


void Polyhedron::average_normals()
{
	int i, j;

	for (i=0; i<nverts; i++) {
		vlist[i]->normal = icVector3(0.0);
		for (j=0; j<vlist[i]->ntris; j++) 
			vlist[i]->normal += vlist[i]->tris[j]->normal;
		normalize(vlist[i]->normal);
	}
}

void Polyhedron::set_local_frame()
{
	int i, j;
	Triangle *temp_t;
	Vertex *temp_v0, *temp_v1, *temp_v2;

	for (i=0; i<ntris; i++) {
		temp_t = tlist[i];
		temp_v0 = temp_t->verts[0];
		temp_v1 = temp_t->verts[1];
		temp_v2 = temp_t->verts[2];
		temp_t->x_dir = icVector2(temp_v1->x-temp_v0->x, temp_v1->y-temp_v0->y);
		normalize(temp_t->x_dir);
		temp_t->y_dir = cross(temp_t->x_dir);
		temp_t->x1 = dot(temp_t->x_dir, icVector2(temp_v1->x-temp_v0->x, temp_v1->y-temp_v0->y));
		temp_t->x2 = dot(temp_t->x_dir, icVector2(temp_v2->x-temp_v0->x, temp_v2->y-temp_v0->y));
		temp_t->y2 = dot(temp_t->y_dir, icVector2(temp_v2->x-temp_v0->x, temp_v2->y-temp_v0->y));
		temp_t->vert_local[0].set(0.0, 0.0);
		temp_t->vert_local[1].set(temp_t->x1, 0.0);
		temp_t->vert_local[2].set(temp_t->x2, temp_t->y2);
		temp_t->edge_normal[0].set(0.0, temp_t->x1);
		temp_t->edge_normal[1].set(-temp_t->y2, temp_t->x2-temp_t->x1);
		temp_t->edge_normal[2].set(temp_t->y2, -temp_t->x2);
		for (j=0; j<3; j++)
			normalize(temp_t->edge_normal[j]);
	}
}

void Polyhedron::compute_vector_separatrices(int saddle_id)
{
	int i, j;
	icVector3 temp_vec;
	icMatrix2x2 temp_m;
	icVector2 solution;
	LineSegmentList *ls, *temp_ls;
	LineSegment *lls;

	// compute separatrices
	for (i=0; i<singularities->num_singularities; i++) {
		if (saddle_id >= 0 && i != saddle_id)
			continue;
		Singularity *ss = singularities->singularities[i];
		if (ss->singularity_type != 5)
			continue;
		ss->separatrices_major = new SeparatricesList(2);
		ss->separatrices_minor = new SeparatricesList(2);


		ls = new LineSegmentList(40);
		ls->direction = 0;
		lls = new LineSegment();
		lls->start_coor = ss->pos;
		lls->end_coor = lls->start_coor + saddle_out * ss->eig_dir[0];
		ls->len = lls->len = length(lls->end_coor - lls->start_coor);
		lls->tri = ss->tri;
		lls->calc_display_attributes();
		ls->append(lls);
		temp_ls = new LineSegmentList(2000);
		trace_one_trajectory_in_domain(500, lls->end_coor, 0, lls->tri, &temp_ls);
		for (j=0; j<temp_ls->num_linesegments; j++){
			ls->append(temp_ls->linesegments[j]);
			temp_ls->linesegments[j]->len = length(temp_ls->linesegments[j]->end_coor - temp_ls->linesegments[j]->start_coor);
			ls->len += temp_ls->linesegments[j]->len;
		}
		delete(temp_ls);
		ss->separatrices_major->append(ls);

		ls = new LineSegmentList(40);
		ls->direction = 0;
		lls = new LineSegment();
		lls->start_coor = ss->pos;
		lls->end_coor = lls->start_coor - saddle_out * ss->eig_dir[0];
		ls->len = lls->len = length(lls->end_coor - lls->start_coor);
		lls->tri = ss->tri;
		lls->calc_display_attributes();
		ls->append(lls);
		temp_ls = new LineSegmentList(2000);
		trace_one_trajectory_in_domain(500, lls->end_coor, 0, lls->tri, &temp_ls);
		for (j=0; j<temp_ls->num_linesegments; j++){
			ls->append(temp_ls->linesegments[j]);
			temp_ls->linesegments[j]->len = length(temp_ls->linesegments[j]->end_coor - temp_ls->linesegments[j]->start_coor);
			ls->len += temp_ls->linesegments[j]->len;
		}
		delete(temp_ls);
		ss->separatrices_major->append(ls);

		ls = new LineSegmentList(40);
		ls->direction = 1;
		lls = new LineSegment();
		lls->start_coor = ss->pos;
		lls->end_coor = lls->start_coor + saddle_out * ss->eig_dir[1];
		ls->len = lls->len = length(lls->end_coor - lls->start_coor);
		lls->tri = ss->tri;
		lls->calc_display_attributes();
		ls->append(lls);
		temp_ls = new LineSegmentList(2000);
		trace_one_trajectory_in_domain(500, lls->end_coor, 1, lls->tri, &temp_ls);
		for (j=0; j<temp_ls->num_linesegments; j++){
			ls->append(temp_ls->linesegments[j]);
			temp_ls->linesegments[j]->len = length(temp_ls->linesegments[j]->end_coor - temp_ls->linesegments[j]->start_coor);
			ls->len += temp_ls->linesegments[j]->len;
		}
		delete(temp_ls);
		ss->separatrices_minor->append(ls);

		ls = new LineSegmentList(40);
		ls->direction = 1;
		lls = new LineSegment();
		lls->start_coor = ss->pos;
		lls->end_coor = lls->start_coor - saddle_out * ss->eig_dir[1];
		ls->len = lls->len = length(lls->end_coor - lls->start_coor);
		lls->tri = ss->tri;
		lls->calc_display_attributes();
		ls->append(lls);
		temp_ls = new LineSegmentList(2000);
		trace_one_trajectory_in_domain(500, lls->end_coor, 1, lls->tri, &temp_ls);
		for (j=0; j<temp_ls->num_linesegments; j++){
			ls->append(temp_ls->linesegments[j]);
			temp_ls->linesegments[j]->len = length(temp_ls->linesegments[j]->end_coor - temp_ls->linesegments[j]->start_coor);
			ls->len += temp_ls->linesegments[j]->len;
		}
		delete(temp_ls);
		ss->separatrices_minor->append(ls);
	}
}

void Polyhedron::analyze_field(int do_analysis, int saddle_id)
{
	determine_singularities();

	if (vec_ten_mode == 0) {
		handle_transversity();

		if (do_analysis == 1) {
			//return;
			compute_vector_separatrices(saddle_id);
		}
	}
	else {
		if (do_analysis == 1) {
			return;
			compute_tensor_separatrices();
		}
	}
}

void Polyhedron::get_model_from_file()
{
	initialize_system();
	initialize(); // initialize everything
	calc_bounding_sphere();
	find_opposite_corners();	
	calc_corner_angles();
	calc_face_normals_and_area();
	average_normals();
	set_local_frame();
	calc_all_lamdas();
}


// NOT MODIFIED FOR DESIGN PROPAGATION YET
void Polyhedron::update_vecfld(int target_region_id, int do_set_zero, const std::vector<const DEInfo*> &design_elements)
{
	/**
	int i, j;
	double len;
	icVector2 temp, temp_result;

	for (i=0; i<ntris; i++)
		tlist[i]->clicked = 0;
	if (do_set_zero == 1) {
		for (i=0; i<nverts; i++) {
			vlist[i]->vec_fld.set(0.0);
			vlist[i]->vec_fld_painting.set(0.0);
		}
	} else {
		for (i=0; i<nverts; i++) 
			vlist[i]->vec_fld = vlist[i]->vec_fld_painting;
	}

	for (j=0; j<singularelements->num_singularelements; j++) {
		if (singularelements->singularelements[j]->order == 0) {
			for (i=0; i<nverts; i++) {
				temp.set(vlist[i]->x - singularelements->singularelements[j]->x,
								 vlist[i]->y - singularelements->singularelements[j]->y);
				len = length(temp);
				normalize(temp);
				vlist[i]->vec_fld += inverse_dist(50.0*len, 1.0e-1) * (singularelements->singularelements[j]->mat * temp);
			} 
		} else if (singularelements->singularelements[j]->order == 1) {
			for (i=0; i<nverts; i++) {
				temp.set(vlist[i]->x - singularelements->singularelements[j]->x,
								 vlist[i]->y - singularelements->singularelements[j]->y);
				len = length(temp);
				normalize(temp);
				for (int pp=0; pp<2; pp++)
					temp_result.entry[pp] = dot(temp, singularelements->singularelements[j]->mat_second[pp]*temp);
				vlist[i]->vec_fld += inverse_dist(50.0*len, 1.0e-1) * singularelements->singularelements[j]->rot_inv * temp_result;
			}
		}
	}

	for (j=0; j<regularelements->num_regularelements; j++) {
		for (i=0; i<nverts; i++) {
			temp.set(vlist[i]->x - regularelements->regularelements[j]->x,
							 vlist[i]->y - regularelements->regularelements[j]->y);
			len = length(temp);
			normalize(temp);
			vlist[i]->vec_fld += inverse_dist(len*50.0, 1.0e-1) * icVector2(regularelements->regularelements[j]->vx, regularelements->regularelements[j]->vy);
		}
	}

	for (i=0; i<nverts; i++){
		normalize(vlist[i]->vec_fld);
	}
	/**/
}

void Polyhedron::write_vecfld_ctrl_file(const char name[128])
{
	int k;
	FILE *fp=fopen(name, "w");

	fprintf(fp, "%d %d\n", singularelements->num_singularelements, regularelements->num_regularelements);
	for (k=0; k<singularelements->num_singularelements; k++) {
		fprintf(fp, "%d %d %f %f %f %f %f %f %f %f\n", k, 
								singularelements->singularelements[k]->type,
								singularelements->singularelements[k]->x, 
								singularelements->singularelements[k]->y, 
								singularelements->singularelements[k]->max_dir.entry[0], 
								singularelements->singularelements[k]->max_dir.entry[1], 
								singularelements->singularelements[k]->min_dir.entry[0], 
								singularelements->singularelements[k]->min_dir.entry[1], 
								singularelements->singularelements[k]->max_mag,
								singularelements->singularelements[k]->min_mag);
	}

	for (k=0; k<regularelements->num_regularelements; k++) {
		fprintf(fp, "%d %f %f %f %f %f %f\n", k, 
								regularelements->regularelements[k]->x,
								regularelements->regularelements[k]->y,
								regularelements->regularelements[k]->vx,
								regularelements->regularelements[k]->vy);
	}
	fclose(fp);
}

void Polyhedron::write_tenfld_ctrl_file(const char name[128])
{
	int k;
	FILE *fp=fopen(name, "w");

	fprintf(fp, "%d %d\n", singularelements->num_singularelements, regularelements->num_regularelements);
	for (k=0; k<singularelements->num_singularelements; k++) {
		fprintf(fp, "%d %d %d %f %f %f %f %f %f %f %f\n", k, 
								singularelements->singularelements[k]->type,
								singularelements->singularelements[k]->order,
								singularelements->singularelements[k]->x, 
								singularelements->singularelements[k]->y, 
								singularelements->singularelements[k]->max_dir.entry[0], 
								singularelements->singularelements[k]->max_dir.entry[1], 
								singularelements->singularelements[k]->min_dir.entry[0], 
								singularelements->singularelements[k]->min_dir.entry[1], 
								singularelements->singularelements[k]->max_mag,
								singularelements->singularelements[k]->min_mag);
	}

	for (k=0; k<regularelements->num_regularelements; k++) {
		fprintf(fp, "%d %f %f %f %f\n", k, 
								regularelements->regularelements[k]->x,
								regularelements->regularelements[k]->y,
								regularelements->regularelements[k]->vx,
								regularelements->regularelements[k]->vy);
	}
	fclose(fp);
}

void Polyhedron::read_vecfld_ctrl_file(const char name[128])
{
	singularelements->finalize();
	regularelements->finalize();
	singularities->finalize();
	singularelements = new SingularElementList(20);
	regularelements = new RegularElementList(20);
	singularities = new SingularityList(20);

	int k, num_se, num_re, dummy;
	FILE *fp=fopen(name, "r");
	SingularElement *se;
	RegularElement *re;

	fscanf(fp, "%d %d\n", &num_se, &num_re);
	for (k=0; k<num_se; k++) {
		se = new SingularElement(vec_ten_mode, 0, 0, 0.0, 0.0,-2);
		fscanf(fp, "%d %d %lf %lf %lf %lf %lf %lf %lf %lf\n", &dummy, 
								&se->order,
								&se->type,
								&se->x, 
								&se->y, 
								&se->max_dir.entry[0], 
								&se->max_dir.entry[1], 
								&se->min_dir.entry[0], 
								&se->min_dir.entry[1], 
								&se->max_mag,
								&se->min_mag);
		se->Construct_mat();
		se->calc_display_attributes();
		singularelements->append(se);
	}

	for (k=0; k<num_re; k++) {
		re = new RegularElement(0.0, 0.0,-2);
		fscanf(fp, "%d %lf %lf %lf %lf\n", &dummy, 
								&re->x,
								&re->y,
								&re->vx,
								&re->vy);
		re->calc_display_attributes();
		regularelements->append(re);
	}
	fclose(fp);
}

void Polyhedron::read_tenfld_ctrl_file(const char name[128])
{
	singularelements->finalize();
	regularelements->finalize();
	singularities->finalize();
	singularelements = new SingularElementList(20);
	regularelements = new RegularElementList(20);
	singularities = new SingularityList(20);

	int k, num_se, num_re, dummy;
	FILE *fp=fopen(name, "r");
	SingularElement *se;
	RegularElement *re;

	fscanf(fp, "%d %d\n", &num_se, &num_re);
	for (k=0; k<num_se; k++) {
		se = new SingularElement(vec_ten_mode, 0, 0, 0.0, 0.0,-2);
		fscanf(fp, "%d %d %d %lf %lf %lf %lf %lf %lf %lf %lf\n", &dummy, 
								&se->type,
								&se->order,
								&se->x, 
								&se->y, 
								&se->max_dir.entry[0], 
								&se->max_dir.entry[1], 
								&se->min_dir.entry[0], 
								&se->min_dir.entry[1], 
								&se->max_mag,
								&se->min_mag);
		se->Construct_mat();
		se->calc_display_attributes();
		singularelements->append(se);
	}

	for (k=0; k<num_re; k++) {
		re = new RegularElement(0.0, 0.0,-2);
		fscanf(fp, "%d %lf %lf %lf %lf\n", &dummy, 
								&re->x,
								&re->y,
								&re->vx,
								&re->vy);
		re->calc_display_attributes();
		regularelements->append(re);
	}
	fclose(fp);
}

void evaluate_vector_in_triangle(double t, double vec_a, double vec_b, double vec_c, double vec_d, double vec_e, double vec_f, double y[3], double dydt[3])
{
	double magnitude;

	dydt[1] = vec_a * y[1] + vec_b  * y[2] + vec_c;
	dydt[2] = vec_d * y[1] + vec_e * y[2] + vec_f;
	magnitude = sqrt(dydt[1]*dydt[1] + dydt[2]*dydt[2]);
	if (magnitude > vector_clip_size) {
		magnitude /= vector_clip_size;
		dydt[1] /= magnitude;
		dydt[2] /= magnitude;
	}
}

void evaluate_tensor_in_triangle(double t, double ten_a, double ten_b, double ten_c, double ten_d, double ten_e, double ten_f, double y[3], double old_dydt[3], double dydt[3])
{
	double magnitude, the_dydt[3];

	the_dydt[1] = ten_a * y[1] + ten_b * y[2] + ten_c;
	the_dydt[2] = ten_d * y[1] + ten_e * y[2] + ten_f;
	magnitude = sqrt(the_dydt[1]*the_dydt[1] + the_dydt[2]*the_dydt[2]);
	if (magnitude > vector_clip_size) {
		magnitude /= vector_clip_size;
		the_dydt[1] /= magnitude;
		the_dydt[2] /= magnitude;
	}
	magnitude = sqrt(the_dydt[1]*the_dydt[1] + the_dydt[2]*the_dydt[2]);

	double angle = atan2(the_dydt[2], the_dydt[1])*0.5;
	double cos_angle = cos(angle), sin_angle = sin(angle);
	
	dydt[1] = magnitude*cos_angle;
	dydt[2] = magnitude*sin_angle;
	if (dydt[1]*old_dydt[1] + dydt[2]*old_dydt[2] < 0.0) {
		dydt[1] = -dydt[1];
		dydt[2] = -dydt[2];
	}
}

void evaluate_vector_in_plane(double t, double y[3], double dydt[3])
{
	double magnitude;

	magnitude = exp(-(y[1]*y[1] + y[2]*y[2]));
	dydt[1] = magnitude*y[2];
	dydt[2] = -magnitude*y[1];
	magnitude = sqrt(dydt[1]*dydt[1] + dydt[2]*dydt[2]);
	if (magnitude > vector_clip_size) {
		magnitude /= vector_clip_size;
		dydt[1] /= magnitude;
		dydt[2] /= magnitude;
	}
}

int test_sign(icVector2 s, icVector2 e, icVector2 t) {
	double res = (e.entry[1] - s.entry[1])*(t.entry[0] - s.entry[0])
						 - (e.entry[0] - s.entry[0])*(t.entry[1] - s.entry[1]);
	if (res > 0.0)
		return 1;
	if (res < 0.0)
		return -1;
	return 0;
}

int get_intersection(icVector2 s, icVector2 e, icVector2 t[2], icVector2 *res)
{
	int i;
	double M[2];

	for (i=0; i<2; i++)
		M[i] = (e.entry[1] - s.entry[1])*(t[i].entry[0] - s.entry[0])
				 - (e.entry[0] - s.entry[0])*(t[i].entry[1] - s.entry[1]);
	if (M[0]*M[1] > 1.0e-20)
		return 0;
	*res = fabs(M[0])/(fabs(M[0])+fabs(M[1]))*t[1] + fabs(M[1])/(fabs(M[0])+fabs(M[1]))*t[0];
	return 1;
}

double test_side(icVector2 line_base_pt, icVector2 line_normal, icVector2 test_pt)
{
	return dot(test_pt - line_base_pt, line_normal);
}

int clip_against_line(icVector2 line_base_pt, icVector2 line_normal, LineSegment **ls)
{
	double side_start, side_end;
	side_start = test_side(line_base_pt, line_normal, (*ls)->start_coor);
	side_end = test_side(line_base_pt, line_normal, (*ls)->end_coor);
	if (side_end * side_start > 0.0)
		return 0;  // no clipping
	double side_new_end = fabs(side_end) / (fabs(side_end) + fabs(side_start));
	(*ls)->end_coor = side_new_end * (*ls)->start_coor + (1-side_new_end) * (*ls)->end_coor;
	return 1;
}

int clip_against_edge(icVector2 edge_s, icVector2 edge_e, LineSegment **ls)
{
	double side_start, side_end;
	icVector2 line_normal = (*ls)->end_coor-(*ls)->start_coor;
	normalize(line_normal);
	line_normal.set(-line_normal.entry[1], line_normal.entry[0]);
	side_start = test_side((*ls)->start_coor, line_normal, edge_s);
	side_end = test_side((*ls)->start_coor, line_normal, edge_e);
	if (side_end * side_start > 0.0)
		return 0;  // no clipping
	double side_new_end = fabs(side_end) / (fabs(side_end) + fabs(side_start));
	(*ls)->end_coor = side_new_end * edge_s + (1-side_new_end) * edge_e;
	return 1;
}

int clip_against_edge_1(icVector2 edge_s, icVector2 edge_e, icVector2 edge_normal, LineSegment **ls)
{
	double side_start, side_end;
	side_start = test_side(edge_s, edge_normal, (*ls)->start_coor);
	side_end = test_side(edge_s, edge_normal, (*ls)->end_coor);
	if (side_end * side_start > 0.0)
		return 0;  // no clipping

	icVector2 line_normal = (*ls)->end_coor-(*ls)->start_coor;
	normalize(line_normal);
	line_normal.set(-line_normal.entry[1], line_normal.entry[0]);
	side_start = test_side((*ls)->start_coor, line_normal, edge_s);
	side_end = test_side((*ls)->start_coor, line_normal, edge_e);
	if (side_end * side_start > 0.0)
		return 0;  // no clipping
	double side_new_end = fabs(side_end) / (fabs(side_end) + fabs(side_start));
	(*ls)->end_coor = side_new_end * edge_s + (1-side_new_end) * edge_e;
	return 1;
}

// return values:
// 0 = continue to another triangle
// 1 = stop due to a critial point
// -1 = stop due to a boundary exit or entry
int Polyhedron::trace_one_trajectory_in_one_triangle(icVector2 start_pos, unsigned char direction, Triangle *tri, Edge **incoming_edge, Triangle **next_tri, icVector2 *next_start_pos, LineSegmentList **ls)
{
	double a[3];
	int nok, nbad;
	int i, j;
	icVector2 temp_vec2, temp_x_dir, temp_y_dir, temp_vec[3], temp_vec_t[2];
	icVector2 temp_vec3;
	LineSegment *ll;
	double temp_x, temp_y, temp_len;
	double bary_a, bary_b;
	Edge *temp_e;
	int tested_signs[3];
	Edge *old_incoming_edge = *incoming_edge;
	int duration = 0;

	*ls = new LineSegmentList(40);

	xp = XP;
	Yp[1] = Yp1;
	Yp[2] = Yp2;
	yp = Yp;

	a[1] = start_pos.entry[0];
	a[2] = start_pos.entry[1];

	while (duration < 10) {
		if (direction == 0) 
			odeint(tri->vecten_a, tri->vecten_b, tri->vecten_c, tri->vecten_d, tri->vecten_e, tri->vecten_f, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		else if (direction == 1)
			odeint(-tri->vecten_a, -tri->vecten_b, -tri->vecten_c, -tri->vecten_d, -tri->vecten_e, -tri->vecten_f, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		else if (direction == 2)
			odeint(-tri->vecten_d, -tri->vecten_e, -tri->vecten_f, tri->vecten_a, tri->vecten_b, tri->vecten_c, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		else if (direction == 3)
			odeint(tri->vecten_d, tri->vecten_e, tri->vecten_f, -tri->vecten_a, -tri->vecten_b, -tri->vecten_c, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		for (i=1; i<kount; i++) {
			ll = new LineSegment();
			ll->tri = tri;
			ll->start_coor.set(yp[1][i], yp[2][i]);
			ll->end_coor.set(yp[1][i+1], yp[2][i+1]);
			tri->convert_local_to_global(ll->start_coor, &temp_vec3);
			tri->convert_local_to_global(ll->end_coor, &temp_vec3);
			(*ls)->append(ll);
			
			// check to see whether it is close to a singular point
			if (tri->the_singularity != NULL) {
				// the end position is close to the critical point
				if (length(tri->the_singularity->pos - ll->end_coor) < 1.0e-6) {
					ll->end_coor = tri->the_singularity->pos;
					ll->calc_display_attributes();
					return 1;
				}

				// the segment is close to the critical point
				temp_x_dir = ll->end_coor-ll->start_coor;
				temp_len = length(temp_x_dir);
				normalize(temp_x_dir);
				temp_y_dir.set(temp_x_dir.entry[1], -temp_x_dir.entry[0]);
				temp_vec2 = tri->the_singularity->pos - ll->start_coor;
				temp_x = dot(temp_vec2, temp_x_dir);
				temp_y = dot(temp_vec2, temp_y_dir);
				if (fabs(temp_y) < 1.0-6 && (temp_x > 0.0 && temp_x <= temp_len)) {
					ll->end_coor = tri->the_singularity->pos;
					ll->calc_display_attributes();
					return 1;
				}
				if (duration > 10) {
					ll->end_coor = tri->the_singularity->pos;
					ll->calc_display_attributes();
					return 1;
				}
			}
			
			// check to see whether it runs into an edge
			// if a boundary edge, finalize
			// if not a boundary edge, continue into another triangle
			tri->get_bary_coor(ll->start_coor, &bary_a, &bary_b);
			if (bary_a < 0.0 || bary_b < 0.0 || 1-bary_a-bary_b < 0.0) {
				int gogo=1;
			}
			tri->get_bary_coor(ll->end_coor, &bary_a, &bary_b);
			if (bary_a < 0.0 || bary_b < 0.0 || 1-bary_a-bary_b < 0.0) {
				temp_vec[0].set(0.0, 0.0);
				temp_vec[1].set(tri->x1, 0.0);
				temp_vec[2].set(tri->x2, tri->y2);
				for (j=0; j<3; j++)
					tested_signs[j] = test_sign(ll->start_coor, ll->end_coor, temp_vec[j]);

/*				
				for (j=0; j<3; j++) {
					if (tested_signs[j] == 0) {
						// the j-th vertex is on the line segment.
						switch (j) {
						case 0:
							ll->end_coor.set(0.0, 0.0);
							break;

						case 1:
							ll->end_coor.set(tri->x1, 0.0);
							break;

						case 2:
							ll->end_coor.set(tri->x2, tri->y2);
							break;
						}
						// find the next triangle and starting position

					}
				}
*/
				temp_vec_t[0] = ll->start_coor;
				temp_vec_t[1] = ll->end_coor;
				double min_dist = 1.0e+20, current_dist;
				icVector2 temp_end_coor;
				for (j=0; j<3; j++) {
					temp_e = tri->edges[j];
					if (ll->start_coor == start_pos && temp_e == old_incoming_edge)
						continue;
					if (tested_signs[j] * tested_signs[(j+1) % 3] == -1) {
						// the j-th edge is on the line segment
						if (get_intersection(temp_vec[j], temp_vec[(j+1) % 3], temp_vec_t, &(temp_end_coor)) == 0)
							continue;
						current_dist = dot(ll->end_coor-ll->start_coor, temp_end_coor-ll->start_coor);
						if (current_dist < 0.0)
							continue;
//						if (current_dist < min_dist) {
							min_dist = current_dist;
							ll->end_coor = temp_end_coor;
							*incoming_edge = temp_e;
							ll->calc_display_attributes();
//						}
					}
				}
				if ((*incoming_edge)->ntris == 1)
					return -1;
				if ((*incoming_edge)->tris[0] == tri) 
					*next_tri = (*incoming_edge)->tris[1];
				else
					*next_tri = (*incoming_edge)->tris[0];
				tri->convert_local_to_global(ll->end_coor, &temp_vec3);
				(*next_tri)->convert_global_to_local(temp_vec3, next_start_pos);
				return 0;
			} else 
				ll->calc_display_attributes();
			// check to see whether it runs into an vertex
			// if a boundary vertex, finalize
			// if not a boundary vertex, continue into another triangle

			// finally, continue inside this triangle
		}
		duration++;
	}
	return 0;
}


int Polyhedron::trace_one_trajectory_in_domain(int num_triangles, icVector2 start_pos, unsigned char direction, Triangle *tri, LineSegmentList **ls) {
	int i, j, result;
	icVector2 next_start_pos = start_pos;
	Triangle *next_tri = tri, *old_next_tri=NULL;
	//LineSegmentList *temp_list;
	icVector2List *temp_vlist;
	Edge *incoming_edge = NULL;
	LineSegment *temp_ls;

	for (i=0; i<num_triangles; i++) {
		old_next_tri = next_tri;
		result = trace_trajectory_in_triangle_vecfld(next_start_pos, direction, next_tri, &incoming_edge, &next_tri, &next_start_pos, &temp_vlist);
		temp_ls = new LineSegment();
		temp_ls->tri = old_next_tri;
		temp_ls->start_coor = *temp_vlist->vectors[0];
		for (j=1; j<temp_vlist->num_vectors-1; j++) {
			if (j %10 != 0)
				continue;
			temp_ls->end_coor = *temp_vlist->vectors[j];
			temp_ls->len = length(temp_ls->end_coor-temp_ls->start_coor);
			temp_ls->calc_display_attributes();
			(*ls)->append(temp_ls);
			temp_ls = new LineSegment();
			temp_ls->tri = old_next_tri;
			temp_ls->start_coor = *temp_vlist->vectors[j];
		}
		temp_ls->end_coor = *temp_vlist->vectors[temp_vlist->num_vectors-1];
		temp_ls->len = length(temp_ls->end_coor-temp_ls->start_coor);
		temp_ls->calc_display_attributes();
		(*ls)->append(temp_ls);

		for(int j = 0; j < temp_vlist->num_vectors; j++) {
			delete temp_vlist->vectors[j];
		}

		temp_vlist->finalize();
		if (result != 0)
			return result;
		if (length(next_tri->verts[0]->vec_fld) == 0.0 || length(next_tri->verts[1]->vec_fld) == 0.0 || length(next_tri->verts[2]->vec_fld) == 0.0)
			return result;
	}
	return result;
}

/*
int Polyhedron::trace_one_trajectory_in_domain(icVector2 start_pos, unsigned char direction, Triangle *tri, LineSegmentList **ls) {
	int i, j, result;
	icVector2 next_start_pos = start_pos;
	Triangle *next_tri = tri, *old_next_tri=NULL;
	LineSegmentList *temp_list;
	Edge *incoming_edge = NULL;

	for (i=0; i<500; i++) {
		if (i==52)
			int go = 1;
		old_next_tri = next_tri;
		result = trace_one_trajectory_in_one_triangle(next_start_pos, direction, next_tri, &incoming_edge, &next_tri, &next_start_pos, &temp_list);
		if (next_tri == old_next_tri)
			int gogo = 1;
		for (j=0; j<temp_list->num_linesegments; j++)
			(*ls)->append(temp_list->linesegments[j]);
		delete(temp_list);
		if (result != 0)
			return result;
		if (length(next_tri->verts[0]->vec_fld) == 0.0 || length(next_tri->verts[1]->vec_fld) == 0.0 || length(next_tri->verts[2]->vec_fld) == 0.0)
			return result;
	}
	return result;
}
*/

int Polyhedron::trace_one_tensor_trajectory_in_domain(int num_triangles, icVector2 start_pos, unsigned char direction, icVector2 initial_dir, Triangle *tri, LineSegmentList **ls) {
	int i, j, result;
	icVector2 next_start_pos = start_pos;
	Triangle *next_tri = tri, *old_next_tri=NULL;
	//LineSegmentList *temp_list;
	icVector2List *temp_vlist;
	Edge *incoming_edge = NULL;
	icVector2 next_initial_dir = initial_dir;
	LineSegment *temp_ls;

	for (i=0; i<num_triangles; i++) {
		old_next_tri = next_tri;
		result = trace_trajectory_in_triangle_tenfld(next_start_pos, direction, next_initial_dir, next_tri, &incoming_edge, &next_tri, &next_start_pos, &next_initial_dir, &temp_vlist);
		temp_ls = new LineSegment();
		temp_ls->tri = old_next_tri;
		temp_ls->start_coor = *temp_vlist->vectors[0];
		for (j=1; j<temp_vlist->num_vectors-1; j++) {
			if (j %10 != 0)
				continue;
			temp_ls->end_coor = *temp_vlist->vectors[j];
			temp_ls->len = length(temp_ls->end_coor-temp_ls->start_coor);
			temp_ls->calc_display_attributes();
			(*ls)->append(temp_ls);
			temp_ls = new LineSegment();
			temp_ls->tri = old_next_tri;
			temp_ls->start_coor = *temp_vlist->vectors[j];
		}
		temp_ls->end_coor = *temp_vlist->vectors[temp_vlist->num_vectors-1];
		temp_ls->len = length(temp_ls->end_coor-temp_ls->start_coor);
		temp_ls->calc_display_attributes();
		(*ls)->append(temp_ls);
		temp_vlist->finalize();
		if (result != 0)
			return result;
		if (next_tri->verts[0]->ten_fld == 0.0 || next_tri->verts[1]->ten_fld == 0.0 || next_tri->verts[2]->ten_fld == 0.0)
			return result;
	}
	return result;
}

/*
int Polyhedron::trace_one_tensor_trajectory_in_domain(int num_triangles, icVector2 start_pos, unsigned char direction, icVector2 initial_dir, Triangle *tri, LineSegmentList **ls) {
	int i, j, result;
	icVector2 next_start_pos = start_pos;
	Triangle *next_tri = tri, *old_next_tri=NULL;
	LineSegmentList *temp_list;
	Edge *incoming_edge = NULL;
	icVector2 next_initial_dir = initial_dir;

	for (i=0; i<num_triangles; i++) {
		old_next_tri = next_tri;
		result = trace_one_tensor_trajectory_in_one_triangle(next_start_pos, direction, next_initial_dir, next_tri, &incoming_edge, &next_tri, &next_start_pos, &next_initial_dir, &temp_list);
		if (next_tri == old_next_tri)
			int gogo = 1;
		for (j=0; j<temp_list->num_linesegments; j++)
			(*ls)->append(temp_list->linesegments[j]);
		delete(temp_list);
		if (result != 0)
			return result;
		if (next_tri->verts[0]->ten_fld == 0.0 || next_tri->verts[1]->ten_fld == 0.0 || next_tri->verts[2]->ten_fld == 0.0)
			return result;
	}
	return result;
}
*/

void Polyhedron::handle_transversity()
{
	int i, j;
	Corner *temp_c;
	icVector2 temp_vec;

	for (i=0; i<nedges; i++) {
		elist[i]->transverse = 0;
		icVector2 connection(elist[i]->verts[0]->x - elist[i]->verts[1]->x, elist[i]->verts[0]->y - elist[i]->verts[1]->y);
		normalize(connection);
		connection = cross(connection);
		double proj[2];
		for (j=0; j<2; j++) {
			temp_vec = elist[i]->verts[j]->vec_fld;
			normalize(temp_vec);
			proj[j] = dot(connection, temp_vec);
		}
		if (proj[0] > 0.0 && proj[1] > 0.0) 
			elist[i]->transverse = 1;
		else if (proj[0] < 0.0 && proj[1] < 0.0)
			elist[i]->transverse = -1;
	}

	for (i=0; i<ntris; i++) {
		for (j=0; j<3; j++) {
			temp_c = clist[3*i+j];
			if (temp_c->e->transverse == 0)
				temp_c->transverse = 0;
			else {
				if (orientation == 0) {
					if (temp_c->e->verts[0] == temp_c->n->v)
						temp_c->transverse = temp_c->e->transverse;
					else
						temp_c->transverse = -temp_c->e->transverse;
				} else {
					if (temp_c->e->verts[0] == temp_c->p->v)
						temp_c->transverse = temp_c->e->transverse;
					else
						temp_c->transverse = -temp_c->e->transverse;
				}
			}
		}
	}
}

int Polyhedron::find_common_region_for_pair_cancellation(Singularity *index_neg, Singularity *index_pos, TriangleList **tl)
{
	TriangleList *ts;

	if(index_neg->separatrices_major->num_separatrices == 0) return 0;

	ts = new TriangleList(20);
	find_region_for_repeller(index_pos, index_neg->tri, &ts);

	LineSegmentList *vertical[2];
	vertical[0] = index_neg->separatrices_major->separatrices[0];
	vertical[1] = index_neg->separatrices_major->separatrices[1];

	if (find_region_for_attractor(index_pos, index_neg->tri, vertical, ts, tl) == 0) {
		ts->finalize();
		return 0;
	}

	ts->finalize();
	return 1;
}

void Polyhedron::find_base_region_based_on_sepatatrix(LineSegmentList *sepa, double percentage, TriangleList **tl_base)
{
	int i;

	double len = 0.0;
	for (i=0; i<sepa->num_linesegments; i++) {
		len += sepa->linesegments[i]->len;
		if (len > sepa->len*percentage) 
			return;
		(*tl_base)->insert_unique(sepa->linesegments[i]->tri);
	}
}

void Polyhedron::find_region_based_on_triangle_list(int direction, TriangleList *tl_in, TriangleList **tl_out)
{
	int i;
	queue *this_list = new queue();
	Corner *temp_c;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;  

	for (i=0; i<tl_in->num_tris; i++) 
		this_list->push(tl_in->tris[i]->index);

	while (this_list->is_empty()) {
		int mm = this_list->pop();
		if (tlist[mm]->temp_use == 1)
			continue;
		tlist[mm]->temp_use = 1;
		(*tl_out)->append(tlist[mm]);

		for (i=0; i<3; i++) {
			temp_c = clist[3*mm+i];
			if (temp_c->o == NULL) 
				continue;
			if (temp_c->o->t->temp_use == 1)
				continue;
			if (direction == 0) {
				if (temp_c->o->transverse == -1)
					continue;
			} else {
				if (temp_c->o->transverse == 1)
					continue;
			}
			this_list->push(temp_c->o->t->index);
		}
	}
	//delete(this_list);
	this_list->finalize();
}

int Polyhedron::contain_other_critical_pts(SingularityList *sl, TriangleList *tl, TriangleList *tl_constraint)
{
	int i;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;
	for (i=0; i<tl_constraint->num_tris; i++) 
		tl_constraint->tris[i]->temp_use = 1;
	for (i=0; i<singularities->num_singularities; i++)
		singularities->singularities[i]->temp_use = 0;
	for (i=0; i<sl->num_singularities; i++)
		sl->singularities[i]->temp_use = 2;  // not covered

	for (i=0; i<tl->num_tris; i++) {
		if (tl->tris[i]->temp_use == 0)
			continue;
		if (tl->tris[i]->the_singularity == NULL)
			continue;
		tl->tris[i]->the_singularity->temp_use++;
	}

	for (i=0; i<singularities->num_singularities; i++) {
		if (singularities->singularities[i]->temp_use == 1)
			return 1;  // some other singularities are covered
	}
	return 0;
}

int Polyhedron::contain_only_required_singularities(SingularityList *sl, TriangleList *tl)
{
	int i;

	for (i=0; i<singularities->num_singularities; i++) 
		singularities->singularities[i]->temp_use = 0;
	for (i=0; i<sl->num_singularities; i++)
		sl->singularities[i]->temp_use = 2;  // not covered

	for (i=0; i<tl->num_tris; i++) {
		if (tl->tris[i]->the_singularity == NULL)
			continue;
		tl->tris[i]->the_singularity->temp_use++;
	}

	for (i=0; i<sl->num_singularities; i++) {
		if (sl->singularities[i]->temp_use == 2)
			return 0;  // some singularities not covered
	}
	for (i=0; i<singularities->num_singularities; i++) {
		if (singularities->singularities[i]->temp_use == 1)
			return 0;  // some other singularities are covered
	}
	return 1;
}

int Polyhedron::check_conley_condition(int which_operation, SingularityList *sl, TriangleList **tl)
{
	// 0 = pair cancellation, 1 = moving singularity
	int i;

	if (is_simply_connected(*tl) == 0)
		return 0;
	if (contain_only_required_singularities(sl, *tl) == 0)
		return 0;

	// check the number of segments in terms of transversality
	CornerList *cl = new CornerList(20);
	find_corners_for_simply_connected_region(*tl, &cl);
	int num_segments = 0;
	int pos[2], first_out_to_in;
	int num_in=0, num_out=0;

	for (i=0; i<cl->num_corners; i++) {
		if (cl->corners[i]->transverse * cl->corners[(i+1) % cl->num_corners]->transverse == -1) {
			if (num_segments >= 2) {   // Conley Index of the region is not trival
				cl->finalize();
				return 0; 
			}
			if (cl->corners[i]->transverse == -1)  // this one is out
				first_out_to_in = num_segments;
			pos[num_segments] = i;
			num_segments++;
		}
		if (cl->corners[i]->transverse == -1) {
			num_out++;
		}
		else if (cl->corners[i]->transverse == 1) {
			num_in++;
		}
	}

	if (which_operation == 0 && num_segments != 2) {
		cl->finalize();
		return 0; 
	}
	if (which_operation == 1 && num_segments != 0) {
		cl->finalize();
		return 0; 
	}

	cl->finalize();
	return 1;
}

int Polyhedron::singularity_pair_cancellation(int cp1, int cp2, double rotate_angle)
{
	Singularity *ss1, *ss2;
	TriangleList *tl = new TriangleList(20);
	Triangle *tri_neg = tlist[cp1], *tri_pos = tlist[cp2];
	int old_index = tri_neg->the_singularity->index;

	ss1 = tri_neg->the_singularity;
	ss2 = tri_pos->the_singularity;

	global_rotate_field_tool(rotate_angle);
	analyze_field(1, old_index);
	ss1 = tri_neg->the_singularity;
	ss2 = tri_pos->the_singularity;

  if (find_common_region_for_pair_cancellation(ss1, ss2, &tl)==0) {
		tl->finalize();
		global_rotate_field_tool(-rotate_angle);
		analyze_field(0, -1);
		return 0;
	}

	CornerList *cl = new CornerList(20);
	find_corners_for_simply_connected_region(tl, &cl);
	smooth_fld_in_region(0, tl);
	tl->finalize();	
	cl->finalize();

	global_rotate_field_tool(-rotate_angle);
	return 1;
}

int compare_vertices_1 (char *ptr1, char *ptr2)
{
  Vertex *v1 = (Vertex *) ptr1;
  Vertex *v2 = (Vertex *) ptr2;

  if (v1->dist_temp < v2->dist_temp)
    return (-1);
  else if (v1->dist_temp > v2->dist_temp)
    return (1);
  else
    return (0);
}

void Polyhedron::search_path_within_on_path(Vertex *the_v, VertexList *the_vl, VertexList **result)
{
	int i, j, kkkk=0;

	for (i=0; i<nverts; i++) {
		vlist[i]->dist_temp = 1.0e+20;
		vlist[i]->status = 1;
	}

  vertex_queue = initialize_skiplist_queue(compare_vertices_1, nverts);
	for (i=0; i<the_vl->num_verts; i++) {
		the_vl->verts[i]->dist_temp = 0.0;
		the_vl->verts[i]->status = 0;
		the_vl->verts[i]->parent = NULL;
		vertex_queue->add((char *)the_vl->verts[i]);
	}
	
	while (vertex_queue->count() > 0) {
		Vertex *temp_v = (Vertex *)(vertex_queue->first());
		vertex_queue->remove((char *)temp_v);
		temp_v->status = -1;
		if (temp_v == the_v)
			break;

		kkkk++;
		for (j=0; j<temp_v->adjacent_edges->num_edges; j++) {
			Vertex *temp_vv;
			Edge *temp_e = temp_v->adjacent_edges->edges[j];
			if (temp_e->on_path == 0)
				continue;
			if (temp_e->verts[0] == temp_v) 
				temp_vv = temp_e->verts[1];
			else
				temp_vv = temp_e->verts[0];
			if (temp_vv->status == -1)
				continue;
			if (temp_vv->status == 1) 
				temp_vv->status = 0;
			else 
				vertex_queue->remove((char *)temp_vv);
			if (temp_vv->dist_temp > temp_v->dist_temp + temp_e->length) {
				temp_vv->dist_temp = temp_v->dist_temp + temp_e->length;
				temp_vv->parent = temp_v;
			}
			if (temp_vv->status == 0)
				vertex_queue->add((char *)temp_vv);
		}
	}
	finalize_skiplist_queue(&vertex_queue);
	
	(*result) = new VertexList(20);
	(*result)->insert_unique(the_v);
	Vertex *current_vert = the_v;
	while (current_vert->parent != NULL) {
		current_vert = current_vert->parent;
		(*result)->insert_unique(current_vert);
	}
}

void Polyhedron::identify_corner_by_vertices(Vertex *v1, Vertex *v2, Corner **c)
{
	int i, j;

	Edge *temp_ee;
	for (i=0; i<v1->adjacent_edges->num_edges; i++) {
		temp_ee = v1->adjacent_edges->edges[i];
		if (temp_ee->verts[0] == v2 || temp_ee->verts[1] == v2) {
			Triangle *temp_tt = temp_ee->tris[0];
			for (j=0; j<3; j++) {
				if (clist[temp_tt->index*3+j]->p->v == v2 && clist[temp_tt->index*3+j]->n->v == v1) {
					(*c) = clist[temp_tt->index*3+j];
					return;
				} else if (clist[temp_tt->index*3+j]->n->v == v2 && clist[temp_tt->index*3+j]->p->v == v1) {
					(*c) = clist[temp_tt->index*3+j]->o;
					return;
				}
			}
		}
	}
}

void Polyhedron::construct_boundary_from_verts()
{
	int i, j;
	VertexList *total = new VertexList(50);
	VertexList *temp_vl, *temp_vl_result;
	Corner *temp_c;

	for (i=0; i<nverts; i++) {
		vlist[i]->on_path = 1;
		vlist[i]->last_touched = -1;
	}
	for (i=0; i<nedges; i++)
		elist[i]->on_path = 1;

	for (i=0; i<region_verts->num_verts; i++) {
		temp_vl = new VertexList(1);
		temp_vl->append(region_verts->verts[(i+1) % region_verts->num_verts]);
		search_path_within_on_path(region_verts->verts[i], temp_vl, &temp_vl_result);
		delete(temp_vl);
		for (j=0; j<temp_vl_result->num_verts-1; j++) 
			total->append(temp_vl_result->verts[j]);
		temp_vl_result->finalize();
	}

	int flag = 1;
	while (flag) {
		flag = 0;
		for (i=0; i<total->num_verts; i++) {
			total->verts[i]->temp_use = 0;
			total->verts[i]->smallest_length = 10000;
		}
		int current_length;
		for (i=0; i<total->num_verts; i++) {
			if (total->verts[i]->temp_use == 0) {
				total->verts[i]->first_use = i;
				total->verts[i]->last_use = i;
			}	else {
				current_length = i - total->verts[i]->last_use;
				if (current_length <total->verts[i]->smallest_length) {
					total->verts[i]->smallest_length = current_length;
					total->verts[i]->smallest_start = total->verts[i]->last_use;
					total->verts[i]->smallest_end = i;
				}
				total->verts[i]->last_use = i;
			}
			total->verts[i]->temp_use++;
		}
		int smallest_id = -1, smallest_start, smallest_end, smallest_length = 100000;
		for (i=0; i<total->num_verts; i++) {
			if (total->verts[i]->temp_use == 1)
				continue;
			current_length = total->verts[i]->first_use + total->num_verts - total->verts[i]->last_use;
			if (current_length < total->verts[i]->smallest_length) {
				total->verts[i]->smallest_length = current_length;
				total->verts[i]->smallest_start = total->verts[i]->last_use;
				total->verts[i]->smallest_end = total->verts[i]->first_use;
			}
			if (smallest_length > total->verts[i]->smallest_length) {
				smallest_length = total->verts[i]->smallest_length;
				smallest_start = total->verts[i]->smallest_start;
				smallest_end = total->verts[i]->smallest_end;
				smallest_id = i;
			}
		}
		if (smallest_id >= 0) {
			flag = 1;
			int len1 = (smallest_end - smallest_start + total->num_verts) % total->num_verts;
			int len2 = total->num_verts - len1;
			if (len1 < len2) {
				int swap = smallest_end;
				smallest_end = smallest_start;
				smallest_start = swap;
				len1 = len2;
			}
			VertexList *temp_total = new VertexList(len1);
			for (i=0; i<len1; i++) {
				j = (smallest_start + i) % total->num_verts;
				temp_total->append(total->verts[j]);
			}
			total->finalize();
			total = temp_total;
		}
	}

	for (i=0; i<total->num_verts; i++) {
		identify_corner_by_vertices(total->verts[i], total->verts[(i+1) % total->num_verts], &temp_c);
		region_boundary->append(temp_c);
	}
	total->finalize();
}

void Polyhedron::generate_region_from_boundary(TriangleList **tl)
{
	int i;
	queue *this_list = new queue();
	Corner *temp_c;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;  
	for (i=0; i<nedges; i++)
		elist[i]->temp_use = 0;  // can cross
	for (i=0; i<region_boundary->num_corners; i++)
		region_boundary->corners[i]->e->temp_use = 1;  // can not cross
	this_list->push(region_boundary->corners[0]->t->index);

	while (this_list->is_empty()) {
		int mm = this_list->pop();
		if (tlist[mm]->temp_use == 1)
			continue;
		tlist[mm]->temp_use = 1;
		(*tl)->append(tlist[mm]);

		for (i=0; i<3; i++) {
			temp_c = clist[3*mm+i];
			if (temp_c->o == NULL) 
				continue;
			if (temp_c->o->t->temp_use == 1)
				continue;
			if (temp_c->e->temp_use == 1)
				continue;
			this_list->push(temp_c->o->t->index);
		}
	}
	for (i=0; i<ntris; i++)
		tlist[i]->clicked = 0;
	for (i=0; i<(*tl)->num_tris; i++)
		(*tl)->tris[i]->clicked = 1;
	//delete(this_list);
	this_list->finalize();
}

int Polyhedron::is_simply_connected(TriangleList *tl)
{
	int i, j;
	int num_verts=0, num_edges=0, num_tris=0;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;
	for (i=0; i<tl->num_tris; i++)
		tl->tris[i]->temp_use=1;

	for (i=0; i<nverts; i++) 
		vlist[i]->temp_use = 0;
	for (i=0; i<nedges; i++) 
		elist[i]->temp_use = 0;

	for (i=0; i<tl->num_tris; i++) {
		if (tl->tris[i]->temp_use == 0)
			continue;
		num_tris++;
		for (j=0; j<3; j++) 
			tl->tris[i]->edges[j]->temp_use++;
	}
	for (i=0; i<nedges; i++) {
		if (elist[i]->temp_use != 1)
			continue;
		elist[i]->verts[0]->temp_use = elist[i]->verts[1]->temp_use = 1;
	}
	
	for (i=0; i<nverts; i++) {
		if (vlist[i]->temp_use > 2)
			return 0;  // the region is non-manifold
	}

	int total_boundary_verts = 0, total_verts_on_this_boundary = 0;
	for (i=0; i<nverts; i++) {
		if (vlist[i]->temp_use == 1) 
			total_boundary_verts++;
	}

	queue *this_list = new queue();
	Vertex *temp_v;
	for (i=0; i<nverts; i++) {
		if (vlist[i]->temp_use == 1) {
			total_verts_on_this_boundary++;
			vlist[i]->temp_use = 2;
			this_list->push(i);
			break;
		}
	}

	while (this_list->is_empty()) {
		int mm = this_list->pop();
		for (i=0; i<vlist[mm]->adjacent_edges->num_edges; i++) {
			if (vlist[mm]->adjacent_edges->edges[i]->temp_use != 1)
				continue;
			if (vlist[mm]->adjacent_edges->edges[i]->verts[0] == vlist[mm]) 
				temp_v = vlist[mm]->adjacent_edges->edges[i]->verts[1];
			else 
				temp_v = vlist[mm]->adjacent_edges->edges[i]->verts[0];
			if (temp_v->temp_use == 2)
				continue;
			temp_v->temp_use = 2;
			total_verts_on_this_boundary++;
			this_list->push(temp_v->index);
		}
	}
	//delete(this_list);
	this_list->finalize();

	if (total_verts_on_this_boundary < total_boundary_verts)
		return 0;  // more than one boundary

	// check genus
	for (i=0; i<nedges; i++) {
		if (elist[i]->temp_use == 0)
			continue;
		num_edges++;
		elist[i]->verts[0]->temp_use = elist[i]->verts[1]->temp_use = 1;
	}
	for (i=0; i<nverts; i++) {
		if (vlist[i]->temp_use == 1) 
			num_verts++;
	}
	if (num_verts - num_edges + num_tris < 1)
		return 0;   // the region contains handle(s)
	return 1;
}

void Polyhedron::find_intersection(TriangleList *tl_in1, TriangleList *tl_in2, TriangleList **tl_out)
{
	int i;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;
	for (i=0; i<tl_in1->num_tris; i++)
		tl_in1->tris[i]->temp_use++;
	for (i=0; i<tl_in2->num_tris; i++)
		tl_in2->tris[i]->temp_use++;

	for (i=0; i<tl_in1->num_tris; i++) {
		if (tl_in1->tris[i]->temp_use == 2)
			(*tl_out)->append(tl_in1->tris[i]);
	}
}

void Polyhedron::find_union(TriangleList *tl_in1, TriangleList *tl_in2, TriangleList **tl_out)
{
	int i;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;
	for (i=0; i<tl_in1->num_tris; i++)
		tl_in1->tris[i]->temp_use=1;
	for (i=0; i<tl_in2->num_tris; i++)
		tl_in2->tris[i]->temp_use=1;

	for (i=0; i<ntris; i++) {
		if (tlist[i]->temp_use > 0)
			(*tl_out)->append(tlist[i]);
	}
}

void Polyhedron::smooth_field()
{
	TriangleList *tl = new TriangleList(20);
	if (vec_ten_mode == 0) {
		for (int i=0; i<nverts; i++)
			vlist[i]->undo_vec_fld = vlist[i]->vec_fld;
	} else {
		for (int i=0; i<nverts; i++)
			vlist[i]->undo_ten_fld = vlist[i]->ten_fld;
	}
	construct_boundary_from_verts();
	generate_region_from_boundary(&tl);
	smooth_fld_in_region(vec_ten_mode, tl);
	analyze_field(1, -1);

	calc_new_texture_coors();
}

void Polyhedron::global_rotate_field_tool(double global_rotate)
{
	int i;
	icVector3 temp;
	icMatrix2x2 temp_mat;
	double cos_theta = cos(global_rotate), sin_theta = sin(global_rotate);

	if (vec_ten_mode == 0) {
		for (i=0; i<nverts; i++){
			vlist[i]->vec_fld.set(cos_theta * vlist[i]->vec_fld.entry[0] - sin_theta * vlist[i]->vec_fld.entry[1], 
														sin_theta * vlist[i]->vec_fld.entry[0] + cos_theta * vlist[i]->vec_fld.entry[1]);
		}
	} else {
		temp_mat.set(cos_theta, sin_theta, -sin_theta, cos_theta);
		for (i=0; i<nverts; i++){
			vlist[i]->ten_fld = othoconjugate(vlist[i]->ten_fld, temp_mat);
		}
	}
}

void Polyhedron::global_rotate_field(double global_rotate)
{
	global_rotate_field_tool(global_rotate);
	analyze_field(1, -1);
	calc_new_texture_coors();
}

void Polyhedron::global_reflect_field_tool()
{
	int i;
	double angle = PI*0.25;

	if (vec_ten_mode == 0) {
		for (i=0; i<nverts; i++) {
			vlist[i]->vec_fld.set(vlist[i]->vec_fld.entry[0], -vlist[i]->vec_fld.entry[1]);
		}
	} else {
		for (i=0; i<nverts; i++) {
//			vlist[i]->ten_fld.set(vlist[i]->ten_fld.entry[0][0], -vlist[i]->ten_fld.entry[0][1],
//														-vlist[i]->ten_fld.entry[1][0], vlist[i]->ten_fld.entry[1][1]);
			vlist[i]->ten_fld = conjugate(vlist[i]->ten_fld, icMatrix2x2(cos(angle), sin(angle), sin(angle), -cos(angle)));
		}
	}

}

void Polyhedron::global_reflect_field()
{
	global_reflect_field_tool();
	analyze_field(1, -1);
	calc_new_texture_coors();
}

int Polyhedron::build_traces_for_moving(Triangle *the_tri, icVector2 pos, Singularity *source, double percentage)
{
	int j;

	move_trace[0]->finalize();
	move_trace[0] = new LineSegmentList(20);
	move_trace[0]->len = 0.0;
	move_trace[0]->direction = 1;
	trace_one_trajectory_in_domain(500, pos, move_trace[0]->direction, the_tri, &move_trace[0]);
	if (move_trace[0]->linesegments[move_trace[0]->num_linesegments-1]->tri != source->tri) {
		return 0;  // fail to connect to the target source
	}
	for (j=0; j<move_trace[0]->num_linesegments; j++){
		move_trace[0]->linesegments[j]->len = length(move_trace[0]->linesegments[j]->end_coor - move_trace[0]->linesegments[j]->start_coor);
		move_trace[0]->len += move_trace[0]->linesegments[j]->len;
	}

	move_trace[1]->finalize();
	move_trace[1] = new LineSegmentList(20);
	move_trace[1]->len = 0.0;
	move_trace[1]->direction = 0;
	trace_one_trajectory_in_domain(500, pos, move_trace[1]->direction, the_tri, &move_trace[1]);
	for (j=0; j<move_trace[1]->num_linesegments; j++){
		move_trace[1]->linesegments[j]->len = length(move_trace[1]->linesegments[j]->end_coor - move_trace[1]->linesegments[j]->start_coor);
		move_trace[1]->len += move_trace[1]->linesegments[j]->len;
	}
	double temp_len = 0.0;
	Triangle *which_tri;
	icVector2 new_pos;
	for (j=0; j<move_trace[1]->num_linesegments; j++){
		temp_len += move_trace[1]->linesegments[j]->len;
		if (temp_len > move_trace[1]->len*percentage) {
			which_tri = move_trace[1]->linesegments[j]->tri;
			new_pos = move_trace[1]->linesegments[j]->start_coor;
			break;
		}
	}
	
	move_trace[1]->finalize();
	move_trace[1] = new LineSegmentList(20);
	move_trace[1]->len = 0.0;
	move_trace[1]->direction = 2;
	trace_one_trajectory_in_domain(500, new_pos, move_trace[1]->direction, which_tri, &move_trace[1]);
	for (j=0; j<move_trace[1]->num_linesegments; j++){
		move_trace[1]->linesegments[j]->len = length(move_trace[1]->linesegments[j]->end_coor - move_trace[1]->linesegments[j]->start_coor);
		move_trace[1]->len += move_trace[1]->linesegments[j]->len;
	}
	move_trace[2]->finalize();
	move_trace[2] = new LineSegmentList(20);
	move_trace[2]->len = 0.0;
	move_trace[2]->direction = 3;
	trace_one_trajectory_in_domain(500, new_pos, move_trace[2]->direction, which_tri, &move_trace[2]);
	for (j=0; j<move_trace[2]->num_linesegments; j++){
		move_trace[2]->linesegments[j]->len = length(move_trace[2]->linesegments[j]->end_coor - move_trace[2]->linesegments[j]->start_coor);
		move_trace[2]->len += move_trace[2]->linesegments[j]->len;
	}
	return 1;
}

void Polyhedron::find_region_for_repeller(Singularity *source, Triangle *the_tri, TriangleList **region_repellor)
{
	// the repeller is a source in both pair cancellation and moving singularities
	int i;
	queue *this_list = new queue();
	Corner *temp_c;
	Triangle *tri = source->tri;

	for (i=0; i<ntris; i++)
		tlist[i]->temp_use = 0;  

	this_list->push(tri->index);

	while (this_list->is_empty()) {
		int mm = this_list->pop();
		if (tlist[mm]->temp_use == 1)
			continue;
		tlist[mm]->temp_use = 1;
		(*region_repellor)->append(tlist[mm]);

		for (i=0; i<3; i++) {
			temp_c = clist[3*mm+i];
			if (temp_c->o == NULL) 
				continue;
			if (temp_c->o->t->the_singularity != NULL && temp_c->o->t != tri && temp_c->o->t != the_tri)
				continue;
			if (temp_c->o->t->temp_use == 1)
				continue;
			if (temp_c->transverse == 1)  // this is different from 3D surface since here the mesh is right-handed, there is left-handed
				continue;
			this_list->push(temp_c->o->t->index);
		}
	}
	for (i=0; i<ntris; i++)
		tlist[i]->clicked = 0;
	for (i=0; i<(*region_repellor)->num_tris; i++)
		(*region_repellor)->tris[i]->clicked = 1;
	//delete(this_list);
	this_list->finalize();
}

int Polyhedron::find_region_for_attractor(Singularity *source, Triangle *the_tri, LineSegmentList *vertical[2], TriangleList *tl_constraint, TriangleList **tl_attractor)
{
	// the attractor is a saddle in pair cancellation, and a regular point in moving singularities
	int i, flag;
	SingularityList *sl = new SingularityList(2);
	TriangleList *tl_base, *tl_region[2], *tl_region_temp1, *tl_region_temp;
	double percentage[2];
	
	int which_operation = 1;
	sl->append(source);
	if (the_tri->the_singularity != NULL) {
		sl->append(the_tri->the_singularity);
		which_operation = 0;
	}

	tl_region[0] = new TriangleList(20);
	tl_region[1] = new TriangleList(20);
	for (i=0; i<2; i++) {
		int old_num_tris;
		double min_percent=0.0, max_percent=2.0, current_percent=max_percent;
	
		old_num_tris = 0;
		int lower_max;
		while (max_percent - min_percent >= 1.0e-2) {
			flag = 0;
			current_percent=(max_percent+min_percent)*0.5;
			tl_base = new TriangleList(20);
			find_base_region_based_on_sepatatrix(vertical[i], current_percent, &tl_base);
			if (tl_base->num_tris == old_num_tris) {
				tl_base->finalize();
				if (lower_max == 1)
					max_percent = current_percent;
				else
					min_percent = current_percent;
				continue;
			}
			if (contain_other_critical_pts(sl, tl_base, tl_constraint) == 1) {
				old_num_tris = tl_base->num_tris;
				tl_base->finalize();
				max_percent = current_percent;
				lower_max = 1;
				flag = 1;
				continue;
			}
			tl_region_temp1 = new TriangleList(20);
			find_region_based_on_triangle_list(1, tl_base, &(tl_region_temp1));
			tl_region_temp = new TriangleList(20);
			find_intersection(tl_region_temp1, tl_constraint, &tl_region_temp);
			tl_region_temp1->finalize();
			if (check_conley_condition(which_operation, sl, &tl_region_temp) == 0) {
				old_num_tris = tl_base->num_tris;
				tl_base->finalize();
				tl_region_temp->finalize();
				max_percent = current_percent;
				lower_max = 1;
				flag = 1;
				continue;
			}
			min_percent = current_percent;
			old_num_tris = tl_base->num_tris;
			lower_max = 0;
			tl_base->finalize();
			tl_region[i]->finalize();
			tl_region[i] = tl_region_temp;
		}
		percentage[i] = current_percent;
	}
	if (tl_region[0]->num_tris == 0 && tl_region[1]->num_tris == 0) {
		tl_region[0]->finalize();
		tl_region[1]->finalize();
		delete(sl);
		return 0;
	}

	tl_region_temp1 = new TriangleList(20);
	find_union(tl_region[0], tl_region[1], &tl_region_temp1);
	(*tl_attractor) = new TriangleList(20);
	find_intersection(tl_region_temp1, tl_constraint, tl_attractor);
	tl_region_temp1->finalize();
	tl_region[0]->finalize();
	tl_region[1]->finalize();

	TriangleList *temp_pl = new TriangleList((*tl_attractor)->num_tris);
	for (i=0; i<(*tl_attractor)->num_tris; i++)
		temp_pl->append((*tl_attractor)->tris[i]);

	if (check_conley_condition(which_operation, sl, &temp_pl) == 1) {
		temp_pl->finalize();
		delete(sl);
		return 1;
	}

	temp_pl->finalize();

	double percentage_ratio = 0.618;
	while (percentage_ratio > 0.05) {
		// rebuild triangles
		for (i=0; i<2; i++) {
			tl_base = new TriangleList(20);
			find_base_region_based_on_sepatatrix(vertical[i], percentage_ratio*percentage[i], &tl_base);
			tl_region_temp = new TriangleList(20);
			find_region_based_on_triangle_list(1, tl_base, &(tl_region_temp));
			tl_base->finalize();
			tl_region[i] = tl_region_temp;
		}
		(*tl_attractor)->finalize();

		tl_region_temp1 = new TriangleList(20);
		find_union(tl_region[0], tl_region[1], &tl_region_temp1);
		(*tl_attractor) = new TriangleList(20);
		find_intersection(tl_region_temp1, tl_constraint, tl_attractor);
		tl_region_temp1->finalize();
		tl_region[0]->finalize();
		tl_region[1]->finalize();
		temp_pl = new TriangleList((*tl_attractor)->num_tris);
		for (i=0; i<(*tl_attractor)->num_tris; i++)
			temp_pl->append((*tl_attractor)->tris[i]);
		if (check_conley_condition(which_operation, sl, &temp_pl) == 1) {
			temp_pl->finalize();
			delete(sl);
			return 1;
		}
		temp_pl->finalize();
		percentage_ratio *= 0.618;
	}

	delete(sl);
	return 0;
}

int Polyhedron::find_common_region_for_moving_source(Singularity *index_pos, Triangle *the_tri, TriangleList **tl)
{
	TriangleList *ts;

	ts = new TriangleList(20);
	find_region_for_repeller(index_pos, the_tri, &ts);

	LineSegmentList *vertical[2];
	vertical[0] = move_trace[1];
	vertical[1] = move_trace[2];

	if (find_region_for_attractor(index_pos, the_tri, vertical, ts, tl) == 0) {
		ts->finalize();
		return 0;
	}
	ts->finalize();
	return 1;
}

void Polyhedron::calc_all_lamdas()
{
	int i, j;

	for (i=0; i<nverts; i++) {
		vlist[i]->floater_lamda = 0.0;
	}
	for (i=0; i<nedges; i++) {
		// floater's metric
		elist[i]->floater_lamda[0] = elist[i]->floater_lamda[1] = 1.0/elist[i]->length;
	}
	for (i=0; i<nverts; i++) {
		for (j=0; j<vlist[i]->adjacent_edges->num_edges; j++) {
			if (vlist[i] == vlist[i]->adjacent_edges->edges[j]->verts[0]) {
				vlist[i]->floater_lamda += vlist[i]->adjacent_edges->edges[j]->floater_lamda[0];
			} else {
				vlist[i]->floater_lamda += vlist[i]->adjacent_edges->edges[j]->floater_lamda[1];
			}
		}
	}
	for (i=0; i<nedges; i++) {
		elist[i]->floater_lamda[0] /= elist[i]->verts[0]->floater_lamda;
		elist[i]->floater_lamda[1] /= elist[i]->verts[1]->floater_lamda;
	}
}

void Polyhedron::smooth_fld_in_region(int vec_ten_mode, TriangleList *tl)
{
	int i, j;
	int total_unknown = 0;
	int total_known = 0;
	double temp_f;

	for (i=0; i<nverts; i++) {
		vlist[i]->temp_use = 0;
	}

	for (i=0; i<nedges; i++)
		elist[i]->temp_use = 0;
	
	for (i=0; i<tl->num_tris; i++) {
		for (j=0; j<3; j++) {
			tl->tris[i]->edges[j]->temp_use++;
		}
	}

	for (i=0; i<nedges; i++) {
		if (elist[i]->temp_use == 2) {
			elist[i]->verts[0]->temp_use = 1;
			elist[i]->verts[1]->temp_use = 1;
		}
	}

	for (i=0; i<nedges; i++) {
		if (elist[i]->temp_use == 1) {
			elist[i]->verts[0]->temp_use = 2;
			elist[i]->verts[1]->temp_use = 2;
		}
	}

	for (i=0; i<nverts; i++) {
		if (vlist[i]->temp_use == 1) {
			total_unknown++;
		} else if (vlist[i]->temp_use == 2) {                                        // the vertex is on the boundary of the region
			total_known++;
		}
	}
	if (total_unknown == 0)  // nothing to do
		return;
	if (total_unknown == nverts)  // can not do it
		return;

	int *unknown = new int[total_unknown];
	double *u=new double[total_unknown];
	double *v=new double[total_unknown];

	int count=0;
	int count_correlation=0;
	for (i=0; i<nverts; i++) {
		vlist[i]->pos_in_matrix = -1;
		if (vlist[i]->temp_use != 1)
			continue;
		unknown[count] = i;
		vlist[i]->pos_in_matrix = count;
		count++;
		for (j=0; j<vlist[i]->adjacent_edges->num_edges; j++) {
			Edge *temp_e = vlist[i]->adjacent_edges->edges[j];

			Vertex *the_other;
			if (temp_e->verts[0] == vlist[i])
				the_other = temp_e->verts[1];
			else
				the_other = temp_e->verts[0];
			if (the_other->temp_use == 1)
				count_correlation++;
		}
	}
	
	double *sa = new double[total_unknown+1+count_correlation];
	int *ija = new int[total_unknown+1+count_correlation];

	for (i=0; i<total_unknown; i++) 
		u[i] = v[i] = 0.0;
	ija[0] = total_unknown + 1;
	int temp_count = ija[0];
	for (i=0; i<total_unknown; i++) {
		sa[i] = 1.0;
		Vertex *temp_v = vlist[unknown[i]];
		for (j=0; j<temp_v->adjacent_edges->num_edges; j++) {
			Edge *temp_e = temp_v->adjacent_edges->edges[j];
			Vertex *the_other;
			if (temp_e->verts[0] == temp_v) {
				the_other = temp_e->verts[1];
				temp_f = temp_e->floater_lamda[0];
			}	else {
				the_other = temp_e->verts[0];
				temp_f = temp_e->floater_lamda[1];
			}
			if (the_other->temp_use == 2) {
				if (vec_ten_mode == 0) {
					u[i] += the_other->vec_fld.entry[0]*temp_f;  // floater spring
					v[i] += the_other->vec_fld.entry[1]*temp_f;  // floater spring
				} else {
					u[i] += the_other->ten_fld.entry[0][0]*temp_f;  // floater spring
					v[i] += the_other->ten_fld.entry[0][1]*temp_f;  // floater spring
				}
			}
			else if (the_other->temp_use == 1) {
				sa[temp_count] = -temp_f;
				ija[temp_count] = the_other->pos_in_matrix;
				temp_count++;
			}
		}
		ija[i+1]=temp_count;
	}

	double *sol_u = new double[total_unknown], *sol_v = new double[total_unknown];
	double try_threshold = 1.0e-12;
	solve_linear_BiCG(sa, ija, sol_u, u, total_unknown, try_threshold);
	solve_linear_BiCG(sa, ija, sol_v, v, total_unknown, try_threshold);

	if (vec_ten_mode == 0) {
		for (i=0; i<total_unknown; i++) {
			vlist[unknown[i]]->vec_fld.set(sol_u[i], sol_v[i]);
			normalize(vlist[unknown[i]]->vec_fld);
		}
	} else {
		for (i=0; i<total_unknown; i++) {
			vlist[unknown[i]]->ten_fld.set(sol_u[i], sol_v[i], sol_v[i], -sol_u[i]);
			double len = sqrt(vlist[i]->ten_fld.entry[0][0]*vlist[i]->ten_fld.entry[0][0]+vlist[i]->ten_fld.entry[0][1]*vlist[i]->ten_fld.entry[0][1]);
			if (len > 0.0)
				vlist[i]->ten_fld *= 1.0/len;
			if (vlist[i]->ten_fld != 0.0) {
				double angle = atan2(vlist[i]->ten_fld.entry[0][1], vlist[i]->ten_fld.entry[0][0])*0.5;
				vlist[i]->vec_fld.set(cos(angle), sin(angle));
			}  else {
				vlist[i]->vec_fld.set(0.0);
			}
		}
	}
	delete(sa);
	delete(ija);
	delete(u);
	delete(v);
	delete(sol_u);
	delete(sol_v);
	delete(unknown);
}

int Polyhedron::singularity_movement(int sid, int tid, double rotate_angle)
{
	int i;

	Singularity *ss;
	Triangle *tri_src = tlist[sid];
	TriangleList *tl = new TriangleList(20);

	global_rotate_field_tool(rotate_angle);
	analyze_field(0, -1);  // this way, sepatrices are not computed
	ss = tri_src->the_singularity;

	if (build_traces_for_moving(tlist[tid], icVector2((tlist[tid]->x1+tlist[tid]->x2)/3.0, tlist[tid]->y2/3.0), ss, 0.05) == 0) {
		tl->finalize();
		global_rotate_field_tool(-rotate_angle);
		analyze_field(0, -1);
		return 0;  // fail to connect
	}
  if (find_common_region_for_moving_source(ss, tlist[tid], &tl)==0) {
		tl->finalize();
		global_rotate_field_tool(-rotate_angle);
		analyze_field(0, -1);   
		return 0;
	}

	CornerList *cl = new CornerList(20);
	find_corners_for_simply_connected_region(tl, &cl);

	icVector2 imagine[3];
	imagine[0].set(-(tlist[tid]->x1+tlist[tid]->x2)/3.0, -tlist[tid]->y2/3.0);
	imagine[1].set(tlist[tid]->x1-(tlist[tid]->x1+tlist[tid]->x2)/3.0, -tlist[tid]->y2/3.0);
	imagine[2].set(tlist[tid]->x2-(tlist[tid]->x1+tlist[tid]->x2)/3.0, tlist[tid]->y2-tlist[tid]->y2/3.0);
	for (i=0; i<3; i++) {
		tlist[tid]->convert_local_to_global_vector(imagine[i], &(tlist[tid]->verts[i]->vec_fld));
		normalize(tlist[tid]->verts[i]->vec_fld);
		tlist[tid]->verts[i]->vec_fld *= 10.0;
	}
	int which_tris;
	for (i=0; i<tl->num_tris; i++) {
		if (tl->tris[i] == tlist[tid]) {
			which_tris = i;
			break;
		}
	}
	if (which_tris < tl->num_tris-1) 
		tl->tris[which_tris] = tl->tris[tl->num_tris-1];
	tl->num_tris--;
	smooth_fld_in_region(0, tl);
	for (i=0; i<3; i++) 
		normalize(tlist[tid]->verts[i]->vec_fld);
	tl->finalize();
	cl->finalize();
	global_rotate_field_tool(-rotate_angle);
	return 1;
}

void Polyhedron::find_next_corner_for_edge(Corner *start_c, Corner **result_c)
{
	Corner *temp_c;

	temp_c = start_c;
	while (temp_c->p->e->temp_use != 1) {
		temp_c = temp_c->p->o;
	}
	(*result_c) = temp_c->p;
}

void Polyhedron::find_corners_for_simply_connected_region(TriangleList *tl, CornerList **cl)
{
	int i, j, eid;
	Corner *start_c, *end_c, *temp_c;

	for (i=0; i<nverts; i++)
		vlist[i]->temp_use = 0;
	for (i=0; i<nedges; i++) 
		elist[i]->temp_use = 0;
	for (i=0; i<ntris; i++) 
		tlist[i]->temp_use = 0;
	for (i=0; i<tl->num_tris; i++) {
		tl->tris[i]->temp_use = 1;
		for (j=0; j<3; j++)
			tl->tris[i]->edges[j]->temp_use++;
	}
	for (i=0; i<nedges; i++) {
		if (elist[i]->temp_use == 2) {
			elist[i]->temp_use = 0;
			continue;
		}
	}		
	// now only the edges on the boundary of the region is left with being 1
	for (i=0; i<nedges; i++) {
		if (elist[i]->temp_use == 1) {
			eid = i;
			break;
		}
	}
	for (i=0; i<elist[eid]->ntris; i++) {
		if (elist[eid]->tris[i]->temp_use == 0)
			continue;
		for (j=0; j<3; j++) {
			if (elist[eid]->tris[i]->corners[j]->e == elist[eid]) {
				end_c = elist[eid]->tris[i]->corners[j];
				(*cl)->append(end_c);
				find_next_corner_for_edge(end_c, &start_c);
				(*cl)->append(start_c);
				temp_c = start_c;
				break;
			}
		}
		break;
	}
	while (temp_c != end_c) {
		find_next_corner_for_edge(temp_c, &temp_c);
		if (temp_c == end_c)
			break;
		(*cl)->append(temp_c);
	}
}

void Polyhedron::reset_zero_field() {
	/*
	singularelements->finalize();
	regularelements->finalize();
	singularities->finalize();
	singularelements = new SingularElementList(20);
	regularelements = new RegularElementList(20);
	singularities = new SingularityList(20);

	if(vec_ten_mode == 0) {
		update_vecfld(1);
	}
	else {
		update_tenfld(1, 0.0);
	}

	analyze_field(1, -1);
	calc_new_texture_coors();
	*/
}


/**
void Polyhedron::removeElementsFromField()
{
	singularelements->finalize();
	regularelements->finalize();
	singularities->finalize();
	singularelements = new SingularElementList(20);
	regularelements = new RegularElementList(20);
	singularities = new SingularityList(20);

	if(vec_ten_mode == 0) {
		update_vecfld(0);
	}
	else {
		update_tenfld(0, 1.);
	}

	analyze_field(1, -1);
	calc_new_texture_coors();
}
/**/

void Polyhedron::undo() {
	/**
	int i;

	if (vec_ten_mode == 0) {
		for (i=0; i<nverts; i++) {
			vlist[i]->redo_vec_fld = vlist[i]->vec_fld;
			vlist[i]->vec_fld = vlist[i]->undo_vec_fld;
		}
	} else {
		for (i=0; i<nverts; i++) {
			vlist[i]->redo_ten_fld = vlist[i]->ten_fld;
			vlist[i]->ten_fld = vlist[i]->undo_ten_fld;
		}
	}

	analyze_field(1, -1);
	calc_new_texture_coors();
	/**/
}


void Polyhedron::redo() {
	/**
	int i;

	if (vec_ten_mode == 0) {
		for (i=0; i<nverts; i++) {
			vlist[i]->undo_vec_fld = vlist[i]->vec_fld;
			vlist[i]->vec_fld = vlist[i]->redo_vec_fld;
		}
	} else {
		for (i=0; i<nverts; i++) {
			vlist[i]->undo_ten_fld = vlist[i]->ten_fld;
			vlist[i]->ten_fld = vlist[i]->redo_ten_fld;
		}
	}

	analyze_field(1, -1);
	calc_new_texture_coors();
	/**/
}



icMatrix2x2 Polyhedron::apply_T(const Vertex &v, const list<SingularElement> &se, const list<RegularElement> &re, float weight_interpolate) {
	double len;
	icVector2 temp;
	icMatrix2x2 ret_val(0, 0, 0, 0);
	list<SingularElement>::const_iterator se_itr, se_itr_end;
	list<RegularElement>::const_iterator re_itr, re_itr_end;

	for(se_itr = se.begin(), se_itr_end = se.end(); se_itr != se_itr_end; se_itr++) {
		// apply when both the element and the vertex is in the same region or the element is global
		if(v.regionID == se_itr->regionID || se_itr->regionID < 0) {
			temp.set(v.x - se_itr->x, v.y - se_itr->y);	
			len = length(temp);
			normalize(temp);

			if(se_itr->order == 0) {
				icVector2 temp_vec = se_itr->mat * temp;
				icMatrix2x2 temp_result(temp_vec.entry[0], temp_vec.entry[1], temp_vec.entry[1], -temp_vec.entry[0]);
				temp_result.rightMultiply(se_itr->rot);
				temp_result.leftMultiply(se_itr->rot_inv);
				ret_val += inverse_dist(50.0 * len, 1.0e-1) * temp_result * (1.0 - weight_interpolate);

// =========== begin higher order stuff ==========================
//				double pp_angle = 2.0*atan2(temp.entry[1], temp.entry[0]);
//				temp_result.set(cos(pp_angle), sin(pp_angle), 0.0,
//												sin(pp_angle), -cos(pp_angle), 0.0,
//												0.0, 0.0, 0.0);
//				vlist[i]->ten_fld += inverse_dist(len) * temp_result;
// ==========  end higher order stuff ===============================

			}
			else {
				icVector2 temp_vec = se_itr->mat * temp;
				double entry[2];
				
				for(int pp = 0; pp < 2; pp++) {
					entry[pp] = dot(temp_vec, se_itr->mat_second[pp] * temp_vec);
				}

				icMatrix2x2 temp_result(entry[0], entry[1], entry[1], -entry[0]);
				temp_result.rightMultiply(se_itr->rot);
				temp_result.leftMultiply(se_itr->rot_inv);
				ret_val += inverse_dist(50.0 * len, 1.0e-1) * temp_result * (1.0 - weight_interpolate);
			}
		}
	}

	for(re_itr = re.begin(), re_itr_end = re.end(); re_itr != re_itr_end; re_itr++) {
		double angle = 2.0 * atan2(re_itr->vy, re_itr->vx);
		icMatrix2x2 temp_mat(cos(angle), sin(angle), sin(angle), -cos(angle));

		// apply when both the element and the vertex is in the same region or the element is global
		if(v.regionID == re_itr->regionID || re_itr->regionID < 0) {
			temp.set(v.x - re_itr->x, v.y - re_itr->y);
			len = length(temp);
			ret_val += inverse_dist(len * 50.0, 1.0e-1) * re_itr->strength * temp_mat * (1.0 - weight_interpolate);
		}
	}

	return ret_val;
}


/*
	target_region_id:		specify region ID to make change 
	do_set_zero:			initialize tensor field to zero if nonzero. Otherwise, edge field from image is used
	weight_interoplate:		blending factor between initial tensor field and design elements
	design_elements:		a set of design elements at current frame defined on each region
*/
void Polyhedron::update_tenfld(int target_region_id, int do_set_zero, double weight_interpolate, const std::vector<const DEInfo*> &design_elements) {
	icVector2 temp;
	icMatrix2x2 temp1;
	const DEInfo *de;
	Vertex *v;

	// clear flag
	for(int i = 0; i < ntris; i++) tlist[i]->clicked = 0;


	//do_set_zero = 1;


	// initialize tensor field
	for(int i = 0; i < nverts; i++) {
		v = vlist[i];

		if(target_region_id >= 0 && target_region_id != v->regionID) continue;

		if(do_set_zero == 1 || v->use_zero_value == true) {
			v->ten_fld.set(0.0);
			//v->ten_fld_painting.set(0.0);
		}
		else {
			v->ten_fld = v->ten_fld_painting * weight_interpolate;
		}
	}

	// interpolate with design elements.
	for(int i = 0; i < nverts; i++) {
		v = vlist[i];

		if(v->regionID < 0) continue;		// this vertex doesn't belong to any region...

		if(target_region_id >= 0 && target_region_id != v->regionID) continue;

		de = design_elements[v->regionID];

		assert(de != NULL);

		if(de->diffusion == true) continue;

		if(de->numKF() > 0) {
			v->ten_fld += apply_T(*v, de->se.kf, de->re.kf, weight_interpolate);
		}

		if(de->numFwd() == 0) {
			if(de->numBwd() == 0) {
				// no design elements for this region... do nothing.
				continue;
			}
			else {
				// only have elements from previous keyframe
				v->ten_fld += apply_T(*v, de->se.bwd_elems, de->re.bwd_elems, weight_interpolate);
			}
		}
		else {
			if(de->numBwd() == 0) {
				v->ten_fld += apply_T(*v, de->se.fwd_elems, de->re.fwd_elems, weight_interpolate);
			}
			else {
				v->ten_fld += apply_T(*v, de->se.fwd_elems, de->re.fwd_elems, weight_interpolate) * de->weight_forward;
				v->ten_fld += apply_T(*v, de->se.bwd_elems, de->re.bwd_elems, weight_interpolate) * (1.f - de->weight_forward);
			}
		}
	}

	// scale
	/**
	for(int j = 0; j < nverts; j++) {
		double temp_len, product;

		product = vlist[j]->ten_fld.entry[0][0]*vlist[j]->ten_fld.entry[0][0]+vlist[j]->ten_fld.entry[0][1]*vlist[j]->ten_fld.entry[0][1];

		if(product == 0) {
			product += 1.e-10;
		}


		temp_len = 1 / product;

		if(product != 0) {
			temp_len = 1 / product;

			vlist[j]->ten_fld *= temp_len;
		}
		else {
		}
	}
	/**/
}

void Polyhedron::convert_vecfld_to_tenfld_tool()
{
	int i; 

	for (i=0; i<nverts; i++) {
		vlist[i]->ten_fld.set(vlist[i]->vec_fld.entry[0], vlist[i]->vec_fld.entry[1],
													vlist[i]->vec_fld.entry[1], -vlist[i]->vec_fld.entry[0]);
	}
}

void Polyhedron::convert_vecfld_to_tenfld()
{
	convert_vecfld_to_tenfld_tool();
	analyze_field(1, -1);
	calc_new_texture_coors();
}

void Polyhedron::convert_tenfld_to_vecfld_tool()
{
	int i;

	for (i=0; i<nverts; i++) {
		vlist[i]->vec_fld.set(vlist[i]->ten_fld.entry[0][0], vlist[i]->ten_fld.entry[0][1]);
	}
}

void Polyhedron::convert_tenfld_to_vecfld()
{
	convert_tenfld_to_vecfld_tool();
	analyze_field(1, -1);
	calc_new_texture_coors();
}


void Polyhedron::compute_tensor_separatrices()
{
	int i, j, pp;
	icVector3 temp_vec;
	icMatrix2x2 temp_m;
	icVector2 solution;
	LineSegmentList *ls, *temp_ls;
	LineSegment *lls;
	
	if (singularities->num_singularities == 0)
		return;
	int ratio_num_triangles = ntris / singularities->num_singularities, num_triangles;

	// compute tensor separatrices
	for (i=0; i<singularities->num_singularities; i++) {
		Singularity *ss = singularities->singularities[i];
	
		if (ss->vecten_index == 1) {
			num_triangles = ratio_num_triangles/20;
			if (num_triangles < 30)
				num_triangles = 30;
		} else {
			num_triangles = ratio_num_triangles/20;
			if (num_triangles < 200)
				num_triangles = 200;
		}

		for (pp=0; pp<ss->num_tensor_dir; pp++) {
			ls = new LineSegmentList(40);
			ls->direction = 0;
			lls = new LineSegment();
			lls->start_coor = ss->pos;
			lls->end_coor = lls->start_coor + saddle_out * ss->tensor_dir[pp];
			ls->len = lls->len = length(lls->end_coor - lls->start_coor);
			lls->tri = ss->tri;
			lls->calc_display_attributes();
			ls->append(lls);
			temp_ls = new LineSegmentList(2000);
			trace_one_tensor_trajectory_in_domain(500, lls->end_coor, 0, ss->tensor_dir[pp], lls->tri, &temp_ls);
			for (j=0; j<temp_ls->num_linesegments; j++){
				ls->append(temp_ls->linesegments[j]);
				temp_ls->linesegments[j]->len = length(temp_ls->linesegments[j]->end_coor - temp_ls->linesegments[j]->start_coor);
				ls->len += temp_ls->linesegments[j]->len;
			}
			delete(temp_ls);
			ss->separatrices_major->append(ls);

			ls = new LineSegmentList(40);
			ls->direction = 0;
			lls = new LineSegment();
			lls->start_coor = ss->pos;
			lls->end_coor = lls->start_coor - saddle_out * ss->tensor_dir[pp];
			ls->len = lls->len = length(lls->end_coor - lls->start_coor);
			lls->tri = ss->tri;
			lls->calc_display_attributes();
			ls->append(lls);
			temp_ls = new LineSegmentList(2000);
		if (i == 51 && pp == 1)
			int gogo = 1;
			trace_one_tensor_trajectory_in_domain(500, lls->end_coor, 1, -1.0*ss->tensor_dir[pp], lls->tri, &temp_ls);
			for (j=0; j<temp_ls->num_linesegments; j++){
				ls->append(temp_ls->linesegments[j]);
				temp_ls->linesegments[j]->len = length(temp_ls->linesegments[j]->end_coor - temp_ls->linesegments[j]->start_coor);
				ls->len += temp_ls->linesegments[j]->len;
			}
			delete(temp_ls);
			ss->separatrices_minor->append(ls);
		}
	}
}

// return values:
// 0 = continue to another triangle
// 1 = stop due to a critial point
// -1 = stop due to a boundary exit or entry
int Polyhedron::trace_one_tensor_trajectory_in_one_triangle(icVector2 start_pos, unsigned char direction, icVector2 initial_dir, Triangle *tri, Edge **incoming_edge, Triangle **next_tri, icVector2 *next_start_pos, icVector2 *next_initial_dir, LineSegmentList **ls)
{
	double a[3];
	int nok, nbad;
	int i, j;
	icVector2 temp_vec2, temp_x_dir, temp_y_dir, temp_vec[3], temp_vec_t[2];
	icVector2 temp_vec3;
	LineSegment *ll;
	double temp_x, temp_y, temp_len;
	double bary_a, bary_b;
	Edge *temp_e;
	int tested_signs[3];
	Edge *old_incoming_edge = *incoming_edge;
	double initial_dir_this[3];
	int duration = 0;

	initial_dir_this[1] = initial_dir.entry[0];
	initial_dir_this[2] = initial_dir.entry[1];

	*ls = new LineSegmentList(5);

	xp = XP;
	Yp[1] = Yp1;
	Yp[2] = Yp2;
	yp = Yp;
	DYDXp[1] = DYDXp1;
	DYDXp[2] = DYDXp2;

	a[1] = start_pos.entry[0];
	a[2] = start_pos.entry[1];

	while (duration < 10) {
		if (direction == 0)
			odeint_tensor(tri->vecten_a, tri->vecten_b, tri->vecten_c, tri->vecten_d, tri->vecten_e, tri->vecten_f, initial_dir_this, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_tensor_in_triangle, rkqs_tensor);
		else
			odeint_tensor(-tri->vecten_a, -tri->vecten_b, -tri->vecten_c, -tri->vecten_d, -tri->vecten_e, -tri->vecten_f, initial_dir_this, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_tensor_in_triangle, rkqs_tensor);
		for (i=1; i<kount; i++) {
			ll = new LineSegment();
			ll->tri = tri;
			ll->start_coor.set(yp[1][i], yp[2][i]);
			ll->end_coor.set(yp[1][i+1], yp[2][i+1]);
			tri->convert_local_to_global(ll->start_coor, &temp_vec3);
			tri->convert_local_to_global(ll->end_coor, &temp_vec3);
			(*ls)->append(ll);
			
			// check to see whether it is close to a singular point
			if (tri->the_singularity != NULL) {
				// the end position is close to the critical point
				if (tri->the_singularity == NULL) {
					ll->calc_display_attributes();
					return 1;
				}

				if (length(tri->the_singularity->pos - ll->end_coor) < 1.0e-6) {
					ll->end_coor = tri->the_singularity->pos;
					ll->calc_display_attributes();
					return 1;
				}

				// the segment is close to the critical point
				temp_x_dir = ll->end_coor-ll->start_coor;
				temp_len = length(temp_x_dir);
				normalize(temp_x_dir);
				temp_y_dir.set(temp_x_dir.entry[1], -temp_x_dir.entry[0]);
				temp_vec2 = tri->the_singularity->pos - ll->start_coor;
				temp_x = dot(temp_vec2, temp_x_dir);
				temp_y = dot(temp_vec2, temp_y_dir);
				if (fabs(temp_y) < 1.0-6 && (temp_x > 0.0 && temp_x <= temp_len)) {
					ll->end_coor = tri->the_singularity->pos;
					ll->calc_display_attributes();
					return 1;
				}

				if (duration > 10) {
					ll->end_coor = tri->the_singularity->pos;
					ll->calc_display_attributes();
					return 1;
				}
			}
			
			// check to see whether it runs into an edge
			// if a boundary edge, finalize
			// if not a boundary edge, continue into another triangle
			tri->get_bary_coor(ll->start_coor, &bary_a, &bary_b);
			if (bary_a < 0.0 || bary_b < 0.0 || 1-bary_a-bary_b < 0.0) {
				int gogo=1;
			}
			tri->get_bary_coor(ll->end_coor, &bary_a, &bary_b);
			if (bary_a < 0.0 || bary_b < 0.0 || 1-bary_a-bary_b < 0.0) {
				temp_vec[0].set(0.0, 0.0);
				temp_vec[1].set(tri->x1, 0.0);
				temp_vec[2].set(tri->x2, tri->y2);
				for (j=0; j<3; j++)
					tested_signs[j] = test_sign(ll->start_coor, ll->end_coor, temp_vec[j]);

/*				
				for (j=0; j<3; j++) {
					if (tested_signs[j] == 0) {
						// the j-th vertex is on the line segment.
						switch (j) {
						case 0:
							ll->end_coor.set(0.0, 0.0);
							break;

						case 1:
							ll->end_coor.set(tri->x1, 0.0);
							break;

						case 2:
							ll->end_coor.set(tri->x2, tri->y2);
							break;
						}
						// find the next triangle and starting position

					}
				}
*/
				temp_vec_t[0] = ll->start_coor;
				temp_vec_t[1] = ll->end_coor;
				double min_dist = 1.0e+20, current_dist;
				icVector2 temp_end_coor;
				for (j=0; j<3; j++) {
					temp_e = tri->edges[j];
					if (ll->start_coor == start_pos && temp_e == old_incoming_edge)
						continue;
					if (tested_signs[j] * tested_signs[(j+1) % 3] == -1) {
						// the j-th edge is on the line segment
						if (get_intersection(temp_vec[j], temp_vec[(j+1) % 3], temp_vec_t, &(temp_end_coor)) == 0)
							continue;
						current_dist = dot(ll->end_coor-ll->start_coor, temp_end_coor-ll->start_coor);
						if (current_dist < 0.0)
							continue;
//						if (current_dist < min_dist) {
							min_dist = current_dist;
							ll->end_coor = temp_end_coor;
							*incoming_edge = temp_e;
							ll->calc_display_attributes();
//						}
					}
				}
				if ((*incoming_edge) == NULL)
					return -1;
				if ((*incoming_edge)->ntris == 1)
					return -1;
				if ((*incoming_edge)->tris[0] == tri) 
					*next_tri = (*incoming_edge)->tris[1];
				else
					*next_tri = (*incoming_edge)->tris[0];
				tri->convert_local_to_global(ll->end_coor, &temp_vec3);
				(*next_tri)->convert_global_to_local(temp_vec3, next_start_pos);
				icVector2 temp_vec2_dir = ll->end_coor - ll->start_coor;
				icVector2 temp_vec3_dir;

				tri->convert_local_to_global_vector(temp_vec2_dir, &temp_vec3_dir);
				(*next_tri)->convert_global_to_local_vector(temp_vec3_dir, next_initial_dir);
				normalize(*next_initial_dir);
				return 0;
			} else 
				ll->calc_display_attributes();
			// check to see whether it runs into an vertex
			// if a boundary vertex, finalize
			// if not a boundary vertex, continue into another triangle

			// finally, continue inside this triangle
		}
		duration++;
	}
	return 1;
}


void Polyhedron::retrieve_divergence_field()
{
	int i, j;
	icVector2 temp, temp_global;

	for (i=0; i<nverts; i++) 
		vlist[i]->div_fld.set(0.0);
	for (i=0; i<ntris; i++) {
		temp.set(tlist[i]->vecten_a + tlist[i]->vecten_d, tlist[i]->vecten_e - tlist[i]->vecten_b);
		tlist[i]->convert_local_to_global_vector(temp, &temp_global);
		for (j=0; j<3; j++) 
			tlist[i]->verts[j]->div_fld += temp_global;
	}
	for (i=0; i<nverts; i++) { 
//		vlist[i]->div_fld *= 1.0/vlist[i]->ntris;
		normalize(vlist[i]->div_fld);
		vlist[i]->vec_fld = vlist[i]->div_fld;
	}

	for (i=0; i<nverts; i++)
		vlist[i]->div_fld.set(0.0);
	for (i=0; i<ntris; i++) {
		icMatrix2x2 temp_mat(tlist[i]->verts[1]->x - tlist[i]->verts[0]->x, tlist[i]->verts[1]->y - tlist[i]->verts[0]->y,
												 tlist[i]->verts[2]->x - tlist[i]->verts[0]->x, tlist[i]->verts[2]->y - tlist[i]->verts[0]->y);
		icMatrix2x2 global_tensor[3];
		for (j=0; j<3; j++)
			global_tensor[j] = tlist[i]->verts[j]->ten_fld;
		
		icVector2 temp_vec;
		temp_vec = inverse(temp_mat)*icVector2(global_tensor[1].entry[0][0] - global_tensor[0].entry[0][0], global_tensor[2].entry[0][0] - global_tensor[0].entry[0][0]);
		double aa = temp_vec.entry[0], bb = temp_vec.entry[1];
		temp_vec = inverse(temp_mat)*icVector2(global_tensor[1].entry[0][1] - global_tensor[0].entry[0][1], global_tensor[2].entry[0][1] - global_tensor[0].entry[0][1]);
		double dd = temp_vec.entry[0], ee = temp_vec.entry[1];
		for (j=0; j<3; j++) 
			tlist[i]->verts[j]->div_fld += icVector2(aa+dd, ee-bb);
	}

	for (i=0; i<nverts; i++) {
		normalize(vlist[i]->div_fld);
		vlist[i]->vec_fld = vlist[i]->div_fld;
	}

	analyze_field(1, -1);
	calc_new_texture_coors();
}

int Polyhedron::trace_trajectory_in_triangle_vecfld(icVector2 start_pos, unsigned char direction, Triangle *tri, Edge **incoming_edge, Triangle **next_tri, icVector2 *next_start_pos, icVector2List **vlist)
{
	double a[3];
	int nok, nbad;
	int i, j;
	icVector2 temp_vec2, temp_x_dir, temp_y_dir, temp_vec[3], temp_vec_t[2];
	icVector2 temp_vec3;
	LineSegment *ll;
	double temp_x, temp_y, temp_len;
	double bary_a, bary_b;
	Edge *temp_e;
	int tested_signs[3];
	Edge *old_incoming_edge = *incoming_edge;
	int duration = 0;

	icVector2 *pos = new icVector2, the_start_pos, *the_end_pos;
	*pos = start_pos;

	(*vlist) = new icVector2List(40);
	(*vlist)->append(pos);


	xp = XP;
	Yp[1] = Yp1;
	Yp[2] = Yp2;
	yp = Yp;

	a[1] = start_pos.entry[0];
	a[2] = start_pos.entry[1];

	while (duration < 10) {
		if (direction == 0) 
			odeint(tri->vecten_a, tri->vecten_b, tri->vecten_c, tri->vecten_d, tri->vecten_e, tri->vecten_f, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		else if (direction == 1)
			odeint(-tri->vecten_a, -tri->vecten_b, -tri->vecten_c, -tri->vecten_d, -tri->vecten_e, -tri->vecten_f, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		else if (direction == 2)
			odeint(-tri->vecten_d, -tri->vecten_e, -tri->vecten_f, tri->vecten_a, tri->vecten_b, tri->vecten_c, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		else if (direction == 3)
			odeint(tri->vecten_d, tri->vecten_e, tri->vecten_f, -tri->vecten_a, -tri->vecten_b, -tri->vecten_c, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_vector_in_triangle, rkqs);
		for (i=1; i<kount; i++) {
			the_end_pos = new icVector2(yp[1][i+1], yp[2][i+1]);
			the_start_pos.set(yp[1][i], yp[2][i]);
			(*vlist)->append(the_end_pos);
			
			// check to see whether it is close to a singular point
			if (tri->the_singularity != NULL) {
				// the end position is close to the critical point
				if (length(tri->the_singularity->pos - *the_end_pos) < 1.0e-6) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}

				// the segment is close to the critical point
				temp_x_dir = *the_end_pos-the_start_pos;
				temp_len = length(temp_x_dir);
				normalize(temp_x_dir);
				temp_y_dir.set(temp_x_dir.entry[1], -temp_x_dir.entry[0]);
				temp_vec2 = tri->the_singularity->pos - the_start_pos;
				temp_x = dot(temp_vec2, temp_x_dir);
				temp_y = dot(temp_vec2, temp_y_dir);
				if (fabs(temp_y) < 1.0-6 && (temp_x > 0.0 && temp_x <= temp_len)) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}
				if (duration > 10) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}
			}
			
			// check to see whether it runs into an edge
			// if a boundary edge, finalize
			// if not a boundary edge, continue into another triangle
			tri->get_bary_coor(*the_end_pos, &bary_a, &bary_b);
			if (bary_a < 0.0 || bary_b < 0.0 || 1-bary_a-bary_b < 0.0) {
				temp_vec[0].set(0.0, 0.0);
				temp_vec[1].set(tri->x1, 0.0);
				temp_vec[2].set(tri->x2, tri->y2);
				for (j=0; j<3; j++)
					tested_signs[j] = test_sign(the_start_pos, *the_end_pos, temp_vec[j]);
				temp_vec_t[0] = the_start_pos;
				temp_vec_t[1] = *the_end_pos;
				double min_dist = 1.0e+20, current_dist;
				icVector2 temp_end_coor;
				for (j=0; j<3; j++) {
					temp_e = tri->edges[j];
					if (the_start_pos == start_pos && temp_e == old_incoming_edge)
						continue;
					if (tested_signs[j] * tested_signs[(j+1) % 3] == -1) {
						// the j-th edge is on the line segment
						if (get_intersection(temp_vec[j], temp_vec[(j+1) % 3], temp_vec_t, &(temp_end_coor)) == 0)
							continue;
						current_dist = dot(*the_end_pos-the_start_pos, temp_end_coor-the_start_pos);
						if (current_dist < 0.0)
							continue;
//						if (current_dist < min_dist) {
							min_dist = current_dist;
							*the_end_pos = temp_end_coor;
							*incoming_edge = temp_e;
					}
				}
				if ((*incoming_edge)->ntris == 1)
					return -1;
				if ((*incoming_edge)->tris[0] == tri) 
					*next_tri = (*incoming_edge)->tris[1];
				else
					*next_tri = (*incoming_edge)->tris[0];
				tri->convert_local_to_global(*the_end_pos, &temp_vec3);
				(*next_tri)->convert_global_to_local(temp_vec3, next_start_pos);
				return 0;
			}
			// check to see whether it runs into an vertex
			// if a boundary vertex, finalize
			// if not a boundary vertex, continue into another triangle

			// finally, continue inside this triangle
		}
		duration++;
	}
	return 0;
}

int Polyhedron::trace_trajectory_in_triangle_tenfld(icVector2 start_pos, unsigned char direction, icVector2 initial_dir, Triangle *tri, Edge **incoming_edge, Triangle **next_tri, icVector2 *next_start_pos, icVector2 *next_initial_dir, icVector2List **vlist)
{
	double a[3];
	int nok, nbad;
	int i, j;
	icVector2 temp_vec2, temp_x_dir, temp_y_dir, temp_vec[3], temp_vec_t[2];
	icVector2 temp_vec3;
	double temp_x, temp_y, temp_len;
	double bary_a, bary_b;
	Edge *temp_e;
	int tested_signs[3];
	Edge *old_incoming_edge = *incoming_edge;
	double initial_dir_this[3];
	int duration = 0;

	initial_dir_this[1] = initial_dir.entry[0];
	initial_dir_this[2] = initial_dir.entry[1];

	icVector2 *pos = new icVector2, the_start_pos, *the_end_pos;
	*pos = start_pos;

	(*vlist) = new icVector2List(40);
	(*vlist)->append(pos);

	xp = XP;
	Yp[1] = Yp1;
	Yp[2] = Yp2;
	yp = Yp;
	DYDXp[1] = DYDXp1;
	DYDXp[2] = DYDXp2;

	a[1] = start_pos.entry[0];
	a[2] = start_pos.entry[1];

	while (duration < 10) {
		if (direction == 0)
			odeint_tensor(tri->vecten_a, tri->vecten_b, tri->vecten_c, tri->vecten_d, tri->vecten_e, tri->vecten_f, initial_dir_this, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_tensor_in_triangle, rkqs_tensor);
		else
			odeint_tensor(-tri->vecten_a, -tri->vecten_b, -tri->vecten_c, -tri->vecten_d, -tri->vecten_e, -tri->vecten_f, initial_dir_this, a, 2, 0.0, 0.25, error_threshold_ode, ode_step, 0.0, &nok, &nbad, evaluate_tensor_in_triangle, rkqs_tensor);
		for (i=1; i<kount; i++) {
			the_end_pos = new icVector2(yp[1][i+1], yp[2][i+1]);
			the_start_pos.set(yp[1][i], yp[2][i]);
			(*vlist)->append(the_end_pos);
			
			// check to see whether it is close to a singular point
			if (tri->the_singularity != NULL) {
				// the end position is close to the critical point
				if (tri->the_singularity == NULL) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}

				if (length(tri->the_singularity->pos - *the_end_pos) < 1.0e-6) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}

				// the segment is close to the critical point
				temp_x_dir = *the_end_pos-the_start_pos;
				temp_len = length(temp_x_dir);
				normalize(temp_x_dir);
				temp_y_dir.set(temp_x_dir.entry[1], -temp_x_dir.entry[0]);
				temp_vec2 = tri->the_singularity->pos - the_start_pos;
				temp_x = dot(temp_vec2, temp_x_dir);
				temp_y = dot(temp_vec2, temp_y_dir);
				if (fabs(temp_y) < 1.0-6 && (temp_x > 0.0 && temp_x <= temp_len)) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}

				if (duration > 10) {
					*the_end_pos = tri->the_singularity->pos;
					return 1;
				}
			}
			
			// check to see whether it runs into an edge
			// if a boundary edge, finalize
			// if not a boundary edge, continue into another triangle
			tri->get_bary_coor(*the_end_pos, &bary_a, &bary_b);
			if (bary_a < 0.0 || bary_b < 0.0 || 1-bary_a-bary_b < 0.0) {
				temp_vec[0].set(0.0, 0.0);
				temp_vec[1].set(tri->x1, 0.0);
				temp_vec[2].set(tri->x2, tri->y2);
				for (j=0; j<3; j++)
					tested_signs[j] = test_sign(the_start_pos, *the_end_pos, temp_vec[j]);

				temp_vec_t[0] = the_start_pos;
				temp_vec_t[1] = *the_end_pos;
				double min_dist = 1.0e+20, current_dist;
				icVector2 temp_end_coor;
				for (j=0; j<3; j++) {
					temp_e = tri->edges[j];
					if (the_start_pos == start_pos && temp_e == old_incoming_edge)
						continue;
					if (tested_signs[j] * tested_signs[(j+1) % 3] == -1) {
						// the j-th edge is on the line segment
						if (get_intersection(temp_vec[j], temp_vec[(j+1) % 3], temp_vec_t, &(temp_end_coor)) == 0)
							continue;
						current_dist = dot(*the_end_pos-the_start_pos, temp_end_coor-the_start_pos);
						if (current_dist < 0.0)
							continue;
//						if (current_dist < min_dist) {
							min_dist = current_dist;
							*the_end_pos = temp_end_coor;
							*incoming_edge = temp_e;
//						}
					}
				}
				if ((*incoming_edge) == NULL)
					return -1;
				if ((*incoming_edge)->ntris == 1)
					return -1;
				if ((*incoming_edge)->tris[0] == tri) 
					*next_tri = (*incoming_edge)->tris[1];
				else
					*next_tri = (*incoming_edge)->tris[0];
				tri->convert_local_to_global(*the_end_pos, &temp_vec3);
				(*next_tri)->convert_global_to_local(temp_vec3, next_start_pos);
				icVector2 temp_vec2_dir = *the_end_pos - the_start_pos;
				icVector2 temp_vec3_dir;

				tri->convert_local_to_global_vector(temp_vec2_dir, &temp_vec3_dir);
				(*next_tri)->convert_global_to_local_vector(temp_vec3_dir, next_initial_dir);
				normalize(*next_initial_dir);
				return 0;
			} 
			// check to see whether it runs into an vertex
			// if a boundary vertex, finalize
			// if not a boundary vertex, continue into another triangle

			// finally, continue inside this triangle
		}
		duration++;
	}
	return 1;
}

void Polyhedron::calc_new_texture_coors()
{
	double vec_len;
	icVector2 temp_vec;
	Vertex *v;
	//Triangle *t;

	if(vec_ten_mode == 0) {
		for(int i = 0; i < nverts; i++) {
			v = vlist[i];

			//temp_vec = vlist[i]->vec_fld * stepsize;
			temp_vec = v->vec_fld * v->stepsize;
			vec_len = length(temp_vec);

			if(vec_len > dmax)  {
					//temp_vec = v->vec_fld * (dmax / vec_len) * stepsize;
					temp_vec = v->vec_fld * (dmax / vec_len) * v->stepsize;
			}

			v->new_ts[0][0] = v->x + temp_vec.entry[0];
			v->new_ts[0][1] = v->y + temp_vec.entry[1];
			v->new_ts[1][0] = v->x - temp_vec.entry[0];
			v->new_ts[1][1] = v->y - temp_vec.entry[1];
		}
	}
	else {
		for(int i = 0; i < nverts; i++) {
			v = vlist[i];

			if(v->ten_fld != 0.0) {
				double angle = atan2(v->ten_fld.entry[0][1], v->ten_fld.entry[0][0])*0.5;
				v->vec_fld.set(cos(angle), sin(angle));
			}
			else {
				v->vec_fld.set(0.0, 0.0);
			}

			double len = sqrt(v->ten_fld.entry[0][0]*v->ten_fld.entry[0][0]+v->ten_fld.entry[0][1]*v->ten_fld.entry[0][1]);
			if (len > 0.0)
				v->ten_fld *= 1.0/len;
		}

		for(int i = 0; i < nverts; i++) {
			v = vlist[i];

			//temp_vec = vlist[i]->vec_fld * stepsize;
			temp_vec = v->vec_fld * v->stepsize;
			vec_len = length(temp_vec);

			if(vec_len > dmax) {
				//temp_vec = vlist[i]->vec_fld * (dmax / vec_len) * stepsize;
				temp_vec = v->vec_fld * (dmax / vec_len) * v->stepsize;
			}

			if(temp_vec.entry[0] >= 0.0) {
				v->new_ts[0][0] = v->x + temp_vec.entry[0];
				v->new_ts[0][1] = v->y + temp_vec.entry[1];
				v->new_ts[2][0] = v->x - temp_vec.entry[1];
				v->new_ts[2][1] = v->y + temp_vec.entry[0];
				v->new_ts[4][0] = v->x - temp_vec.entry[0];
				v->new_ts[4][1] = v->y - temp_vec.entry[1];
				v->new_ts[6][0] = v->x + temp_vec.entry[1];
				v->new_ts[6][1] = v->y - temp_vec.entry[0];
			}
			else {
				v->new_ts[0][0] = v->x - temp_vec.entry[0];
				v->new_ts[0][1] = v->y - temp_vec.entry[1];
				v->new_ts[2][0] = v->x + temp_vec.entry[1];
				v->new_ts[2][1] = v->y - temp_vec.entry[0];
				v->new_ts[4][0] = v->x + temp_vec.entry[0];
				v->new_ts[4][1] = v->y + temp_vec.entry[1];
				v->new_ts[6][0] = v->x - temp_vec.entry[1];
				v->new_ts[6][1] = v->y + temp_vec.entry[0];
			}

			if (temp_vec.entry[1] >= 0.0) {
				v->new_ts[1][0] = v->x + temp_vec.entry[0];
				v->new_ts[1][1] = v->y + temp_vec.entry[1];
				v->new_ts[3][0] = v->x - temp_vec.entry[1];
				v->new_ts[3][1] = v->y + temp_vec.entry[0];
				v->new_ts[5][0] = v->x - temp_vec.entry[0];
				v->new_ts[5][1] = v->y - temp_vec.entry[1];
				v->new_ts[7][0] = v->x + temp_vec.entry[1];
				v->new_ts[7][1] = v->y - temp_vec.entry[0];
			}
			else {
				v->new_ts[1][0] = v->x - temp_vec.entry[0];
				v->new_ts[1][1] = v->y - temp_vec.entry[1];
				v->new_ts[3][0] = v->x + temp_vec.entry[1];
				v->new_ts[3][1] = v->y - temp_vec.entry[0];
				v->new_ts[5][0] = v->x + temp_vec.entry[0];
				v->new_ts[5][1] = v->y + temp_vec.entry[1];
				v->new_ts[7][0] = v->x - temp_vec.entry[1];
				v->new_ts[7][1] = v->y + temp_vec.entry[0];
			}
		}
	}

	/*
	if(ignore_at_singularity) {
		for(int i = 0; i < ntris; i++) {
			t = tlist[i];

			if(t->the_singularity != NULL) {
				for(int j = 0; j < 3; j++) {
					v = t->verts[j];

					switch(vec_ten_mode) {
						case 0:
							for(int k = 0; k < 2; k++) {
								v->new_ts[k][0] = v->x;
								v->new_ts[k][1] = v->y;
							}

							break;

						case 1:
							for(int k = 0; k < 8; k++) {
								v->new_ts[k][0] = v->x;
								v->new_ts[k][1] = v->y;
							}

							break;
					}
				}
			}
		}
	}
	*/
}

void Polyhedron::determine_singularities()
{
	int i, j;
	icVector2 temp_vec;
	icMatrix2x2 temp_m;
	icVector2 solution;
	double bary_a, bary_b;  // barycentric coordinates of (0, 0), (x1, 0), and (x2, y2) for critical point solution (temp_t->vec_x, temp_t->vec_y)
	double temp;
	double sol[3];
	icMatrix2x2 temp_tensor[3];
	unsigned int temp_index;
	Triangle *temp_t;
	Vertex *temp_v0, *temp_v1, *temp_v2;
	double temp1, deter, deter2;

	singularities->finalize();
	singularities = new SingularityList(20);

	if (vec_ten_mode == 0) {
		for (i=0; i<ntris; i++) {
			temp_t = tlist[i];
			temp_t->the_singularity = NULL;
			temp_v0 = temp_t->verts[0];
			temp_v1 = temp_t->verts[1];
			temp_v2 = temp_t->verts[2];

			temp_t->vecten_c = dot(temp_v0->vec_fld, temp_t->x_dir);
			temp_t->vecten_f = dot(temp_v0->vec_fld, temp_t->y_dir);

			temp_vec = 1.0/temp_t->x1*(temp_v1->vec_fld-temp_v0->vec_fld);

			temp_t->vecten_a = dot(temp_vec, temp_t->x_dir);
			temp_t->vecten_d = dot(temp_vec, temp_t->y_dir);

			temp_vec = 1.0/temp_t->y2*((temp_v2->vec_fld-temp_v0->vec_fld)-temp_t->x2*temp_vec);

			temp_t->vecten_b  = dot(temp_vec, temp_t->x_dir);
			temp_t->vecten_e = dot(temp_vec, temp_t->y_dir);

			temp_index = temp_t->quick_index(0);
			if (temp_index != 1 && temp_index != -1) 
				continue;
			temp_m.set(temp_t->vecten_a, temp_t->vecten_b, temp_t->vecten_d, temp_t->vecten_e);
			solution = inverse(temp_m)*icVector2(-temp_t->vecten_c, -temp_t->vecten_f);
			temp_t->get_bary_coor(solution, &bary_a, &bary_b);
			if (bary_a > 1.0e-6 && bary_b > 1.0e-6 && 1.0-bary_a-bary_b > 1.0e-6) {
				Singularity *ss = new Singularity();
				ss->index = singularities->num_singularities;
				ss->tri = tlist[i];
				ss->vecten_index = temp_index;
				ss->pos = solution;
				ss->vec_ten_mode = 0;

				deter = determinant(temp_m);		
				if (deter>1.0e-12) {
					deter2 = (temp_t->vecten_a+temp_t->vecten_e)*(temp_t->vecten_a+temp_t->vecten_e) - 4.0*deter;
					if (deter2 >= 0.0) {
						ss->eig_value[0].set(0.5*((temp_t->vecten_a+temp_t->vecten_e)+sqrt(deter2)), 0.0);
						ss->eig_value[1].set(0.5*((temp_t->vecten_a+temp_t->vecten_e)-sqrt(deter2)), 0.0);

						if (ss->eig_value[0].r > 0.0) {  // source
							ss->singularity_type = 0;
						} else {  // sink
							ss->singularity_type = 1;
						}
					} else {
						ss->eig_value[0].set(0.5*(temp_t->vecten_a+temp_t->vecten_e), sqrt(-deter2)*0.5);
						ss->eig_value[1].set(0.5*(temp_t->vecten_a+temp_t->vecten_e), -sqrt(-deter2)*0.5);
						if (ss->eig_value[0].r == 0.0) {  // pure center
							ss->singularity_type = 2;
						} else if (ss->eig_value[0].r > 0.0)  {  // repelling focus
							ss->singularity_type = 3;
						} else {  // attracting focus
							ss->singularity_type = 4;
						}
					}
				}
				else if (deter<-1.0e-12) {
					ss->singularity_type = 5;
					
					// eigen[0] is positive, and eigen[1] is negative
					deter2 = (temp_t->vecten_a+temp_t->vecten_e)*(temp_t->vecten_a+temp_t->vecten_e) - 4.0*deter;
					ss->eig_value[0].set(0.5*((temp_t->vecten_a+temp_t->vecten_e)+sqrt(deter2)), 0.0);
					ss->eig_value[1].set(0.5*((temp_t->vecten_a+temp_t->vecten_e)-sqrt(deter2)), 0.0);

					// calculate eigenvectors using local frame
					if (fabs(temp_t->vecten_b)<1.0e-12 &&  fabs(temp_t->vecten_d)<1.0e-12) {
						if (temp_t->vecten_a > 0.0) {
							ss->eig_dir[0].set(1.0, 0.0);
							ss->eig_dir[1].set(0.0, 1.0);
						} else {
							ss->eig_dir[1].set(1.0, 0.0);
							ss->eig_dir[0].set(0.0, 1.0);
						}
					} else if (fabs(temp_t->vecten_b) > fabs(temp_t->vecten_d)) {
						temp1 = ss->eig_value[0].r-temp_t->vecten_a;
						temp = 1.0/sqrt(temp_t->vecten_b*temp_t->vecten_b+temp1*temp1);
						ss->eig_dir[0].set(temp_t->vecten_b*temp, temp1*temp);

						temp1 = ss->eig_value[1].r-temp_t->vecten_a;
						temp = 1.0/sqrt(temp_t->vecten_b*temp_t->vecten_b+temp1*temp1);
						ss->eig_dir[1].set(temp_t->vecten_b*temp, temp1*temp);
					} else {
						temp1 = ss->eig_value[0].r-temp_t->vecten_e;
						temp = 1.0/sqrt(temp_t->vecten_d*temp_t->vecten_d+temp1*temp1);
						ss->eig_dir[0].set(temp1*temp, temp_t->vecten_d*temp);

						temp1 = ss->eig_value[1].r-temp_t->vecten_e;
						temp = 1.0/sqrt(temp_t->vecten_d*temp_t->vecten_d+temp1*temp1);
						ss->eig_dir[1].set(temp1*temp, temp_t->vecten_d*temp);
					}
				}
				ss->calc_display_attributes();
				singularities->append(ss);
				temp_t->the_singularity = ss;
			} 
		}

	}
	else {
		int total_index = 0, total_pos = 0, total_neg = 0;
		for (i=0; i<ntris; i++) {
			tlist[i]->the_singularity = NULL;
			temp_index = tlist[i]->quick_index(1);
			for (j=0; j<3; j++) 
				tlist[i]->convert_global_to_local_tensor(tlist[i]->verts[j]->ten_fld, &temp_tensor[j]);
			tlist[i]->vecten_c = temp_tensor[0].entry[0][0];
			tlist[i]->vecten_f = temp_tensor[0].entry[0][1];
			tlist[i]->vecten_a = (temp_tensor[1].entry[0][0]-tlist[i]->vecten_c)/tlist[i]->x1;
			tlist[i]->vecten_d = (temp_tensor[1].entry[0][1]-tlist[i]->vecten_f)/tlist[i]->x1;
			tlist[i]->vecten_b = (temp_tensor[2].entry[0][0]-tlist[i]->vecten_c-tlist[i]->vecten_a*tlist[i]->x2)/tlist[i]->y2;
			tlist[i]->vecten_e = (temp_tensor[2].entry[0][1]-tlist[i]->vecten_f-tlist[i]->vecten_d*tlist[i]->x2)/tlist[i]->y2;
			tlist[i]->the_singularity = NULL;
			if (temp_index != 1 && temp_index != -1) {
				tlist[i]->clicked = 0;
				continue;
			}

			temp_m.set(tlist[i]->vecten_a, tlist[i]->vecten_b, tlist[i]->vecten_d, tlist[i]->vecten_e);
			solution = inverse(temp_m)*icVector2(-tlist[i]->vecten_c, -tlist[i]->vecten_f);
			tlist[i]->get_bary_coor(solution, &bary_a, &bary_b);
			if (bary_a > 1.0e-9 && bary_b > 1.0e-9 && 1.0-bary_a-bary_b > 1.0e-9) {
				Singularity *ss = new Singularity();
				ss->index = singularities->num_singularities;
				ss->tri = tlist[i];
				ss->vecten_index = temp_index;

				ss->separatrices_major->finalize();
				ss->separatrices_minor->finalize();
				
				ss->separatrices_major = new SeparatricesList(3);
				ss->separatrices_minor = new SeparatricesList(3);

				ss->pos = solution;
				ss->vec_ten_mode = 1;
				if (temp_index == 1) {
					total_pos++;
				}
				if (temp_index == -1) {
					total_neg++;
				}
				if (temp_index == 1)
					ss->singularity_type = 0;
				if (temp_index == -1)
					ss->singularity_type = 5;
				if (fabs(tlist[i]->vecten_e) < fabs(tlist[i]->vecten_d)) {
					SolveCubic(-tlist[i]->vecten_d, 2.0*tlist[i]->vecten_a-tlist[i]->vecten_e, tlist[i]->vecten_d+2.0*tlist[i]->vecten_b, tlist[i]->vecten_e, &ss->num_tensor_dir, sol);
					for (int pp=0; pp<ss->num_tensor_dir; pp++) 
						ss->tensor_dir[pp].set(sol[pp], 1.0);
				} else {
					SolveCubic(tlist[i]->vecten_e, tlist[i]->vecten_d+2.0*tlist[i]->vecten_b, 2.0*tlist[i]->vecten_a-tlist[i]->vecten_e, -tlist[i]->vecten_d, &ss->num_tensor_dir, sol);
					for (int pp=0; pp<ss->num_tensor_dir; pp++) 
						ss->tensor_dir[pp].set(1.0, sol[pp]);
				}
				for (int pp=0; pp<ss->num_tensor_dir; pp++) {
					normalize(ss->tensor_dir[pp]);
					temp = (tlist[i]->vecten_a*ss->tensor_dir[pp].entry[0]*ss->tensor_dir[pp].entry[0]*ss->tensor_dir[pp].entry[0]
							 +	(tlist[i]->vecten_b+2.0*tlist[i]->vecten_d)*ss->tensor_dir[pp].entry[0]*ss->tensor_dir[pp].entry[0]*ss->tensor_dir[pp].entry[1]
							 +	(2.0*tlist[i]->vecten_e-tlist[i]->vecten_a)*ss->tensor_dir[pp].entry[0]*ss->tensor_dir[pp].entry[1]*ss->tensor_dir[pp].entry[1]
							 -	tlist[i]->vecten_b*ss->tensor_dir[pp].entry[1]*ss->tensor_dir[pp].entry[1]*ss->tensor_dir[pp].entry[1]);
					if (temp < 0.0)
						ss->tensor_dir[pp] = -ss->tensor_dir[pp];
				}
				ss->calc_display_attributes();
				singularities->append(ss);
				tlist[i]->the_singularity = ss;
			}
			total_index += temp_index;
		}
	}
}	

void Polyhedron::singularity_pair_cancellation_field(int cp1, int cp2)
{
	int i, result, sid, tid;

	if (singularities->singularities[cp1]->vecten_index * singularities->singularities[cp2]->vecten_index != -1)
		return;
	if (singularities->singularities[cp1]->vecten_index == -1) {
		sid = singularities->singularities[cp1]->tri->index;
		tid = singularities->singularities[cp2]->tri->index;
	} else {
		sid = singularities->singularities[cp2]->tri->index;
		tid = singularities->singularities[cp1]->tri->index;
	}
	region_verts->finalize();
	region_boundary->finalize();
	region_verts = new VertexList(20);
	region_boundary = new CornerList(20);
	int	original_vec_ten_mode = vec_ten_mode;
	if (original_vec_ten_mode == 1) {
		vec_ten_mode = 0;
		convert_tenfld_to_vecfld_tool();
		analyze_field(0, -1);
		for (i=0; i<nverts; i++)
			vlist[i]->undo_ten_fld = vlist[i]->ten_fld;
	}
	for (i=0; i<nverts; i++) 
		vlist[i]->undo_vec_fld = vlist[i]->vec_fld;

	double div = tlist[tid]->vecten_a + tlist[tid]->vecten_e, curl = tlist[tid]->vecten_b  - tlist[tid]->vecten_d;
	double rotate_angle = atan2(curl, div);
	result = singularity_pair_cancellation(sid, tid, rotate_angle);
	if (result != 1) {
		for (i=1; i<3; i++) {
			result = singularity_pair_cancellation(sid, tid, rotate_angle + i*PI/12);
			if (result==1)
				break;
			result = singularity_pair_cancellation(sid, tid, rotate_angle - i*PI/12);
			if (result==1)
				break;
		}
	}
	if (original_vec_ten_mode == 0) {
		analyze_field(1, -1);
	} else {
		vec_ten_mode = 1;
		convert_vecfld_to_tenfld();
	}

	calc_new_texture_coors();
}

void Polyhedron::singularity_movement_field(int cp1, int tid)
{
	int i, result, sid, is_saddle = 0; 

	if (tlist[tid]->the_singularity != NULL)
		return;
	sid = singularities->singularities[cp1]->tri->index;
	if (singularities->singularities[cp1]->vecten_index == -1) 
		is_saddle = 1;
	region_verts->finalize();
	region_boundary->finalize();
	region_verts = new VertexList(20);
	region_boundary = new CornerList(20);

	int original_vec_ten_mode = vec_ten_mode;
	if (original_vec_ten_mode == 1) {
		vec_ten_mode = 0;
		convert_tenfld_to_vecfld_tool();
		analyze_field(0, -1);
		for (i=0; i<nverts; i++)
			vlist[i]->undo_ten_fld = vlist[i]->ten_fld;
	}
	if (is_saddle == 1) {
		global_reflect_field_tool();
		analyze_field(0, -1);
	}
	for (i=0; i<nverts; i++)
		vlist[i]->undo_vec_fld = vlist[i]->vec_fld;

	double div = tlist[tid]->vecten_a + tlist[tid]->vecten_e, curl = tlist[tid]->vecten_b  - tlist[tid]->vecten_d;
	double rotate_angle = atan2(curl, div);

	result = singularity_movement(sid, tid, rotate_angle);
	if (result != 1) {
		for (i=1; i<3; i++) {
			result = singularity_movement(sid, tid, rotate_angle+i*PI/12);
			if (result==1)
				break;
			result = singularity_movement(sid, tid, rotate_angle-i*PI/12);
			if (result==1)
				break;
		}
	}
	if (is_saddle == 1)
		global_reflect_field_tool();
	if (original_vec_ten_mode == 0) {
		analyze_field(1, -1);
	} else {
		vec_ten_mode = 1;
		convert_vecfld_to_tenfld();
	}

	calc_new_texture_coors();
}


void Polyhedron::copySingularElementList(SingularElementList **list) {
	*list = new SingularElementList(*singularelements);
}

void Polyhedron::copyRegularElementList(RegularElementList **list) {
	*list = new RegularElementList(*regularelements);
}

void Polyhedron::setSingularElementList(SingularElementList *list) {
	singularelements->finalize();

	if(list) 
		singularelements = new SingularElementList(*list);
	else
		singularelements = new SingularElementList(20);
}

void Polyhedron::setRegularElementList(RegularElementList *list) {
	regularelements->finalize();

	if(list)
		regularelements = new RegularElementList(*list);
	else
		regularelements = new RegularElementList(20);
}
